;;; $Id: alphat.scm,v 1.2 1994/10/04 21:09:36 queinnec Exp $

;;; This file offers a transformation to rename all local variables.
;;; This is a kind of alphatization. In fact, objectify already did
;;; that alphatization implicitly since variables are converted to
;;; different instances.  The currentx transformation gives explicit
;;; numbers for variables so it is easy for a compiler to generate
;;; different names.

;;; Exportations:
;;;                   (alphatize! Program) : Program
;;; end of exportations.

(define (alphatize! o)
  (local-alphatize! o '()) )

;;; Renamed variable have their initial name and are suffixed with an
;;; index (a natural number).

(define-class Renamed-Local-Variable Local-Variable 
  ( index       ; a natural number.
    ) )

;;; The clone function is used to rename variables.

(let ((counter 0))
  (define-method (clone (o Local-Variable))
    (make-Renamed-Local-Variable 
     (Local-Variable-name o)
     (Local-Variable-mutable? o)
     (Local-Variable-dotted? o)
     (begin (set! counter (+ counter 1))
            counter ) ) ) )

;;; R is an Alist that maps old local variables to new renamed ones.

(define-generic (local-alphatize! (o Program) r)
  (update-walk! local-alphatize! o r) )

;;; These first methods are sufficient if you perform the transformation
;;; right after the objectification. If you do it later you will need
;;; to add some more methods.

(define-method (local-alphatize! (o Local-Reference) r)
  (let* ((var (Local-Reference-variable o))
         (p (assq var r)) )
    (when (pair? p)
      (set-Local-Reference-variable! o (cdr p)) )
    o ) )

(define-method (local-alphatize! (o Function) r)
  (let* ((vars (Function-variables o))
         (new-vars (map clone vars)) )
    (set-Function-variables! o new-vars)
    (set-Function-body! 
     o (local-alphatize! 
        (Function-body o) (append (map cons vars new-vars) r) ) )
    o ) )

(define-method (local-alphatize! (o Fix-Let) r)
  (let* ((vars (Fix-Let-variables o))
         (new-vars (map clone vars)) )
    (set-Fix-Let-variables! o new-vars)
    (set-Fix-Let-arguments!
     o (local-alphatize! (Fix-Let-arguments o) r) )
    (set-Fix-Let-body! 
     o (local-alphatize! 
        (Fix-Let-body o) (append (map cons vars new-vars) r) ) )
    o ) )

;;; Add some more methods in case they are needed ?!

;(define-method (local-alphatize! (o Box-Creation) r)
;  (let ((var (Box-Creation-variable o))
;        (p (assq var r)) )
;    (when (pair? p)
;      (set-Box-Creation-variable! o (cdr p)) )
;    (call-next-method) ) )

;(define-method (local-alphatize! (o Preallocated-Fix-Let) r)
;  (let* ((vars (Fix-Let-variables o))
;         (new-vars (map clone vars))
;         (temps (Preallocated-Fix-Let-temporaries o))
;         (new-temps (map clone temps)) )
;    (set-Preallocated-Fix-Let-temporaries! o new-temps)
;    (set-Fix-Let-variables! o new-vars)
;    (set-Fix-Let-arguments!
;     o (local-alphatize! (Fix-Let-arguments o) r) )
;    (set-Fix-Let-body! 
;     o (local-alphatize! 
;        (Fix-Let-body o) (append (map cons vars new-vars) r) ) )
;    o ) )

;;; end of rename.scm
