;;; $Id: colltemp.scm,v 1.3 1994/10/04 21:09:36 queinnec Exp $

;;; Exportations:
;;;        (gather-temporaries! Flattened-Program) : Flattened-Program
;;; end of exportations.

;;; Collect local variables to allocate them in one go. Any local
;;; variables of any let form internal to a lambda is collected in
;;; order to be preallocated when the lambda is invoked (and not when
;;; the let form is evaluated). This transformation must be done after
;;; lifting.

;;; Basically a function definition such as the left one becomes the right 
;;; one where &aux is the Common Lisp lambda keyword.
;;;   (lambda ( ... )                 (lambda ( ... &aux x ...)
;;;     ...                             ...
;;;     (let ((x ...)) ... )            (begin (set! x ...) ... )
;;;     ... )                           ... )
;;; In fact the Fix-Let node is turned into a Preallocated-Fix-Let node.
;;; Pay attention to name clashes.

;;; NOTE: To perform this transformation clashes wrt call/cc! So if
;;; you want to be purely Scheme, don't do this or, alternatively, box
;;; every variable.

;;;==================================================================
;;; New classes.

(define-class With-Temp-Function Function
  ( temporaries      ; a list of Local-Variables
    ) )

;;; A Preallocated-Let is similar to a Fix-Let except that its
;;; semantics states that its local variables are already allocated by
;;; the most enclosing lambda form.

(define-class Preallocated-Fix-Let Fix-Let ())

;;;==================================================================
;;; Collect temporary variables into functions.

(define (gather-temporaries! o)
  (collect-temporaries! o #f) )

;;; Collect all temporary variables from O. Store these variables in WTFUN. 

(define-generic (collect-temporaries! (o Program) wtfun)
  (update-walk! collect-temporaries! o wtfun) )

;;; Replace Functions with With-Temp-Functions. Recurse on body.

(define-method (collect-temporaries! (o Function) wtfun)
  (let ((newfun (make-With-Temp-Function
                 (Function-variables o)
                 (Function-body o)
                 '() )))
    (set-Function-body! newfun 
                        (collect-temporaries! (Function-body o) newfun) )
    newfun ) )

;;; For now, only variables in Let are considered as temporaries. The
;;; Fix-Let form is turned into a Preallocated-Fix-Let.

(define-method (collect-temporaries! (o Fix-Let) wtfun)
  (adjoin-temporary-variables! wtfun (Fix-Let-variables o))
  (make-Preallocated-Fix-Let 
   (Fix-Let-variables o)
   (collect-temporaries! (Fix-Let-arguments o) wtfun)
   (collect-temporaries! (Fix-Let-body o) wtfun) ) )

;;; Adjoin temporary variables to the next enclosing lambda.  Don't
;;; try to avoid duplicates, variables are all different since
;;; objectification alphatizes them.

(define (adjoin-temporary-variables! wtfun newvars)
  (let ((temps (With-Temp-Function-temporaries wtfun)))
    (set-With-Temp-Function-temporaries! 
     wtfun (append newvars temps) ) ) )

;;; end of colltemp.scm
