;;; $Id: extract.scm,v 1.3 1994/09/30 19:41:39 queinnec Exp $

;;; Exportations:
;;;          (extract-things! Program) : Flattened-Program
;;;          some classes.
;;; end of exportations.

;;; Collect all functions and quotations and store them in the fields
;;; of a Flattened-Program object. This object contains anything
;;; needed to understand the program. This should be done after the 
;;; collect-temporary variables analyzis.

;;;==================================================================
;;; New classes.

;;; Keep a reference to the the original function definition.

(define-class Closure-Creation Form
  ( definition         ; the associated Function-Definition
    free               ; a Free-Environment listing all closed variables
    ) )

;;; Definitions of functions are indexed within a Flattened-Program
;;; instance. They have a structure similar to Flat-Function except
;;; that their Free field contains a (real) list of variables (to
;;; avoid walking them).

(define-class Function-Definition Form
  ( variables          ; list of local variables
    body               ; a Form
    temporaries        ; list of temporary variables
    free               ; list of free variables
    index              ; Every closure is given an index (a natural)
    ) )

;;; Quotations are collected then referred to through global variables.
;;; The 'name' of a Quotation-Variable is a natural number, the value field
;;; contains the associated quoted value.

(define-class Quotation-Variable Variable 
  ( value              ; a Scheme value.
    ) )

;;; This class describes a transformed program with lifted quotations
;;; and functions.

(define-class Flattened-Program Program 
  ( body               ; the body of the program (a Form)
    quotations         ; list of Quotation-Variables
    definitions        ; list of Function-Definitions
    ) )

;;;==================================================================
;;; Extract quotations and functions definitions to the highest level.

(define (extract-things! o)
  (let ((flatprg (make-Flattened-Program o '() '())))
    (set-Flattened-Program-body! flatprg (extract! o flatprg))
    flatprg ) )

(define-generic (extract! (o Program) flatprg)
  (update-walk! extract! o flatprg) )

(define-method (extract! (o Constant) flatprg)
  (let* ((qv* (Flattened-Program-quotations flatprg))
         (qv  (make-Quotation-Variable (length qv*) (Constant-value o))) )
    (set-Flattened-Program-quotations! flatprg (cons qv qv*))
    (make-Global-Reference qv) ) )

(define-method (extract! (o Flat-Function) flatprg)
  (set-Flat-Function-body! o (extract! (Flat-Function-body o) flatprg))
  (let* ((freevars (let extract ((free (Flat-Function-free o)))
                     (if (Free-Environment? free)
                         (cons (Reference-variable 
                                (Free-Environment-first free) )
                               (extract (Free-Environment-others free)) )
                         '() ) ))
         (variables (Flat-Function-variables o))
         (body (Flat-Function-body o))
         (aux (Flat-Function-temporaries o))
         (newdef (adjoin-definition! flatprg 
                                     variables body aux freevars )) )
    (make-Closure-Creation newdef (Flat-Function-free o)) ) )

;;; May try to share Function-Definitions.                      FUTURE

(define (adjoin-definition! flatprg variables body aux free)
  (let* ((definitions (Flattened-Program-definitions flatprg))
         (newindex    (length definitions))
         (newdef      (make-Function-Definition 
                       variables body aux free newindex )) )
    (set-Flattened-Program-definitions! flatprg (cons newdef definitions))
    newdef ) )

;;; end of extract.scm
