;;; $Id: predenv.scm,v 1.2 1994/10/24 19:36:21 queinnec Exp $

;;; This file contains the definition of the environment that are used
;;; for compilation as well as some utilities to access them.

;;; Exportations:
;;;       (extend-environment r variables) : Local-Environment
;;;       (find-variable? name g) : Reference
;;;       (find-predefined-variable? name g) : Predefined-Variable
;;; end of exportations.

;;;==================================================================
;;; Environment classes.

(define-class Environment Object () :virtual)

;;; Actually `predefined' and `global' are lists of variables
;;; Use hash-table to speedup searching in the global environment    FUTURE

(define-class Global-Environment Environment
  ( predefined     ; predefined global variables 
    global         ; user-defined global variables 
    ) )

;;; It is not useful to use a hash-table for local variables.

(define-class Local-Environment Environment
  ( variables      ; local variables (a list of Local-Variable)
    ) )

;;;==================================================================
;;; Operations on environments.

;;; extend a local environment with a list of local variables.

(define-generic (extend-environment (r Environment) variables))

(define-method (extend-environment (r Local-Environment) variables)
  (make-Local-Environment
   (append variables (Local-Environment-variables r)) ) )

;;; find a variable with a given name in the local environment or in
;;; the global environment. In that latter case, first try the
;;; user-global mutable environment then the predefined immutable one.

(define-generic (find-variable? name (r Environment)))

(define-method (find-variable? name (r Local-Environment))
  (let scan ((vars (Local-Environment-variables r)))
    (if (pair? vars)
        (if (eq? name (Variable-name (car vars)))
            (make-Local-Reference (car vars))
            (scan (cdr vars)) )
        #f ) ) )

(define-method (find-variable? name (g Global-Environment))
  (define (scan vars reference-maker)
    (if (pair? vars)
        (if (eq? name (Variable-name (car vars)))
            (reference-maker (car vars))
            (scan (cdr vars) reference-maker) )
        #f ) )
  (or (scan (Global-Environment-global g) make-Global-Reference)
      (scan (Global-Environment-predefined g) make-Predefined-Reference) ) )

;;; Sometimes one has to find predefined variables independently of
;;; the local or global environment. This is the case for `unspecified'
;;; and `cons' for some transformation rules (see objify.scm)

(define-generic (find-predefined-variable? name (g Global-Environment)))

(define-method (find-predefined-variable? name (g Global-Environment))
  (define (scan vars)
    (if (pair? vars)
        (if (eq? name (Variable-name (car vars)))
            (car vars)
            (scan (cdr vars)) )
        #f ) )
  (scan (Global-Environment-predefined g)) )

;;; end of predenv.scm
