;;; $Id: progr.scm,v 1.6 1995/06/26 12:16:25 queinnec Exp $

;;; This file defines all the classes to represent programs as they are
;;; produced by objify.scm.

;;; Exportation: 
;;;                all classes and related functions.
;;; End of exportations.

;;; Variables are not Programs! They represent bindings. This
;;; distinction is needed by the meta-generic code-walker that only
;;; recurses on instances of programs (see walk.scm file).

(define-class Variable Object 
  ( name ; Name of the variable (a symbol or a string or a number
         ; depending on the exact subclass).
    ) )

(define-class Global-Variable Variable 
  ( initialized?  ; Boolean (tells if the variable surely has a value 
                  ; before being read).
    ) )

(define-class Predefined-Variable Variable 
  ( description ; a description of a predefined variable (an instance
                ; of class description)
    ) )

(define-class Local-Variable Variable 
  ( mutable?      ; Boolean (tells if the variable is assigned)
    dotted?       ; Boolean (tells if the variable is dotted as Z in
                  ; the (lambda (x y . z) 33) abstraction)
    ) )

;;; Descriptions for predefined bindings, these objects will describe
;;; the value of the binding. 

(define-class Description Object () :virtual)

(define-class Functional-Description Description
  ( comparator    ; Function (in fact, = or >=) Used to compare the number
                  ; of terms in an application and the real arity of the 
                  ; function currently described.
    arity         ; Natural (the required number of arguments)
    ) )

(define-class Constant-Description Description
  ( value         ; ???
    ) )

;;; The Program class will be enriched later with new fields. It
;;; represents a full program not a local expression.

(define-class Program Object () :virtual)

;;; The Form class is the mother class of all expressions.

(define-class Form Program () :virtual)

;;; This is the superclass of all references.

(define-class Reference Form 
  ( variable         ; a Variable
    ) :virtual )

(define-class Local-Reference Reference ())

(define-class Global-Reference Reference ())

(define-class Predefined-Reference Reference ())

(define-class Assignment Form
  ( form
    ) :virtual )

(define-class Local-Assignment Assignment
  ( reference 
    ) )

(define-class Global-Assignment Assignment
  ( variable      ; a Global-Variable instance.
    ) )

(define-class Function Form 
  ( variables
    body
    ) )

(define-class Alternative Form 
  ( condition 
    consequent 
    alternant
    ) )

(define-class Sequence Form 
  ( first
    last
    ) )

(define-class Constant Form 
  ( value
    ) )

(define-class Application Form 
  ( arguments   ; arguments of the application (instance of Arguments or
                ; No-Arguments)
    ) :virtual )

(define-class Regular-Application Application 
  ( function
    ) )

(define-class Predefined-Application Application 
  ( variable
    ) )

(define-class Fix-Let Form 
  ( variables
    arguments
    body
    ) )

(define-class Arguments Form 
  ( first
    others
    ) )

(define-class No-Argument Form ())

(define-class Monitoring Form
  ( handler
    body 
    ) )

(define-class Escaper Form
  ( variable
    body
    ) )

;;; end of progr.scm
