/* $Id: dmports.c,v 1.6 1997/08/22 12:05:05 queinnec Exp $  
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////

 * DOCUMENTATION dmports.c
 * This file replaces the runtime1.8/Clib/ports.c file of Bigloo.
 * It allows Bigloo to serve DMeroon requests while waiting on a
 * C stream. It must be compiled and considered by the linker before
 * the libbigloo.a (to shadow the equivalent ports.o file). 
 */

#include "dmeroon.h"

DM_DefineVersionString(dmports,
  "$Id: dmports.c,v 1.6 1997/08/22 12:05:05 queinnec Exp $");

/* Define a macro to replace fread with.
 * This assumes that the original ports.c file has been converted so
 * that fread was changed into DM_fread.
 */

#define DM_long_duration 100*1000*1000

#define DM_fread(ptr, size, nitems, stream) \
  ( fflush(stdout), fflush(stderr), /* To keep things printed in order. */ \
    DM_wait_until_fd_ready(DMbgl_context, \
                           DM_long_duration, \
                           fileno(stream) ), \
    fread(ptr, size, nitems, stream) )

/* Define a macro to replace fgets with.
 * This assumes that the original ports.c file has been converted so
 * that fgets was changed into DM_fgets.
 */

#define DM_fgets(ptr, size, stream) \
  ( fflush(stdout), fflush(stderr), /* To keep things printed in order. */ \
    DM_wait_until_fd_ready(DMbgl_context, \
                           DM_long_duration, \
                           fileno(stream) ), \
    fgets(ptr, size, stream) )

/* Include the original (in fact, fread was replaced by DM_fread and
 * same for fgets) Bigloo source file. */

#include "ports.c"

/* RESOURCE DM_dmports_initialize
 * This initialisation function allows to check whether that file shadows 
 * the original one from Bigloo library.
 */

void
DM_dmports_initialize (DM_Context *context) 
{
  DM_LinkVersionStringIntoSite(dmports);
}

/* end of dmports.c */
