;;; $Id: macros.scm,v 1.17 1996/08/29 15:33:49 queinnec Exp $

;;; This file contains macros (and only macros) used in the sources of
;;; Icslas. It presupposes a prologue that defines how macros are
;;; defined. This prologue depends on the Scheme interpreter or
;;; compiler chosen to run Icslas. The macros of this file are
;;; themselves customised depending on the current features.

;;; Conditional compilation: The (meta) variable icslas-features
;;; defines the features of Icslas to be taken into account when
;;; loading (or compiling Icslas).  With that macro, we can
;;; conditionalize definitions with boolean expressions as in:
;;;       (or elk (and (not bigloo) Scheme->C)).

(define-icslas-macro (if-icslas-feature condition th-en . el-se)
  ((lambda ()
     (define (feature-present? condition)
       (if (pair? condition) 
           (case (car condition)
             ((and) 
              (if (pair? (cdr condition))
                  (if (pair? (cddr condition))
                      (and (feature-present? (cadr condition))
                           (feature-present? (cons 'and (cddr condition))) )
                      (feature-present? (cadr condition)) )
                  #f ) )
             ((or) 
              (if (pair? (cdr condition))
                  (if (pair? (cddr condition))
                      (or (feature-present? (cadr condition))
                          (feature-present? (cons 'or (cddr condition))) )
                      (feature-present? (cadr condition)) )
                  #t ) )
             ((not) (not (feature-present? (cadr condition))))
             (else (member condition *icslas-features*)) )
           (member condition *icslas-features*) ) )
     (if (feature-present? condition) 
         th-en 
         (if (pair? el-se) (car el-se) '#f) ) ) ) )

;;; I find these macros to be useful and I tend to use them.

(if-icslas-feature (or Bigloo Bigloo++ Icslas Scm Gambit)
  (begin 
    (define-icslas-macro (when condition . body)
      `(if ,condition (begin . ,body) #f) ) 
    (define-icslas-macro (unless condition . body)
      `(if ,condition #f (begin . ,body)) ) )
  )

;;;=================================================================
;;; The sources of Icslas use the monitor form to control exceptions.
;;; Of course it must be defined as a macro in existing Scheme
;;; implementations.  The exception is raised by the diagnose
;;; function. Thanks to Rodget's thesaurus for the suggestion of
;;; the "diagnose" verb. 

(if-icslas-feature (not Icslas)
  (define-icslas-macro (monitor handler . body)
    `(do-monitor ,handler (lambda () . ,body)) )
  )

;;; end of macros.scm
