;;; $Id: monitor.scm,v 1.3 1996/11/17 18:50:38 queinnec Exp $

;;; This file simulates the monitor facility of Icslas in a regular
;;; Scheme implementation. It assumes that there exist
;;;   a protected-eval function
;;; 	(protected-eval thunk) computes the thunk and returns a vector 
;;;     of its value. If an error occurs, it returns a list describing
;;;     the error.
;;;   a protected-error function that signals an error to the underlying 
;;;     Scheme implementation.

;;; Computes thunk under the monitoring of handler.
;;; *current-handler* simulates a dynamic variable.

(define (do-monitor handler thunk)
  (let* ((old *current-handler*)
         (new (lambda (exc c?)
                (set! *current-handler* old)
                (let ((r (handler exc c?)))
                  (cond 
                   (c? r)
                   (else (set! *current-handler* old)
                         (diagnose
                          (make-Underlying-Exception 
                           "Cannot continue a non continuable exception" 
                           exc '() )
                          #f ) ) ) ) )) )
    (set! *current-handler* new)
    (let ((result (protected-eval thunk)))
      (set! *current-handler* old)
      (if (vector? result)
          (vector-ref result 0)
          (new (make-Underlying-Exception 
                "Underlying Scheme exception"
                result '() ) 
               #f ) ) ) ) )

;;; This is the function to signal an exception.

(define (diagnose exception continuable?)
  (let* ((ch *current-handler*)
         (r  (ch exception continuable?)) )
    (set! *current-handler* ch)
    r ) )

;;; This is the initial handler. It simply calls the error mechanism
;;; of the underlying Scheme implementation.

(define *current-handler*
  (lambda (exc c?)
    (protected-error 'diagnose "No current handler" (list exc c?)) ) )

;;; end of monitor.scm
