;;; $Id: boot.scm,v 1.5 1995/06/25 18:00:03 queinnec Exp $

;;; This file defines part of the bootstrap operations.
;;; It is able to process (read, expand, compile) all the sources of Icslas.

;;; NOTES:
;;; If you want to expand Icslas using Meroonet then you need to have Meroonet
;;; present (loaded) so you can expand Meroonet (see the definition of the
;;; define-generic function which uses parse-variable-specifications).

;;;==================================================================
;;; Byte compile any file leaving a .xlo besides any .scm file.

(define (byte-compile-icslas-sources features)
  (monitor byte-compile-icslas-sources-handler
    (call/cc 
     (lambda (exit)
       (define-method (byte-compile-icslas-sources-handler (exc Exception) c?)
         (exit exc) )
       (process-icslas-sources
        features
        (lambda (filename)
          (report-progress "Compiling" filename "...")
          (byte-compile-file 
           (lambda (e) (walk e enriched-icslas-expansion-environment))
           g.icslas 
           (string-append "./" filename ".scm") ) ) ) ) ) ) )
;;; (byte-compile-icslas-sources 
;;;  (append '(Meroonet TraditionalMacros AddLibrary) icslas-features))

(define-generic (byte-compile-icslas-sources-handler (exc) c?))

(define-method 
  (byte-compile-icslas-sources-handler (exc Compilation-Warning) c?)
  (basic-handler exc c?) )

;;;==================================================================
;;; Returns the list of all the expressions of all sources.

(define (read-all-sources features)
  (let ((bodies '()))
    (process-icslas-sources
     features
     (lambda (basefilename)
        ;;(display `("reading" ,basefilename) stderr)
       (let ((e* (file->list (string-append basefilename ".scm"))))
         ;;(newline stderr)
         (set! bodies (append! bodies e*)) ) ) )
    bodies ) )

;;; expand all expressions.
;;; Here they are expanded until turned into essential Scheme that's why
;;; it uses the enriched-icslas-expansion-environment. If one wants to 
;;; use Icslas in regular Scheme, just use the icslas-expansion-environment
;;; instead to remove all Icslas-related macros (when and others) that are
;;; defined with define-icslas-macro.

(define (expand-all-forms e*)
  (define (boot-walk e)
    (enriched-icslas-expansion-environment e) )
  (if (pair? e*)
      (let ((e (car e*)))
        ;;(display `("to be expanded" ,e) stderr)
        (let ((ne (walk e boot-walk)))
          ;;(display `("giving" ,ne) stderr)(newline stderr)(newline stderr)
          (cons ne (expand-all-forms (cdr e*))) ) )
      '() ) )

;;; read, expand, bytecompile all sources.

(define (bytecompile-all-sources features)
  (call/cc 
   (lambda (exit)
     (monitor icslas-handler
       (define-method (icslas-handler (exc Exception) c?)
         (exit "failed bootstrap") )
       (display "Reading..." stderr)
       (let ((e* (read-all-sources features)))
         ;;(list->file e* "o/all.scm")
         (newline stderr)
         (display "Expanding..." stderr)
         (let ((ee* (expand-all-forms e*)))
           (list->file ee* "o/all.expanded.scm")
           (newline stderr)
           (display "ByteCompiling..." stderr)
           (let* ((exp `(begin (begin . ,ee*)
                               (Byte-Code-main) ))
                  (bc  (compile->Byte-Code exp g.icslas)) )
             (newline stderr)
             (set! bytecompiledboot bc)
             bc ) ) ) ) ) ) )

(define bytecompiledboot #f)

;;;==================================================================
;;; The error handler.
;;; It defines new global variables on the fly.

(define-generic (icslas-handler (exc) continuable?)
  (basic-handler exc continuable?) )

;;;==================================================================
;;; Cross fingers...
;;; NOTE: Actually this imposes to use Meroonet in order to expand Meroonet.

(define (boot)
  (byte-compile-icslas-sources
   '(TraditionalMacros AddLibrary ByteCode Meroonet) ) )

;;; end of boot.scm
