;;; $Id: extrdf.scm,v 1.4 1995/07/29 18:02:29 queinnec Exp $

;;; This file allows to extract the names that are defined in a
;;; regular Scheme file. It is used to build interfaces, for instance
;;; modules definition for Bigloo. Not to be dependent of Scheme
;;; implementations, this file only produces the defined names (and
;;; signatures when known) in a signature file: f.sig. This file has
;;; to be processed after (see for instance mkbglmdl.scm).

;;; Caution, this file is evaluated in the TARGET directory!
;;; NOTE: this file must not use macros at all to be portable.

;;; Load the configuration parameters. This configuration file is 
;;; generated in the TARGET directory.

(load "config.scm")

;;;==================================================================
;; Read a whole file into a list of expressions.

(define (file->list filename)
  (call-with-input-file filename
    (lambda (in)
      (let loop ((e (read in)))
        (if (eof-object? e)
            '()
            (cons e (loop (read in))) ) ) ) ) )

;;; This is the function that extracts all defined names from a file
;;; and outputs them in a file with .SIG as extension.

(define (extract-defined-names basefile)
  (let* ((e*      (file->list (string-append basefile ".scm")))
         (sigfile (string-append basefile ".SIG")) )
    (call-with-output-file sigfile
      (lambda (out)
        (display ";;; automatically expanded (Don't touch!)" out)
        (newline out)
        (newline out)
        (for-each
         (lambda (e) 
           (if (and (pair? e) (eq? (car e) 'define))
               (begin (write (cadr e) out)
                      (newline out) ) )
           (display ".") )
         e* )
        (display ";;; eof." out)
        (newline out) ) ) ) )

;;;==================================================================
;;; Load the processor of Icslas sources.

(load (string-append TOP "/icslas.scm"))

;;; Load the precise features describing what must be generated.  This
;;; file should have already be generated in the TARGET/Bootstrap
;;; directory.

(load "Bootstrap/features.scm")

;;;==================================================================
;;; Everything is ready: process every source files (due to
;;; process-icslas-sources, they are processed in order which is very
;;; convenient for state-macros).

(define (work)
  (process-icslas-sources
   icslas-features
   (lambda (basefile)
     (display "extracting names from ")
     (display basefile)
     (extract-defined-names basefile)
     (newline) ) )
  (display "all signatures extracted.")
  (newline) )

;;;Test: (work)

;;; end of extrdf.scm
