;;; $Id: mkbglmdl.scm,v 1.6 1995/08/22 19:13:23 queinnec Exp $

;;; This file takes signatures files and converts them into Bigloo modules.

;;; Caution, this file is evaluated in the TARGET directory!
;;; NOTE: this file must not use macros at all to be portable.

;;; Load the configuration parameters. This configuration file is 
;;; generated in the TARGET directory.

(load "config.scm")

;;;==================================================================
;;; Load some utilities

(load (string-append TOP "/Bootstrap/filutl.scm"))

;;; But redefine %read to be read.

(define %read read)

;;; This is the function that converts signatures into a Bigloo module.

(define (generate-bigloo-module basefile)
  (let* ((e*      (file->list (string-append basefile ".sig")))
         (module  (string-append basefile ".BGL"))
         (scmfile (string-append basefile ".scm"))
         (module-name (extract-filename-sans-extension
                       (extract-basefilename basefile) ))
         (imported-modules (determine-imported-modules *modules* module-name))
         (forced-modules (determine-forced-modules *modules* module-name)) )
    (call-with-output-file module
      (lambda (out)
        (display ";;; automatically expanded (Don't touch!)" out)
        (newline out)
        (write `(module ,(string->symbol module-name)
                   ;; the body of the module
                   (include ,(string-append TARGET "/" scmfile))
                   ;; the Bigloo specific macros emulating Icslas
                   ;; special form (ie the monitor form).
                   (load (macbgl ,(string-append TARGET "/Bigloo/macbgl.bgl")))
                   ;; the support functions for monitor
                   (import (rtbgl ,(string-append TARGET "/Bigloo/rtbgl.bgl")))
                   ;; DMeroon library.
                   (import (dm ,(string-append TARGET "/DMeroon/dm.bgl")))
                   ;; the exportations
                   (export . ,e*)
                   ;; force all previous modules
                   (force . ,forced-modules)
                   ;; import all other modules.
                   (import . ,imported-modules) )
                 out )
        (newline out)
        (for-each 
         (lambda (e) (write e out) (newline out))
         ;; This is the body of the module
         `(  (display ,module-name)
             ) )
        (newline out) ) ) ) )

;;;==================================================================
;;; A module should import all other modules (except itself!).
;;; First all basefiles are kept in the *modules* variable.
;;; But to respect the correct loading order it must force every module
;;; that must be evaluated before it.

(define *modules* '())

(define (determine-imported-modules modules module-name)
  (let make ((modules modules))
    (if (pair? modules)
        (let ((modname (extract-filename-sans-extension
                        (extract-basefilename (car modules)) )))
          (if (string=? module-name modname)
              (make (cdr modules))
              (cons `(,(string->symbol modname)
                      ,(string-append TARGET "/" (car modules) ".bgl") )
                    (make (cdr modules)) ) ) )
        '() ) ) )

(define (determine-forced-modules modules module-name)
  (let make ((modules modules))
    (if (pair? modules)
        (let ((modname (extract-filename-sans-extension
                        (extract-basefilename (car modules)) )))
          (if (string=? module-name modname)
              '()
              (cons (string->symbol modname)
                    (make (cdr modules)) ) ) )
        '() ) ) )

;;;==================================================================
;;; Load the processor of Icslas sources.

(load (string-append TOP "/icslas.scm"))

;;; Load the precise features describing what must be generated. This
;;; file should have already be generated in the TARGET/Bootstrap
;;; directory.

(load "Bootstrap/features.scm")

;;;==================================================================
;;; Two auxiliary files are generated.
;;; Tmp/modules is the sequence of all required modules for Bigloo
;;;             including the Bigloo/rtbgl module
;;; Tmp/main.bgl is the main module of Bigloo.

(define (generate-tmp-modules modules)
  (call-with-output-file "Tmp/modules"
    (lambda (out)
      (for-each (lambda (basefile)
                  (display basefile out)
                  (newline out) )
                modules ) ) ) )

(define (generate-tmp-main-bgl modules)
  (call-with-output-file "Tmp/main.BGL"
    (lambda (out)
      (let ((imports (determine-imported-modules modules "")))
        (write `(module main (main main)
                  (load (macbgl ,(string-append TARGET "/Bigloo/macbgl.bgl")))
                  (import (rtbgl ,(string-append TARGET "/Bigloo/rtbgl.bgl"))
                          (dm ,(string-append TARGET "/DMeroon/dm.bgl"))
                          . ,imports )
                  (force rtbgl dm . ,(map car imports))
                  (include ,(string-append TOP "/Bigloo/main.scm")) )
               out )
        (newline out) ) ) ) )

;;; NOTE: pourquoi ne pas utiliser la mecanique generale avec main     FUTURE
;;; indique dans icslas.scm ???

;;;==================================================================
;;; Everything is ready: process every source files (due to
;;; process-icslas-sources, they are processed in order which is very
;;; convenient for state-macros).

;;; Not only are Bigloo modules produced but also a file mentioning
;;; all of them: TARGET/Tmp/modules and a TARGET/Tmp/main.bgl

(define (work)
  ;; Collect all needed modules (they're needed for importations)
  (process-icslas-sources
   icslas-features
   (lambda (basefile)
     (set! *modules* (cons basefile *modules*)) ) )
  ;; Put module names in order
  (set! *modules* (reverse *modules*))
  ;; Generate auxiliary files
  (generate-tmp-modules *modules*)
  (generate-tmp-main-bgl *modules*)
  ;; Now generate modules
  (process-icslas-sources
   icslas-features
   (lambda (basefile)
     (display "generating Bigloo module for ")
     (display basefile)
     (generate-bigloo-module basefile)
     (newline) ) )
  (display "all Bigloo modules generated.")
  (newline) )

(work)

;;; end of mkbglmdl.scm
