;;; $Id: BC2dm.scm,v 1.4 1995/09/23 18:20:26 queinnec Exp $

;;; This file defines how to convert a ByteCoded module into a DMeroon
;;; object so it can be run by DM_byte_eval. Of course, it requires
;;; DMeroon to be present. It also knows how to store a module into a
;;; file.

;;; Convert a ByteCode module (a Meroonet object) into a DMeroon object.

(define (Byte-Compilation->DMeroon bc)
  (let* ((bytes (Byte-Compilation-bytes bc))
         (byte-length (Byte-Compilation-index bc))
         (quotations (Byte-Compilation-quotations bc))
         (quotation-length (length quotations))
         (frees (Global-Environment-global (Byte-Compilation-global bc)))
         (free-length (length frees))
         (module (DM-allocate3 DM-Module-class
                               byte-length
                               quotation-length
                               free-length )) )
    (DM-local-set DM-reference-type
                  module
                  DM-Module-name-field
                  (scheme->DMeroon (Byte-Compilation-name bc)) )
    (do ((i 0 (+ i 1)))
        ((>= i byte-length))
      (DM-local-indexed-set DM-nat1-type
                            module
                            DM-Module-byte-field
                            i
                            (vector-ref bytes i) ) )
    (do ((i (- quotation-length 1) (- i 1))
         (quotations quotations (cdr quotations)) )
        ((null? quotations))
      (let* ((qv    (car quotations))
             (value (BC-Quotation-Variable-value qv)) )
        (DM-local-indexed-set DM-reference-type
                              module
                              DM-Module-quotation-field
                              i
                              (scheme->DMeroon value) ) ) )
    (do ((i (- free-length 1) (- i 1))
         (frees frees (cdr frees)) )
        ((null? frees))
      (let* ((gv   (car frees))
             (name (symbol->string (Global-Variable-name gv))) )
        (DM-local-indexed-set DM-reference-type
                              module
                              DM-Module-free-field
                              i
                              (scheme->DMeroon name) ) ) )
    (DM-set-initialized module)
    module ) )

;;; This name is imposed since it is used by the byte-compile-file
;;; function from BCdrive.scm file.

(define (Byte-Compilation->file bc target)
  (let ((module (Byte-Compilation->DMeroon bc)))
    (DM-file-out-module module target) ) )

;;; Compile a module and store it in a file.
;;; FUTURE make the buffer be flushed directly in a file..............

(define (icslas-byte-compile-file expand g.init source . target)
  (monitor byte-compile-file-handler
    (let ((name (extract-basefilename
                 (extract-filename-sans-extension source) )))
      (report-progress "Compiling" name "...")
      (let* ((e*     (expand (file->list source)))
             (bc     (compile->Byte-Code name `(begin . ,e*) g.init))
             (module (Byte-Compilation->DMeroon bc))
             (target (if (pair? target) (car target)
                         (byte-compile-compute-target-filename
                          source ".xlo" ) ))
             ;; for now, create a very BIG buffer               FUTURE
             (be (DM-allocate1 DM-OutBuffer-class 10000)) )
        (DM-local-set DM-nat3-type be DM-Buffer-index-field 0)
        (DM-set-initialized be)
        (report-progress "Marshaling" name "...")
        (DM-marshal be module)
        (report-progress "Recording" name "...")
        (call-with-output-file target
          (lambda (out)
            (let ((n (DM-local-get DM-nat3-type be DM-Buffer-index-field)))
              (do ((i 0 (+ i 1)))
                  ((>= i n))
                (write-char
                 (integer->char
                  (DM-local-indexed-get DM-nat1-type
                                        be
                                        DM-Buffer-byte-field
                                        i ) )
                 out ) ) ) ) )
        (report-progress "Done.") ) ) ) )
;;; Test: (icslas-byte-compile-file icslas-expand g.icslas 
;;;         "../../Tmp/single.scm" "Test/single.xlo" )

;;; end of BC2dm.scm
