;;; $Id: BCcode.scm,v 1.3 1995/07/28 12:41:19 queinnec Exp $

;;; This file defines the codop of the virtual ByteCode machine.

;;; Some codops must be sequential. For instance, if a codop that
;;; needs one argument in short form (one byte) has code N, then the
;;; same codop with an argument in long form (two bytes) should have
;;; code N+1, with an argument in long long form (four bytes) should
;;; have code N+2. If N+3 exists then it is the specialized case for a
;;; 0 argument, N+4 for 1 etc. until N+7 for the argument 4.  
;;;   To sum up:
;;;           ---- Bytecodes ---------    -- value of the argument ------------
;;;           N     arg
;;;           N+1   arg1 arg2              256*arg1 + arg2
;;;           N+2   arg1 arg2 arg3 arg4    256*(256*(256*(arg1)+arg2)+arg3)+arg4
;;;           N+3                          0
;;;           N+4                          1
;;;           N+5                          2
;;;           N+6                          3
;;;           N+7                          4

;;; The arrays of instruction names and sizes.

(define *instruction-name* (make-vector 256 'Illegal))

(define *instruction-size* (make-vector 256 255))

;;; The macro to define instructions.

(define-icslas-macro (define-instruction call code)
  (let ((iname (string->symbol
                (apply string-append 
                       (map symbol->string 
                            (list 'BC- (car call) '-code) ) ) )))
    `(begin
       (define ,iname ,code)
       (vector-set! *instruction-name* ,code ',(car call))
       (vector-set! *instruction-size* ,code ,(length call))
       ',(car call) ) ) )

;;; This long list defines instruction names and sizes.

(define-instruction (nop) 0)
(define-instruction (link-env) 1)
(define-instruction (unlink-env) 2)
(define-instruction (return) 3)
(define-instruction (primitive-invoke rank) 4)
(define-instruction (primitive-goto rank) 5)
(define-instruction (pop-function-and-invoke) 6)
(define-instruction (pop-function-and-goto) 7)
(define-instruction (push-value) 9)

(define-instruction (load-nat1 i)  10)
(define-instruction (load-nat2 i j) 11)
(define-instruction (load-nat3 i j k l) 12)
(define-instruction (load-nat4 i j k l m n o p) 13)
(define-instruction (load-negative-nat1 i) 15)
(define-instruction (load-negative-nat2 i j) 16)
(define-instruction (load-negative-nat3 i j k l) 17)
(define-instruction (load-negative-nat4 i j k l m n o p) 18)
(define-instruction (load-char1 i) 19)

(define-instruction (short-global-ref i) 20)
(define-instruction (long-global-ref i j) 21)
(define-instruction (long-long-global-ref i j k l) 22)
(define-instruction (short-quotation-ref i) 25)
(define-instruction (long-quotation-ref i j) 26)
(define-instruction (long-long-quotation-ref i j k l) 26)
(define-instruction (short-predefined-ref i) 30)
(define-instruction (long-predefined-ref i j) 31)
(define-instruction (long-long-predefined-ref i j k l) 32)

(define-instruction (box-ref) 40)
(define-instruction (box-ref-and-check) 41)
(define-instruction (pop-box-and-write) 42)
(define-instruction (boxify-variable index) 43)

(define-instruction (create-closure-and-short-goto+ i) 45)
(define-instruction (create-closure-and-long-goto+ i j) 46)
(define-instruction (create-closure-and-long-long-goto+ i j k l) 47)

(define-instruction (short-shallow-reference index) 50)
(define-instruction (long-shallow-reference index) 51)
(define-instruction (long-long-shallow-reference index) 52)
(define-instruction (shallow-reference0) 53)
(define-instruction (shallow-reference1) 54)
(define-instruction (shallow-reference2) 55)
(define-instruction (shallow-reference3) 56)
(define-instruction (shallow-reference4) 57)

(define-instruction (short-create-activation-frame size) 60)
(define-instruction (long-create-activation-frame size) 61)
(define-instruction (long-long-create-activation-frame size) 62)
(define-instruction (create-activation-frame0) 63)
(define-instruction (create-activation-frame1) 64)
(define-instruction (create-activation-frame2) 65)
(define-instruction (create-activation-frame3) 66)
(define-instruction (create-activation-frame4) 67)

(define-instruction (short-pop-in-frame rank) 70)
(define-instruction (long-pop-in-frame rank) 71)
(define-instruction (long-long-pop-in-frame rank) 72)
(define-instruction (pop0-in-frame) 73)
(define-instruction (pop1-in-frame) 74)
(define-instruction (pop2-in-frame) 75)
(define-instruction (pop3-in-frame) 76)
(define-instruction (pop4-in-frame) 77)

(define-instruction (short-depth1-reference index) 80)
(define-instruction (long-depth1-reference index) 81)
(define-instruction (long-long-depth1-reference index) 82)
(define-instruction (depth1-reference0) 83)
(define-instruction (depth1-reference1) 84)
(define-instruction (depth1-reference2) 85)
(define-instruction (depth1-reference3) 86)
(define-instruction (depth1-reference4) 87)

(define-instruction (short-check->=arity arity) 90)
(define-instruction (long-check->=arity arity) 91)
(define-instruction (long-long-check->=arity arity) 92)
(define-instruction (check->=arity0) 93)
(define-instruction (check->=arity1) 94)
(define-instruction (check->=arity2) 95)
(define-instruction (check->=arity3) 96)
(define-instruction (check->=arity4) 97)

(define-instruction (short-check-=arity arity+1) 100)
(define-instruction (long-check-=arity arity+1) 101)
(define-instruction (long-long-check-=arity arity+1) 102)
(define-instruction (check-=arity0) 103)
(define-instruction (check-=arity1) 104)
(define-instruction (check-=arity2) 105)
(define-instruction (check-=arity3) 106)
(define-instruction (check-=arity4) 107)

(define-instruction (short+short-deep-reference i j) 110)
(define-instruction (short+long-deep-reference i j k) 111)
(define-instruction (short+long-long-deep-reference i j k l m) 112)
(define-instruction (long+short-deep-reference i j k) 113)
(define-instruction (long+long-deep-reference i j k l) 114)

(define-instruction (if-false-short-goto+ i) 120)
(define-instruction (if-false-long-goto+ i j) 121)
(define-instruction (if-false-long-long-goto+ i j k l) 122)
(define-instruction (short-goto+ offset) 125)
(define-instruction (long-goto+ i j) 126)
(define-instruction (long-long-goto+ i j k l) 127)

(define-instruction (short-create-activation-frame-and-push size) 130)
(define-instruction (long-create-activation-frame-and-push size) 131)
(define-instruction (long-long-create-activation-frame-and-push size) 132)
(define-instruction (create-activation-frame0-and-push) 133)
(define-instruction (create-activation-frame1-and-push) 134)
(define-instruction (create-activation-frame2-and-push) 135)
(define-instruction (create-activation-frame3-and-push) 136)
(define-instruction (create-activation-frame4-and-push) 137)

(define-instruction (suicide) 140)
(define-instruction (suspend) 141)
(define-instruction (fork-and-short-goto+ offset) 142)
(define-instruction (fork-and-long-goto+ i j) 143)
(define-instruction (fork-and-long-long-goto+ i j k l) 144)

(define-instruction (run-module) 145)

(define-instruction (push-handler) 150)
(define-instruction (pop-handler) 151)
(define-instruction (short-create-escape-frame i) 155)
(define-instruction (long-create-escape-frame i j) 156)
(define-instruction (long-long-create-escape-frame i j k l) 157)

(define-instruction (load-false) 200)
(define-instruction (load-true) 201)
(define-instruction (load-nil) 202)
(define-instruction (load-zero) 203)
(define-instruction (load-one) 204)

;;; These are pseudo instructions that serves various purposes.

(define-instruction (apply)         	252)
(define-instruction (callcc)        	253)
(define-instruction (handle-exception)	254)
(define-instruction (module-border)	255)    ; sentinel

;;; end of BCcode.scm
