;;; $Id: BCdrive.scm,v 1.7 1995/07/07 19:34:11 queinnec Exp $

;;; This file defines the main functions to invoke the compiler on a file,
;;; to bytecompile it and to store the result in an object file.

;;; More than a macro system can be present.
(if-icslas-feature EPS
  (define (icslas-expand e)
    (report-progress "Expanding...")
    (walk e icslas-macroexpand) ) )

(if-icslas-feature TraditionalMacros
  (define (icslas-expand e)
    (report-progress "Expanding...")
    (walk e enriched-icslas-expansion-environment) ) )

;;; The main function to byte-compile a file.

(define (byte-compile-file expand g.init source . target)
  (monitor byte-compile-file-handler
    (let ((name (extract-basefilename
                 (extract-filename-sans-extension source) )))
      (report-progress "Compiling" name "...")
      (let ((e* (expand (file->list source))))
        (list->file e* (byte-compile-compute-target-filename 
                        source "esm" ))
        (let* ((bc     (compile->Byte-Code name `(begin . ,e*) g.init))
               (target (if (pair? target) (car target)
                           (byte-compile-compute-target-filename 
                            source "xlo" ) )) )
          (Byte-Compilation->file bc target) ) ) ) ) )

;;; Compute the name of the file where to store the object code.
;;; By default, the object file has the name of the source with .xlo as 
;;; extension. If a XLA directory exists in the directory of the source or
;;; one of its parent directory, the object file will go there with a 
;;; corresponding name. For instance, if /a/b/XLA exists then
;;;   /a/b.scm             -> /a/b.xlo
;;;   /a/b/c.scm           -> /a/b/XLA/c.xlo
;;;   /a/b/c/d.scm         -> /a/b/XLA/c/d.xlo
;;;   /a/bb/c/d.scm        -> /a/bb/c/d.xlo
        
(define (byte-compile-compute-target-filename source extension)
  (let ((file (extract-filename-sans-extension source)))
    (let search ((dir  (extract-directory file))
                 (name (extract-basefilename file)) )
      (if (file-exists? (string-append dir "/XLA"))
          (string-append dir "/XLA/" name "." extension)
          (if (> (string-length dir) 0)
              (search (extract-directory dir) 
                      (string-append (extract-basefilename dir) "/" name) )
              (string-append file "." extension) ) ) ) ) )
;;; Tests: (byte-compile-compute-target-filename "/a.scm")
;;; (byte-compile-compute-target-filename "/b/a.scm")
;;; (byte-compile-compute-target-filename "/usr/users/queinnec/icslas/a.scm")

;;; The exception handler. 
;;; Usually resignal the exception but for Uninitialized-Variables error.

(define-generic (byte-compile-file-handler (exc) c?)
  (diagnose exc c?) )

;;; While byte-compiling an expression and if a Uninitialized-Variable
;;; error is raised this only means that the module compilation did
;;; know for sure that the variable was used before being
;;; written. Since the module may be loaded in an environment where
;;; this variable may be defined this is not a sure error. Return #f
;;; to stop the initialization analysis which leaves the variable
;;; uninitialized.

(define-method (byte-compile-file-handler (exc Uninitialized-Variable) c?)
  (if c? #f (call-next-method)) )

;;; end of BCdrive.scm
