;;; $Id: BCobj.scm,v 1.12 1995/07/14 16:09:20 queinnec Exp $

;;;==================================================================
;;; These are the classes needed for the simulation.  There are the
;;; classes needed to implement Scheme values as well as the classes
;;; required to execute Scheme programs.

(define-class BCRT Object () :virtual)

;;; Predefined values:

(define-class BCRT-Value BCRT
  ( scheme ; the equivalent underlying Scheme value (for DEBUG)
    ) )

(define BCRT-false (make-BCRT-Value #f))
(define BCRT-true (make-BCRT-Value #t))
(define BCRT-nil (make-BCRT-Value '()))
(define BCRT-uninitialized (make-BCRT-Value "uninitialized"))
(define BCRT-unspecified (make-BCRT-Value (if #f 1)))
(define BCRT-end-of-file (make-BCRT-Value "<EOF>"))

;;; Scheme classes

(define-class BCRT-Pair BCRT
  ( tail
    head 
    ) )

(define-class BCRT-Vector BCRT
  ( (* item)
    ) )

(define-class BCRT-Symbol BCRT
  ( name
    ) )

(define-class BCRT-String BCRT
  ( (* character)
    ) )

(define-class BCRT-Character BCRT
  ( code
    ) )

(define-class BCRT-Port BCRT
  ( fd                                  ; UN*X file descriptor
    closed?
    buffer   
    index    
    ) )

(define-class BCRT-Input-Port BCRT-Port
  ( poken 
    zone   ; a String to temporarily read strings or symbols.
    zoneindex 
    ) )

(define-class BCRT-Output-Port BCRT-Port
  ( ) )

;;; This unique type contains integers or floats.

(define-class BCRT-number BCRT
  ( value
    ) )
(define-class BCRT-fixnum BCRT-number ())
(define-class BCRT-float  BCRT-number ())

(define-class BCRT-closure BCRT
  ( pc
    environment     ; the enclosed local lexical environment
    globalenv       ; the global environment of the module where the
                    ; function was defined. (the module can be found from it).
    ) )

(define-class BCRT-primitive BCRT
  ( arity
    address
    name      ; for debug
    ) )

;;;  Local environment.

(define-class BCRT-Environment BCRT
  ( (= next  :initializer (lambda () BCRT-uninitialized))
    (* value :initializer (lambda (i) BCRT-uninitialized))
    ) )

(define-class BCRT-Null-Environment BCRT ())

(define-class BCRT-Box BCRT
  ( value
    ;; type 
    ) )

(define-class BCRT-Named-Box BCRT-Box
  ( name
    ) )

;;; Continuations.

(define-class BCRT-Continuation BCRT
  ( next
    ) :virtual )

(define-class BCRT-Internal-Continuation BCRT-Continuation
  ( address   ; a Scheme function
    ) )

(define-class BCRT-Handler-Continuation BCRT-Continuation
  ( handler
    ) )

(define-class BCRT-Skip-Handler-Continuation BCRT-Continuation
  ( alternate
    ) )

(define-class BCRT-Value-Continuation BCRT-Continuation
  ( value
    ;; type
    ) )

;;; Similar to a closure but for the inherited `next' field.

(define-class BCRT-Context-Continuation BCRT-Continuation
  ( pc
    env
    genv
    ) )

;;; This continuation marks bind-exit continuations.

(define-class BCRT-Escaper-Context-Continuation BCRT-Context-Continuation ())

(define-class BCRT-Thread BCRT
  ( user
    genv
    env
    pc
    val
    cont
    ) )

;;;==================================================================
;;; Anomalies

(define-class BCRT-Anomaly BCRT 
  ( msg
    hints 
    ) )

(define-class BCRT-Underlying-Anomaly BCRT-Anomaly ())

(define-class BCRT-Obsolete-Continuation-Anomaly BCRT-Anomaly ())

(define-class BCRT-Non-Continuable-Exception-Anomaly BCRT-Anomaly ())

;;;==================================================================
;;; This is the structure of module nearly ready to be executed in
;;; memory. 

(define-class BCRT-Module BCRT
  ( name                ; name (a String) of the module.
    (* byte)            ; the bytecodes.
    (* quotation)       ; the ordered quotations.
    (* varname)         ; the Strings naming global variables.
    ) )

;;; When a bytecoded module is loaded, a Module-Env is allocated to
;;; hold its global mutable variables. The rest of the module is
;;; immutable (ie quotations and bytecodes) so the Module can be
;;; reached throught the module field. The *genv* register should
;;; always designate a Module-Env.

(define-class BCRT-Module-Environment BCRT
  ( module      ; a BCRT-Module
    (* element) ; Shared boxes (or values)
    ) )

(define-class BCRT-User BCRT
  ( name        ; a String
    toplevel    ; A Named-Environment
    ;; should it keep the list of all installed modules ???     FUTURE
    ) )

(define-class BCRT-Named-Env BCRT
  ( mapping   ; map from Strings (scheme symbols for now) to Boxes.
    ) )

;;; end of BCobj.scm
