;;; $Id: testeval.scm,v 1.2 1995/09/23 18:20:29 queinnec Exp $

;;; This file tests Icslas when included in a regular interpreter.
;;; These are the tests for OScheme.

;;; This is a little toplevel function.

(define (icslas)
  (let ()
    (display "Welcome to Icslas")
    (newline)
    (let loop ()
      (display "?? ")
      (let ((e (read)))
        (newline)
        (let ((result (icslas-eval e)))
          (display "== ")
          (display (DMeroon->scheme result))
          (newline) ) )
      (loop) ) ) )

(define icslas-thread
  (DM-make DM-Thread-class 
           DM-user                      ; user
           DM-nil                       ; genv (to be patched)
           DM-null-environment          ; env
           0                            ; pc
           DM-uninitialized             ; val
           DM-return-continuation       ; cont
           DM-nil                       ; left
           DM-nil                       ; right
           #t                           ; jumpbuf (to be patched)
           ) )

(define *last-dmeroon-module* DM-nil)

(define (icslas-eval e)
  (let ((module 
         (monitor byte-compile-file-handler
           (let* ((ne (icslas-expand e))
                  (bc     (compile->Byte-Code "toplevel" ne g.icslas)) )
             (Byte-Compilation->DMeroon bc) ) ) ))
    (set! *last-dmeroon-module* module)
    (DM-exec-thread icslas-thread module) ) )

(define-method (byte-compile-file-handler (exc Compilation-Exception) c?)
  (DM-error exc) )

(define *last-dmeroon-result* DM-nil)

(define *erroneous-tests-to-skip*
  '( (set! xyzzy 3)
     ) )

(define *benign-tests-to-skip*
  '( (begin (set! a (cons 'a 'b)) (set-car! a 'c) a)
     (begin (set! a (cons 'a 'b)) (set-cdr! a 'd) a)
     ) )

;;; First test: an expression is read, compiled into a module object
;;; and bytee-eval is run on it.

(define (test-decoder o)
  '--- )

(define (test-embedded-icslas file)
  (let ((ch (error-handler)))
    (suite-test
     file
     "?? " "== " #t
     (lambda (read print err)
       (set! DM-error err)
       ;; Hack, define f t 
       (icslas-eval '(begin (set! f #f)(set! t #t)))
       (lambda ()
         (set-error-handler! 
          (lambda erroneous 
            (set-error-handler! ch)
            (err erroneous) ) )
         (let retry ((e (read)))
           (cond 
            ((member e *erroneous-tests-to-skip*)
             (err '***) )
            ((member e *benign-tests-to-skip*)
             (set! e (read)) ; skip result
             (retry (read)) )
            (else
             (let ((result (icslas-eval e)))
               (set! *last-dmeroon-result* result)
               (print (DMeroon->scheme result test-decoder)) ) ) ) ) ) )
     equal? )
    (set-error-handler! ch) ) )
;;; Test: (test-embedded-icslas "../../Test/scheme.tst")

;;; Second test: an expression is read, compiled into a module,
;;; marshaled in a buffer, then unmarshaled by DMeroon in order to be
;;; run.

(define (icslas-marshal-eval e)
  (let* ((be (DM-field-value thetcp DM-Connection-emission-field))
         (br (DM-field-value thetcp DM-Connection-reception-field))
         (module 
          (monitor byte-compile-file-handler
            (let* ((ne (icslas-expand e))
                   (bc (compile->Byte-Code "toplevel" ne g.icslas)) )
              (Byte-Compilation->DMeroon bc) ) ) ))
    (set! *last-dmeroon-module* module)
    ;; reset index of the emission buffer
    (DM-local-set DM-nat3-type be DM-Buffer-index-field 0)
    (DM-marshal be module)
    ;; copy the emission buffer into the reception buffer.
    (DM-local-copy
     (DM-local-length DM-nat3-type be DM-Buffer-byte-field)
     br
     (DM-indexed-compute-offset br DM-Buffer-byte-field 0)
     be
     (DM-indexed-compute-offset be DM-Buffer-byte-field 0) )
    ;; reset index/maximum of the reception buffer
    (DM-local-set DM-nat3-type br DM-Buffer-index-field 0)
    (DM-local-set DM-nat3-type br DM-Buffer-maximum-field
                  (DM-local-get DM-nat3-type be DM-Buffer-index-field) )
    ;; Use the C function to unmarshal the module
    (let ((newmodule (DM-busy-read-object br)))
      (DM-exec-thread icslas-thread newmodule) ) ) )

;;; Prepare a connection for test.
;;; TEMP: most of the fields are empty since they are useless.
(define (create-thetcp sz)
  (let* ((be (DM-allocate1 DM-OutBuffer-class sz))
         (br (DM-allocate1 DM-InBuffer-class sz))
         (ce (DM-make DM-BoundedStack-class 0 2 DM-nil DM-nil))
         (cr (DM-make DM-BoundedStack-class 0 2 DM-nil DM-nil))
         (conn (DM-make DM-Connection-class
                        0               ; fd
                        5               ; retries
                        0               ; options
                        0               ; family
                        0               ; port
                        0               ; address
                        0               ; zero
                        br              ; reception
                        be              ; emission
                        0               ; clocker
                        DM-nil          ; site
                        '()             ; objects
                        ) ) )
    (DM-local-set DM-reference-type be DM-Buffer-port-field conn)
    (DM-local-set DM-reference-type br DM-Buffer-port-field conn)
    (DM-local-set DM-reference-type be DM-Buffer-cache-field ce)
    (DM-local-set DM-reference-type br DM-Buffer-cache-field cr)
    (DM-set-initialized be)
    (DM-set-initialized br)
    conn ) )

(define thetcp (create-thetcp 1024))

(define last-unmarshaled-object #f)

(define (test-marshaled-icslas file)
  (let ((ch (error-handler)))
    (suite-test
     file
     "?? " "== " #t
     (lambda (read print err)
       (set! DM-error err)
       ;; Hack, define f t 
       (icslas-eval '(begin (set! f #f)(set! t #t)))
       (lambda ()
         (set-error-handler! 
          (lambda erroneous 
            (set-error-handler! ch)
            (err erroneous) ) )
         (let retry ((e (read)))
           (cond 
            ((member e *erroneous-tests-to-skip*)
             (err '***) )
            ((member e *benign-tests-to-skip*)
             (set! e (read)) ; skip result
             (retry (read)) )
            (else
             (let ((result (icslas-marshal-eval e)))
               (set! *last-dmeroon-result* result)
               (print (DMeroon->scheme result test-decoder)) ) ) ) ) ) )
     equal? )
    (set-error-handler! ch) ) )
;;; Test: (test-marshaled-icslas "../../Test/scheme.tst")

;;; end of testeval.scm
