/* $Id: xlaenv.c,v 1.50 1998/06/01 19:30:41 queinnec Exp $ */

#include "xla.h"

DM_DefineStringWithCstring(xlaenv_version,
  "$Id: xlaenv.c,v 1.50 1998/06/01 19:30:41 queinnec Exp $");

/*
 * Preallocate all 256 regular characters.
 * There is a C array that contains all wrapped characters and a DMeroon
 * ImmutableVector referencing them. This won't work for Unicode characters.
 */

#define DefineCharacter(i) { \
  DM_SupportPrefixInitialization(DM_Char1_wrapped, \
                                 DM_character_##i##_object) \
  DM_ReferenceToClass(Char1), \
  DM_ProxyInitialization(), \
  { { { \
    (i) \
   } } } \
}

#define Define2Characters(i) \
  DefineCharacter(i), \
  DefineCharacter(i+1)

#define Define4Characters(i) \
  Define2Characters(i), \
  Define2Characters(i+2)

#define Define8Characters(i) \
  Define4Characters(i), \
  Define4Characters(i+4)

#define Define16Characters(i) \
  Define8Characters(i), \
  Define8Characters(i+8)

#define Define32Characters(i) \
  Define16Characters(i), \
  Define16Characters(i+16)

#define Define64Characters(i) \
  Define32Characters(i), \
  Define32Characters(i+32)

#define Define128Characters(i) \
  Define64Characters(i), \
  Define64Characters(i+64)

#define Define256Characters(i) \
  Define128Characters(i), \
  Define128Characters(i+128)

DM_Char1_wrapped DM_character_objects[256] = {
  Define256Characters(0)
};

#define ReferenceToCharacter(i) \
  DM_FromC(&DM_character_objects[i])

#define RefersTo2Characters(i) \
    ReferenceToCharacter(i+0), \
    ReferenceToCharacter(i+1),

#define RefersTo4Characters(i) \
    RefersTo2Characters(i+0) \
    RefersTo2Characters(i+2)
 
#define RefersTo8Characters(i) \
    RefersTo4Characters(i+0) \
    RefersTo4Characters(i+4)
 
#define RefersTo16Characters(i) \
    RefersTo8Characters(i+0) \
    RefersTo8Characters(i+8)
 
#define RefersTo32Characters(i) \
    RefersTo16Characters(i+0) \
    RefersTo16Characters(i+16)
 
#define RefersTo64Characters(i) \
    RefersTo32Characters(i+0) \
    RefersTo32Characters(i+32)
 
#define RefersTo128Characters(i) \
    RefersTo64Characters(i+0) \
    RefersTo64Characters(i+64)
 
#define RefersTo256Characters(i) \
    RefersTo128Characters(i+0) \
    RefersTo128Characters(i+128)
 
DM_ImmutableVector_256_wrapped DM_characters_object = {
  DM_SupportPrefixInitialization(DM_ImmutableVector_256_wrapped,
                                 DM_characters_object)
  DM_ReferenceToClass(ImmutableVector),
  DM_ProxyInitialization(), 
  { { { 
    256,
    {
      RefersTo256Characters(0)
    }
  } } }
};

/* RESOURCE DM_link_module
 * This function allocates a ModuleEnvironment for a module using the
 * hashtable representing the toplevel of the user. The toplevel is a
 * hashtable mapping Strings to NamedBoxes.
 */

DM_ModuleEnvironment*
DM_link_module (DM_Context* context,
                DM_User     *user, 
                DM_Module   *module) {
  DM_Field* free_field = DM_ReferenceToFieldOfClass(free, Module);
  DM_nat3 i, nvars;
  DM_ModuleEnvironment* me;
  DM_String* varname;
  DM_ErrorCode errcode;

  errcode = DMeroon_indexed_field_length(context, 
                                         &nvars,
                                         DM_NULL,
                                         (DM_Object*)module,
                                         free_field );
  /* Reserved for dmscm.c: DM_CheckReturnedStatus(context, errcode); */
  me = DM_Allocate1(context, ModuleEnvironment, nvars);
  me->module = module;
  for ( i=0 ; i<nvars ; i++ ) {
    errcode = DMeroon_indexed_field_value(context,
                                          &varname,
                                          DM_NULL,
                                          (DM_Object*)module,
                                          free_field,
                                          i );
    /* Reserved for dmscm.c: DM_CheckReturnedStatus(context, errcode); */
    me->element[i] = DM_find_associated_box(context, 
                                            &(user->toplevel), 
                                            varname );
  }
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, me);
  }
  return me;
}

/* RESOURCE DM_find_associated_box
 * Find if there is a Box with that name in the toplevel hashtable, otherwise
 * create such a box. Double the hashtable if needed.
 */

DM_NamedBox*
DM_find_associated_box (DM_Context*   context,
                        DM_Hashtable* *toplevel, 
                        DM_String*    varname ) {
  DM_NamedBox *namedbox = DM_extract_namedbox(context, *toplevel, varname);
  
  if ( namedbox == DM_NULL ) {
    namedbox         = DM_Allocate0(context, NamedBox);
    namedbox->value  = DM_TheConstant(uninitialized);
    namedbox->name   = varname;
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, namedbox);
    }
    DM_hashtable_insert(context, toplevel, (DM_Object*)namedbox);
  }
  return namedbox;
}

/* RESOURCE DM_extract_namedbox
 * Find a namedBox in a hashtable.
 */

DM_NamedBox*
DM_extract_namedbox (DM_Context*   context,
                     DM_Hashtable* ht,
                     DM_String*    string ) 
{
  DM_nat3 length   = string->character_length;
  char    *cstring = string->character;

  DM_HashtableExtract( context, 
                       ht,
                       DM_NamedBox*,
                       DM_hashpjw(length, cstring),
                       (    (length == object->name->character_length)
                         && (strncmp(cstring, 
                                     object->name->character,
                                     length) == 0) )
                        );
}

/* RESOURCE DM_exec_thread
 * A function to interact with the ByteCode interpreter.
 * Just give it a thread and a module and it will change that thread
 * to run that module and returns its final value. 
 * Caution: most of the fields of the Thread are kept as they stand.
 */

DM_Object*
DM_exec_thread (DM_Context* context,
                DM_Thread*  thread,
                DM_Module*  module ) {
  thread->env  = (DM_Environment*)(DM_ReferenceTo(DM_null_environment));
  thread->pc   = 0;
  /* leave the other fields of the thread as they are. */
  thread->genv = DM_link_module(context, thread->user, module);
  /* Protect the evaluation                          FUTURE */
  return DM_byte_eval(context, thread);
}

#ifdef DM_DEBUG

/* RESOURCE DM_show_object
 * Display an object. This function may be used as the print function 
 * of a Scheme interpreter.
 *
 * Note: the wrapped_port may be an OutputPort as well as stderr.
 */

static void
DM_print_support_fixnum (DM_Context*      context,
                         DM_SupportFixnum i, 
                         DM_SupportPointer wrapped_port ) {

#ifdef DM_HAS_LONG_LONG_ENTITIES
  long long int j = i;
  DM_C_long_long_int_print(context, &j, wrapped_port);
#else
  long int j = i;
  DM_C_long_int_print(context, &j, wrapped_port);
#endif

}

static void
DM_print_support_floatnum (DM_Context*        context,
                           DM_SupportFloatnum i, 
                           DM_SupportPointer  wrapped_port ) {
  long double j = i;
  DM_C_long_double_print(context, &j, wrapped_port);
}

void
DM_show_object (DM_Context*       context,
                DM_Object*        o,
                DM_SupportPointer wrapped_stream ) {

  /* When Icslas is embedded, primitives may return these values,
   * which are not DMeroon values. It is of course also possible to
   * get a NULL pointer. */

  if ( o == NULL ) {
    DM_print_char_star(context, "<0x0>", 5, wrapped_stream);
    return;
  }

  /* FUTURE utiliser le nom des constantes                 FUTURE */

  if ( o == (DM_Object*)(DM_TheConstant(nil)) ) {
    DM_print_char_star(context, "()", 2, wrapped_stream);
    goto end;
  }
  
  if ( o == (DM_Object*)(DM_ReferenceTo(DM_end_of_file)) ) {
    DM_print_char_star(context, "#<EOF>", 6, wrapped_stream);
    goto end;
  }
  
  if ( o == (DM_Object*)(DM_TheConstant(uninitialized)) ) {
    DM_print_char_star(context, "#<OVNI>", 7, wrapped_stream);
    goto end;
  }
  
  if ( DM_IsA(context, o, Int4) ) {
    DM_print_support_fixnum(context, ((DM_Int4*)o)->value, wrapped_stream);
    goto end;
  }

  if ( DM_IsA(context, o, Float2) ) {
    DM_DeclareNarrowedObject(f, Float2, o);
    DM_print_support_floatnum(context, f->value, wrapped_stream);
    goto end;
  }

  if ( DM_IsA(context, o, Boolean) ) {
    if ( DM_IsA(context, o, True) ) {
      DM_print_char_star(context, "#t", 2, wrapped_stream);
      goto end;
    }
    if ( DM_IsA(context, o, False) ) {
      DM_print_char_star(context, "#f", 2, wrapped_stream);
      goto end;
    }
    goto unknown;
  }

  if ( DM_IsA(context, o, Symbol) ) {
    DM_DeclareNarrowedObject(s, Symbol, o);
    
    DM_print_char_star(context, s->character, 
                       s->character_length, wrapped_stream);
    goto end;
  }

  if ( DM_IsA(context, o, String) || DM_IsA(context, o, MutableString) ) {
    DM_String *str = (DM_String*)o;
    DM_nat3   size = str->character_length;
    
    DM_print_char_star(context, "\"", 1, wrapped_stream);
    if ( str->character[size-1] == '\0' ) {
      DM_print_char_star(context, str->character, size-1, wrapped_stream);
    } else {
      DM_print_char_star(context, str->character, size, wrapped_stream);
    }
    DM_print_char_star(context, "\"", 1, wrapped_stream);
    goto end;
  }

  if ( DM_IsA(context, o, Character) ) {
    DM_Character *ch = (DM_Character*)o;
    char s[4] = "#\\?";

    s[2] = ch->code;
    DM_print_char_star(context, s, 3, wrapped_stream);
    goto end;
  }

  if (    DM_IsA(context, o, ImmutablePair)
       || DM_IsA(context, o, MutablePair) ) {
    DM_MutablePair *pair = (DM_MutablePair*)o;

    DM_print_char_star(context, "(", 1, wrapped_stream);
    DM_show_object(context, pair->value, wrapped_stream);
    DM_print_char_star(context, " . ", 3, wrapped_stream);
    DM_show_object(context, pair->rest, wrapped_stream);
    DM_print_char_star(context, ")", 1, wrapped_stream);
    goto end;
  }

  DM_DebugAtLevel(8,

    /* Additional methods for debugging. */

    if ( DM_IsA(context, o, NamedBox) ) {
      DM_NamedBox *box = (DM_NamedBox*)o;

      DM_print_char_star(context, "#<NamedBox(", 0, wrapped_stream);
      DM_show_object(context, (DM_Object*)(box->name), wrapped_stream);
      DM_print_char_star(context, "):", 2, wrapped_stream);
      DM_show_object(context, box->value, wrapped_stream);
      DM_print_char_star(context, ">", 1, wrapped_stream);
      goto end;
    }

    if ( DM_IsA(context, o, Box) ) {
      DM_Box *box = (DM_Box*)o;

      DM_print_char_star(context, "#<Box:", 0, wrapped_stream);
      DM_show_object(context, box->value, wrapped_stream);
      DM_print_char_star(context, ">", 0, wrapped_stream);
      goto end;
    }

    if ( DM_IsA(context, o, Primitive) ) {
      DM_DeclareNarrowedObject(p, Primitive, o);
      DM_print_char_star(context, "#<Primitive:", 0, wrapped_stream);
      DM_show_object(context, (DM_Object*)(p->name), wrapped_stream);
      DM_print_char_star(context, ">", 0, wrapped_stream);
      goto end;
    }

    if ( DM_IsA(context, o, SpecialPrimitive) ) {
      DM_DeclareNarrowedObject(sp, SpecialPrimitive, o);
      DM_print_char_star(context, "#<SpecialPrimitive:", 0, wrapped_stream);
      DM_show_object(context, (DM_Object*)(sp->name), wrapped_stream);
      DM_print_char_star(context, ">", 0, wrapped_stream);
      goto end;
    }

    if ( DM_IsA(context, o, Environment) ) {
      DM_Environment *env  = (DM_Environment*)o;
      DM_nat3        nargs = env->value_length;
      int            i;

      DM_print_char_star(context, "#<Environment:",0,wrapped_stream);
      for ( i=0 ; i<(nargs-1) ; i++ ) {
        DM_print_char_star(context, "[", 0, wrapped_stream);
        DM_print_support_fixnum(context, i, wrapped_stream);
        DM_print_char_star(context, "]=", 0, wrapped_stream);
        DM_show_object(context, env->value[i], wrapped_stream);
        DM_print_char_star(context, "\n", 0, wrapped_stream);
      }
      DM_print_char_star(context, ">", 0, wrapped_stream);
      goto end;
    }

    if ( DM_IsA(context, o, NullEnvironment) ) {
      DM_print_char_star(context, "#<NullEnvironment>", 0, wrapped_stream);
      goto end;
    }

    if ( DM_IsA(context, o, ValueContinuation) ) {
      DM_DeclareNarrowedObject(k, ValueContinuation, o);

      DM_print_char_star(context, "#<ValueContinuation:",0,wrapped_stream);
      DM_show_object(context, k->value, wrapped_stream);
      DM_print_char_star(context, ">\n", 0, wrapped_stream);
      DM_show_object(context, (DM_Object*)(k->next), wrapped_stream);
      goto end;
    }

    if ( DM_IsA(context, o, ContextContinuation) ) {
      DM_DeclareNarrowedObject(k, ContextContinuation, o);

      DM_print_char_star(context, "#<ContextContinuation:pc=", 0, wrapped_stream);
      DM_print_support_fixnum(context, k->pc, wrapped_stream);
      DM_print_char_star(context, ">\n", 0, wrapped_stream);
      DM_show_object(context, (DM_Object*)(k->next), wrapped_stream);
      goto end;
    }

    if ( DM_IsA(context, o, Continuation) ) {
      DM_print_char_star(context, "#<Continuation>", 0, wrapped_stream);
      goto end;
    }

    if ( DM_IsA(context, o, Closure) ) {
      DM_DeclareNarrowedObject(c, Closure, o);

      DM_print_char_star(context, "#<Closure:pc=", 0, wrapped_stream);
      DM_print_support_fixnum(context, c->pc, wrapped_stream);
      DM_print_char_star(context, ">", 1, wrapped_stream);
      goto end;
    }

);

  if ( DM_is_procedure(context, o) != DM_SupportFalse ) {
    char line[50];
    
    sprintf(line, "#<Procedure:%lu>", (DM_SupportAddress)(o));
    DM_print_char_star(context, line, 0, wrapped_stream);
    goto end;
  }

  unknown :
    DM_Abort(context, DME_UNSHOWABLE);

  end : {
    return;
  }
}

#endif /* DM_DEBUG */

/*
 * A long set of primitive functions.
 *
 * Arity is already checked by the invoking machinery (see xlabci.c).
 * When the function has a fix arity, it takes the same number of arguments
 * in C, it takes in Scheme. When the arity is non-fix, the C version 
 * takes the arguments with the argc,argv convention.
 *
 * These functions just return a value that will be stored in the val
 * register and the bytecode interpreter is restarted with a resume 
 * operation that gives this val register to the continuation.
 */

#define DM_DefinePredicate(name,class) \
DM_SupportBoolean \
DM_##name (DM_Context *context, DM_Object* a) { \
  if ( DM_is_a(context, a, DM_ReferenceToClass(class)) ) { \
    return DM_SupportTrue; \
  } else { \
    return DM_SupportFalse; \
  } \
} \
DM_DefinePrimitive(name,1)

DM_DefinePredicate(is_boolean,Boolean);

DM_SupportBoolean
DM_not (DM_Context* context, DM_Object* a) {
  if ( a == DM_SupportFalse ) {
    return DM_SupportTrue; 
  } else {
    return DM_SupportFalse; 
  }
}

DM_DefinePrimitive(not,1);

DM_SupportBoolean
DM_eq (DM_Context* context, DM_Object* a, DM_Object* b) {    
  if ( a == b ) {
    return DM_SupportTrue;
  } else {
    return DM_SupportFalse;
  }
}

DM_DefinePrimitive(eq,2);

DM_SupportBoolean
DM_eqv (DM_Context* context, DM_Object *a, DM_Object* b) {  
  if ( a == b ) {
    return DM_SupportTrue;
  } else if (    DM_IsA(context, a, Int4) && DM_IsA(context, b, Int4)
              && ( ((DM_Int4*)a)->value == ((DM_Int4*)b)->value ) ) {
    return DM_SupportTrue;
  } else if (    DM_IsA(context, a, Float2) && DM_IsA(context, b, Float2)
              && ( ((DM_Float2*)a)->value == ((DM_Float2*)b)->value ) ) {
    return DM_SupportTrue;
  } else {
    return DM_SupportFalse;
  }
}

DM_DefinePrimitive(eqv,2);

DM_SupportBoolean 
DM_is_pair (DM_Context* context, DM_Object *a) {
  if (    DM_IsA(context, a, ImmutablePair) 
       || DM_IsA(context, a, MutablePair) ) {
    return DM_SupportTrue; 
  } else {
    return DM_SupportFalse; 
  }
}

DM_DefinePrimitive(is_pair,1);

DM_Object*
DM_cons (DM_Context* context, 
         DM_Object* a, 
         DM_Object* b ) {

#ifdef DM_ICSLAS_PREFERS_MUTABILITY
  DM_MutablePair   *result = DM_Allocate0(context, MutablePair);
#else
  DM_ImmutablePair *result = DM_Allocate0(context, ImmutablePair);
#endif

  result->value = a;
  result->rest  = b;
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, result);
  }
  return (DM_Object*)result;
}

DM_DefinePrimitive(cons,2);

DM_Object*
DM_car (DM_Context* context,
        DM_Object* a) {
  if (    DM_IsA(context, a, ImmutablePair)
       || DM_IsA(context, a, MutablePair) ) {
    return ((DM_MutablePair*)a)->value;
  } else {
    DM_Abort(context, DME_NOT_A_PAIR);
    /* NOT REACHED */
    return DM_NULL; 
  }
}

DM_DefinePrimitive(car,1);

DM_Object*
DM_cdr (DM_Context* context,
        DM_Object* a) {
  if ( DM_IsA(context, a, ImmutablePair)
       || DM_IsA(context, a, MutablePair) ) {
    return ((DM_MutablePair*)a)->rest;
  } else {
    DM_Abort(context, DME_NOT_A_PAIR);
    /* NOT REACHED */
    return DM_NULL; 
  }
}

DM_DefinePrimitive(cdr,1);

DM_Object*
DM_set_car (DM_Context* context,
            DM_Object* a,
            DM_Object* b ) {
  if ( DM_IsA(context, a, MutablePair) ) {
    DM_DeclareNarrowedObject(pair, MutablePair, a);

    pair->value = b;
    return a;
  } else {
    DM_Abort(context, DME_NOT_A_MUTABLE_PAIR);
    /* NOT REACHED */
    return DM_NULL; 
  }
}

DM_DefinePrimitive(set_car,2);

DM_Object*
DM_set_cdr (DM_Context* context,
            DM_Object* a,
            DM_Object* b ) {
  if ( DM_IsA(context, a, MutablePair) ) {
    DM_DeclareNarrowedObject(pair, MutablePair, a);

    pair->rest = b;
    return a;
  } else {
    DM_Abort(context, DME_NOT_A_MUTABLE_PAIR);
    /* NOT REACHED */
    return DM_NULL; 
  }
}

DM_DefinePrimitive(set_cdr,2);

DM_Object*
DM_list (DM_Context* context,
         DM_nat3 argc, 
         DM_Object* (*argv)[1] ) {
  DM_Object *result = DM_TheConstant(nil);
  
  while ( argc > 0 ) {

#ifdef DM_ICSLAS_PREFERS_MUTABILITY
    DM_MutablePair   *r = DM_Allocate0(context, MutablePair);
#else
    DM_ImmutablePair *r = DM_Allocate0(context, ImmutablePair);
#endif

    r->value = (*argv)[--argc];
    r->rest  = result;
    result   = (DM_Object*)r;
  }
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, result);
  }
  return result;
}

DM_DefinePrimitive(list,-1);

DM_SupportBoolean 
DM_is_null (DM_Context* context,
            DM_Object* a) { 
  if ( a == DM_TheConstant(nil) ) {
    return DM_SupportTrue; 
  } else {
    return DM_SupportFalse; 
  }
}

DM_DefinePrimitive(is_null,1);

DM_DefinePredicate(is_symbol,Symbol);

DM_Object*
DM_symbol_to_string (DM_Context* context,
                     DM_Object* symbol) {
  if ( DM_IsA(context, symbol, Symbol) ) {
    DM_DeclareNarrowedObject(s, Symbol, symbol);
    DM_String* str = DM_Allocate1(context, String, s->character_length);

    memcpy(str->character, s->character, s->character_length);
    return (DM_Object*)str;
  } else {
    DM_Abort(context, DME_NOT_A_SYMBOL);
    /* NOT REACHED */
    return DM_NULL;
  }
}

DM_DefinePrimitive(symbol_to_string,1);  

DM_Object*
DM_string_to_symbol (DM_Context* context,
                     DM_Object* string) 
{
    DM_nat3   len;
    char      *s;
    DM_Symbol *symbol;

  if ( DM_IsA(context, string, String) ) {
    DM_DeclareNarrowedObject(str, String, string);
    len = str->character_length;
    s   = str->character;
  } else if ( DM_IsA(context, string, MutableString) ) {
    DM_DeclareNarrowedObject(str, MutableString, string);
    len = str->character_length;
    s   = str->character;
  } else {
    DM_Abort(context, DME_NOT_A_STRING);
  }
  symbol = DM_Allocate1(context, Symbol, len);
  memcpy(symbol->character, s, len);
  return (DM_Object*)(DM_intern_symbol(context, symbol));
}

DM_DefinePrimitive(string_to_symbol,1);

DM_DefinePredicate(is_integer,Int4);
DM_DefinePredicate(is_real,Float2);

DM_SupportBoolean 
DM_is_number (DM_Context* context,
              DM_Object* o) {
  if ( DM_IsA(context, o, Int4) || DM_IsA(context, o, Float2) ) {
    return DM_SupportTrue;
  } else {
    return DM_SupportFalse;
  }
}

DM_DefinePrimitive(is_number,1);

#define DM_DefineArithmeticPredicate(name,op) \
DM_SupportBoolean DM_##name (DM_Context *context, \
                             DM_Object* a, \
                             DM_Object* b) { \
  if ( DM_IsA(context, a, Int4) && DM_IsA(context, b, Int4) \
      && ( ((DM_Int4*)a)->value op ((DM_Int4*)b)->value ) ) { \
    return DM_SupportTrue; \
  } else if ( DM_IsA(context, a, Float2) && DM_IsA(context, b, Float2) \
             && ( ((DM_Float2*)a)->value op ((DM_Float2*)b)->value ) ) { \
    return DM_SupportTrue; \
  } else { \
    return DM_SupportFalse; \
  } \
} \
DM_DefinePrimitive(name,2)

DM_DefineArithmeticPredicate(binary_numeric_equal,==);
DM_DefineArithmeticPredicate(binary_numeric_less,<);
DM_DefineArithmeticPredicate(binary_numeric_greater,>);
DM_DefineArithmeticPredicate(binary_numeric_less_equal,<=);
DM_DefineArithmeticPredicate(binary_numeric_greater_equal,>=);

DM_Object*
DM_plus (DM_Context* context,
         DM_nat3 argc, 
         DM_Object* (*argv)[1]) {
  DM_int4            iresult = 0;
  DM_float2          fresult;
  DM_Object           *final;
  DM_reference       operand;

  while ( argc-- > 0 ) {
    operand = *(argv[argc]);
    if ( DM_IsA(context, operand, Int4) ) {
      iresult += ((DM_Int4*)operand)->value;
    } else if ( DM_IsA(context, operand, Float2) ) {
      goto floats;
    } else {
      DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
    }
  }
  final = ((DM_Object*)(DM_Allocate0(context, Int4)));
  ((DM_Int4*)final)->value = iresult;
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, final);
  }
  return final;
  
 floats:
  fresult = (DM_float2)iresult;
  fresult += ((DM_Float2*)operand)->value;
  while ( argc-- > 0 ) {
    operand = *(argv[argc]);
    if ( DM_IsA(context, operand, Int4) ) {
      fresult += (DM_float2)(((DM_Int4*)operand)->value);
    } else if ( DM_IsA(context, operand, Float2) ) {
      fresult += ((DM_Float2*)operand)->value;
    } else {
      DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
    }
  }
  final = ((DM_Object*)(DM_Allocate0(context, Float2)));
  ((DM_Float2*)final)->value = fresult;
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, final);
  }
  return final;
}

DM_DefinePrimitive(plus,-1);

DM_Object*
DM_times (DM_Context* context,
          DM_nat3 argc,
          DM_Object* (*argv)[1]) {
  DM_int4            iresult = 1;
  DM_float2          fresult;
  DM_Object           *final;
  DM_reference       operand;

  while ( argc-- > 0 ) {
    operand = *(argv[argc]);
    if ( DM_IsA(context, operand, Int4) ) {
      iresult *= ((DM_Int4*)operand)->value;
    } else if ( DM_IsA(context, operand, Float2) ) {
      goto floats;
    } else {
      DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
    }
  }
  final = ((DM_Object*)(DM_Allocate0(context, Int4)));
  ((DM_Int4*)final)->value = iresult;
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, final);
  }
  return final;
  
 floats:
  fresult = (DM_float2)iresult;
  fresult *= ((DM_Float2*)operand)->value;
  while ( argc-- > 0 ) {
    operand = *(argv[argc]);
    if ( DM_IsA(context, operand, Int4) ) {
      fresult *= (DM_float2)(((DM_Int4*)operand)->value);
    } else if ( DM_IsA(context, operand, Float2) ) {
      fresult *= ((DM_Float2*)operand)->value;
    } else {
      DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
    }
  }
  final = ((DM_Object*)(DM_Allocate0(context, Float2)));
  ((DM_Float2*)final)->value = fresult;
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, final);
  }
  return final;
}

DM_DefinePrimitive(times,-1);

DM_Object*
DM_difference (DM_Context* context,
               DM_nat3 argc,
               DM_Object* (*argv)[1]) {
  DM_int4            iresult = 0;
  DM_float2          fresult = 0.0;
  DM_Object           *final;
  DM_reference       operand;

  switch ( argc ) {
  case 0 : 
    return DM_ReferenceTo(DM_fixnum_0);

  case 1 : {
    operand = *(argv[0]);
    if ( DM_IsA(context, operand, Int4 ) ) {
      iresult = 0 - ((DM_Int4*)operand)->value;
      goto return_fixnum;
    } else if ( DM_IsA(context, operand, Float2 ) ) {
      fresult = 0.0 - ((DM_Float2*)operand)->value;
      goto return_floatnum;
    } else {
      DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
    }
  }

  default : {
    operand = *(argv[0]);
    if ( DM_IsA(context, operand, Int4 ) ) {
        iresult = ((DM_Int4*)operand)->value;
      } else if ( DM_IsA(context, operand, Float2) ) {
        fresult = ((DM_Float2*)operand)->value;
        goto floats;
      } else {
        DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
      }
    while ( argc-- > 1 ) {
      operand = *(argv[argc]);
      if ( DM_IsA(context, operand, Int4 ) ) {
        iresult -= ((DM_Int4*)operand)->value;
      } else if ( DM_IsA(context, operand, Float2) ) {
        goto floats;
      } else {
        DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
      }
    }
    goto return_fixnum;

  floats:
    fresult = (DM_float2)iresult;
    fresult -= ((DM_Float2*)operand)->value;
    while ( argc-- > 1 ) {
      operand = *(argv[argc]);
      if ( DM_IsA(context, operand, Int4) ) {
        fresult -= (DM_float2)(((DM_Int4*)operand)->value);
      } else if ( DM_IsA(context, operand, Float2) ) {
        fresult -= ((DM_Float2*)operand)->value;
      } else {
        DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
      }
    }
    goto return_floatnum;
  }
  }
  
 return_fixnum:
  final = ((DM_Object*)(DM_Allocate0(context, Int4)));
  ((DM_Int4*)final)->value = iresult;
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, final);
  }
  return final;

 return_floatnum:
  final = ((DM_Object*)(DM_Allocate0(context, Float2)));
  ((DM_Int4*)final)->value = fresult;
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, final);
  }
  return final;
}

DM_DefinePrimitive(difference,-2);

DM_Object*
DM_divide (DM_Context* context,
           DM_nat3 argc,
           DM_Object* (*argv)[1]) {
  DM_float2          fresult;
  DM_Object           *final;
  DM_reference       operand;

  switch ( argc ) {
  case 0 : {
    DM_Abort(context, DME_DIVISION_BY_ZERO);
    /* NOT REACHED */
    return DM_NULL;
  }

  case 1 : {
    operand = *(argv[0]);
    if ( DM_IsA(context, operand, Int4 ) ) {
      fresult = 1.0 / (DM_float2)(((DM_Int4*)operand)->value);
      goto return_floatnum;
    } else if ( DM_IsA(context, operand, Float2 ) ) {
      fresult = 1.0 / ((DM_Float2*)operand)->value;
      goto return_floatnum;
    } else {
      DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
    }
  }

  default : {
    operand = *(argv[0]);
    if ( DM_IsA(context, operand, Int4 ) ) {
        fresult = (DM_float2)(((DM_Int4*)operand)->value);
      } else if ( DM_IsA(context, operand, Float2) ) {
        fresult = ((DM_Float2*)operand)->value;
      } else {
        DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
      }
    fresult /= ((DM_Float2*)operand)->value;
    while ( argc-- > 1 ) {
      operand = *(argv[argc]);
      if ( DM_IsA(context, operand, Int4) ) {
        fresult /= (DM_float2)(((DM_Int4*)operand)->value);
      } else if ( DM_IsA(context, operand, Float2) ) {
        fresult /= ((DM_Float2*)operand)->value;
      } else {
        DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
      }
    }
    final = ((DM_Object*)(DM_Allocate0(context, Float2)));
    ((DM_Int4*)final)->value = fresult;
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, final);
    }
    return final;
  }
  }
 return_floatnum:
  final = ((DM_Object*)(DM_Allocate0(context, Float2)));
  ((DM_Int4*)final)->value = fresult;
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, final);
  }
  return final;
}

DM_DefinePrimitive(divide,-2);

DM_Object*
DM_quotient (DM_Context *context,
             DM_Object* a, 
             DM_Object* b ) {
  if ( DM_IsA(context, a, Int4 ) && DM_IsA(context, b, Int4 ) ) {
    DM_Int4           *result        = DM_Allocate0(context, Int4);
    
    result->value = ((DM_Int4*)a)->value / ((DM_Int4*)b)->value;
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, result);
    }
    return (DM_Object*)result;
  } else {
    DM_Abort(context, DME_NOT_A_NUMBER);
    /* NOT REACHED */
    return DM_NULL;
  }
}

DM_DefinePrimitive(quotient,2);

DM_Object*
DM_remainder (DM_Context* context,
              DM_Object* a, 
              DM_Object* b) {
  if ( DM_IsA(context, a, Int4 ) && DM_IsA(context, b, Int4 ) ) {
    DM_Int4           *result        = DM_Allocate0(context, Int4);
    
    result->value = ((DM_Int4*)a)->value % ((DM_Int4*)b)->value;
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, result);
    }
    return (DM_Object*)result;
  } else {
    DM_Abort(context, DME_NOT_A_NUMBER);
    /* NOT REACHED */
    return DM_NULL;
  }
}

DM_DefinePrimitive(remainder,2);

DM_Object*
DM_modulo (DM_Context* context,
           DM_Object* a, 
           DM_Object* b) {
  if ( DM_IsA(context, a, Int4 ) && DM_IsA(context, b, Int4 ) ) {
    DM_Int4           *result        = DM_Allocate0(context, Int4);
    
    result->value = ((DM_Int4*)a)->value % ((DM_Int4*)b)->value;
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, result);
    }
    return (DM_Object*)result;
  } else {
    DM_Abort(context, DME_NOT_A_NUMBER);
    /* NOT REACHED */
    return DM_NULL;
  }
}

DM_DefinePrimitive(modulo,2);

DM_Object*
DM_floor (DM_Context* context,
          DM_Object* a) {
  if ( DM_IsA(context, a, Float2 ) ) {
    DM_DeclareNarrowedObject(f, Float2, a);
    DM_Float2 *result = DM_Allocate0(context, Float2);
    
    result->value = floor(f->value);
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, result);
    }
    return (DM_Object*)result;
  } else {DM_Abort(context, DME_NOT_A_NUMBER);
    /* NOT REACHED */
    return DM_NULL;
  }    
}

DM_DefinePrimitive(floor,1);

DM_Object*
DM_ceiling (DM_Context* context,
            DM_Object* a) {
  if ( DM_IsA(context, a, Float2 ) ) {
    DM_DeclareNarrowedObject(f, Float2, a);
    DM_Float2 *result = DM_Allocate0(context, Float2);
    
    result->value = ceil(f->value);
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, result);
    }
    return (DM_Object*)result;
  } else {DM_Abort(context, DME_NOT_A_NUMBER);
    /* NOT REACHED */
    return DM_NULL;
  }    
}

DM_DefinePrimitive(ceiling,1);

DM_Object*
DM_truncate (DM_Context* context,
             DM_Object* a) {
  if ( DM_IsA(context, a, Float2 ) ) {   
    DM_Abort(context, DME_NOT_YET_IMPLEMENTED);
    /* NOT REACHED */
    return DM_NULL;
  } else {
    DM_Abort(context, DME_NOT_AN_INEXACT);
    /* NOT REACHED */
    return DM_NULL;
  }
}

DM_DefinePrimitive(truncate,1);

DM_Object*
DM_round (DM_Context* context,
          DM_Object* a) {
  if ( DM_IsA(context, a, Float2 ) ) {  
    DM_Abort(context, DME_NOT_YET_IMPLEMENTED);
    /* NOT REACHED */
    return DM_NULL;
  } else {
    DM_Abort(context, DME_NOT_AN_INEXACT);
    /* NOT REACHED */
    return DM_NULL;
  }
}

DM_DefinePrimitive(round,1);

DM_Object*
DM_sqrt (DM_Context* context,
         DM_Object* a) {
  if ( DM_IsA(context, a, Float2 ) ) {
    DM_DeclareNarrowedObject(f, Float2, a);
    DM_Float2 *result = DM_Allocate0(context, Float2);
    
    result->value = sqrt(f->value);
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, result);
    }
    return (DM_Object*)result;
  } /* *****                            FUTURE
  if ( DM_IsA(context, a, Int4 ) ) {
    DM_Int4 *result = DM_Allocate0(context, Int4);
    
    result->value = sqrt(a->value);
    return (DM_Object*)result;
  }  ***** */
  else {
    DM_Abort(context, DME_NOT_A_FLOAT);
    /* NOT REACHED */
    return DM_NULL;
  }
}

DM_DefinePrimitive(sqrt,1);

DM_Object*
DM_expt (DM_Context* context,
         DM_Object* a,
         DM_Object* b ) {
  if ( DM_IsA(context, a, Float2 ) && DM_IsA(context, b, Float2 ) ) {
    DM_DeclareNarrowedObject(f, Float2, a);
    DM_DeclareNarrowedObject(g, Float2, b);
    DM_Float2 *result = DM_Allocate0(context, Float2);
    
    result->value = pow(f->value, g->value);
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, result);
    }
    return (DM_Object*)result;
  } else {
    DM_Abort(context, DME_NOT_A_FLOAT);
    /* NOT REACHED */
    return DM_NULL;
  }
}

DM_DefinePrimitive(expt,2);

DM_DefinePredicate(is_string,String);

DM_Object*
DM_make_string (DM_Context* context,
                DM_nat3 argc,
                DM_Object* (*argv)[1]) {
  if ( argc < 1 ) {
    DM_Abort(context, DME_WRONG_ARITY);
    /* NOT REACHED */
    return DM_NULL;
  } else {
    DM_reference n = *(argv[0]);
    
    if ( DM_IsA(context, n, Int4 ) ) {
      DM_nat3   size    = ((DM_Int4*)n)->value;
      DM_String *result = DM_Allocate1(context, String, size);

      if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
        DM_SetSharable(context, result);
      }
      switch ( argc ) {
        case 1 : {
          return (DM_Object*)result;
        }
        case 2 : {
          DM_nat3  i;
          DM_reference ch = *(argv[1]);
          
          if ( ! DM_IsA(context, ch, Character ) ) {
            DM_Abort(context, DME_NOT_A_CHARACTER);
          } 
          for ( i=0 ; i<size ; i++ ) {
            result->character[i] = ((DM_Character*)ch)->code;
          }
          return (DM_Object*)result;
        }
        default : {
          DM_Abort(context, DME_WRONG_ARITY);
          /* NOT REACHED */
          return DM_NULL;
        }
      }
    } else {
      DM_Abort(context, DME_NOT_A_NUMBER);
      /* NOT REACHED */
      return DM_NULL;
    }
  }
}

DM_DefinePrimitive(make_string,-1);

DM_Object*
DM_string (DM_Context* context,
           DM_nat3 argc,
           DM_Object* (*argv)[1]) {
  DM_nat3   i;

#ifdef DM_ICSLAS_PREFERS_MUTABILITY
  DM_String *result = DM_Allocate1(context, MutableString, argc);
#else
  DM_String *result = DM_Allocate1(context, String, argc);
#endif

  for ( i=0 ; i<argc ; i++ ) {
    DM_reference operand = *(argv[i]);

    if ( ! DM_IsA(context, operand, Character ) ) {
      DM_Abort(context, DME_NOT_A_CHARACTER);
      /* NOT REACHED */
      return DM_NULL;
    } 
    result->character[i] = ((DM_Character*)operand)->code;
  }
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, result);
  }
  return (DM_Object*)result;
}

DM_DefinePrimitive(string,-1);

DM_Object*
DM_string_copy (DM_Context* context,
                DM_Object* a) {
  DM_Object* result;

  if ( ! DM_IsA(context, a, String ) ) {
    DM_Abort(context, DME_NOT_A_STRING);
    /* NOT REACHED */
    return DM_NULL;
  }
  result = DM_clone(context, a);
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, result);
  }
  return (DM_Object*)result;
}

DM_DefinePrimitive(string_copy,1);

DM_DefinePredicate(is_char, Character);

DM_Object*
DM_is_vector (DM_Context* context,
              DM_Object* a) { 
  if ( DM_IsA(context, a, ImmutableVector)
       || DM_IsA(context, a, MutableVector ) ) {
    return DM_SupportTrue; 
  } else {
    return DM_SupportFalse; 
  }
}

DM_DefinePrimitive(is_vector,1);

DM_Object* 
DM_make_vector (DM_Context* context,
                DM_nat3 argc, 
                DM_Object* (*argv)[1]) {
  if ( argc < 1 ) {
    DM_Abort(context, DME_WRONG_ARITY);
    /* NOT REACHED */
    return DM_NULL;
  }
  { DM_reference n = *(argv[0]);
    
  if ( ! DM_IsA(context, n, Int4 ) ) {
    DM_Abort(context, DME_NOT_A_NUMBER);
    /* NOT REACHED */
    return DM_NULL;
  }
    { DM_nat3   size    = ((DM_Int4*)n)->value;

#ifdef DM_ICSLAS_PREFERS_MUTABILITY
      DM_MutableVector *result = DM_Allocate1(context, MutableVector, size);
#else
      DM_ImmutableVector *result = DM_Allocate1(context, ImmutableVector, size);
#endif

      if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
        DM_SetSharable(context, result);
      }
      switch ( argc ) {
        case 1 : return (DM_Object*)result;
        case 2 : {
          DM_nat3  i;
          DM_reference content = *(argv[1]);
          
          for ( i=0 ; i<size ; i++ ) {
            result->item[i] = content;
          }
          return (DM_Object*)result;
        }
        default : {
          DM_Abort(context, DME_WRONG_ARITY);
          /* NOT REACHED */
          return DM_NULL;
        }
        }
    }
  }
}

DM_DefinePrimitive(make_vector,-2);

DM_Object*
DM_vector (DM_Context* context,
           DM_nat3 argc, 
           DM_Object* (*argv)[1]) {
  DM_nat3            i;

#ifdef DM_ICSLAS_PREFERS_MUTABILITY
  DM_MutableVector  *result   = DM_Allocate1(context, MutableVector, argc);
#else
  DM_ImmutableVector *result  = DM_Allocate1(context, ImmutableVector, argc);
#endif

  for ( i=0 ; i<argc ; i++ ) {
    result->item[i] = *(argv[i]);
  }
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, result);
  }
  return (DM_Object*)result;
}

DM_DefinePrimitive(vector,-1);

DM_Object*
DM_is_procedure (DM_Context* context,
                 DM_Object* o) {
  if ( DM_IsA(context, o, Closure) ) 
    return DM_SupportTrue;
  if ( DM_IsA(context, o, Primitive) )
    return DM_SupportTrue;
  if ( DM_IsA(context, o, Continuation) )
    return DM_SupportTrue;
  if ( DM_IsA(context, o, SpecialPrimitive) )
    return DM_SupportTrue;
  else return DM_SupportFalse;
}

DM_DefinePrimitive(is_procedure,1);

DM_DefineSpecialPrimitive(apply,-3,DM_APPLY_CODE);

DM_DefineSpecialPrimitive(callcc,1,DM_CALLCC_CODE);

/******************************************************************
 * standard Unix ports viewed from Icslas.
 */

DM_DefinePredicate(is_input_port,InputPort);
DM_DefinePredicate(is_output_port,OutputPort);

DM_InBuffer_1024_wrapped DM_stdin_buffer_object = {
  DM_SupportPrefixInitialization(DM_InBuffer_wrapped,
                                 DM_stdin_buffer_object)
  DM_ReferenceToClass(InBuffer),
  DM_ProxyInitialization(),
  { { {
        (void*)(DM_ReferenceTo(DM_stdin)),  /* port */
        DM_NULL,    			/* cache */
        DM_NULL,			/* mark */
        0,                      	/* index */
        0,                      	/* maximum */
        0,                      	/* amount */
        DM_NULL,			/* next */
        DM_INITIAL_BUFFER_SIZE, 	/* byte_length */
        { 0,                    	/* bytes... */
        }
      } } }
};

DM_InputPort_wrapped DM_stdin_object = {
  DM_SupportPrefixInitialization(DM_InputPort_wrapped,
                                 DM_stdin_object)
  DM_ReferenceToClass(InputPort),
  DM_ProxyInitialization(),
  { { {
        0,                       /* fd */
        5,                       /* retries */
        DM_PORT_DEFAULT_OPTIONS, /* options */
        (void*)(DM_ReferenceTo(DM_stdin_buffer)),
        0,                       /* poken */
        0                        /* zoneindex */
      } } }
};

DM_OutBuffer_1024_wrapped DM_stdout_buffer_object = {
  DM_SupportPrefixInitialization(DM_OutBuffer_wrapped,
                                 DM_stdout_buffer_object)
  DM_ReferenceToClass(OutBuffer),
  DM_ProxyInitialization(),
  { { {
        (void*)(DM_ReferenceTo(DM_stdout)), 	/* port */
        DM_NULL,    			/* cache */
        DM_NULL,			/* mark */
        0,                      	/* index */
        0,                      	/* maximum */
        0,                      	/* amount */
        DM_NULL,			/* next */
        DM_INITIAL_BUFFER_SIZE, 	/* byte_length */
        { 0,                    	/* bytes... */
        }
      } } }
};

DM_OutputPort_wrapped DM_stdout_object = {
  DM_SupportPrefixInitialization(DM_OutputPort_wrapped,
                                 DM_stdout_object)
  DM_ReferenceToClass(OutputPort),
  DM_ProxyInitialization(),
  { { {
        1,                       /* fd */
        5,                       /* retries */
        DM_PORT_DEFAULT_OPTIONS, /* options */
        (void*)(DM_ReferenceTo(DM_stdout_buffer))
      } } }
};

DM_OutBuffer_1024_wrapped DM_stderr_buffer_object = {
  DM_SupportPrefixInitialization(DM_OutBuffer_wrapped,
                                 DM_stderr_buffer_object)
  DM_ReferenceToClass(OutBuffer),
  DM_ProxyInitialization(),
  { { {
        (void*)(DM_ReferenceTo(DM_stderr)),    	/* port */
        DM_NULL,    			/* cache */
        DM_NULL,			/* mark */
        0,                      	/* index */
        0,                      	/* maximum */
        0,                      	/* amount */
        DM_NULL,			/* next */
        DM_INITIAL_BUFFER_SIZE,    	/* byte_length */
        { 0,                       	/* bytes... */
        }
      } } }
};

DM_OutputPort_wrapped DM_stderr_object = {
  DM_SupportPrefixInitialization(DM_OutputPort_wrapped,
                                 DM_stderr_object)
  DM_ReferenceToClass(OutputPort),
  DM_ProxyInitialization(),
  { { {
        2,                       /* fd */
        5,                       /* retries */
        DM_PORT_DEFAULT_OPTIONS, /* options */
        (void*)(DM_ReferenceTo(DM_stderr_buffer))
      } } }
};

DM_Object*
DM_current_input_port (DM_Context* context) {
  return DM_ReferenceTo(DM_stdin);            /* TEMPORARY */
}

DM_DefinePrimitive(current_input_port,0);

DM_Object*
DM_current_output_port (DM_Context* context) {
  return DM_ReferenceTo(DM_stdout);           /* TEMPORARY */
}

DM_DefinePrimitive(current_output_port,0);

DM_Object*
DM_open_input_file (DM_Context* context,
                    DM_Object* a) {
  DM_Object* port;

  if ( ! DM_IsA(context, a, String ) ) 
    DM_Abort(context, DME_NOT_A_STRING);
  port = DM_open_input_port(context, ((DM_String*)a)->character);
  if ( port != DM_SupportFalse ) {
    return port;
  } else {
    DM_Abort(context, DME_CANNOT_OPEN_FILE);
    /* NOT REACHED */
    return DM_NULL;
  }
}

/*
 * Initialize an instance of InputPort. The 'number' field must be
 * present to activate the InputPort.
 */

DM_InputPort*
DM_establish_input_port (DM_Context*   context,
                         DM_InputPort* inputport) {
  DM_InBuffer *buffer = DM_Allocate1(context, InBuffer, DM_INITIAL_BUFFER_SIZE);
  DM_BoundedStack *cache = DM_Allocate1(context, BoundedStack, 2);
    
  inputport->options   = DM_PORT_DEFAULT_OPTIONS;
  inputport->buffer    = (void*)buffer;
  inputport->poken     = 0;
  inputport->zoneindex = 0;
  buffer->port         = (DM_Port*)inputport;
  buffer->cache        = (void*)cache;
  buffer->index        = 0;
  buffer->maximum      = 0;
  buffer->amount       = 0;
  /* Input cache has no extractor. */
  cache->index         = 0;
  if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
    DM_SetSharable(context, cache);
    DM_SetSharable(context, inputport);
    DM_SetSharable(context, buffer);
  }
  DM_activate_port(context, inputport->number, 
                   (DM_Port*)inputport, DM_PORT_TO_READ);
  return inputport;
}

/*
 * Opens a file and return a Port instance (with its associated Buffer).
 * If there is a problem, return DM_SupportFalse.
 */

DM_Object*
DM_open_input_port(DM_Context* context,
                   char *filename) {
  int fd;
    
  DM_ResetErrno();
  if ( (fd = open(filename, O_RDONLY, DM_OPEN_MODMASK)) < 0 ) {
    return DM_SupportFalse;
  } else {
    DM_InputPort *inputport = DM_Allocate0(context, InputPort);

   inputport->number    = fd;
   return (DM_Object*)(DM_establish_input_port(context, inputport));
  }
}

DM_DefinePrimitive(open_input_file,1);

DM_Object*
DM_open_output_file (DM_Context* context,
                     DM_Object* file) {
  if ( DM_IsA(context, file, String ) ) {
    DM_DeclareNarrowedObject(s, String, file);
    char         *filename = s->character;
    int          fd;
    
    DM_ResetErrno();
    if ( (fd = open(filename, O_WRONLY, DM_OPEN_MODMASK)) < 0 ) {
      DM_Abort(context, DME_CANNOT_OPEN_FILE);
      /* NOT REACHED */
      return DM_NULL;
    } else {
      DM_OutBuffer  *buffer = 
        DM_Allocate1(context, OutBuffer,DM_INITIAL_BUFFER_SIZE);
      DM_OutputPort *port   = DM_Allocate0(context, OutputPort);
      DM_BoundedStack *cache  = DM_Allocate1(context, BoundedStack,2);
      
      port->number        = fd;
      port->options       = DM_PORT_DEFAULT_OPTIONS;
      port->buffer        = (void*)buffer;
      buffer->port        = (void*)port;
      buffer->cache       = cache;
      buffer->index       = 0;
      buffer->maximum     = 0;
      buffer->amount      = 0;
      /* No extractor for a cache towards a file. */
      cache->index        = 0;
      if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
        DM_SetSharable(context, cache);
        DM_SetSharable(context, port);
        DM_SetSharable(context, buffer);
      }
      DM_activate_port(context, fd, (DM_Port*)port, DM_PORT_TO_WRITE);
      return (DM_Object*)port;
    }
  } else {
    DM_Abort(context, DME_NOT_A_STRING);
    /* NOT REACHED */
    return DM_NULL;
  }
}

DM_DefinePrimitive(open_output_file,1);

DM_Object*
DM_close_input_port (DM_Context* context,
                     DM_Object*  port) {
  if ( DM_IsA(context, port, InputPort ) ) {
    DM_DeclareNarrowedObject(p, InputPort, port);

    if ( close(p->number) < 0 ) {
      DM_SetOption(p, DM_PORT_FAILED_CLOSE);
      DM_Abort(context, DME_CANNOT_CLOSE);
    }
    DM_SetOption(p, DM_PORT_IS_CLOSED);
    return DM_ReferenceTo(DM_unspecified);
  } else {
    DM_Abort(context, DME_NOT_AN_INPUTPORT);
    /* NOT REACHED. */
    return port;
  }
}

DM_DefinePrimitive(close_input_port,1);

/* should rather be called finish_, take poken and flush into consideration
   do the rest in Scheme                     FUTURE */

DM_Object*
DM_close_output_port (DM_Context* context,
                      DM_Object*  port) {
  if ( DM_IsA(context, port, OutputPort ) ) {
    DM_DeclareNarrowedObject(p, OutputPort, port);

    if ( close(p->number) < 0 ) {
      DM_SetOption(p, DM_PORT_FAILED_CLOSE);
      DM_Abort(context, DME_CANNOT_CLOSE);
    } 
    DM_SetOption(p, DM_PORT_IS_CLOSED);
    return DM_ReferenceTo(DM_unspecified);
  } else {
    DM_Abort(context, DME_NOT_AN_OUTPUTPORT);
    /* NOT REACHED. */
    return port;
  }
}

DM_DefinePrimitive(close_output_port,1);

DM_Object*
DM_fill_input_port (DM_Context* context,
                    DM_Object*  port) {
  if ( DM_IsA(context, port, InputPort ) ) {
    DM_DeclareNarrowedObject(p, InputPort, port);
    DM_InBuffer *buffer = (DM_InBuffer*)(p->buffer);

    if ( DM_HasOption(p, DM_PORT_IS_CLOSED) ) 
      DM_Abort(context, DME_ALREADY_CLOSED_PORT);
    DM_fill_buffer(context, buffer);
    return DM_ReferenceTo(DM_unspecified);
  } else {
    DM_Abort(context, DME_NOT_AN_INPUTPORT);
    /* NOT REACHED. */
    return port;
  }
}

DM_DefinePrimitive(fill_input_port,1);

DM_Object*
DM_flush_output_port (DM_Context* context,
                      DM_Object* port) {
  if ( DM_IsA(context, port, OutputPort ) ) {
    DM_DeclareNarrowedObject(p, OutputPort, port);
    DM_OutBuffer *buffer = (DM_OutBuffer*)(p->buffer);

    if ( DM_HasOption(p, DM_PORT_IS_CLOSED) )
      DM_Abort(context, DME_ALREADY_CLOSED_PORT);
    DM_flush_buffer(context, buffer);
    return DM_NULL;
  } else {
    DM_Abort(context, DME_NOT_AN_OUTPUTPORT);
    /* NOT REACHED. */
    return port;
  }
}

DM_DefinePrimitive(flush_output_port,1);

DM_Object*
DM_do_read_char (DM_Context* context,
                 DM_Object* port) {
  if ( DM_IsA(context, port, InputPort ) ) {
    DM_DeclareNarrowedObject(p, InputPort, port);

    if ( DM_HasOption(p, DM_PORT_IS_CLOSED) ) {
      return DM_ReferenceTo(DM_end_of_file);
    } else { 
      DM_InBuffer *buffer = (DM_InBuffer*)(p->buffer);
      
      if ( buffer->index < buffer->maximum ) {
        char c = buffer->byte[(buffer->index)++];
        DM_ImmutableVector *DM_characters = 
          (DM_ImmutableVector*) DM_ReferenceTo(DM_characters);
        return DM_characters->item[(DM_nat1)c];
      }
      return DM_SupportFalse;
    }
  } else {
    DM_Abort(context, DME_NOT_AN_INPUTPORT);
    /* NOT REACHED. */
    return port;
  }
}

DM_DefinePrimitive(do_read_char,1);

DM_Object*
DM_do_write_char (DM_Context* context,
                  DM_Object* ch, 
                  DM_Object* port) {
  char c;

  if ( DM_IsA(context, port, OutputPort ) ) {
    DM_DeclareNarrowedObject(p, OutputPort, port);
    if ( DM_HasOption(p, DM_PORT_IS_CLOSED) )
      return DM_SupportFalse;
    if ( DM_IsA(context, ch, Character ) ) {
      DM_DeclareNarrowedObject(cc, Character, ch);
      c = cc->code;
      DM_print_char_star(context, &c, 1, port);
      return ch;
    } else {
      DM_Abort(context, DME_NOT_A_CHARACTER);
      /* NOT REACHED. */
      return port;
    }
  } else {
    DM_Abort(context, DME_NOT_AN_OUTPUTPORT);
    /* NOT REACHED. */
    return port;
  }
}

DM_DefinePrimitive(do_write_char,2);     

DM_Object*
DM_do_write_string (DM_Context* context,
                    DM_Object* str, 
                    DM_Object* port ) {
  if ( DM_IsA(context, str, String) ) {
    DM_DeclareNarrowedObject(s, String, str);
    DM_print_char_star(context,
                       s->character, 
                       s->character_length, 
                       port);
    return str;
  } else {
    DM_Abort(context, DME_NOT_A_STRING);
    /* NOT REACHED */
    return DM_NULL;
  }
}
 
DM_DefinePrimitive(do_write_string,2);

DM_Object*
DM_display (DM_Context* context,
            DM_nat3     argc, 
            DM_Object*  (*argv)[1]) {
  
  switch ( argc ) {
  case 1: {
    DM_Object* o = (void*)argv[0];

    DM_DebugAtLevel(0,
      DM_show_object(context, o, DM_current_output_port(context));
);

    return DM_SupportTrue;
  }
  case 2: {
    DM_Object* o    = (void*)argv[0];
    DM_Object* port = (void*)argv[1];

    DM_DebugAtLevel(0,
      DM_show_object(context, o, port);
);

    return DM_SupportTrue;
  }
  default: {
    DM_Abort(context, DME_WRONG_ARITY);
    /* NOT REACHED */
    return DM_NULL;
    }
  }
}

DM_DefinePrimitive(display,-2);

/******************************************************************
 * DMeroon primitives
 */

DM_DefinePrimitive(is_a,2);

DM_Object*
DM_class_of (DM_Context* context,
             DM_Object*  o ) {
  return (DM_Object*)(DM_ObjectToClass(context, o));
}

DM_DefinePrimitive(class_of,1);

/******************************************************************
 * This is the predefined environment containing all predefined 
 * primitives. This part of code is generated automatically from
 * BCpred.scm in order to keep the same indexes.

 * NOTE: To include it here saves all the declarations of the above
 * primitive objects.
 */

#include "xlapred.c"


/******************************************************************
 * Initialize this module.
 * NOTE: if Symbol can be subclassed into a Mutable class, there is a
 * problem when inserting a mutable thing in the oblist. In fact,
 * string->symbol and gensym intern the symbols they create and intern
 * is not a function invokable by the user so the problem is avoided.
 */

void 
DM_xlaenv_initialize (DM_Context* context) {

  /* Just to avoid warnings */
  DM_xlaenv_version_string_object = DM_xlaenv_version_string_object;

  /* The oblist is already initialized (in dmcomm.c). */

  /* Compute and set up the exact size of the predefined environment. */
  { DM_Environment *predef = (DM_Environment*)
      DM_ReferenceTo(DM_predefined_environment);
    int               i;
    
    for ( i=predef->value_length ; i>=0 ; i-- ) {
      if ( predef->value[i] == DM_TheConstant(uninitialized) ) {
        predef->value_length = i;
        break;
      }
    }
  }

  /* Setup caches in std* buffers. */
  { DM_BoundedStack *cache  = DM_Allocate1(context, BoundedStack,2);
    DM_InBuffer    *buffer = (DM_InBuffer*)(DM_ReferenceTo(DM_stdin_buffer));
    
    cache->index = 0;
    buffer->cache = cache;
    DM_SetSharable(context, cache);
  }
  { DM_BoundedStack *cache  = DM_Allocate1(context, BoundedStack,2);
    DM_OutBuffer   *buffer = (DM_OutBuffer*)(DM_ReferenceTo(DM_stdout_buffer));
    
    cache->index = 0;
    buffer->cache = cache;
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, cache);
    }
  }
  { DM_BoundedStack *cache  = DM_Allocate1(context, BoundedStack,2);
    DM_OutBuffer   *buffer = (DM_OutBuffer*)(DM_ReferenceTo(DM_stderr_buffer));
    
    cache->index = 0;
    buffer->cache = cache;
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, cache);
    }
  }
}

/* end of xlaenv.c */
