/* $Id: xlaobj.c,v 1.20 1997/03/26 15:45:27 queinnec Exp $ */

#include "xla.h"

DM_DefineStringWithCstring(xlaobj_version,
  "$Id: xlaobj.c,v 1.20 1997/03/26 15:45:27 queinnec Exp $");

/*
 * These objects are preallocated.
 *
 *   The null environment. 
 *      This object has its own class and a status similar to DM_nil.
 *
 *   The unspecified object (I chose #f) is defined as a macro in xla.h
 *   The end_of_file object.
 * 
 *   The Fixnums 0 and 1.
 */

DMeroon_object DM_null_environment_object = {
  DM_SupportPrefixInitialization(DMeroon_object,
                                 DM_null_environment_object)
  DM_ReferenceToClass(NullEnvironment),
  DM_ProxyInitialization()
};

DMeroon_object DM_end_of_file_object = {
  DM_SupportPrefixInitialization(DMeroon_object,
                                 DM_end_of_file_object)
  DM_ReferenceToClass(Object),
  DM_ProxyInitialization()
};

DM_Int4_wrapped DM_fixnum_0_object = {
  DM_SupportPrefixInitialization(DM_Int4_wrapped,
                                 DM_fixnum_0_object)
  DM_ReferenceToClass(Int4),
  DM_ProxyInitialization(),
  { { {
    0
  } } }
};

DM_Int4_wrapped DM_fixnum_1_object = {
  DM_SupportPrefixInitialization(DM_Int4_wrapped,
                                 DM_fixnum_1_object)
  DM_ReferenceToClass(Int4),
  DM_ProxyInitialization(),
  { { {
    1
  } } }
};

/*
 * The User instance describing the user.

 * NOTE: A statically allocated string to hold the user name.
 * There is a limit on the size of the user name. (remove that      FUTURE)
 */

DM_DefineStringWithCstring(user_name,"??nemo??");

DM_User_wrapped DM_user_object = {
  DM_SupportPrefixInitialization(DM_User_wrapped,
                                 DM_user_object)
  DM_ReferenceToClass(User),
  DM_ProxyInitialization(),
  { { {
    DM_ReferenceToString(user_name),
    DM_NULL,                           /* toplevel: to be filled */
  } } }
};

/*
 * Icslas expose some values as a dictionary that will be published
 * in the site object.
 * The structure of the Icslas dictionary is as follows:
 *    Icslas
 *      classes
 *        DM_Thread_class 
 *        ...
 *      characters
 *      threads
 *      predefined environment
 *      constants
 *      ...
 */

DM_Publish(900,"predefined environment",
           DM_ReferenceTo(DM_predefined_environment),
           DM_TheConstant(nil)
           );

/* Dictionary of Icslas-related constants. */
DM_Publish(910,"null environment",
           DM_ReferenceTo(DM_null_environment),
           DM_TheConstant(nil)
           );
DM_Publish(911,"end_of_file",
           DM_ReferenceTo(DM_end_of_file),
           DM_ReferenceTo(DM_910_dictionary)
           );
DM_Publish(912,"zero",
           DM_ReferenceTo(DM_fixnum_0),
           DM_ReferenceTo(DM_911_dictionary)
           );
/* Don't give `uninitialized' to the user ! */
DM_Publish(901,"constants",
           DM_ReferenceTo(DM_912_dictionary),
           DM_ReferenceTo(DM_900_dictionary)
           );

/* incorporate characters. */
DM_Publish(920,"characters",
           DM_ReferenceTo(DM_characters),
           DM_ReferenceTo(DM_901_dictionary)
           );

/* incorporate description of instructions. */
DM_Publish(931,"names",
           DM_ReferenceTo(DM_instruction_names),
           DM_TheConstant(nil)
           );
DM_Publish(932,"sizes",
           DM_ReferenceTo(DM_instruction_sizes),
           DM_ReferenceTo(DM_931_dictionary)
           );
DM_Publish(930,"instructions",
           DM_ReferenceTo(DM_932_dictionary),
           DM_ReferenceTo(DM_920_dictionary)
           );

/* Dictionary of Icslas classes. */
DM_Publish(801,"MutablePair",
           DM_ReferenceToClass(MutablePair),
           DM_TheConstant(nil)
           );
DM_Publish(802,"MutableString",
           DM_ReferenceToClass(MutableString),
           DM_ReferenceTo(DM_801_dictionary)
           );
/* etc for all other classes                   FUTURE */
DM_Publish(800,"classes",
           DM_ReferenceTo(DM_802_dictionary),
           DM_ReferenceTo(DM_930_dictionary)
           );

/* RESOURCE DM_icslas_dictionary
 * The top dictionary describing Icslas. It is exported to be inserted
 * in the 'information' field of the site.
*/
DM_Publish(icslas,"Icslas",
           DM_ReferenceTo(DM_800_dictionary),
           DM_TheConstant(nil)
           );

/******************************************************************
 * Initialize the User instance, give it a Toplevel for its global variables.
 */

void
DM_xlaobj_initialize (DM_Context* context) {
  DM_User      *user      = (DM_User*)DM_ReferenceTo(DM_user);
  DM_Hashtable *toplevel  = 
    DM_Allocate1(context, Hashtable, DM_HASHTABLE_INITIAL_SIZE);
  DM_String    *user_name = (DM_String*)(user->name);
  int          name_len;
  char         *name;

  /* Just to avoid warnings. */
  DM_xlaobj_version_string_object = DM_xlaobj_version_string_object;
  DM_icslas_dictionary_object = DM_icslas_dictionary_object;

  user->toplevel = toplevel;

  if ( ( name = getlogin() ) == NULL ) {
    struct passwd *pw;

    if ( ( pw = getpwuid(getuid()) ) == NULL ) {
       DM_Abort(context, DME_USERNAME_PROBLEM);
    } else {
      name = pw->pw_name;
    }
    name_len = strlen(name);
    if ( name_len > user_name->character_length )
      DM_Abort(context, DME_TOO_LONG_USERNAME);
    memcpy(user_name->character, name, name_len);
    user_name->character_length = name_len;
  }

  /* Add the Icslas dictionary to the published dictionaries of the
     site. We add it as the second published dictionary, normally
     after the DMeroon one but before the user one. */

  { DM_Site* site = DM_TheCurrentSite();
    DM_Dictionary* dmeroon_dict = site->information;
    DM_Dictionary* user_dict    = dmeroon_dict->rest;
    DM_Dictionary* icslas_dict = 
      (DM_Dictionary*)(DM_ReferenceTo(DM_icslas_dictionary));

    dmeroon_dict->rest = icslas_dict;
    icslas_dict->rest  = user_dict;
  }
  
}

/* end of xlaobj.c */
