;;; $Id: mkcmod.scm,v 1.13 1997/03/27 19:55:25 queinnec Exp $

;;; This file converts a ByteCoded module into a C file.

;;; NOTE: If you combine many such modules, symbols and any quotations
;;; containing symbols would be incorrect since symbols must be
;;; uniquely shared across modules. Albeit this is correct if there is
;;; only one module (during tests for example).
;;; NOTE: In fact this is still incorrect since the created symbols do not
;;; belong to the oblist.

;;; FUTURE: 1 in a quotation is not shared with the preallocated 1.
;;; neither is 0.

;;;==================================================================
;;; Read a module from file.xlo and convert it into a file.c module.

(define (file->BCRT-C-file filename)
  (call-with-output-file
      (string-append (extract-filename-sans-extension filename) ".c")
    (lambda (out)
      (BCRT-Module->C-file 
       (file->BCRT-Module filename)
       out ) ) ) )

(define (BCRT-Module->C-file bcrt-module out)
  (set! *quotes* '())
  (display "#include \"dmeroon.h\"" out)
  (newline out)
  (display "#include \"xla.h\"" out)
  (newline out)
  (newline out)
  ;; always output the name of the module as last quotation
  ;; so there is at least one quotation.
  (Scheme->C-file 
   (convert-to-scheme (BCRT-Module-name bcrt-module))
   "module_name"
   out )
  (newline out)
  ;; 
  (display "/* Names of global variables */" out)
  (newline out)
  (let ((n (BCRT-Module-varname-length bcrt-module)))
    (do ((i 0 (+ i 1)))
        ((>= i n))
      (Scheme->C-file 
       (convert-to-scheme (BCRT-Module-varname bcrt-module i))
       (string-append "var" (number->string i))
       out ) ) )
  (newline out)
  ;;
  (display "/* Quotations */" out)
  (newline out)
  (let ((n (BCRT-Module-quotation-length bcrt-module)))
    (do ((i 0 (+ i 1)))
        ((>= i n))
      (Scheme->C-file
       (convert-to-scheme (BCRT-Module-quotation bcrt-module i))
       (string-append "quote" (number->string i))
       out ) ) )
  ;;
  (newline out)
  (display "DM_DefineModulePrefix(" out)
  (display (convert-to-scheme (BCRT-Module-name bcrt-module)) out)
  (display "," out)
  (display (BCRT-Module-byte-length bcrt-module) out)
  (display "," out)
  (display (+ 1 (BCRT-Module-quotation-length bcrt-module)) out)
  (display "," out)
  (display (+ 1 (BCRT-Module-varname-length bcrt-module)) out)
  (display ") /* Module */" out)
  (newline out)
  (let ((n (BCRT-Module-byte-length bcrt-module)))
    (display "DM_ModuleByteCode(" out)
    (display n out)
    (display ")    /* bytecodes */" out)
    (do ((i 0 (+ i 1)))
        ((>= i n))
      (if (= 0 (modulo i 16))(newline out))
      (let ((code (BCRT-Module-byte bcrt-module i)))            
        (display code out) )
      (if (< i (- n 1)) (display ", " out)) )
    (newline out) )
  ;;
  (let ((n (BCRT-Module-quotation-length bcrt-module)))
    (display "DM_ModuleQuotations(" out)
    (display (+ n 1) out)
    (display ")    /* quotations */" out)
    (newline out)
    (do ((i 0 (+ i 1)))
        ((>= i n))
      (display "DM_ReferenceTo(quote" out)
      (display i out)
      (display ")," out) 
      (newline out) ) )
  ;; add the reference to the string naming the module as an extra quotation.
  (display "DM_ReferenceTo(module_name)" out)
  (newline out)
  ;;
  (let ((n (BCRT-Module-varname-length bcrt-module)))
    (cond 
     ;;; Add a dummy variable.
     ((= n 0) 
      ;; HACK: 0 is the real size even if one item is generated.
      (display "DM_ModuleVariableNames(0)    /* variable names */" out)
      (newline out)
      (display "DM_ReferenceTo(DM_nil)" out)
      (newline out) )
     (else
      (display "DM_ModuleVariableNames(" out)
      (display n out)
      (display ")    /* variable names */" out)
      (newline out)
      (do ((i 0 (+ i 1)))
          ((>= i n))
        (display "DM_ReferenceTo(var" out)
        (display i out)
        (display ")" out)
        (if (< i (- n 1)) (display ", " out))
        (newline out) ) ) ) )
  (display "DM_DefineModulePostfix();" out)
  (newline out)
  (newline out)
  (display "/* eof */" out)
  (newline out) )

;;;==================================================================
;;; Compile quotations.
;;; Share them maximally (they must not be circular).

;;; This variable memorizes tuples (scheme-value . name). They are
;;; memorized after the corresponding C code is output on file.

(define *quotes* '())

(define (Scheme->C-file o name out)
  (let ((v (assoc o *quotes*)))
    (if (pair? v)
        (Scheme-alias->C-file v name out)
        (cond
         ((string? o)  (Scheme-string->C-file o name out))
         ((symbol? o)  (Scheme-symbol->C-file o name out))
         ((char? o)    (Scheme-char->C-file o name out))
         ((pair? o)    (Scheme-pair->C-file o name out))
         ((null? o)    (Scheme-null->C-file o name out))
         ((integer? o) (Scheme-integer->C-file o name out))
         ((real? o)    (Scheme-float->C-file o name out))
         ((boolean? o) (Scheme-boolean->C-file o name out))
         (else (ByteCode-error "Cannot convert to C" o)) ) )
    (set! *quotes* (cons (cons o name) *quotes*)) ) )

(define (Scheme-alias->C-file v name out)
  (display "#define " out)
  (display name out)
  (display "_object " out)
  (display (cdr v) out)
  (display "_object" out)
  (newline out) )

(define (Scheme-integer->C-file o name out)
  (display "DM_DefineIcslasFixnum(" out)
  (display name out)
  (display "," out)
  (display o out)
  (display ");" out)
  (newline out) )

(define (Scheme-float->C-file o name out)
  (display "DM_DefineIcslasFloatnum(" out)
  (display name out)
  (display "," out)
  (display o out)
  (display ");" out)
  (newline out) )

(define (Scheme-boolean->C-file o name out)
  (display "#define " out)
  (display name out)
  (display "_object DM_" out)
  (display (if o "true" "false") out)
  (display "_object" out)
  (newline out) )  
  
(define (Scheme-null->C-file o name out)
  (display "#define " out)
  (display name out)
  (display "_object DM_nil_object" out)
  (newline out) )

(define (Scheme-pair->C-file o name out)
  (Scheme->C-file (car o) (string-append name "_a") out)
  (Scheme->C-file (cdr o) (string-append name "_d") out)
  (display "DM_DefineIcslasPair(" out)
  (display name out)
  (display ");" out)
  (newline out) )

(define (Scheme-char->C-file o name out)
  (display "#define " out)
  (display name out)
  (display "_object DM_character_objects[" out)
  (display (char->integer o) out)
  (display "]" out)
  (newline out) )

(define (Scheme-string->C-file o name out)
  (display "DM_DefineIcslasString(" out)
  (display name out)
  (display "," out)
  (write o out)
  (display "," out)
  (display (string-length o) out)
  (display ");" out)
  (newline out) )

(define (Scheme-symbol->C-file o name out)
  (display "DM_DefineIcslasSymbol(" out)
  (display name out)
  (display "," out)
  (write (symbol->string o) out)
  (display "," out)
  (display (string-length (symbol->string o)) out)
  (display ");" out)
  (newline out) )

;;;==================================================================
;;; Also generate the predefined environment from g.icslas.

;;; This variable defines what names do not correspond to C-written
;;; primitives. Some will be programmed some day, others will be
;;; written in Scheme. 

(define *ignored-names* '())

;;; This variable associates Scheme names to C names. 
;;; Boolean predicates have a C name that start with is_.

(define *predefined-translations* '())

;;; Define the predefined environment

(load (string-append TOP "/ByteCode/predenv.scm"))

;;; This function converts a Scheme symbol into a string naming the
;;; global variable in C. The diphtongue -> usual in Scheme is
;;; converted into _to_.  This is also the case for special characters
;;; such as >, <, = which are made more vocal. Some other letters are
;;; removed.

(define (C-equivalent-name symbol)
  (if (string? symbol) symbol
      ;; don't translate these symbols.
      (if (member symbol *ignored-names*)
          #f 
          (let ((v (assq symbol *predefined-translations*)))
            (if (pair? v)
                (cdr v)
                (list->string
                 (let convert ((pname (string->list (symbol->string symbol))))
                   (cond 
                    ((pair? pname)
                     (case (car pname)
                       ((#\? #\! #\/) (convert (cdr pname)))
                       ((#\>)
                        (append (string->list "_greater")
                                (convert (cdr pname)) ) )
                       ((#\<)
                        (append (string->list "_less")
                                (convert (cdr pname)) ) )
                       ((#\=)
                        (append (string->list "_equal")
                                (convert (cdr pname)) ) )
                       ((#\-) 
                        (if (and (pair? (cdr pname))
                                 (char=? #\> (cadr pname)) )
                            (append (string->list "_to_")
                                    (convert (cddr pname)) )
                            (cons #\_ (convert (cdr pname))) ) )
                       (else (cons (char-downcase (car pname))
                                   (convert (cdr pname)) )) ) )
                    (else '()) ) ) ) ) ) ) ) )

(define (find-maximal-index g)
  (let scan ((m 0) (g g))
    (if (pair? g)
        (let* ((pv (car g))
               (i  (BC-Predefined-Variable-rank pv)) )
          (if (> i m)
              (scan i (cdr g))
              (scan m (cdr g)) ) )
        m ) ) )

(define (g.icslas->C-predefined-environment g basefile)
  (define size (+ 1 (find-maximal-index g)))
  (let ((gv (make-vector size #f)))
    (let loop ((g g))
      (if (pair? g)
          (let* ((pv (car g))
                 (i  (BC-Predefined-Variable-rank pv)) )
            (if (vector-ref gv i)
                (Byte-Code-error "index conflict" i)
                (vector-set! gv i pv) )
            (loop (cdr g)) ) ) )
    (call-with-output-file (string-append basefile ".h")
      (lambda (out)
        (display "/* Automatically generated, don't touch! */" out)
        (newline out)
        (display "DM_ExternalizePredefinedEnvironment(" out)
        (display (+ size 1) out)
        (display ");" out)
        (newline out)
        (display "/* eof */" out)
        (newline out) ) )
    (call-with-output-file (string-append basefile ".c")
      (lambda (out)
        (display "/* Automatically generated, don't touch! */" out)
        (newline out)
        (display "DM_DefinePredefinedEnvironmentPrefix(" out)
        (display (+ size 1) out)
        (display ")" out)
        (newline out)
        (do ((i 0 (+ i 1)))
            ((>= i size))
          (let ((pv (vector-ref gv i)))
            (if pv
                (let ((name (C-equivalent-name (Predefined-Variable-name pv)))
                      (desc (Predefined-Variable-description pv)) )
                  (cond
                   ((not name) 
                    (display "DM_TheConstant(uninitialized)," out)
                    (newline out) )
                   ((Functional-Description? desc)
                    (display "DM_ReferenceTo(DM_" out)
                    (display name out)
                    (display "_primitive),  /* " out)
                    (display i out)
                    (display " */" out)
                    (newline out) )
                   (else 
                    (display "DM_ReferenceTo(DM_" out)
                    (display name out)
                    (display "),  /* " out)
                    (display i out)
                    (display " */" out)
                    (newline out) ) ) )
                ;; just leave a place
                (begin
                  (display "DM_TheConstant(uninitialized)," out)
                  (newline out) ) ) ) )
        (display "DM_DefinePredefinedEnvironmentPostfix();" out)
        (newline out)
        (display "/* eof */" out)
        (newline out) ) ) ) )
;;; Test: (g.icslas->C-predefined-environment g.icslas "ByteCode/c/xlapred.c")

;;; end of mkcmod.scm
