;;; $Id: mkcodop.scm,v 1.10 1997/02/28 19:50:35 queinnec Exp $

;;; This file dumps the ByteCode interpreter into C structures.
;;; It generates xlacodop.c and xlacodop.h files that describe
;;; ByteCode operations, arity and names.

;;; Caution, this file is evaluated in the TARGET directory!
;;; NOTE: this file must not use macros at all to be portable.

;;; Load the configuration parameters. This configuration file is 
;;; generated in the TARGET directory.

(load "config.scm")

;;;==================================================================
;;; Load the processor of Icslas sources.

(load (string-append TOP "/icslas.scm"))

;;; These are the features to load the Scheme simulated ByteCode interpreter.

(define *icslas-features*
  '(Bigloo               ;; use Bigloo for that.
    Meroonet
    TraditionalMacros
    FrontEnd
    Utilities
    FrameWork
    ByteCode
    debug               ; for BCRT-Module etc.
    ) )

;;; Load the ByteCode simulator

(process-icslas-sources
 *icslas-features*
 (lambda (basefile)
   (load (string-append TOP "/" basefile ".scm")) ) )

;;;==================================================================
;;; Dump the instruction-{name,size} tables.

(define (make-codop-files basefile)
  (make-codop-header-file basefile)
  (make-codop-sizes-and-names-file basefile) )

(define (make-codop-sizes-and-names-file basefile)
  (call-with-output-file (string-append basefile ".c")
    (lambda (out)
      (display "/* Automatically generated (Don't touch!). */" out)
      (newline out)
      (newline out)
      (display "#include \"xla.h\" " out)
      (newline out)
      (newline out)
      (display "DM_DefineInstructionSizesPrefix" out)
      (do ((i 0 (+ i 1)))
          ((>= i 256))
        (if (= 0 (modulo i 16)) (newline out))
        (let ((size (vector-ref *instruction-size* i)))
          (display size out)
          (if (not (= i 255)) (display ", " out)) ) )
      (newline out)
      (display "DM_DefineInstructionSizesPostfix;" out)
      (newline out)
      (newline out)
      (display "DM_DefineStringWithCstring(instruction_illegal," out)
      (write (symbol->string 'illegal) out)
      (display ");" out)
      (newline out)
      (do ((i 0 (+ i 1)))
          ((>= i 256))
        (let ((name (vector-ref *instruction-name* i)))
          (if (not (eq? name 'illegal))
              (begin
                (display "DM_DefineStringWithCstring(instruction_name" out)
                (display i out)
                (display "," out)
                (write (symbol->string name) out)
                (display ");" out)
                (newline out) ) ) ) )
      (newline out)
      (display "DM_DefineInstructionNamesPrefix" out)
      (newline out)
      (do ((i 0 (+ i 1)))
          ((>= i 256))
        (let ((name (vector-ref *instruction-name* i)))
          (if (eq? name 'illegal)
              (display "DM_ReferenceToString(instruction_illegal)" out)
              (begin
                (display "DM_ReferenceToString(instruction_name" out)
                (display i out)
                (display ")" out) ) )
          (if (not (= i 255))  (display "," out))
          (newline out) ) )
      (display "DM_DefineInstructionNamesPostfix;" out)
      (newline out)
      (newline out)
      (display "/* eof */" out)
      (newline out) ) ) )

;;; Take a symbol naming an instruction and return a string that will be used
;;; in C to designate the same thing.

(define (compute-c-instruction-name name)
  (list->string
   (map char-upcase
        (append
         (string->list (symbol->string 'DM_))
         (let convert ((chars (string->list (symbol->string name))))
           (if (pair? chars)
               (let ((ch (car chars)))
                 (case ch
                   ((#\-)  
                    (cons #\_ (convert (cdr chars))) )
                   ((#\+)
                    (append 
                     (if (pair? (cdr chars))
                         (string->list (symbol->string '_with_))
                         (string->list (symbol->string '_forward)) )
                     (convert (cdr chars)) ) )
                   ((#\>)
                    (append (string->list (symbol->string 'greater_))
                            (convert (cdr chars)) ) )
                   ((#\=)
                    (append (string->list (symbol->string 'equal_))
                            (convert (cdr chars)) ) )
                   ((#\<)
                    (append (string->list (symbol->string 'less_))
                            (convert (cdr chars)) ) )
                   (else (cons ch (convert (cdr chars)))) ) )
               '() ) )
         (string->list (symbol->string '_code)) ) ) ) )

(define (make-codop-header-file basefile)
  (call-with-output-file (string-append basefile ".h")
    (lambda (out)
      (display "/* Automatically generated (Don't touch!). */" out)
      (newline out)
      (newline out)
      (do ((i 0 (+ i 1)))
          ((>= i 256))
        (let ((name (vector-ref *instruction-name* i)))
          (if (not (eq? name 'illegal))
              (let ((cname (compute-c-instruction-name name)))
                (display "#define " out)
                (display cname out)
                (display " " out)
                (display i out)
                (newline out) ) ) ) )
      (display "/* eof */" out)
      (newline out) ) ) )

;;; The real work now:
;;;(make-codop-files "ByteCode/c/xlacodop")

;;; end of mkcodop.scm
