;;; $Id: Cenv.scm,v 1.2 1994/10/01 16:28:50 queinnec Exp $

;;; Define the predefined global environment for C compilation.

;;; Exportations:
;;;                   (initial-global-environment) : Global-Environment
;;; end of exportations.

;;; This function creates a new global environment. Its predefined
;;; part is static and shared (but it is fortunately immutable).

(define (predefined-global-env)
  (make-Global-Environment predefined-env '()) )

;;; The predefined environment is just a list of bindings. 

(define predefined-env '())

;;; Descriptions of functions for C are richer, they know how to
;;; generate a direct call to the C function.

(define-class Functional-Description-for-C Functional-Description
  ( generator
    ) )

;;; This macro enriches the predefined environment.

(define-icslas-macro (defprimitive name Cname arity)
  `(let ((v (make-Predefined-Variable 
             ',name (make-Functional-Description-for-C
                    = ,arity 
                    (make-predefined-application-generator ',Cname) ) )))
     (set! predefined-env (cons v predefined-env))
     ',name ) )


(defprimitive cons "SCM_cons" 2)
(defprimitive car "SCM_car" 1)
(defprimitive cdr "SCM_cdr" 1)
(defprimitive set-car! "SCM_set_car" 2)
(defprimitive set-cdr! "SCM_set_cdr" 2)
(defprimitive pair? "SCM_pairp" 1)
(defprimitive null? "SCM_nullp" 1)
(defprimitive symbol? "SCM_symbolp" 1)
(defprimitive stringp "SCM_stringp" 1)
(defprimitive eq? "SCM_eqp" 2)
(defprimitive integer? "SCM_fixnump" 1)
(defprimitive procedure? "SCM_procedurep" 1)

;;; Use macros instead of calling functions. 

(defprimitive + "SCM_Plus" 2)
(defprimitive - "SCM_Minus" 2)
(defprimitive * "SCM_Times" 2)
(defprimitive / "SCM_Quotient" 2)
(defprimitive remainder "SCM_Remainder" 2)
(defprimitive <= "SCM_LeP" 2)
(defprimitive >= "SCM_GeP" 2)
(defprimitive = "SCM_EqnP" 2)
(defprimitive < "SCM_LtP" 2)
(defprimitive > "SCM_GtP" 2)

(defprimitive call/ep "SCM_callep" 1)
(defprimitive print "SCM_print" 1)

;;; Not really true since SCM_prin is only monadic whereas display may
;;; have an optional port:

(defprimitive display "SCM_prin" 1)

;;; Define as well some global constants

(define-icslas-macro (definitial name value)
  `(let ((v (make-Predefined-Variable 
             ',name (make-Constant-Description ,value) )))
     (set! predefined-env (cons v predefined-env))
     ',name ) )

;;; Apply is a special case. It has to be invoked specially as an nary
;;; subr.  It is not inlined.

(begin
  (set! predefined-env
        (append (map (lambda (name) 
                       (make-Predefined-Variable name #f) )
                     '(list apply call/cc) )
                predefined-env ))
  'apply )

;;;==================================================================
;;; Predefine some global variables. 
;;; One is needed for objectify: the `unspecified' variable which holds
;;; a regular value with no specific properties. 

(definitial unspecified "SCM_unspecified")

(definitial t   "SCM_true")
(definitial f   "SCM_false")
(definitial nil "SCM_nil")

;;; end of Cenv.scm
