;;; $Id: Ctest.scm,v 1.4 1996/08/26 18:43:25 queinnec Exp $

;;; Quick and Dirty tests of the C code generation.

;;; The generic function to handle exceptions while testing.

(define-generic (test-C-compiler-handler (exc) c?))

(define-method (test-C-compiler-handler (exc Compilation-Warning) c?)
  (flush-output-port stdout)
  (display (Compilation-Warning-phase exc) stderr)
  (display " Warning: " stderr)
  (display (Compilation-Warning-message exc) stderr)
  (newline stderr)
  (for-each (lambda (hint) 
              (show hint stderr)
              (newline stderr) )
            (Compilation-Warning-culprits exc) ) )

(define *tests-to-skip*
  '( (SET! XYZZY 3)
     ) )

;;; NOTE: Tests that contain calls to call/cc are transformed to use
;;; call/ep.  Of course, on scheme.tst they will fail on continuations
;;; called out of their dynamic extent. 

(define (test-C-compiler file)
  (suite-test
   file
   "Scheme? "
   "Scheme= "
   #t
   (lambda (read check err)
     (define-method (test-C-compiler-handler (exc Exception) c?)
       (err exc) )
     (lambda ()
       (monitor test-C-compiler-handler
         (let ((e (read)))
           (cond ((member e *tests-to-skip*)
                  (compiler-main-error "Test to skip") )
                 ((appear? 'call/cc e)
                  (check (test-expression 
                          `((lambda (call/cc) ,e) call/ep) )) )
                 (else (check (test-expression e))) ) ) ) ) )
   naive-match ) )
;;; Test: (test-C-compiler "../DEA/scheme.tst")

;;; Compile a single Sexpression E. 
;;; Write it in a file, compile then run this program, reads its output 
;;; (normally a single Sexpression) and return it.

(define (test-expression e)
  (define tmpfile "/tmp/tmp.scm")
  (call-with-output-file tmpfile
    (lambda (out) (write e out)) )
  ;; This is the body of (compiler-main (list tmpfile))
  (let ((cmd (make-C-Compilation #f #f #f #f)))
    (scan-options! cmd (list tmpfile))
    (when (Compilation-verbose? cmd)
      (display "Generate C code...")(newline) )
    (let ((e `(begin . ,(file->list (Compilation-source cmd)))))
      ;; NOTE: no macro-expansion here for the sake of tests.
      (call-with-output-file (C-Compilation-cfile cmd)
        (lambda (out) (compile->C e (predefined-global-env) out)) ) )
    (when (Compilation-verbose? cmd)
      (display "Calling C compiler...")(newline) )
    (call-external-compiler cmd) )
  (run-compiled-program "/tmp/a.out") )

(define (run-compiled-program aout)
  (let ((status 0)
        (run-cmd (string-append
                  "/tmp/a.out >/tmp/tmplog ; "
                  "sed -e 's:#<:\<:g' -e 's:@::g' < /tmp/tmplog > /tmp/log" )) )
    (set! status (system run-cmd))
    (unless (= status 0)
      (compiler-main-error "C execution problem" status) )
    (let ((e (call-with-input-file "/tmp/log" read)))
      (if (eof-object? e)
          (compiler-main-error "No result" e)
          e ) ) ) )

;;; end of Ctest.scm
