/* $Id: dmc.h,v 1.14 1998/06/03 17:55:51 queinnec Exp $  
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////

 * DOCUMENTATION dmc.h
 * This file defines the interface of DMeroon for C.
 */

#ifndef DMeroon_C_H
#define DMeroon_C_H

/* There are two options:
 *   (1) Use Boehm's GC     [Define DM_USE_BOEHM_GC]
 *   (2) Use no GC          [Define DM_USE_NO_GC]
 */

#if defined(DM_USE_BOEHM_GC)

/* Include the header of Boehm's GC. */
#include "gc.h"
/* and use Boehm's allocator. */
# define DM_MALLOC GC_malloc
/* Boehm's GC zeroes allocated objects. */
#undef DM_NON_ZEROIC_MALLOC

#endif /* DM_USE_BOEHM_GC */


#if defined(DM_USE_NO_GC)

#define DM_MALLOC DMsm_malloc
#define DMsm_PAGE_SIZE  4*1024
#undef DM_NON_ZEROIC_MALLOC

#endif /* DM_USE_NO_GC */

#define DM_SUPPORT_LANGUAGE "C"

#define DMEROON_TAG 56423

/* 
 * In C, C references are similar to DMeroon references ie references
 * are to the first word of the content. The class and proxy fields must
 * be accessed backward from that reference. These macros allow to 
 * recognize a DMeroon object (depending from the point of view).
 */

#define DM_IsDMeroonInstance(wrapped_p) \
   ( ( wrapped_p ) && \
     ( DMEROON_TAG == (DM_ToC(DM_FromSupport(wrapped_p)))->_tag ) )

#define DM_IsObject(o) \
   ( ( o ) && (DMEROON_TAG == (DM_ToC(o))->_tag) )

/* 
 * There is no distinction between a pointer to a DMeroon object as 
 * seen from the C support language and as seen from DMeroon. 
 */

#define DM_SupportPointer  DM_Object*

#define DM_FromSupport(dmeroon) ((void*)(dmeroon))

#define DM_ToSupport(dmeroon) ((DM_SupportPointer)(dmeroon))

/* 
 * Layout of a DMeroon value:
 *
 * typedef struct DMeroon_object {
 *   _tag;                    <-- C prefix
 *   _padding;
 *   DM_reference _class;     <-- DMeroon prefix
 *   proxy information;
 *   blah blah blah;          <-- DMeroon content  <== DMeroon or C reference
 * } DMeroon_object;
 */

#define DM_SupportPrefix() \
  DM_nat3 _tag; \
  DM_nat3 _padding;

#define DM_SupportPrefixInitialization(type, varname) \
  DMEROON_TAG, \
  sizeof(type),

/* 
 * This is the precise function that initializes both DMeroon and the
 * binding language (here C).
 */

#define DM_INITIALIZE DM_c_initialize

#endif /* DMeroon_C_H */

/* DOCUMENTATION
 * This is the second part of this header file. 
 * It defines the functions that are required by DMeroon to run.
 *
 * DM_initialize should be called before you use DMeroon (somewhere in
 *   your main).
 *
 * DM_allocate_bare_instance allocates an instance of <class> with a given
 *   size (expressed in bytes).
 *
 * DM_is_object tests if a pointer is the address of a DMeroon object.
 *
 * NOTE: These prototypes use DMeroon definitions so they must be processed
 * after all DMeroon header files.
 */

#ifdef DMeroon_SUPPORT_INCLUDED_AGAIN

extern DM_Object*
DM_allocate_bare_instance (DM_Context *context,
                           DM_Class   *class, 
                           DM_nat3    size );

extern DM_c_boolean
DM_is_object (DM_SupportPointer wrapped_p);

extern void
DM_finalize_exit (void *exit, void *client_data);

extern void
DM_set_exit_finalization (DM_Exit *exit);

extern void
DM_print_char_star (DM_Context *context,
                    char       *string, 
                    DM_nat3    len,
                    void       *output_port);

extern void DM_c_initialize (DM_Context *context);

#endif /*  DMeroon_SUPPORT_INCLUDED_AGAIN */

/* end of dmc.h */
