#! /usr/local/bin/wish -f
# $Id: inspect.tcl,v 1.7 1996/05/21 11:37:30 queinnec Exp $

# A procedure to print on the stdout and immediately flush the output.

proc outputCommand { cmd } {
    puts stdout $cmd
    flush stdout
}

# Immediately send something so DMeroon knows inspect.tcl is up.

outputCommand "inspector"

wm title . "DMeroon inspector"

button .quit -text "Quit" -command { 
    outputCommand "quit"
    exit
}

set helpTextForInspector {\
    This inspector allows you to enquire DMeroon at work.
\$Id: inspect.tcl,v 1.7 1996/05/21 11:37:30 queinnec Exp $

It must be started from DMeroon with a command like:
    (DM-fork-inspector "${TOP}/Commands/inspect.tcl") in Scheme
or  DM_fork_inspector("${TOP}/Commands/inspect.tcl")  in C
where TOP is the directory where DMeroon is.

Your clicks and addresses are forwarded to DMeroon which outputs
the associated results on its stderr.  When giving the address of a
DMeroon instance, you may choose to display it in text or
bytes. Memory slices may also be dumped, their size may be given by
the scale. Addresses may be harmlessly prefixed by "0x", the address
field supports pasting in addresses with mouse button 2. Special 
DMeroon objects can be obtained directly via their names (this menu
may be torn off with mouse button 2).

Enjoy (this is my first Tcl attempt!).
}

# a width of 600 seems to be sufficient on most machines depending on the
# default font that is used.

button .help -text "Help" -command {
    outputCommand "help"
    toplevel .helptext
    wm title .helptext "Help for DMeroon inspector"
    message .helptext.text -width 600 -text $helpTextForInspector
    frame .helptext.frame
    button .helptext.frame.ok -text "OK" -command { destroy .helptext }
    button .helptext.frame.reload -text "Source" \
            -command { source [ info script ] }
    pack .helptext.frame.ok -side left
    pack .helptext.text .helptext.frame    
}

button .types -text "Show types" -command {
    outputCommand "types"
}

button .ports -text "Show ports" -command {
    outputCommand "ports"
}

button .site -text "Show site" -command {
    outputCommand "site"
}

button .constants -text "Constants" -command {
    outputCommand "constants"
}

# a queue of last queried addresses is maintained.

set memorized 0
set maxmemorized 5
set showers { }

frame .frame

entry .frame.address -exportselection yes -textvariable address -relief sunken

focus .frame.address

bind .frame.address <Return> { 
    handleAddress
}

bind .frame.address <ButtonRelease-2> {
    set address [ selection get STRING ]
    handleAddress
}

label .frame.label -text "0x"

button .frame.doit -text "Show!" -command {
    handleAddress
}

proc handleAddress { } {
    global address memorized mode maxmemorized showers
    if [ regexp -nocase {^(0x)?([0-9a-f]+)$} $address w p address ] {
        outputCommand [ makeCommand "0x$address" ]
        set memorized [ expr $memorized + 1 ]
        set tempcommand [ makeCommand "0x$address" ]
        button .show$memorized -text "$tempcommand" \
                -command [ list outputCommand $tempcommand ]
        pack .show$memorized -fill x
        lappend showers .show$memorized
        if { $memorized > $maxmemorized } {
            destroy [ lindex $showers 0 ]
            set showers [ lreplace $showers 0 0 ]
        }
    } else {
        error "Illegal address"
    }
}

pack .frame.label .frame.address .frame.doit -side left

# This variable tells whether requests to show an object should
# display it in bytes or text.

frame .choices

set size 32

radiobutton .choices.showbyte -text "bytes" -variable mode -value "bytes"
radiobutton .choices.showtext -text "text" -variable mode -value "print"
radiobutton .choices.showdump -text "dump" -variable mode -value "dump"
radiobutton .choices.showentry -text "entry" -variable mode -value "entry"

set mode "print"

scale .size -label "slice size" -from 0 -to 256 -orient horizontal \
        -command "set size"
.size set $size

pack .choices.showbyte .choices.showtext .choices.showdump \
        .choices.showentry -side left

proc makeCommand { address } {
    global mode size
    if { $mode == "dump" } {
        return "dump $address $size"
    } else {
        return "$mode $address"
    }
}

pack .quit -fill x
pack .help -fill x
pack .choices 
pack .size -fill x
pack .types -fill x 
pack .ports -fill x 
pack .site  -fill x
pack .constants -fill x 
#pack .objects -fill x
pack .frame -fill x 

# end of inspect.tcl
