#! /bin/sh
# $Id: propose.sh,v 1.14 1998/08/23 08:54:16 queinnec Exp $

# Syntax: propose.sh SUPPORT TARGET
SUPPORT=$1
TARGET=$2
OK=true

# This script is run if TOP/Imakefile detects some unspecified variables.
# It tries to interactively solve the problem. It's easy to add new
# cases to it: just add two lines below: one for proposing a choice
# (gives inside bracket a non-ambiguous prefix to name it), the other
# line sets appropriately the SUPPORT variable to the name of the 
# config/$SUPPORT.mkf file that describes how to regenerate DMeroon.

################################ ZERO: Confirmation

cat <<eof

                Welcome to the Scheme/Icslas/DMeroon world!

This script will ask you a few questions to determine what you want.
You may want to regenerate DMeroon with some support language or 
rebuild the runtime for a Scheme compiler for some target language. 

    Do you want to rebuild 
        [D]     DMeroon...............library
        [C]     Scheme to C...........compiler
        [J]     Scheme to Java........compiler
        [B]     Scheme to ByteCode....compiler

    Which one do you choose?
eof
        read answer
        case "$answer" in
        [dD]*)  WORLD=world ;;
        [cC]*)  SUPPORT=C
                WORLD=regenerate.runtime ;;
        [jJ]*)  SUPPORT=Java 
                WORLD=regenerate.runtime ;;
        [bB]*)  SUPPORT=ByteCode
                WORLD=regenerate.runtime ;;
        *)      cat <<eof
*** Not a legal choice!
eof
                exit 11 ;;
        esac

################################ ONE: try to identify the support language.
# This section is only needed for DMeroon.

if [ -z "$SUPPORT" ]
then 
        OK=false
        cat <<eof

*** Missing variable SUPPORT!            Dont be worry, I take care of that.
You may actually use one of the following support language:
        [b]     Bigloo
        [b9]	Bigloo 1.9
        [c]     C (gcc)
        [ec]    C (egcs)
        [cpp]	C++
        [i]     Icslas
        [j]     Java
        [o]     OScheme
        [oi]    OScheme+Icslas
	[p]     Pico
        [stk]   STk
        [s2c]   Scheme->C

    Which one do you choose?
eof
        read lang
        case "$lang" in
        b9*)		SUPPORT=bigloo19 ;;
        b*)     	SUPPORT=bigloo ;;
        c)      	SUPPORT=c ;;
        ec)      	SUPPORT=cegcs ;;
        cpp|C++)      	SUPPORT=cpp ;;
        i*)     	SUPPORT=icslas ;;
        j*)     	SUPPORT=java ;;
        oi)     	SUPPORT=oi ;;
        o*)     	SUPPORT=oscheme ;;
	p*)     	SUPPORT=pico ;;
        s2c*)   	SUPPORT=scheme2c ;;
        stk*)   	SUPPORT=stk ;;
        *)      cat <<eof
*** Not a legal choice!
eof
                exit 13 ;;
        esac
fi

###############################TWO: identify the current machine
# If ARCH is already defined then assume it is correct.

if [ -z "$ARCH" ]
then 
        cat <<eof

Let me see what kind of machine you have...

eof
make -f Imakefile /tmp/dmarch
cat <<eof

You seem to have a `/tmp/dmarch`.

eof
        ARCH=`/tmp/dmarch`
else
        cat <<eof

I assume you have a ${ARCH} machine.

eof
        echo "echo $ARCH" > /tmp/dmarch
        chmod a+x /tmp/dmarch
fi

###############################THREE: determine where to compile
# The target name is made of the name of the support language and the
# type of machine where it will be compiled.

if [ -z "$TARGET" ] 
then 
        OK=false
        TARGET=`pwd`/o/"$SUPPORT.$ARCH" 
fi

# Use an absolute path
case "$TARGET" in
/*)     : ;;
*)      OK=false
        TARGET=`pwd`/$TARGET
        cat <<eof

TARGET should be an absolute path!      Dont be worry, I take care of that.
eof
esac

###############################FOUR: propose to run a command
if $OK
then
        exit 0
else
        cat <<eof

I propose you to run the following command:

        make -f Imakefile SUPPORT=$SUPPORT TARGET=$TARGET WORLD=${WORLD} work

this means that I will create the TARGET directory mentioned above,
I will populate it with files and compile them there. 
In case of problems (who knows ?), cd to TARGET, edit the generated
Makefile (mainly some wrong pathnames) and re-run 'make world'...

   Should I proceed (yes,no)[y] ?
eof

        read answer
        case "$answer" in
        n*)     echo "Too bad!"
                exit 15
                ;;
        *)     
                make -f Imakefile mkdir SUPPORT=$SUPPORT \
                        TARGET=$TARGET WORLD=${WORLD} 
                exec make -f Imakefile work SUPPORT=$SUPPORT \
                        TARGET=$TARGET WORLD=${WORLD} 
                ;;
        esac        
fi

# end of propose.sh
