;;; $Id: c.tst,v 1.6 1995/12/13 18:05:12 queinnec Exp $

;;; This file tests the c.scm file that implements management of C
;;; structures. It also provides examples of use.

'(load "c.scm")
   ---

;;; This is the root class for C structures.
(DM-print-class C-class (current-output-port))
   ---
;;; Define a structure with a unique char field.
(define c1-class 
  (C-create-structure 'c1 (list DM-C-char-type)) )
   ---
(DM-print-class c1-class (current-output-port))
   ---
;;; Fields can be retrieved with ->Field
;;; (DM-ClassToField is unsafe and reserved for internal use only).
(eq? (->Field c1-class 0) (DM-ClassToField c1-class 0))
   #t
;;; allocate an instance of a C structure.
(DM-allocate c1-class)
   ---
;;; test C-get to access a C structure.
(let ((o (DM-allocate c1-class)))
  (C-get o (->Field c1-class 0)) )
   ---
;;; test C-set! to modify a C structure
(let ((o (DM-allocate c1-class)))
  (C-set! o (->Field c1-class 0) #\A) )
   ---
(let ((o (DM-allocate c1-class)))
  (C-set! o (->Field c1-class 0) #\A)
  (C-get o (->Field c1-class 0)) )
   #\A
;;; test DM-make to allocate and initialize a C structure
(let ((o (DM-make c1-class #\B)))
  (C-get o (->Field c1-class 0)) )
   #\B

;;; Test ->C-Type coercer
(eq? DM-C-short-int-type (->C-Type 'short))
   #t
;;; define a class using this coercer
(define c2-class 
  (C-create-structure 'c2 
                      (map ->C-Type '((long int)
                                      "unsigned_char" ))) )
   ---
(DM-print-class c2-class (current-output-port))
   ---
;;; test again ->Field
(let ((o (DM-make c2-class 12345 45)))
  (C-set! o (->Field c2-class 1) 56)
  (C-get o (->Field c2-class 1)) )
   56

;;; define a class without fields
(define c4-class
  (C-create-structure 'c4 '()) )
   ---
(DM-print-class c4-class (current-output-port))
   ---
;;; Define a class with named fields
(define c3-class
  (C-create-structure 'c3
         (list (cons 'x DM-C-unsigned-int-type)
               (cons "y" DM-C-void-star-type) ) ) )
   ---
(DM-print-class c3-class (current-output-port))
   ---
;;; test ->Field on symbols and strings.
(let ((o (DM-make c3-class 12345 45)))
  (C-set! o (->Field c3-class 'x) 56)
  (C-set! o (->Field c3-class 'x) 56) )
   56
(let ((o (DM-make c3-class 12345 45)))
  (C-set! o (->Field c3-class "y") 56)
  (C-set! o (->Field c3-class "y") 56) )
   56


" *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
                      All c.tst tests successfully run!
 *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***"
   ---

;;; end of c.tst
