;;; $Id: dict.tst,v 1.3 1995/09/19 11:54:08 queinnec Exp $

;;; testing dictionaries

;;; Testing dictionary at level 1
(define data1 (DM-make DM-Nat2-class 22))
   ---
(define data2 (DM-make DM-Int3-class -33))
   ---
;;; Finding a given entry
(let ((d DM-nil))
  (eq? DM-nil (dict-find d "foo")) )
   #t
(let ((d (DM-make DM-Dictionary-class DM-nil data1 (->String "foo"))))
   (unveil d) )
   ---
(let ((d (DM-make DM-Dictionary-class DM-nil data1 (->String "foo"))))
   (dict-find d "foo") )
   ---
(let ((d (DM-make DM-Dictionary-class DM-nil data1 (->String "foo"))))
   (eq? data1 (DM-field-value (dict-find d "foo") DM-Dictionary-value-field)) )
   #t

;;; search something absent
(let ((d (DM-make DM-Dictionary-class DM-nil data1 (->String "foo"))))
   (eq? DM-nil (dict-find d "one")) )
   ---
;;; adjoin one entry
(let ((d DM-nil))
  (unveil (dict-adjoin! d data1 "one")) )
   ---
;;; and find it
(let ((d DM-nil))
  (set! d (dict-adjoin! d data1 "one"))
  (eq? data1 (DM-field-value (dict-find d "one") DM-Dictionary-value-field)) )
   #t
;;; adjoin two different entries
(let ((d DM-nil))
  (set! d (dict-adjoin! d data1 "one"))
  ;;(unveil d)
  ;;(unveil (dict-find d "one"))
  ;;(unveil (dict-find d "two"))
  (set! d (dict-adjoin! d data2 "two"))
  (unveil d) )
   ---
;;; and search through them
(let ((d DM-nil))
  (set! d (dict-adjoin! d data1 "one"))
  (set! d (dict-adjoin! d data2 "two"))
  (eq? data1 (DM-field-value (dict-find d "one") DM-Dictionary-value-field)) )
   #t
(let ((d DM-nil))
  (set! d (dict-adjoin! d data1 "one"))
  (set! d (dict-adjoin! d data2 "two"))
  (eq? data2 (DM-field-value (dict-find d "two") DM-Dictionary-value-field)) )
   #t
;;; Modify an existing entry
(let ((d DM-nil))
  (set! d (dict-adjoin! d data1 "one"))
  (set! d (dict-adjoin! d data2 "two"))
  (set! d (dict-adjoin! d data2 "one"))
  (eq? data2 (DM-field-value (dict-find d "one") DM-Dictionary-value-field)) )
   #t

;;; testing dictionary at deeper levels
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d data1 '("aaa" "one")))
  (unveil d)
  (newline)
  (display (dict-rec-list d)) )
   ---
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d data1 '("aaa" "one")))
  (eq? data1 (DM-field-value (dict-rec-find d '("aaa" "one"))
                          DM-Dictionary-value-field )) )
   #t
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d data1 '("aaa" "one")))
  (set! d (dict-rec-adjoin! d data2 '("aaa" "two")))
  (unveil d)
  (newline)
  (display (dict-rec-list d)) )
   ---
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d data1 '("aaa" "one")))
  (set! d (dict-rec-adjoin! d data2 '("aaa" "two")))
  (eq? data1 (DM-field-value (dict-rec-find d '("aaa" "one"))
                          DM-Dictionary-value-field )) )
   #t
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d data1 '("aaa" "one")))
  (set! d (dict-rec-adjoin! d data2 '("aaa" "two")))
  (eq? data2 (DM-field-value (dict-rec-find d '("aaa" "two"))
                          DM-Dictionary-value-field )) )
   #t
;;; modifying a deeper pathname
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d data1 '("aaa" "one")))
  (set! d (dict-rec-adjoin! d data2 '("aaa" "one")))
  (eq? data2 (DM-field-value (dict-rec-find d '("aaa" "one"))
                          DM-Dictionary-value-field )) )
  #t
;;; modifying a value that was once a dictionary.
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d DM-nil '("aaa" "one")))
  (set! d (dict-rec-adjoin! d (->String "A is here") '("aaa" "one")))
  (set! d (dict-rec-adjoin! d (->String "A just!") '("aaa")))
  (unveil d) )
   ---
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d DM-nil '("aaa" "one")))
  (set! d (dict-rec-adjoin! d (->String "A is here") '("aaa" "one")))
  (set! d (dict-rec-adjoin! d (->String "A just!") '("aaa")))
  (DM-is-a (dict-rec-find d '("aaa")) DM-Dictionary-class) )
   #t
;;; adding multiple entries at multiple levels
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d DM-nil '("aaa" "one")))
  (set! d (dict-rec-adjoin! d DM-nil '("aaa" "two")))
  (set! d (dict-rec-adjoin! d DM-nil '("bbb" "three" "chetire")))
  (set! d (dict-rec-adjoin! d DM-nil '("ccc")))
  (unveil d) )
   ---
;;; and modifying one of them
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d DM-nil '("aaa" "one")))
  (set! d (dict-rec-adjoin! d DM-nil '("ccc")))
  (set! d (dict-rec-adjoin! d DM-nil '("aaa" "two")))
  (set! d (dict-rec-adjoin! d DM-nil '("bbb" "three" "chetire")))
  (set! d (dict-rec-adjoin! d data1 '("ccc")))
  (eq? data1 (DM-field-value (dict-rec-find d '("ccc"))
                          DM-Dictionary-value-field )) )
   #t
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d DM-Object-class '("types" "Object")))
  (set! d (dict-rec-adjoin! d DM-Class-class '("types" "Class")))
  (set! d (dict-rec-adjoin! d DM-Type-class '("types" "Type")))
  (set! d (dict-rec-adjoin! d DM-nil '("nil")))
  (eq? DM-Class-class 
       (DM-field-value (dict-rec-find d '("types" "Class"))
                    DM-Dictionary-value-field ) ) )
   #t
(let ((d DM-nil))
  (set! d (dict-rec-adjoin! d DM-Object-class '("types" "Object")))
  (set! d (dict-rec-adjoin! d DM-Class-class '("types" "Class")))
  (set! d (dict-rec-adjoin! d DM-nil '("nil")))
  (set! d (dict-rec-adjoin! d DM-Type-class '("types" "Type")))
  (eq? DM-nil
       (DM-field-value (dict-rec-find d '("nil"))
                    DM-Dictionary-value-field ) ) )
   #t


;;; Testing DM-publish!
;;; at level 1
(dict-rec-list (DM-field-value DM-site DM-Site-information-field))
   ---
(DM-publish! data1 "fb")
   ---
(dict-rec-list (DM-field-value DM-site DM-Site-information-field))
   ---
(eq? data1 
     (DM-field-value (dict-rec-find 
                   (DM-field-value DM-site DM-Site-information-field)
                   '("user" "fb") )
                  DM-Dictionary-value-field ) )
   #t
(eq? data1 (DM-retrieve "fb"))
   #t
;;; at deeper level
(DM-publish! data2 "f" "b" "d")
   ---
(eq? data2
 (DM-field-value (dict-rec-find (DM-field-value DM-site DM-Site-information-field)
                             '("user" "f" "b" "d") )
              DM-Dictionary-value-field ) )
   #t
(eq? data2 (DM-retrieve "f" "b" "d"))
   #t

" *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
                      All dict.tst tests successfully run!
 *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***"
   ---

;;; end of dict.tst
