;;; $Id: dmeroon.tst,v 1.33 1995/12/15 16:02:48 queinnec Exp $
;;;///////////////////////////////////////////////////////////////////////
;;;                      //          //
;;;                     // DMeroon  //
;;;                    //          //
;;; A continuation-based causal distributed class-based object system.
;;; Christian.Queinnec@polytechnique.fr
;;;///////////////////////////////////////////////////////////////////////

;;; This file contains tests for the Scheme binding of DMeroon.

;;; First print the types
(DM-print-types (current-output-port))
   ---

;;; Test DMeroon membership
(DM-is-object #t)
   #f
(DM-is-object DM-Class-class)
   #t
(DM-is-object DM-nat1-type)
   #t

;;; Indexes are independent of bindings.
(DM-TypeToIndex DM-nought-type)
   0
(DM-TypeToIndex DM-nat3-type)
   3

;;; Check instantiation links
(eq? (DM-ObjectToClass DM-Object-class)
     (DM-ObjectToClass DM-Class-class) )
   #t
(eq? (DM-ObjectToClass DM-nat1-type)
     DM-DMeroonType-class )
   #t

;;; Check the inheritance tree.
;;; Remember that classes appear before fields in a class instance.
(DM-ClassToSuperLength DM-Object-class)
   1
(DM-ClassToSuperLength DM-Class-class)
   2
(eq? (DM-ClassToSuper DM-Object-class 0) DM-Object-class)
   #t
(eq? (DM-ClassToSuper DM-Class-class 0) DM-Object-class)
   #t
(eq? (DM-ClassToSuper DM-Class-class 1) DM-Class-class)
   #t

;;; Check class membership
(DM-is-a DM-Object-class DM-Class-class)
   #t
(DM-is-a DM-Object-class DM-Object-class)
   #t
(DM-is-a DM-Class-class DM-Object-class)
   #t
(DM-is-a DM-Class-class DM-Class-class)
   #t
(DM-is-a DM-Object-class DM-Field-class)
   #f
(DM-is-a DM-Class-name-field DM-Field-class)
   #t
(DM-is-a DM-Class-name-field DM-Class-class)
   #f

;;; Check field access.
;;; This is now difficult since fields are stored after superclasses in
;;; class instances. Fortunately, DM-ClassToField* functions are primitive,
;;; so we just test them.
(DM-ClassToFieldLength DM-String-class)
   1
(DM-ClassToFieldLength DM-Class-class)
   4
(DM-ClassToFieldLength DM-Object-class)
   0
(eq? (DM-ClassToField DM-String-class 0) DM-String-character-field)
   #t
(eq? (DM-ClassToField DM-Type-class 1) DM-Type-index-field)
   #t
(eq? (DM-ClassToField DM-Type-class 3) DM-Type-size-field)
   #t


;;; Test the computation of offsets.

;;; First, test fields that can be accessed with a simple displacement
(= (DM-local-get DM-nat1-type DM-nat3-type DM-Type-index-field)
   (DM-TypeToIndex DM-nat3-type) )
   #t 
(= (DM-local-get DM-nat1-type DM-nat3-type DM-Type-size-field)
   (DM-TypeToSize DM-nat3-type) )
   #t 
(= (DM-local-get DM-nat1-type DM-nat3-type DM-Type-alignment-field)
   (DM-TypeToAlignment DM-nat3-type) )
   #t 
;;; The length of an indexed field not preceded by indexed fields should
;;; be accessed without problems.
(= (DM-local-length DM-nat3-type DM-Class-class DM-Class-super-field)
   (DM-ClassToSuperLength DM-Class-class) )
   #t
;;; and fields that are accessed with a displacement plus an index
(eq? (DM-local-indexed-get DM-reference-type DM-Class-class DM-Class-super-field 0)
     (DM-ClassToSuper DM-Class-class 0) )
   #t
(eq? (DM-local-indexed-get DM-reference-type DM-Class-class DM-Class-super-field 1)
     (DM-ClassToSuper DM-Class-class 1) )
   #t

;;; Second, test fields after an indexed field.
(= (DM-local-length DM-nat3-type DM-Class-class DM-Class-field-field)
   (DM-ClassToFieldLength DM-Class-class) )
   #t
(eq? (DM-local-indexed-get DM-reference-type DM-Class-class DM-Class-field-field 1)
     (DM-ClassToField DM-Class-class 1) )
   #t
(eq? (DM-local-indexed-get DM-reference-type DM-Class-class DM-Class-field-field 0)
     (DM-ClassToField DM-Class-class 0) )
   #t
;;; accessing a zero-length indexed field
(DM-local-length DM-nat3-type DM-Object-class DM-Class-field-field)
   0

;;; exercize some errors
(DM-local-get DM-nat3-type DM-Class-class DM-Class-name-field)
   *** ; Type mismatch
(DM-local-get DM-data-type DM-Class-class DM-Class-name-field)
   *** ; Type mismatch
(DM-local-get DM-nat3-type DM-Class-class DM-Type-name-field)
   *** ; no such field
(DM-local-indexed-get DM-reference-type DM-Class-class DM-Class-name-field 123)
   *** ; not an indexed field
(DM-local-get DM-reference-type DM-Class-class DM-Class-field-field)
   *** ; not a regular field
(DM-local-indexed-get DM-reference-type DM-Class-class DM-Class-super-field 123)
   *** ; out of range index
(DM-local-indexed-get DM-reference-type DM-Class-class DM-Class-field-field 123)
   *** ; out of range index
(DM-local-indexed-get DM-reference-type DM-Class-class DM-Class-field-field -1)
   *** ; out of range index

;;; Test access to types
(eq? DM-nought-type (DM-index-to-type 0))
   #t

;;; We just check that DM-size seems to work. It is needed by show-bytes.
(DM-size DM-nil)
   ---
(DM-size DM-Class-class)
   ---

;;; Test printing. Once these functions work, debugging becomes easier.
;;; Transforms DMeroon strings into Scheme strings. Remember that
;;; DMeroon strings end with a trailing null.
;;; REMOVED these tests since DM-string->string and unveil do not
;;; belong to DMeroon kernel.
;(substring (DM-string->string (DM-ClassToName DM-Class-class)) 0 5)
;   "Class"
;(unveil (DM-ClassToName DM-type-class))
;   ---
;(unveil DM-nat3-type)
;   ---
;(unveil DM-Class-class)
;   ---
;;; show-bytes is useful to see the content in bytes of objects.
;;; First define it. 
(define (show-bytes o . stream)
  (let ((stream (if (pair? stream) (car stream) (current-output-port)))
        (o      (if (DM-is-object o) o (if (integer? o) (DM-gimme o) #f))) )
    (DM-print-bytes o (+ DM-prefix-size (DM-size o)) stream) ) )
   ---
(show-bytes DM-Class-class)
   ---
(show-bytes (DM-location DM-Class-class))
   ---
;;; Bigloo also has string ports, but dont test them since they are
;;; not regular Scheme.
;(let ((out (open-output-string)))
;  (show-bytes DM-Class-class out) 
;  (write (flush-output-port out)) )
;   ---

;;; Allocation
;;; Simple classes without indexed fields.
(DM-allocate0 DM-Type-class)
   ---
;;; With one indexed field
(DM-allocate1 DM-String-class 3)
   ---
;;; verify size
(DM-local-length DM-nat3-type (DM-allocate1 DM-String-class 3) 
                 DM-String-character-field )
   3
;;; With two indexed fields
(DM-allocate2 DM-Class-class 2 3)
   ---
;;; verify sizes
(DM-local-length DM-nat3-type (DM-allocate2 DM-Class-class 2 3)
                 DM-Class-super-field )
   2
(DM-local-length DM-nat3-type (DM-allocate2 DM-Class-class 2 3)
                 DM-Class-field-field )
   3
;;; test allocation errors
(DM-allocate1 DM-Type-class 33)
   *** ; too much sizes
(DM-allocate2 DM-String-class 3 44)
   *** ; too much sizes
(DM-allocate0 DM-String-class)
   *** ; missing sizes
(DM-allocate0 DM-Class-class)
   *** ; missing sizes
(DM-allocate1 DM-Class-class 2)
   *** ; missing sizes
;;; Test arity
(DM-allocate0 DM-Type-class 33)
   *** ; too much sizes
(DM-allocate3 DM-String-class 3 44)
   *** ; too much sizes
(DM-allocate1 DM-String-class 3 44)
   *** ; too much sizes
(DM-allocate1 DM-String-class)
   *** ; missing sizes
(DM-allocate2 DM-Class-class)
   *** ; missing sizes
(DM-allocate0 DM-Class-class 2)
   *** ; missing sizes

;;; Test alignments.
;;; Normally malloc or any substitute must return a pointer to a space
;;; suitable for any kind of data ie aligned on the most stringent type.
;;; This does not seem to be always the case.
;;; This test tries to allocate Class which is a stringent class.
(let* ((align (DM-TypeToAlignment DM-most-stringent-type))
       ;; number of misaligned allocations
       (result 0)
       (sizes '( 1 2 3 4 5 6 7 8 9 10
                 11 12 13 14 15 16 17 18 19 20
                 21 22 23 24 25 26 27 28 29 30
                 31 32 33 34 35 36 37 38 39 40
                 45 54 65 77 89 ) )
       (addresses (map (lambda (i)
                         (DM-location
                          (DM-allocate-bare-instance 
                           DM-Class-class i DM-nil) ) )
                       sizes )) )
  (display `(most stringent alignment is ,align))(newline)
  (do ((a addresses (cdr a))
       (s sizes (cdr s)) )
      ((null? a) result)
    (display `(allocated ,(car s) bytes at ,(car a)))
    (if (= 0 (modulo (car a) align))
        (display " OK")
        (begin (display "      Big problem here <<<<<------- ")
               (set! result (+ result 1)) ) )
    (newline) ) )
    0
;;; Check that statically allocated data are correctly aligned.
(let* ((align (DM-TypeToAlignment DM-most-stringent-type))
       ;; number of misaligned allocations
       (result 0)
       (objects (list 
                 DM-Object-class
                 DM-nil
                 DM-Class-class
                 DM-Class-name-field
                 DM-Class-options-field
                 DM-Class-super-field
                 DM-Class-field-field
                 DM-Field-class
                 DM-Field-name-field
                 DM-Field-options-field
                 DM-Field-introducing-field
                 DM-Field-offset-field
                 DM-Type-class
                 DM-Type-name-field
                 DM-Type-index-field
                 DM-Type-alignment-field
                 DM-Type-size-field
                 DM-String-class
                 DM-String-character-field
                 DM-ImmutableVector-class
                 DM-MutableVector-class ))
       (addresses (map DM-location objects)) )
  (display `(most stringent alignment is ,align))(newline)
  (do ((a addresses (cdr a))
       (s objects (cdr s)) )
      ((null? a) result)
    (display `(,(car s) statically allocated at ,(car a)))
    (if (= 0 (modulo (car a) align))
        (display " OK")
        (begin (display "      Big problem here <<<<<------- ")
               (set! result (+ result 1)) ) )
    (newline) ) )
    0
;;; Try to read the most stringent type in a weird position. It is not
;;; necessary to write in the instance to test alignment, reading is
;;; sufficient.
(let* ((align (DM-TypeToAlignment DM-most-stringent-type))
       (o (DM-allocate-bare-instance DM-Object-class 20 DM-nil)) )
  (DM-local-read DM-most-stringent-type o align) )
   ---
;;; This leads always to a bus error soon or later.
;;; Use it if you care to know about aligment rules.
'''
(let* ((align (DM-TypeToAlignment DM-most-stringent-type))
       (o (DM-allocate-bare-instance DM-Object-class 20 DM-nil)) )
  (DM-showme o 32)
  (let loop ((i align))
    (display `(try to read at ,o + ,i))(newline)
    (DM-local-read DM-most-stringent-type o i)
    (if (> i 0) (loop (/ i 2))) ) )
   ---
;;; This is not necessary an error
;(let* ((align (DM-TypeToAlignment DM-most-stringent-type))
;       (o (DM-allocate-bare-instance DM-Object-class 20 DM-nil)) )
;  (DM-local-read DM-most-stringent-type o (/ align 2)) )
;   ***
;;; NOTE on Sony, the most stringent type is float with an alignment
;;; of 8.  Running the test above shows that 4 is sufficient as
;;; alignment constraint. 
                          

;;; Test class membership of freshly allocated instances
(DM-is-a (DM-allocate2 DM-Class-class 2 3) DM-Object-class)
   #t
(DM-is-a (DM-allocate2 DM-Class-class 2 3) DM-Class-class)
   #t
(DM-is-a (DM-allocate2 DM-Class-class 2 3) DM-Type-class)
   #f

;;; Testing another function of the library. Pay attention to the
;;; final NULL added at the end of strings. This is tested here since
;;; there is an hidden allocation.
;;; REMOVED since it does not belong to DMeroon kernel.
;(->String "bar")
;   ---
;(DM-string->string (->String "bar"))
;   ---
;(DM-string->string (->String 'bar))
;   ---
;(let ((s (->String 'bar)))
;  (eq? s (->String s)) )
;   #t
;(equal? "bar" (DM-string->string (->String "bar")))
;   #t

;;; Compare repeated character fields.
(DM-of-of-cmp (DM-ClassToName DM-Class-class) DM-String-character-field
              (DM-ClassToName DM-Class-class) DM-String-character-field )
   #t
(DM-of-of-cmp (DM-ClassToName DM-Class-class) DM-String-character-field
              (DM-ClassToName DM-Object-class) DM-String-character-field )
   #f
(DM-of-of-cmp DM-Class-class DM-Class-name-field
              DM-Class-name-field DM-Class-name-field )
   *** ;; no such field
(DM-of-of-cmp DM-Class-class DM-Class-super-field
              DM-Class-name-field DM-Field-name-field )
   *** ;; not an indexed field
(DM-of-of-cmp DM-Class-class DM-Class-super-field
              DM-String-class DM-String-character-field )
   *** ;; not the same repeated type.

;;; Compare a Scheme string and a DMeroon repetition of characters.
(DM-str-of-cmp "Class" (DM-ClassToName DM-Class-class) DM-String-character-field)
   #t
(DM-str-of-cmp "Classs" (DM-ClassToName DM-Class-class) DM-String-character-field)
   #f
(DM-str-of-cmp "Clas" (DM-ClassToName DM-Class-class) DM-String-character-field)
   #f
;;; compare with an inner repetition of chars.
;;; REMOVED dictionary does not contain an indexed field now.
;(let ((infos (DM-local-get DM-reference-type DM-site
;                     (DM-ClassToField DM-Site-class 4) )))
;  (DM-str-of-cmp "foo" infos DM-Dictionary-name-field
;                             DM-String-character-field ) )
;   #f
;(let ((infos (DM-local-get DM-reference-type DM-site
;                     (DM-ClassToField DM-Site-class 4) )))
;  (DM-str-of-cmp "DMeroon" infos DM-Dictionary-name-field
;                                 DM-String-character-field ) )
;   #t
;;; Test giving a precise field
;;;         FUTURE to be done

;;; Test the retrieval of named fields. This uses the previous function.
;;; Pay attention that fields of predefined classes are defined with names
;;; in lowercase.
;;; REMOVED tests using the generic function ->Field which does not belong
;;; to DMeroon kernel.
(eq? (DM-char-star-to-field DM-Class-class "field")
     DM-Class-field-field )
   #t
(DM-char-star-to-field DM-Class-class "foobar")
   #f
;(eq? (->Field DM-Class-class DM-Class-field-field) DM-Class-field-field)
;   #t
(eq? (DM-number-to-field DM-Class-class 2) (DM-ClassToField DM-Class-class 2))
   #t
;;; Wait for a class to be defined.
;(eq? (->Field DM-Class-class 'field) (->Field DM-Class-class "field"))
;   #t
(eq? (DM-string-to-field DM-Class-class (DM-FieldToName DM-Class-field-field))
     DM-Class-field-field )
   #t
(eq? (DM-string-to-field DM-Class-class (DM-ClassToName DM-Class-class))
     DM-Class-field-field )
   #f


;;; Testing writes in objects.
;;; Fortunately there exists some predefined mutable classes (MutableVector
;;; and Dictionary). Test vectors after since they are indexed.
;;; One: mutate the first field.
(let ((d (DM-allocate0 DM-Dictionary-class)))
  (DM-local-set DM-reference-type 
                d
                DM-Dictionary-rest-field
                d )
  #t )
   ---
(let ((d (DM-allocate0 DM-Dictionary-class)))
  (DM-local-set DM-reference-type 
                d
                DM-Dictionary-rest-field
                d )
  (eq? d (DM-local-get DM-reference-type 
                       d
                       DM-Dictionary-rest-field )) )
   #t
;;; Two mutate the second field
(let ((d (DM-allocate0 DM-Dictionary-class)))
  (DM-local-set DM-reference-type 
                d
                DM-Dictionary-value-field
                d ) )
   ---
(let ((d (DM-allocate0 DM-Dictionary-class)))
  (DM-local-set DM-reference-type 
                d
                DM-Dictionary-value-field
                d )
  (eq? d (DM-local-get DM-reference-type 
                       d
                       DM-Dictionary-value-field )) )
   #t
;;; test the returned value of mutation
(let ((d (DM-allocate0 DM-Dictionary-class)))
  (DM-local-set DM-reference-type 
                d
                DM-Dictionary-value-field
                d )
  (eq? d (DM-local-set DM-reference-type 
                       d
                       DM-Dictionary-value-field
                       DM-nil )) )
   #t
;;; On indexed fields now
(let ((v (DM-allocate1 DM-MutableVector-class 4)))
  (DM-local-indexed-set DM-reference-type v 
                        DM-MutableVector-item-field 1 v ) )
   ---
(let ((v (DM-allocate1 DM-MutableVector-class 4)))
  (DM-local-indexed-set DM-reference-type v 
                        DM-MutableVector-item-field 1 v)
  (eq? v (DM-local-indexed-get DM-reference-type v 
                               DM-MutableVector-item-field 1)) )
   #t
(let ((v (DM-allocate1 DM-MutableVector-class 4)))
  (DM-local-indexed-set DM-reference-type v 
                        DM-MutableVector-item-field 1 v)
  (eq? v (DM-local-indexed-set DM-reference-type v 
                               DM-MutableVector-item-field 1
                               DM-nil )) )
   #t


;;; Testing class creation through the user interface.
;;; Just one field and no options
(DM-create-class "Point" DM-Object-class 1
                 "-field" "x" )
   ---
(let ((c (DM-create-class "Point" DM-Object-class 1
                          "-field" "x" )))
  (DM-allocate0 c) )
   ---
(let ((c (DM-create-class "Point" DM-Object-class 1
                          "-field" "x" )))
  (DM-is-a (DM-allocate0 c) c) )
   #t
;;; To test the immutability of a field, the object must be initialized
;;; otherwise all writing in it are allowed. 
;;; 13 X11 95, this test is no longer portable since paving ensures that 
;;; objects are always initialized.
;(let* ((c (DM-create-class "Point" DM-Object-class 1
;                           "-field" "x" ))
;       (o (DM-allocate0 c))
;       (value DM-Class-class) )
;  (DM-set-initialized o) )
;   #f ; Object not yet initialized 
(let* ((c (DM-create-class "Point" DM-Object-class 1
                           "-field" "x" ))
       (o (DM-allocate0 c))
       (value DM-Class-class) )
  (DM-local-set DM-reference-type o (DM-ClassToField c 0) value)
  (DM-set-initialized o) )
   #t
(let* ((c (DM-create-class "Point" DM-Object-class 1
                           "-field" "x" ))
       (o (DM-allocate0 c))
       (value DM-Class-class) )
  (DM-local-set DM-reference-type o (DM-ClassToField c 0) value)
  (DM-set-initialized o)
  (DM-local-set DM-reference-type o (DM-ClassToField c 0) value) ) 
   *** ; immutable field

;;; test now the definition of a class using symbols as names
(DM-create-class 'Point DM-Object-class 1
  "-field" 'x )
   ---
(let ((c (DM-create-class 'Point DM-Object-class 1
                          "-field" 'x )))
  (DM-is-a (DM-allocate0 c) c) )
   #t
;;; tests ->Field on a field named after a symbol.
(let ((c (DM-create-class 'Point DM-Object-class 1
                          "-field" 'x )))
  (eq? (DM-ClassToField c 0) (->Field c 'x)) )
   #t

;;; One field and some options
;;; The mutable option
(DM-create-class "Point" DM-Object-class 1
                 "-field" "x" "-mutable" )
   ---
(let* ((c (DM-create-class "Point" DM-Object-class 1
                           "-field" "x" "-mutable" ))
       (o (DM-allocate0 c))
       (value DM-Class-class) )
  (DM-local-set DM-reference-type o (DM-ClassToField c 0) value) )
   ---
(let* ((c (DM-create-class "Point" DM-Object-class 1
                           "-field" "x" "-mutable" ))
       (o (DM-allocate0 c))
       (value DM-Class-class) )
  (DM-local-set DM-reference-type o (DM-ClassToField c 0) value)
  (eq? (DM-local-get DM-reference-type o (DM-ClassToField c 0)) value) )
   #t
;;; the immotile option
(DM-create-class "Point" DM-Object-class 1
                 "-field" "x" "-mutable" 
                 "-immotile" )
   ---
(DM-create-class "Point" DM-Object-class 1
                 "-field" "x" 
                 "-immotile" )
   ---
;;; the indexed field option
(DM-create-class "Vector" DM-Object-class 1
                 "-field" "item" "-indexed" DM-nat3-type "-mutable" )
   ---
(let* ((c (DM-create-class "Vector" DM-Object-class 1
                 "-field" "item" "-indexed" DM-nat3-type "-mutable" ))
       (o (DM-allocate1 c 4))
       (value DM-Class-class) )
  (DM-local-indexed-set DM-reference-type o (DM-ClassToField c 0) 2 value)
  (eq? (DM-local-indexed-get DM-reference-type o (DM-ClassToField c 0) 2) value) )
   #t

;;; more than one field
(DM-create-class "Point" DM-Object-class 2
                 "-field" "x" "-type" DM-nat1-type "-mutable"
                 "-field" "y" "-type" DM-nat2-type "-mutable" )
   ---

;;; Create a class with one field
(define Test1-class
  (DM-create-class "Test1" DM-Object-class 1
   "-field" "head" "-type" DM-nat1-type "-mutable" ) )
   ---
(DM-allocate0 Test1-class)
   ---
(DM-is-a (DM-allocate0 Test1-class) Test1-class)
   #t
(DM-is-a (DM-allocate0 Test1-class) DM-Object-class)
   #t
(DM-is-a (DM-allocate0 Test1-class) DM-Type-class)
   #f
(unveil (DM-allocate0 Test1-class))
   ---
(let ((o (DM-allocate0 Test1-class)))
  (DM-local-set DM-nat1-type o (DM-ClassToField Test1-class 0) 33)
  (unveil o)
  (DM-local-get DM-nat1-type o (DM-ClassToField Test1-class 0)) )
   33

;;; Create a class with two fields, the second is indexed.
(define Test2-class
  (DM-create-class "Test2" DM-Object-class 2
   "-field" "head" "-type" DM-nat1-type "-mutable"
   "-field" "content" "-type" DM-data-type "-mutable"
                      "-indexed" DM-nat2-type  ) )
   ---
(DM-allocate1 Test2-class 5)
   ---
(DM-is-a (DM-allocate1 Test2-class 5) Test2-class)
   #t
(DM-is-a (DM-allocate1 Test2-class 5) DM-Object-class)
   #t
(DM-is-a (DM-allocate1 Test2-class 5) DM-Type-class)
   #f
;;; Remove these unveil forms since the content field has type data
;;; and is not initialized at that time.
(let ((o (DM-allocate1 Test2-class 5)))
  (DM-local-set DM-nat1-type o (DM-ClassToField Test2-class 0) 33)
  ;(unveil o)
  (DM-local-get DM-nat1-type o (DM-ClassToField Test2-class 0)) )
   33
(let ((o (DM-allocate1 Test2-class 5))
      (value 'foobar) )
  (DM-local-indexed-set DM-data-type o (DM-ClassToField Test2-class 1) 3 value)
  ;(unveil o)
  (eq? value (DM-local-indexed-get DM-data-type o 
                             (DM-ClassToField Test2-class 1) 3 )) )
   #t

;;; Test2 could have been defined by inheritance from test1 ...
(define Test2a-class
  (DM-create-class "Test2a" Test1-class 1
   "-field" "content" "-type" DM-data-type "-mutable"
                      "-indexed" DM-nat2-type  ) )
   ---
(DM-allocate1 Test2a-class 5)
   ---
(DM-is-a (DM-allocate1 Test2a-class 5) Test2a-class)
   #t
(DM-is-a (DM-allocate1 Test2a-class 5) Test1-class)
   #t
(DM-is-a (DM-allocate1 Test2a-class 5) Test2-class)
   #f
(let ((o (DM-allocate1 Test2a-class 5)))
  (DM-local-set DM-nat1-type o (DM-ClassToField Test2a-class 0) 33)
  ;(unveil o)
  (DM-local-get DM-nat1-type o (DM-ClassToField Test2a-class 0)) )
   33
(let ((o (DM-allocate1 Test2a-class 5))
      (value 'foobar) )
  (DM-local-indexed-set DM-data-type o (DM-ClassToField Test2a-class 1) 3 value)
  ;(unveil o)
  (eq? value (DM-local-indexed-get DM-data-type o 
                             (DM-ClassToField Test2a-class 1) 3 )) )
   #t

;;; Test3 with three fields ie one regular field after an indexed one.
(define Test3-class
  (DM-create-class "Test3" Test2a-class 1
   "-field" "tail" "-type" DM-char1-type "-mutable" ) )
   ---
;(unveil (DM-allocate1 Test3-class 5))
;   ---
(DM-is-a (DM-allocate1 Test3-class 5) Test3-class)
   #t
(DM-is-a (DM-allocate1 Test3-class 5) Test2a-class)
   #t
(DM-is-a (DM-allocate1 Test3-class 5) Test2-class)
   #f
(let ((o (DM-allocate1 Test3-class 5)))
  (DM-local-set DM-nat1-type o (DM-ClassToField Test3-class 0) 33)
  ;(unveil o)
  (DM-local-get DM-nat1-type o (DM-ClassToField Test3-class 0)) )
   33
(let ((o (DM-allocate1 Test3-class 5))
      (value 'foobar) )
  (DM-local-indexed-set DM-data-type o (DM-ClassToField Test3-class 1) 3 value)
  ;(unveil o)
  (eq? value (DM-local-indexed-get DM-data-type o (DM-ClassToField Test3-class 1) 3)) )
   #t
(let ((o (DM-allocate1 Test3-class 5)))
  (DM-local-set DM-char1-type o (DM-ClassToField Test3-class 2) #\Q)
  ;(unveil o)
  (DM-local-get DM-char1-type o (DM-ClassToField Test3-class 2)) )
   #\Q

;;; Some error on DM-create-class
(DM-create-class "test" DM-Object-class 1)
   *** ; should contain one field

;;; Testing a little more padding. A repeated field of type char1 involves
;;; some dynamical padding after it to get realigned. It is sufficient
;;; to unveil allocated objects since they must be read to be unveiled.
(define Testpad1-class
  (DM-create-class "Testpad1" DM-Object-class 3
   "-field" "c" "-type" DM-char1-type "-mutable"
   "-field" "char" "-type" DM-char1-type "-mutable"
                   "-indexed" DM-nat1-type
  "-field" "n1" "-type" DM-nat2-type ) )
   ---
(DM-print-class Testpad1-class (current-output-port))
   ---
;(unveil (DM-allocate1 Testpad1-class 4))
;   ---
;(unveil (DM-allocate1 Testpad1-class 3))
;   ---
;;; Test with a second more stringent field. This proves that the padding-log
;;; is not sufficient to deal with that kind of situation. It allows to
;;; resynchronize addresses for the n1 field, but not for n2!
(define Testpad1a-class
  (DM-create-class "Testpad1a" DM-Object-class 4
   "-field" "c" "-type" DM-char1-type "-mutable"
   "-field" "char" "-type" DM-char1-type "-mutable"
                   "-indexed" DM-nat1-type
  "-field" "n1" "-type" DM-nat2-type
  "-field" "n2" "-type" DM-nat3-type ) )
   ---
(DM-print-class Testpad1a-class (current-output-port))
   ---
;(unveil (DM-allocate1 Testpad1a-class 4))
;   ---
;(unveil (DM-allocate1 Testpad1a-class 3))
;   ---
;;; Test with a second repetition after a repetition.
(define Testpad2-class
  (DM-create-class "Testpad2" DM-Object-class 5
   "-field" "c" "-type" DM-char1-type "-mutable"
   "-field" "char" "-type" DM-char1-type "-mutable"
                   "-indexed" DM-nat1-type
   "-field" "x" "-type" DM-char1-type "-mutable"
                "-indexed" DM-nat1-type
   "-field" "n1" "-type" DM-nat2-type
   "-field" "n2" "-type" DM-nat3-type ) )
   ---
(DM-print-class Testpad2-class (current-output-port))
   ---
;(unveil (DM-allocate2 Testpad2-class 4 3))
;   ---
;(unveil (DM-allocate2 Testpad2-class 3 4))
;   ---
;(unveil (DM-allocate2 Testpad2-class 3 5))
;   ---
;;; It is possible to have some padding after the repetition factor if
;;; it is a small fixnum and the repetition is a stringent one.
(define Testpad3-class
  (DM-create-class "Testpad3" DM-Object-class 2
   "-field" "c" "-type" DM-float1-type "-mutable" "-indexed" DM-nat1-type
   "-field" "f" "-type" DM-float2-type "-mutable" ) )
   ---
(let ((o (DM-allocate1 Testpad3-class 0)))
  (DM-local-length DM-nat1-type o (DM-ClassToField Testpad3-class 0)) )
   0
(let ((o (DM-allocate1 Testpad3-class 1)))
  (DM-local-length DM-nat1-type o (DM-ClassToField Testpad3-class 0)) )
   1
;;; compute-indexed-offset does not exist any longer.
;(let* ((o (DM-allocate1 Testpad3-class 1)))
;  (> (DM-compute-indexed-offset o (DM-ClassToField Testpad3-class 0) 0)
;     (DM-compute-offset o (DM-ClassToField Testpad3-class 0)) ) )
;   #t

                        
;;; Test immutability of fields. 
(DM-local-set DM-reference-type 
        DM-Class-class 
        (DM-ClassToField DM-Class-class 0)
        DM-Type-class )
   *** ; immutable field (DM-ClassToName)
(DM-local-indexed-set DM-reference-type 
                DM-Class-class
                DM-Class-super-field
                1
                DM-Class-class )
   *** ; immutable field
;;; testing immutability of fields for dynamically built classes.
(define Test3a-class
  (DM-create-class "Test3a" Test2a-class 2
    "-field" "tail" "-type" DM-char1-type "-immutable"
    "-field" "xtail" "-type" DM-nat1-type "-immutable"
                     "-indexed" DM-nat1-type ) )
   ---
(DM-local-set DM-char1-type (DM-allocate1 Test3a-class 5) 
        (DM-ClassToField Test3a-class 3)
        #\Q )
   *** ; immutable field
(DM-local-indexed-set DM-nat1-type (DM-allocate1 Test3a-class 5) 
                (DM-ClassToField Test3a-class 4)
                44 )
   *** ; immutable field

;;; For every type, there is an associated class with a single field
;;; of that type. Test them.
(DM-make DM-Nat1-class 0)
   ---
(DM-make DM-Nat1-class 113)
   ---
(DM-make DM-Nat1-class 255)
   ---
(DM-make DM-Nat1-class 256)
   ***
(DM-make DM-Nat1-class -113)
   ***
(DM-make DM-Nat1-class -12213)
   ***
(DM-make DM-Nat2-class 0)
   ---
;;; #x1234 = 4660 !
(DM-make DM-Nat2-class 4660)
   ---
;;; #xffff = 65535
(DM-make DM-Nat2-class 65535)
   ---
(DM-make DM-Nat2-class (+ 1 65535))
   ***
(DM-make DM-Nat2-class -113)
   ***
(DM-make DM-Nat2-class -12213)
   ***
(DM-make DM-Nat3-class 0)
   ---
;;; #x12345678 = 305419896
(DM-make DM-Nat3-class 305419896)
   ---
(DM-make DM-Nat3-class -113)
   ***
(DM-make DM-Nat3-class -12213)
   ***
;;; cannot test limits of nat4 from Bigloo
;;; Moreover if paving is allowed the initial value is part of a pointer
;;; and therefore likely to be out of range of a fixnum. Remove these
;;; tests.
;(DM-make DM-Nat4-class 0)
;   ---
;(DM-make DM-Nat4-class 305419896)
;   ---
;(DM-make DM-Nat4-class -113)
;   ***
;(DM-make DM-Nat4-class -12213)
;   ***
(DM-make DM-Reference-class DM-nil)
   ---
(DM-make DM-Reference-class 'foo)
   ***
(DM-make DM-Data-class DM-nil)
   ---
(DM-make DM-Data-class 'foo)
   ---
;;; cannot test DM-Code-class
(DM-make DM-Int1-class 0)
   ---
(DM-make DM-Int1-class 127)
   ---
(DM-make DM-Int1-class 128)
   ***
(DM-make DM-Int1-class -127)
   ---
(DM-make DM-Int1-class -129)
   ***
(DM-make DM-Int2-class 0)
   ---
(DM-make DM-Int2-class 127)
   ---
(DM-make DM-Int2-class 128)
   ---
;;; #x7fff = 32767
(DM-make DM-Int2-class 32767)
   ---
(DM-make DM-Int2-class (+ 1 32767))
   ***
(DM-make DM-Int2-class -127)
   ---
(DM-make DM-Int2-class (- 32767))
   ---
(DM-make DM-Int2-class (- (+ 32767 1)))
   ***
;;; cannot test limits of int3 and int4 with Bigloo on a 32 bits machine.
(DM-make DM-Int3-class 0)
   ---
(DM-make DM-Int3-class 127)
   ---
(DM-make DM-Int3-class 128)
   ---
;;; #x7fffff = 8388607
(DM-make DM-Int3-class 8388607)
   ---
(DM-make DM-Int3-class (- 8388607))
   ---
;;; Remove these tests for the same reason, nat4 tests were removed.
;(DM-make DM-Int4-class 8388607)
;   ---
;(DM-make DM-Int4-class (- 8388607))
;   ---
(DM-make DM-Char1-class #\q)
   ---
(DM-make DM-NetNat2-class 0)
   ---
;;; #xabcd = 43981
(DM-make DM-NetNat2-class 43981)
   ---
(DM-make DM-NetNat2-class (+ 1 65535))
   ***
(DM-make DM-NetNat2-class -56)
   ***
(DM-make DM-NetNat3-class 0)
   ---
;;; #xabcdef = 11259375
(DM-make DM-NetNat3-class 11259375)
   ---
(DM-make DM-NetNat3-class -56)
   ***
(DM-make DM-Float1-class 3.14e+2)
   ---

;;; This is a very long series of tests, jump to %%% to skip it.
;;; Testing read and write for all basic types.
(define Testf-class
  (DM-create-class "Testf" DM-Object-class 26
   "-field" "nat1" "-type" DM-nat1-type "-mutable"
   "-field" "nat2" "-type" DM-nat2-type "-mutable"
   "-field" "nat3" "-type" DM-nat3-type "-mutable"
   "-field" "reference" "-type" DM-reference-type "-mutable"
   "-field" "pointer" "-type" DM-data-type "-mutable"
   "-field" "code" "-type" DM-code-type "-mutable"
   "-field" "int1" "-type" DM-int1-type "-mutable"
   "-field" "int2" "-type" DM-int2-type "-mutable"
   "-field" "int3" "-type" DM-int3-type "-mutable"
   "-field" "char1" "-type" DM-char1-type "-mutable"
   "-field" "netnat2" "-type" DM-netnat2-type "-mutable"
   "-field" "netnat3" "-type" DM-netnat3-type "-mutable"
   "-field" "float1" "-type" DM-float1-type "-mutable"
   "-field" "xnat1" "-type" DM-nat1-type "-mutable" 
                    "-indexed" DM-nat1-type
   "-field" "xnat2" "-type" DM-nat2-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xnat3" "-type" DM-nat3-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xreference" "-type" DM-reference-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xpointer" "-type" DM-data-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xcode" "-type" DM-code-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xint1" "-type" DM-int1-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xint2" "-type" DM-int2-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xint3" "-type" DM-int3-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xchar1" "-type" DM-char1-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xnetnat2" "-type" DM-netnat2-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xnetnat3" "-type" DM-netnat3-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xfloat1" "-type" DM-float1-type "-mutable"
                    "-indexed" DM-nat1-type ) )
   ---
;;; Note: from now we use the general DM-allocate rather than the 
;;; specific DM-allocate<i> functions.
(DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2)
   ---
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 0))
      (ty DM-nat1-type)
      (value 111) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 0))
      (ty DM-nat1-type)
      (value (* 256 111)) )
  (DM-local-set ty o fd value) )
   *** ; too big nat1
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 0))
      (ty DM-nat1-type)
      (value car) )
  (DM-local-set ty o fd value) )
   *** ; not a number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 0))
      (ty DM-nat1-type)
      (value (- 111)) )
  (DM-local-set ty o fd value) )
   *** ; not positive

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 1))
      (ty DM-nat2-type)
      (value (* 256 111)) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 1))
      (ty DM-nat2-type)
      (value car) )
  (DM-local-set ty o fd value) )
   *** ; not a number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 1))
      (ty DM-nat2-type)
      (value (* 256 256 111)) )
  (DM-local-set ty o fd value) )
   *** ; too big nat2
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 1))
      (ty DM-nat2-type)
      (value (- 111)) )
  (DM-local-set ty o fd value) )
   *** ; not positive

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 2))
      (ty DM-nat3-type)
      (value (* 256 256 111)) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 2))
      (ty DM-nat3-type)
      (value car) )
  (DM-local-set ty o fd value) )
   *** ; not a number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 2))
      (ty DM-nat3-type)
      (value (- 111)) )
  (DM-local-set ty o fd value) )
   *** ; not positive

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 3))
      (ty DM-reference-type)
      (value DM-Object-class) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 3))
      (ty DM-reference-type)
      (value 333) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; not a reference

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 4))
      (ty DM-data-type)
      (value DM-Object-class) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 4))
      (ty DM-data-type)
      (value 335) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 5))
      (ty DM-code-type)
      (value DM-Object-class) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 5))
      (ty DM-code-type)
      (value 333) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 6))
      (ty DM-int1-type)
      (value 33) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 6))
      (ty DM-int1-type)
      (value -33) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 6))
      (ty DM-int1-type)
      (value car) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; not a number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 6))
      (ty DM-int1-type)
      (value 333) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; Too big number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 6))
      (ty DM-int1-type)
      (value -333) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; Too big number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 7))
      (ty DM-int2-type)
      (value (* 256 33)) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 7))
      (ty DM-int2-type)
      (value (* -256 33)) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 7))
      (ty DM-int2-type)
      (value car) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; not a number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 7))
      (ty DM-int2-type)
      (value (* 256 256 33)) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; too big number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 7))
      (ty DM-int2-type)
      (value (* -256 256 33)) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; too big number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 8))
      (ty DM-int3-type)
      (value (* 256 256 33)) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 8))
      (ty DM-int3-type)
      (value (* -256 256 33)) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 8))
      (ty DM-int3-type)
      (value car) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; not a number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 9))
      (ty DM-char1-type)
      (value #\Q) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 9))
      (ty DM-char1-type)
      (value car) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; not a number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 10))
      (ty DM-netnat2-type)
      (value 56423) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 10))
      (ty DM-netnat2-type)
      (value car) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; not a number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 11))
      (ty DM-netnat3-type)
      (value (* 56426 255)) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 11))
      (ty DM-netnat3-type)
      (value car) )
  (DM-local-set ty o fd value)
  (equal? value (DM-local-get ty o fd)) )
   *** ; not a number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 12))
      (ty DM-float1-type)
      (value 3.14) )
  (DM-local-set ty o fd value)
  (< (abs (- value (DM-local-get ty o fd))) 0.0001) )
  #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 12))
      (ty DM-float1-type)
      (value 3) )
  (DM-local-set ty o fd value)
  (= value (DM-local-get ty o fd)) )
  *** ; not a float

;;; Testing now indexed read/writes.

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 13))
      (ty DM-nat1-type)
      (value 111) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 13))
      (ty DM-nat1-type)
      (value (* 256 111)) )
  (DM-local-indexed-set ty o fd 1 value) )
   *** ; too big nat1
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 14))
      (ty DM-nat1-type)
      (value car) )
  (DM-local-indexed-set ty o fd 1 value) )
   *** ; not a number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 14))
      (ty DM-nat1-type)
      (value (- 111)) )
  (DM-local-indexed-set ty o fd 1 value) )
   *** ; not positive

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 14))
      (ty DM-nat2-type)
      (value (* 256 111)) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 14))
      (ty DM-nat2-type)
      (value car) )
  (DM-local-indexed-set ty o fd 1 value) )
   *** ; not a number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 14))
      (ty DM-nat2-type)
      (value (* 256 256 111)) )
  (DM-local-indexed-set ty o fd 1 value) )
   *** ; too big nat2
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 14))
      (ty DM-nat2-type)
      (value (- 111)) )
  (DM-local-indexed-set ty o fd 1 value) )
   *** ; not positive

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 15))
      (ty DM-nat3-type)
      (value (* 256 256 111)) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 15))
      (ty DM-nat3-type)
      (value car) )
  (DM-local-indexed-set ty o fd 1 value) )
   *** ; not a number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 15))
      (ty DM-nat3-type)
      (value (- 111)) )
  (DM-local-indexed-set ty o fd 1 value) )
   *** ; not positive

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 16))
      (ty DM-reference-type)
      (value DM-Object-class) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 16))
      (ty DM-reference-type)
      (value 333) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; not a reference

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 17))
      (ty DM-data-type)
      (value DM-Object-class) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 17))
      (ty DM-data-type)
      (value 335) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 18))
      (ty DM-code-type)
      (value DM-Object-class) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 18))
      (ty DM-code-type)
      (value 333) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 19))
      (ty DM-int1-type)
      (value 33) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 19))
      (ty DM-int1-type)
      (value -33) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 19))
      (ty DM-int1-type)
      (value car) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; not a number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 19))
      (ty DM-int1-type)
      (value 333) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; Too big number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 19))
      (ty DM-int1-type)
      (value -333) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; Too big number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 20))
      (ty DM-int2-type)
      (value (* 256 33)) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 20))
      (ty DM-int2-type)
      (value (* -256 33)) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 20))
      (ty DM-int2-type)
      (value car) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; not a number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 20))
      (ty DM-int2-type)
      (value (* 256 256 33)) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; too big number
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 20))
      (ty DM-int2-type)
      (value (* -256 256 33)) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; too big number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 21))
      (ty DM-int3-type)
      (value (* 256 256 33)) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 21))
      (ty DM-int3-type)
      (value (* -256 256 33)) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 21))
      (ty DM-int3-type)
      (value car) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; not a number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 22))
      (ty DM-char1-type)
      (value #\Q) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 22))
      (ty DM-char1-type)
      (value car) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; not a number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 23))
      (ty DM-netnat2-type)
      (value 56423) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 23))
      (ty DM-netnat2-type)
      (value car) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; not a number

(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 24))
      (ty DM-netnat3-type)
      (value (* 255 56423)) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 24))
      (ty DM-netnat3-type)
      (value car) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
   *** ; not a number

;;; Bigloo stores floats as doubles so take care when comparing floats.
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 25))
      (ty DM-float1-type)
      (value 3.14) )
  (DM-local-indexed-set ty o fd 1 value)
  (< (abs (- value (DM-local-indexed-get ty o fd 1))) 0.0001) )
  #t
(let ((o (DM-allocate Testf-class 2 3 4 5 6 7 8 7 6 5 4 3 2))
      (fd (DM-ClassToField Testf-class 25))
      (ty DM-float1-type)
      (value 3) )
  (DM-local-indexed-set ty o fd 1 value)
  (equal? value (DM-local-indexed-get ty o fd 1)) )
  *** ; not a float
;;; end of the long series of tests %%%

;;; Test local and code types.
(define Local-class
  (DM-create-class "Local" DM-Object-class 3
     "-field" "reference" "-type" DM-reference-type "-mutable"
     "-field" "data"      "-type" DM-data-type      "-mutable"
     "-field" "code"      "-type" DM-code-type      "-mutable" ) )
   ---
(let ((o (DM-allocate Local-class))
      (fd (DM-ClassToField Local-class 1))
      (ty DM-data-type)
      (value 3) )           ; test with a fixnum
  (DM-local-set ty o fd value)
  (eq? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Local-class))
      (fd (DM-ClassToField Local-class 1))
      (ty DM-data-type)
      (value car) )           ; test with a function
  (DM-local-set ty o fd value)
  (eq? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Local-class))
      (fd (DM-ClassToField Local-class 1))
      (ty DM-data-type)
      (value 3.14) )           ; test with a float
  (DM-local-set ty o fd value)
  (eq? value (DM-local-get ty o fd)) )
   #t
(let ((o (DM-allocate Local-class))
      (fd (DM-ClassToField Local-class 1))
      (ty DM-data-type)
      (value (lambda (x) y)) )           ; test with a closure
  (DM-local-set ty o fd value)
  (eq? value (DM-local-get ty o fd)) )
   #t
;;; NOTE: Actually I don't know how to test `code' fields portably in Scheme
;;; since I do not know how to get and store a C function address.


;;; Test classes with three indexed fields.
;;; Seems that DM_compute_offset neglects the displacement of the second
;;; indexed field ?!
(define test3indexed-class
  (DM-create-class "Test3indexed" DM-Object-class 7 
    "-field" 'a "-type" DM-nat2-type
    "-field" 'b "-indexed" DM-nat1-type "-type" DM-char1-type
    "-field" 'c "-type" DM-nat4-type
    "-field" 'd "-indexed" DM-nat2-type "-type" DM-nat2-type
    "-field" 'e "-type" DM-nat3-type "-mutable"
    "-field" 'f "-indexed" DM-nat1-type "-type" DM-nat2-type
    "-field" 'g "-type" DM-nat1-type ) )
   ---
(DM-print-class test3indexed-class (current-output-port))
   ---
(define a3 
  (let ((o (DM-allocate3 test3indexed-class 3 7 5)))
    (show-bytes o)
    o ) )
   ---
(begin 
  (DM-local-set DM-nat3-type a3 (DM-ClassToField test3indexed-class 4) 55)
  (show-bytes a3)
  (DM-local-length DM-nat2-type a3 (DM-ClassToField test3indexed-class 3)) )
   7
 
;;; Verify the layout of every predefined class.
(define (create-simple-offset t offset)
  (let* ((itemtypindex (DM-OffsetToItemTypeIndex offset))
         (itemtyp      (DM-index-to-type itemtypindex))
         (reptypindex  (DM-OffsetToRepetitionTypeIndex offset))
         (reptyp       (DM-index-to-type reptypindex)) )
    (if (= reptypindex DM-nought-index)
        (DM-PackOffset (DM-OffsetPad t itemtyp)
                        reptypindex
                        itemtypindex )
        (DM-PackOffset (DM-OffsetPad t reptyp)
                        reptypindex
                        itemtypindex ) ) ) )
   ---
(define (verify-layout class)
  (display `(checking ,(DM-string->string (DM-ClassToName class))))(newline)
  (let ((fields-number (DM-ClassToFieldLength class)))
    (let loop ((i                 0)
               (t                 0)
               (previous-indexed? #t) )
      (if (< i fields-number)
          (let* ((field          (DM-ClassToField class i))
                 (offsets-number (DM-FieldToOffsetLength field))
                 (last-offset    (DM-FieldToOffset field (- offsets-number 1)))
                 (reptypindex    (DM-OffsetToRepetitionTypeIndex last-offset))
                 (itemtypindex   (DM-OffsetToItemTypeIndex last-offset))
                 (indexed?       (not (= reptypindex DM-nought-index)))
                 (prevlim        (- offsets-number 1))
                 (new-offset     (create-simple-offset 
                                  (if previous-indexed?
                                      (if (> offsets-number 1) 0 t)
                                      t )
                                  last-offset )) )
            ;; check the previous offsets 
            (if (> offsets-number 1)
                (let ((previous-field (DM-ClassToField class (- i 1))))
                  (do ((j 0 (+ j 1)))
                      ((>= j prevlim))
                    (if (not (= (DM-FieldToOffset previous-field j)
                                (DM-FieldToOffset field j) ))
                        (DM-error "error on offset" class i j) ) )  ) )
            ;; compare the current one
            (if (not (= new-offset (DM-FieldToOffset field (- offsets-number 1))))
                (DM-error "error on offset" class i) )
            ;; recurse on next field.
            (loop (+ i 1) 
                  (if indexed?
                      0
                      (let ((t    (DM-OffsetToTranslation new-offset))
                            (type (DM-index-to-type itemtypindex)) )
                        (DM-OffsetExtend (DM-OffsetPad t type) type) ) )
                  indexed? ) ) ) ) ) )  
   ---
;;; start at 1.
(do ((i 1 (+ i 1)))
    ((>= i (DM-local-length DM-nat3-type 
                            DM-predefined-classes
                            DM-ImmutableVector-item-field )))
  (verify-layout (DM-local-indexed-get DM-reference-type 
                                       DM-predefined-classes
                                       DM-ImmutableVector-item-field 
                                       i )) )
   ---


" *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
                      All dmeroon.tst tests successfully run!
 *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***"
   ---

;;; end of dmeroon.tst
