;;; $Id: fill.tst,v 1.8 1995/09/11 18:21:19 queinnec Exp $

;;; Tests enhanced allocations with DM-instantiate and DM-update.
;;; Some tests on DM-make and simple classes already appear in the 
;;; dmeroon.tst file.

;;; Test allocations with DM-make.

(define DM-Null-class
  (DM-create-class "Null" DM-Object-class 0) )
   ---
(DM-allocate DM-Null-class)
   ---
(DM-make DM-nil)
   *** ; not a class
(DM-make DM-nil 0 2 DM-nil)
   *** ; not a class
(DM-make DM-Null-class)
   ---
(DM-make DM-Null-class 0)
   *** ; too much args
(DM-make DM-String-class)
   *** ; missing size
(DM-make DM-String-class 0)
   ---
(DM-make DM-String-class 3 #\b #\a #\r)
   ---
(let ((s (DM-make DM-String-class 3 #\b #\a #\r)))
  (DM-local-indexed-get DM-char1-type s (DM-ClassToField DM-String-class 0) 1) )
   #\a
(DM-make DM-String-class 3 #\b #\a #\r DM-nil)
   *** ; too much args
(DM-make DM-String-class #\b #\a #\r DM-nil)
   *** ; not a size

;;; Create a class with one field
(define Test1-class
  (DM-create-class "Test1" DM-Object-class 1
   "-field" "head" "-type" DM-nat1-type "-mutable" ) )
   ---
(DM-make Test1-class 44)
   ---
;;; Create a class with two fields, the second is indexed.
(define Test2-class
  (DM-create-class "Test2" DM-Object-class 2
   "-field" "head" "-type" DM-nat1-type "-mutable"
   "-field" "content" "-type" DM-data-type "-mutable"
                      "-indexed" DM-nat2-type  ) )
   ---
(DM-make Test2-class 44 3 'a 'b 'c)
   ---
(DM-make Test1-class 44 'a 'b)
   *** ; too much args
(DM-make Test1-class)
   *** ; missing args
(DM-make DM-Class-class DM-nil 0 2 DM-nil DM-nil 3 DM-nil DM-nil DM-nil)
   ---
;;; more difficult tests with weirder classes
;;; Testing a little more padding. A repeated field of type char1 involves
;;; some dynamical padding after it to get realigned. It is sufficient
;;; to unveil allocated objects since they must be read to be unveiled.
(define Testpad1-class
  (DM-create-class "Testpad1" DM-Object-class 3
   "-field" "c" "-type" DM-char1-type "-mutable"
   "-field" "char" "-type" DM-char1-type "-mutable"
                   "-indexed" DM-nat1-type
  "-field" "n1" "-type" DM-nat2-type ) )
   ---
(DM-make Testpad1-class #\Q 3 #\b #\a #\r (* 256 111))
   ---
;;; Test with a second more stringent field. This proves that the padding-log
;;; is not sufficient to deal with that kind of situation. It allows to
;;; resynchronize addresses for the n1 field, but not for n2!
(define Testpad1a-class
  (DM-create-class "Testpad1a" DM-Object-class 4
   "-field" "c" "-type" DM-char1-type "-mutable"
   "-field" "char" "-type" DM-char1-type "-mutable"
                   "-indexed" DM-nat1-type
  "-field" "n1" "-type" DM-nat2-type
  "-field" "n2" "-type" DM-nat3-type ) )
   ---
(DM-make Testpad1a-class #\Q 3 #\b #\a #\r (* 256 111) (* 256 256 111))
   ---
;;; Test with a second repetition after a repetition.
(define Testpad2-class
  (DM-create-class "Testpad2" DM-Object-class 5
   "-field" "c" "-type" DM-char1-type "-mutable"
   "-field" "char" "-type" DM-char1-type "-mutable"
                   "-indexed" DM-nat1-type
   "-field" "x" "-type" DM-char1-type "-mutable"
                "-indexed" DM-nat1-type
   "-field" "n1" "-type" DM-nat2-type
   "-field" "n2" "-type" DM-nat3-type ) )
   ---
(DM-make Testpad2-class #\Q 3 #\b #\a #\r 4 #\f #\o #\o #\Q 
         (* 256 111) (* 256 256 111) )
   ---
;;; This is a very long series of tests, jump to %%% to skip it.
;;; Testing read and write for all basic types.
(define Testf-class
  (DM-create-class "Testf" DM-Object-class 26
   "-field" "nat1" "-type" DM-nat1-type "-mutable"
   "-field" "nat2" "-type" DM-nat2-type "-mutable"
   "-field" "nat3" "-type" DM-nat3-type "-mutable"
   "-field" "reference" "-type" DM-reference-type "-mutable"
   "-field" "pointer" "-type" DM-data-type "-mutable"
   "-field" "code" "-type" DM-code-type "-mutable"
   "-field" "int1" "-type" DM-int1-type "-mutable"
   "-field" "int2" "-type" DM-int2-type "-mutable"
   "-field" "int3" "-type" DM-int3-type "-mutable"
   "-field" "char1" "-type" DM-char1-type "-mutable"
   "-field" "netnat2" "-type" DM-netnat2-type "-mutable"
   "-field" "netnat3" "-type" DM-netnat3-type "-mutable"
   "-field" "float1" "-type" DM-float1-type "-mutable"
   "-field" "xnat1" "-type" DM-nat1-type "-mutable" 
                    "-indexed" DM-nat1-type
   "-field" "xnat2" "-type" DM-nat2-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xnat3" "-type" DM-nat3-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xreference" "-type" DM-reference-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xpointer" "-type" DM-data-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xcode" "-type" DM-code-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xint1" "-type" DM-int1-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xint2" "-type" DM-int2-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xint3" "-type" DM-int3-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xchar1" "-type" DM-char1-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xnetnat2" "-type" DM-netnat2-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xnetnat3" "-type" DM-netnat3-type "-mutable"
                    "-indexed" DM-nat1-type
   "-field" "xfloat1" "-type" DM-float1-type "-mutable"
                    "-indexed" DM-nat1-type ) )
   ---
(unveil (DM-make Testf-class
         11 (* 256 111) (* 256 256 111)
         DM-nil car cdr
         -11 (* 256 -111) (* 256 256 -111)
         #\Q 56423 (* 255 56423) 3.14 
   3 1 2 3
   3 1111 2222 3333
   3 111111 222222 333333
   3 DM-nil DM-nil DM-nil 
   3 car car car
   3 cdr cdr cdr
   3 -1 -2 -3
   3 -111 -222 -333
   3 -111111 -222222 -333333
   3 #\f #\o #\o
   3 56423 56424 56425
   3 (* 255 56423) (* 255 56424) (* 255 56425)
   3 3.3 4.4 -5.5 ))
   ---
;;; Another padding test.
(define Testpad4-class
  (DM-create-class "Testpad4" DM-Object-class 5
   "-field" "a3" "-type" DM-netnat3-type "-mutable"
   "-field" "a2" "-type" DM-netnat2-type "-mutable" 
   "-field" "b3" "-type" DM-netnat3-type "-mutable"
   "-field" "b2" "-type" DM-netnat2-type "-mutable" 
   "-field" "l"  "-type" DM-data-type "-mutable" ) )
   ---
(let ((o (DM-make Testpad4-class 1 2 3 4 #f)))
  (DM-local-get DM-netnat2-type o (DM-ClassToField Testpad4-class 3)) )
   4
(let ((o (DM-make Testpad4-class 1 2 3 4 #t)))
  (DM-local-get DM-data-type o (DM-ClassToField Testpad4-class 4)) )
   #t

;;; Testing DM-instantiate

(DM-instantiate DM-Nat3-class
  "-value" 44 )
   ---
;; test repeated field
(DM-instantiate DM-String-class
  "-character" 3 #\F #\o #\O )
   ---
;; test zero-repeated field
(DM-instantiate DM-String-class
  "-character" 0)
   ---
;; test lot of fields in order
(DM-instantiate DM-Type-class
  "-name" (->String "foo")
  "-index" 33
  "-alignment" 22
  "-size"  44
  "-repsize" 55
  "-reader" car
  "-writer" cdr
  "-printer" read
)
   ---
;; test lot of fields in disorder
(DM-instantiate DM-Type-class
  "-index" 33
  "-size"  44
  "-alignment" 22
  "-repsize" 55
  "-name" (->String "foo")
  "-reader" car
  "-printer" read
  "-writer" cdr
)
   ---
;; lot of fields with repeated fields in order
(DM-instantiate DM-Class-class
   "-name" (->String "foo")
   "-options" 0
   "-super" 2 DM-Object-class DM-Class-class
   "-field" 0 )
   ---
;; lot of fields with repeated fields in disorder
(DM-instantiate DM-Class-class
   "-field" 0
   "-super" 2 DM-Object-class DM-Class-class
   "-name" (->String "foo")
   "-options" 0 )
   ---

;;; exercize some errors.
(DM-instantiate 33)
   *** ; not a class
(DM-instantiate DM-Nat2-class)
   *** ; missing field
(DM-instantiate DM-Nat3-class
  "foobar" 34 )
   *** ; no such field
(DM-instantiate DM-Class-class
   "-field" 0
   "-name" (->String "foo")
   "-options" 0 )
   ***
(DM-instantiate DM-Nat3-class
  "-foobar" 34 )
   *** ; No such field
(DM-instantiate DM-Nat3-class
  "-value" 33
  "-value" 44 )
   *** ; overdefined field

;;; check DM-update now

;;; update without parameter
(DM-update DM-Nat2-class (DM-make DM-Nat2-class 33))
   ---
(DM-local-get DM-nat2-type
        (DM-update DM-Nat2-class (DM-make DM-Nat2-class 33))
        DM-Nat2-value-field )
   33
;; with more than one field
(DM-update DM-Type-class
 (DM-instantiate DM-Type-class
  "-name" (->String "foo")
  "-index" 33
  "-alignment" 22
  "-size"  44
  "-repsize" 55
  "-reader" car
  "-writer" cdr
  "-printer" read
  ))
   ---
;; update a single field
(DM-string->string
 (DM-TypeToName (DM-update DM-Type-class
                       (DM-instantiate DM-Type-class
                                       "-name" (->String "foo")
                                       "-index" 33
                                       "-alignment" 22
                                       "-size"  44
                                       "-repsize" 55
                                       "-reader" car
                                       "-writer" cdr
                                       "-printer" read
                                       ) 
                       "-name" (->String "foobar") )) )
   "foobar"
;; update a single repeated field
(DM-local-length 
 DM-nat3-type
 (DM-update DM-Class-class
            (DM-instantiate DM-Class-class
                            "-field" 0
                            "-super" 2 DM-Object-class DM-Class-class
                            "-name" (->String "foo")
                            "-options" 0 )
            "-super" 3 DM-nil DM-nil DM-nil )
 DM-Class-super-field )
   3
;; update two fields, one being repeated
(DM-local-get
 DM-nat3-type
 (DM-update DM-Class-class
            (DM-instantiate DM-Class-class
                            "-field" 0
                            "-super" 2 DM-Object-class DM-Class-class
                            "-name" (->String "foo")
                            "-options" 0 )
            "-super" 3 DM-nil DM-nil DM-nil
            "-options" 44 )
 DM-Class-options-field )
   44
;; update two fields, one being zero-repeated
(DM-local-get
 DM-nat3-type
 (DM-update DM-Class-class
            (DM-instantiate DM-Class-class
                            "-field" 0
                            "-super" 2 DM-Object-class DM-Class-class
                            "-name" (->String "foo")
                            "-options" 0 )
            "-super" 0
            "-options" 44 )
 DM-Class-options-field )
   44
;;; Add a field ie refine the class
(DM-update DM-ImmutablePair-class
           (DM-make DM-ImmutableLink-class DM-nil)
           "-value" (->String "foo") )
   ---
(DM-is-a (DM-update DM-ImmutablePair-class
                    (DM-make DM-ImmutableLink-class DM-nil)
                    "-value" (->String "foo") )
          DM-ImmutablePair-class )
   #t
(DM-string->string
 (DM-local-get DM-reference-type
         (DM-update DM-ImmutablePair-class
                    (DM-make DM-ImmutableLink-class DM-nil)
                    "-value" (->String "foo") )
         DM-ImmutablePair-value-field ) )
   "foo"   
        
;;; exercice errors
(DM-update 33 DM-nil)
   *** ; not a class
(DM-update DM-Nat3-class (DM-make DM-Int1-class 3))
   *** ; Not a refined class
(DM-update DM-Nat3-class (DM-make DM-Nat3-class 3)
           "-value" 33
           "-value" 44 )
   *** ; overdefined
(DM-update DM-ImmutablePair-class
           (DM-make DM-ImmutableLink-class DM-nil)
           )
   *** ; missing field
(DM-update DM-ImmutablePair-class
           (DM-make DM-ImmutableLink-class DM-nil)
           "-rest" (->String "foo") )
   *** ; overdefined or missing fields


" *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
                      All fill.tst tests successfully run!
 *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***"
   ---

;;; end of fill.tst
