;;; $Id: migr.tst,v 1.19 1995/09/11 18:21:21 queinnec Exp $
;;; This file tests the migration/encoding/decoding process.
;;; It is mainly centered on the functions of migr.scm and dmcomm.c

;;; Some functions to investigate the Entries table.
;;; Remember that the entries table is stored in a volatile field so it
;;; must be reread each time.
(define (entries)
  (DM-local-get DM-reference-type DM-site
          (->Field DM-Site-class "entries") ) )
   ---
;;; extract the physical size and the number of current entries.
(define (entries-size)
  (cons (DM-local-get DM-nat3-type
                (entries)
                (->Field DM-Hashtable-class "size") )
        (DM-local-length DM-nat3-type 
                   (entries)
                   (->Field DM-Hashtable-class "item") ) ) )
   ---
;;; cannot test directly DM-extract-entry since it requires a nat4,
;;; a type that most Scheme implementations ignore.
(define a-value (DM-make DM-ImmutableLink-class DM-nil))
   ---
(DM-create-entry a-value DM-general-clock)
   ---
(unveil (entries))
   ---
(entries-size)
   ---
;;; check that the backpointer from the entry to the object is filled
;;; as well as the object field in the Entry.
(eq? a-value 
     (DM-local-get DM-reference-type 
             (Object.entry a-value)
             (->Field DM-Entry-class "object") ) )
   #t
;;; check idempotence of DM-create-entry
(let ((entry (Object.entry a-value)))
  (eq? entry (DM-create-entry a-value DM-general-clock)) )
   #t
(eq? (DM-create-entry a-value DM-general-clock)
     (DM-create-entry a-value DM-general-clock) ) )
   #t
;;; Make the Entries hashtable double
(let ((actual-size (entries-size)))
  (let loop ()
    (set! a-value (DM-make DM-ImmutableLink-class a-value))
    (DM-create-entry a-value DM-general-clock)
    (let ((new-size (entries-size)))
      (display new-size)(newline)
      (if (> (cdr new-size) (cdr actual-size))
          'doubled
          (if (> (car new-size) (car actual-size))
              (begin (set! actual-size new-size)
                     (loop) )
              'problem ) ) ) ) )
   doubled
;;; Test now the removal of a single entry.
(DM-remove-entry (entries) (Object.entry a-value))
   ---
;;; verify the Entry is no longer present
(eq? DM-nil (Object.entry a-value))
   #t
;;; reinsert it in preparation for the next test.
(DM-create-entry a-value DM-general-clock)
   ---
;;; remove a  number of entries.
(let ((actual-size (entries-size)))  
  (let loop ()
    (if (eq? a-value DM-nil)
        'finished
        (let ((entry (Object.entry a-value)))
          (if (not (eq? entry DM-nil))
              (DM-remove-entry (entries) entry) )
          (set! a-value (DM-local-get DM-reference-type a-value 
                                DM-ImmutableLink-rest-field ))
          (let ((new-size (entries-size)))
            (display new-size)(newline)
            (if (< (car new-size) (car actual-size))
                (begin (set! actual-size new-size)
                       (loop) )
                'problem ) ) ) ) ) )
   finished

;;; test now the reference counters.
;;; test that the number of the Entry is reset to zero.
(let ((entry (DM-create-entry DM-nil DM-general-clock)))
  (DM-increment-counter entry)
  (DM-decrement-counter entry)
  (= 0 (DM-local-get DM-nat4-type entry DM-Entry-number-field)) )
   #t
;;; first add some values, increment their refcount then decrement them.
(let ((n 10)
      (actual-size (entries-size)) )
  (let loop ((i n))
    (if (>= i 0)
        (begin
          (set! a-value (DM-make DM-ImmutableLink-class a-value))
          (DM-increment-counter (DM-create-entry a-value DM-general-clock))
          (loop (- i 1)) ) ) )
  (let loop ((i n))
    (if (>= i 0)
        (begin
          (DM-decrement-counter (Object.entry a-value))
          (set! a-value (DM-local-get DM-reference-type a-value 
                                DM-ImmutableLink-rest-field ))
          (loop (- i 1)) ) ) )
  (equal? actual-size (entries-size)) )
   #t

;;; Check that mutation on a non-externalized objects do not increment
;;; the general clock.
(define (current-time)
  (DM-local-get DM-nat4-type 
          DM-general-clock 
          (->Field DM-Clock-class "time") ) )
   ---
(let ((time 0)
      (v (DM-make DM-MutableVector-class 3 DM-nil DM-nil DM-nil)) )
  (set! time (current-time))
  (DM-local-indexed-set DM-reference-type
                  v
                  DM-MutableVector-item-field
                  1
                  v )
  (unveil v)
  (- (current-time) time) )
   0
(unveil DM-general-clock)
   ---
;;; externalize and mutate an object
(let ((time 0)
      (v (DM-make DM-MutableVector-class 3 DM-nil DM-nil DM-nil)) )
  (DM-create-entry v DM-general-clock)
  (set! time (current-time))
  (DM-local-indexed-set DM-reference-type
                  v
                  DM-MutableVector-item-field
                  2
                  v )
  (unveil v)
  (- (current-time) time) )
   1
(unveil DM-general-clock)
   ---
;;; Test if an idempotent mutation prevents the incrementation of the clock
(let ((time 0)
      (v (DM-make DM-MutableVector-class 3 DM-nil DM-nil DM-nil)) )
  (DM-create-entry v DM-general-clock)
  (DM-local-indexed-set DM-reference-type
                  v
                  DM-MutableVector-item-field
                  2
                  v )
  (set! time (current-time))
  (DM-local-indexed-set DM-reference-type
                  v
                  DM-MutableVector-item-field
                  2
                  v )
  (unveil v)
  (- (current-time) time) )
   0
(unveil DM-general-clock)
   ---

;;; Test reference shortcuts.
;;; First in writing
(let* ((o (DM-make DM-MutableVector-class 2 DM-nil DM-nil))
       (exit (DM-make DM-Exit-class
                      0 0 o DM-nil DM-MutableVector-class
                      0 0 DM-nil 0 0 0 0 )) )
  (show-bytes exit)
  ;; write a reference to an Exit
  (show-bytes o)
  (DM-local-indexed-set DM-reference-type 
                  o
                  DM-MutableVector-item-field
                  1
                  exit )
  (show-bytes o)
  ;; verify we have o instead of exit 
  (eq? o (DM-local-indexed-get DM-reference-type 
                         o
                         DM-MutableVector-item-field
                         1 )) )
   #t
;;; second in reading
(let* ((exit (DM-make DM-Exit-class
                      0 0 DM-nil ; <<-- the object
                      DM-nil DM-MutableVector-class
                      0 0 DM-nil 0 0 0 0 ))
       (o (DM-make DM-MutableVector-class 2 DM-nil exit)) )
  (show-bytes exit)
  (show-bytes o)
  ;; Patch the Exit to be associated now to a value
  ;; cannot use DM-set which involves the coherency protocol,
  ;; patch with deeper unsafe primitives.
  (DM-local-write DM-reference-type 
                  exit 
                  (DM-compute-offset exit (DM-ClassToField DM-Exit-class 2))
                  o )
  (show-bytes exit)
  ;; verify we have o instead of exit 
  (let ((no (DM-local-indexed-get DM-reference-type 
                            o
                            DM-MutableVector-item-field
                            1 )))
    (show-bytes o)
    (eq? o no) ) )
   #t



;;; Test conversion between nat4 and Entries.
(eq? DM-nil (DM-string-to-entry "0123456"))
   #t
(let ((entry (DM-externalize (DM-make DM-Nat1-class 12))))
  (DM-string->string (DM-entry-to-string entry)) )
   ---
(unveil (DM-entry-to-string (DM-externalize (DM-make DM-Nat1-class 12))))
   ---
(let ((entry (DM-externalize (DM-make DM-Nat1-class 12))))
  (eq? entry (DM-string-to-entry (DM-entry-to-string entry))) )
   #t
;;; externalize another object (which is already externalized)
(let ((entry (DM-externalize DM-general-clock)))
  (eq? entry (DM-string-to-entry (DM-entry-to-string entry))) )
   #t
;;; externalize another object not already externalized
(let ((entry (DM-externalize (->String "foobar"))))
  (eq? entry (DM-string-to-entry (DM-entry-to-string entry))) )
   #t


;;; Check DM-externalize
(define b-value (DM-make (DM-create-class "Pair" DM-Object-class 2
                                          "-field" "x" "-field" "y" )
                         DM-nil DM-nil ))
   ---
(DM-externalize b-value)
   ---
;;; check that the class had been externalized
(eq? DM-nil (Object.entry (DM-ObjectToClass b-value)))
   #f
;;; Test with an explicit clock
(define c-value (DM-make (DM-create-class "Pair" DM-Object-class 2
                                          "-field" "x" "-mutable"
                                          "-field" "y" "-mutable" )
                         DM-nil DM-nil ))
   ---
(DM-externalize c-value (DM-make DM-Clock-class 0 0))
   ---
;;; check that the class had been externalized
(eq? DM-nil (Object.entry (DM-ObjectToClass c-value)))
   #f



;;; Create a class with all (non pointer) types.
(define Testf-class
  (DM-create-class "Testf" DM-Object-class 25
   "-field" "nat1" "-type" DM-nat1-type "-mutable"
   "-field" "nat2" "-type" DM-nat2-type "-mutable"
   "-field" "nat3" "-type" DM-nat3-type "-mutable"
   "-field" "nat4" "-type" DM-nat3-type "-mutable"
   "-field" "int1" "-type" DM-int1-type "-mutable"
   "-field" "int2" "-type" DM-int2-type "-mutable"
   "-field" "int3" "-type" DM-int3-type "-mutable"
   "-field" "int4" "-type" DM-int3-type "-mutable"
   "-field" "int1" "-type" DM-int1-type "-mutable"
   "-field" "int2" "-type" DM-int2-type "-mutable"
   "-field" "int3" "-type" DM-int3-type "-mutable"
   "-field" "int4" "-type" DM-int3-type "-mutable"
   "-field" "char1" "-type" DM-char1-type "-mutable"
   "-field" "netnat2" "-type" DM-netnat2-type "-mutable"
   "-field" "netnat3" "-type" DM-netnat3-type "-mutable"
   "-field" "float1" "-type" DM-float1-type "-mutable"
   "-field" "float2" "-type" DM-float2-type "-mutable"
   "-field" "float1" "-type" DM-float1-type "-mutable"
   "-field" "float2" "-type" DM-float2-type "-mutable"
   "-field" "float1" "-type" DM-float1-type "-mutable"
   "-field" "float2" "-type" DM-float2-type "-mutable"
   "-field" "float1" "-type" DM-float1-type "-mutable"
   "-field" "float2" "-type" DM-float2-type "-mutable"
   "-field" "float1" "-type" DM-float1-type "-mutable"
   "-field" "float2" "-type" DM-float2-type "-mutable" ) )
   ---
;;; Test encode/decode on this class. This is done on a single site
;;; frobnicating with buffers within a connection.
(let ()
  (define c 1)
  (define b 2)
  (define o 3)
  (set! b (DM-allocate DM-Buffer-class 64))
  (set! c (DM-make DM-Connection-class
                   0 5 0   ; 
                   0 0 0 0 ; sockaddr
                   0 0 b b 0 ))
  (set! o (DM-make Testf-class
                   11 
                   #x1234
                   #x12345678
                   #x12345678  ; nat4
                   11 
                   #x1234
                   #x12345678
                   #x12345678  ; int4
                   (- 11)
                   (- #x1234)
                   (- #x12345678)
                   (- #x12345678)  ; int4
                   #\Q
                   #x1234          ; netnat2
                   #x0abcde        ; netnat3
                   0.0          ; float1
                   0.0          ; float2
                   3.14         ; float1
                   2.78e+5      ; float2
                   3.14e-4
                   2.78e-7
                   -3.14e+4
                   -2.78e+7
                   -3.14e-4
                   -2.78e-7 ))
  ;; look at the instance
  ;;(unveil o (current-output-port))(newline)
  (show-bytes o)
  ;; encode/decode every field.
  (let loop ((i 0))
    (if (< i (DM-ClassToFieldLength Testf-class))
        (let* ((field         (DM-ClassToField Testf-class i))
               (value         (field-value o field))
               (offset-number (DM-FieldToOffsetLength field))
               (offset        (DM-FieldToOffset field (- offset-number 1)))
               (translation   (DM-OffsetToTranslation offset))
               (itemtyp       (offset.item-type offset)) )
          (display `(sending field ,i with value ,value))(newline)
          (set-field-value! b 0 (->Field DM-Buffer-class "index"))
          (DM-encode itemtyp o translation c)
          (set-field-value! b 
                            (field-value b (->Field DM-Buffer-class "index"))
                            (->Field DM-Buffer-class "maximum") )
          (set-field-value! b 0 (->Field DM-Buffer-class "index"))
          (DM-decode itemtyp o translation c)
          (let ((new (field-value o field)))
            (display `(received ,new))(newline)
            (if (or (equal? value new)
                    (and (number? value)
                         (let ((delta (/ (- value new) value)))
                           (display `(relative delta is ,delta ))
                           (newline)
                           (< (abs delta) 1.e-6) ) ) )
                (loop (+ i 1))
                (begin (show-bytes o)
                       (show-bytes b)
                       #f ) ) ) )
        #t ) ) )
   #t
;;; Test again after repeating twice the encoding/decoding process.
;;; Normally, floats may change a little the first time but not the
;;; second time.  This is what really happens on SonyNews (the error
;;; is less than 1e-6).
(let ()
  (define c 1)
  (define b 2)
  (define o 3)
  (set! b (DM-allocate DM-Buffer-class 64))
  (set! c (DM-make DM-Connection-class
                   0 5 0   ; 
                   0 0 0 0 ; sockaddr
                   0 0 b b 0 ))
  (set! o (DM-make Testf-class
                   11 
                   #x1234
                   #x12345678
                   #x12345678  ; nat4
                   11 
                   #x1234
                   #x12345678
                   #x12345678  ; int4
                   (- 11)
                   (- #x1234)
                   (- #x12345678)
                   (- #x12345678)  ; int4
                   #\Q
                   #x1234          ; netnat2
                   #x0abcde        ; netnat3
                   0.0          ; float1
                   0.0          ; float2
                   3.14         ; float1
                   2.78e+5      ; float2
                   3.14e-4
                   2.78e-7
                   -3.14e+4
                   -2.78e+7
                   -3.14e-4
                   -2.78e-7 ))
  ;; look at the instance
  ;;(unveil o (current-output-port))(newline)
  (show-bytes o)
  ;; encode/decode every field.
  (let loop ((i 0))
    (if (< i (DM-ClassToFieldLength Testf-class))
        (let* ((field         (DM-ClassToField Testf-class i))
               (value         (field-value o field))
               (new           'wait)
               (newnew        'wait)
               (offset-number (DM-FieldToOffsetLength field))
               (offset        (DM-FieldToOffset field (- offset-number 1)))
               (translation   (DM-OffsetToTranslation offset))
               (itemtyp       (offset.item-type offset)) )
          (display `(sending field ,i with value ,value))(newline)
          ;; first encoding
          (set-field-value! b 0 (->Field DM-Buffer-class "index"))
          (DM-encode itemtyp o translation c)
          (set-field-value! b 
                            (field-value b (->Field DM-Buffer-class "index"))
                            (->Field DM-Buffer-class "maximum") )
          (set-field-value! b 0 (->Field DM-Buffer-class "index"))
          (DM-decode itemtyp o translation c)
          (set! new (field-value o field))
          (display `(received ,new))(newline)
          ;; second encoding
          (set-field-value! b 0 (->Field DM-Buffer-class "index"))
          (DM-encode itemtyp o translation c)
          (set-field-value! b 
                            (field-value b (->Field DM-Buffer-class "index"))
                            (->Field DM-Buffer-class "maximum") )
          (set-field-value! b 0 (->Field DM-Buffer-class "index"))
          (DM-decode itemtyp o translation c)
          (set! newnew (field-value o field))
          (display `(received again ,newnew))(newline)
          ; comparison
          (if (or (equal? value newnew)
                  (and (number? value)
                       (let ((delta1 (/ (- value new) value))
                             (delta2 (/ (- new newnew) new)) )
                         (display `(first relative delta is ,delta1))
                         (newline)
                         (display `(second relative delta is ,delta2))
                         (newline)
                         (and (< (abs delta1) 1.e-6)
                              (< (abs delta2) 1.e-6) ) ) ) )
              (loop (+ i 1))
              (begin (show-bytes o)
                     (show-bytes b)
                     #f ) ) )
        #t ) ) )
   #t

;;; Let's test now the upper level of send/receive.
;;; First define a predicate to compare objects.
(define (egal? o new)
  (define (scan i t)
    (define class (DM-ObjectToClass o))
    (if (< i (DM-ClassToFieldLength class))
        (let* ((field          (DM-ClassToField class i))
               (offsets-number (DM-FieldToOffsetLength field))
               (last-offset    (DM-FieldToOffset field (- offsets-number 1)))
               (reptypindex    (DM-OffsetToRepetitionTypeIndex last-offset))
               (reptyp         (DM-index-to-type reptypindex))
               (itemtypindex   (DM-OffsetToItemTypeIndex last-offset))
               (itemtyp        (DM-index-to-type itemtypindex)) )
          (if (= DM-nought-index reptypindex)
              ;; Regular field
              (begin
                (set! t (pad t itemtyp))
                (display `(egal? itemtyp on ,t))(newline)
                (and (egal? (DM-local-read itemtyp o t)
                            (DM-local-read itemtyp new t) )
                     (scan (+ i 1) (extend (pad t itemtyp) itemtyp)) ) )
              ;; Indexed field
              (begin
                (set! t (pad t reptyp))
                (let ((rep (DM-local-read reptyp o t)))
                  (display `(egal? reptyp on ,t (,rep)))(newline)
                  (and (= rep (DM-local-read reptyp new t))
                       (begin
                         (let enum ((j 0)(t (extend (pad t reptyp) reptyp)))
                           (if (< j rep)
                               (begin
                                 (set! t (pad t itemtyp))
                                 (display `(egal? repitemtyp on ,t))(newline)
                                 (and (egal? (DM-local-read itemtyp o t)
                                             (DM-local-read itemtyp new t) )
                                      (enum (+ j 1) (extend t itemtyp)) ) )
                               (scan (+ i 1) (realign t)) ) ) ) ) ) ) ) )
        #t ) )
  (or (eq? o new)
      ;; NOTE: What follows is no longer required for test-exchange1
      ;; (but still for test-exchange2) since when a site receives an
      ;; Exit corresponding to one of its Entry, it immediately
      ;; substitutes to this Exit the real value.
      (if (DM-is-a new DM-Exit-class)
          ;; compare the original object with the object associated to the
          ;; entry remotely referenced by the Exit instance got as `new'.  
          (egal? o (let* ((nat4-name (DM-exit-to-string new))
                          (entry     (DM-string-to-entry nat4-name)) )
                     (DM-local-get DM-reference-type entry DM-Entry-object-field) ))
          #f )
      ;; Compare recursively the content of the objects.
      (and (egal? (DM-ObjectToClass o) (DM-ObjectToClass new))
           (scan 0 0) ) ) )
   ---
;;; Test a little that egal? predicate
(egal? DM-nil DM-nil)
   #t
(egal? 13 13)
   #t
(egal? (DM-make DM-Nat1-class 13) (DM-make DM-Nat1-class 13))
   #t
(egal? (DM-make DM-ImmutableVector-class 1 DM-Class-class)
       (DM-make DM-ImmutableVector-class 1 DM-Class-class) )
   #t
(egal? (DM-make DM-ImmutableVector-class 1 DM-Class-class)
       (DM-make DM-MutableVector-class 1 DM-Class-class) )
   #f
(egal? (DM-make DM-ImmutableVector-class 1 DM-Class-class)
       (DM-make DM-ImmutableVector-class 2 DM-Class-class DM-nil) )
   #f

;;; Now test the exchange of object.
;;; This test simulates a connection from localhost to localhost.
;;; In that case, when an Exit is rebuilt, it is identified to the 
;;; corresponding Exit and substituted by its value. 
(define (test-exchange1 o)
  (define c 1)
  (define be 2)
  (define br 22)
  (set! be (DM-allocate DM-Buffer-class 64))
  (set! br (DM-allocate DM-Buffer-class 64))
  (set! c (DM-make DM-Connection-class
                   0 5 0
                   0 
                   (DM-local-get DM-netnat2-type DM-site DM-Site-port-field)
                   0 ; The ip address of the site is a too large number.
                   0
                   0 0 br be 0 ))
  ;; patch the connection to have the correct ip address
  (DM-set-ip-address c (DM-OffsetToTranslation
                        (DM-FieldToOffset DM-Tcp-address-field 0) )
                     (DM-String->string
                      (DM-local-get DM-reference-type DM-site DM-Site-name-field) )
                     0 )
  ;; show the connection
  ;(show-bytes c)
  (let ((new 'wait))
    (display `(sending ,o))(newline)
    ;(unveil o (current-output-port))(newline)
    (set-field-value! be 0 (->Field DM-Buffer-class "index"))
    (DM-send c o)
    (show-bytes o)
    (set-field-value! be
                      (field-value be (->Field DM-Buffer-class "index"))
                      (->Field DM-Buffer-class "maximum") )
    (set-field-value! be 0 (->Field DM-Buffer-class "index"))
    (show-bytes be)
    ;; take care of not disturbing the buffer we read if the receiver
    ;; wants to send back some commands to the emitter. Since the
    ;; field is immutable, bypass protection. We just exchange the
    ;; emission and reception buffers.
    (DM-local-write DM-reference-type
                    c
                    (DM-OffsetToTranslation
                     (DM-FieldToOffset
                      (->Field DM-Connection-class "emission") 0 ) ) 
                    br )
    (DM-local-write DM-reference-type
                    c
                    (DM-OffsetToTranslation
                     (DM-FieldToOffset
                      (->Field DM-Connection-class "reception") 0 ) ) 
                    be )
    (set-field-value! br 0 (->Field DM-Buffer-class "index"))
    (set! new (DM-receive c))
    (display `(received ,new))(newline)
    (show-bytes new)
    ;; NOTE: html-unveil also externalizes values.
    ;(html-unveil new (current-output-port))(newline)
    ;; comparison
    (egal? o new) ) )
   ---
;;; test exchange on predefined objects.
(test-exchange1 DM-nil)
   #t
(test-exchange1 DM-nat2-type)
   #t
(test-exchange1 DM-Class-class)
   #t
;;; test other kind of objects with known class 
;;; but without references inside them.
(test-exchange1 (DM-make DM-Nat1-class 32))
   #t
;;; send a repetition of characters
(test-exchange1 (DM-make DM-String-class 3 #\F #\o #\O))
   #t
;;; test object with a trivial reference
(test-exchange1 (DM-make DM-Reference-class DM-nil))
   #t
(test-exchange1 (DM-make DM-Reference-class DM-nat3-type))
   #t
;;; test repeated objects with trivial references
(test-exchange1 (DM-make DM-ImmutableVector-class
                        2 DM-nil DM-nat1-type ))
   #t
;;; test object with non trivial references.
;;; First test they are received as eq?
(let ()
  (define c 1)
  (define be 2)
  (define br 22)
  (define o (DM-make DM-Reference-class 
                     (DM-make DM-Nat2-class 1234) ) )
  (set! be (DM-allocate DM-Buffer-class 64))
  (set! br (DM-allocate DM-Buffer-class 64))
  (set! c (DM-make DM-Connection-class
                   0 5 0
                   0 
                   (DM-local-get DM-netnat2-type DM-site DM-Site-port-field)
                   0 ; The ip address of the site is a too large number.
                   0
                   0 0 br be 0 ))
  ;; patch the connection to have the correct ip address
  (DM-set-ip-address c (DM-OffsetToTranslation
                        (DM-FieldToOffset DM-Tcp-address-field 0) )
                     (DM-String->string
                      (DM-local-get DM-reference-type DM-site DM-Site-name-field) )
                     0 )
  ;; show the connection
  ;(show-bytes c)
  (let ((new 'wait))
    (display `(sending ,o))(newline)
    ;(unveil o (current-output-port))(newline)
    (set-field-value! be 0 (->Field DM-Buffer-class "index"))
    (DM-send c o)
    (show-bytes o)
    (set-field-value! be
                      (field-value be (->Field DM-Buffer-class "index"))
                      (->Field DM-Buffer-class "maximum") )
    (set-field-value! be 0 (->Field DM-Buffer-class "index"))
    (show-bytes be)
    ;; take care of not disturbing the buffer we read if the receiver
    ;; wants to send back some commands to the emitter. Since the
    ;; field is immutable, bypass protection. We just exchange the
    ;; emission and reception buffers.
    (DM-local-write DM-reference-type
                    c
                    (DM-OffsetToTranslation
                     (DM-FieldToOffset
                      (->Field DM-Connection-class "emission") 0 ) ) 
                    br )
    (DM-local-write DM-reference-type
                    c
                    (DM-OffsetToTranslation
                     (DM-FieldToOffset
                      (->Field DM-Connection-class "reception") 0 ) ) 
                    be )
    (set-field-value! br 0 (->Field DM-Buffer-class "index"))
    (set! new (DM-receive c))
    (display `(received ,new))(newline)
    (show-bytes new)
    ;; NOTE: html-unveil also externalizes values.
    ;(html-unveil new (current-output-port))(newline)
    ;; O and NEW are structurally equal but not eq, but their first field
    ;; are eq.
    (eq? (DM-local-get DM-reference-type o (DM-ClassToField DM-Reference-class 0))
         (DM-local-get DM-reference-type new (DM-ClassToField DM-Reference-class 0))) ) )
   #t
;;; continue regular tests.
(test-exchange1 (DM-make DM-Reference-class 
                        (DM-make DM-Nat2-class 1234) ) )
   #t
(test-exchange1 (DM-make DM-Reference-class 
                        (DM-make DM-Reference-class
                                 (DM-make DM-Nat2-class 1234) ) ) )
   #t
;;; test repeated field with non trivial references.
(test-exchange1 (DM-make DM-ImmutableVector-class
                        4 (DM-make DM-Reference-class DM-nat3-type)
                          DM-nil
                          (DM-make DM-Nat2-class 1234)
                          (DM-make DM-Reference-class 
                                   (DM-make DM-Reference-class
                                            (DM-make DM-Nat2-class 1234) ) )))
   #t
;;; test object with non predefined class.
;;; Only an Exit is made on the receiver.
(define Pair-class (DM-create-class "Pair" DM-Object-class 2
                           "-field" "x" "-mutable"
                           "-field" "y" "-mutable" ))
   ---
(test-exchange1 (DM-make Pair-class
                        DM-nil DM-nat4-type ))
   #t
(test-exchange1 (DM-make Testf-class
                   11 
                   #x1234
                   #x12345678
                   #x12345678  ; nat4
                   11 
                   #x1234
                   #x12345678
                   #x12345678  ; int4
                   (- 11)
                   (- #x1234)
                   (- #x12345678)
                   (- #x12345678)  ; int4
                   #\Q
                   #x1234          ; netnat2
                   #x0abcde        ; netnat3
                   0.0          ; float1
                   0.0          ; float2
                   3.14         ; float1
                   2.78e+5      ; float2
                   3.14e-4
                   2.78e-7
                   -3.14e+4
                   -2.78e+7
                   -3.14e-4
                   -2.78e-7 ) )
   #t


;;; This test simulates a connection from no host to localhost
;;; The difference lies with receiving Exit items which are no longer
;;; recognized as being equivalent to the sent Entry.
(define (test-exchange2 o)
  (define c 1)
  (define be 2)
  (define br 22)
  (set! be (DM-allocate DM-Buffer-class 64))
  (set! br (DM-allocate DM-Buffer-class 64))
  (set! c (DM-make DM-Connection-class
                   0 5 0
                   0 
                   (DM-local-get DM-netnat2-type DM-site DM-Site-port-field)
                   0 ; The ip address of the site is a too large number.
                   0
                   0 0 br be 0 ))
  ;; DON'T patch the connection to have the correct ip address
  ;(DM-set-ip-address c (DM-OffsetToTranslation
  ;                      (DM-FieldToOffset DM-Tcp-address-field 0) )
  ;                  (DM-String->string
  ;                    (DM-local-get DM-reference-type DM-site DM-Site-name-field) )
  ;                   0 )
  ;; show the connection
  ;(show-bytes c)
  (let ((new 'wait))
    (display `(sending ,o))(newline)
    ;(unveil o (current-output-port))(newline)
    (set-field-value! be 0 (->Field DM-Buffer-class "index"))
    (DM-send c o)
    (show-bytes o)
    (set-field-value! be
                      (field-value be (->Field DM-Buffer-class "index"))
                      (->Field DM-Buffer-class "maximum") )
    (set-field-value! be 0 (->Field DM-Buffer-class "index"))
    (show-bytes be)
    ;; take care of not disturbing the buffer we read if the receiver
    ;; wants to send back some commands to the emitter. Since the
    ;; field is immutable, bypass protection. We just exchange the
    ;; emission and reception buffers.
    (DM-local-write DM-reference-type
                    c
                    (DM-OffsetToTranslation
                     (DM-FieldToOffset
                      (->Field DM-Connection-class "emission") 0 ) ) 
                    br )
    (DM-local-write DM-reference-type
                    c
                    (DM-OffsetToTranslation
                     (DM-FieldToOffset
                      (->Field DM-Connection-class "reception") 0 ) ) 
                    be )
    (set-field-value! br 0 (->Field DM-Buffer-class "index"))
    (set! new (DM-receive c))
    (display `(received ,new))(newline)
    (show-bytes new)
    ;; NOTE: html-unveil also externalizes values.
    ;(html-unveil new (current-output-port))(newline)
    ;; comparison
    (egal? o new) ) )
   ---
;;; test exchange on predefined objects.
(test-exchange2 DM-nil)
   #t
(test-exchange2 DM-nat2-type)
   #t
(test-exchange2 DM-Class-class)
   #t
;;; test other kind of objects with known class 
;;; but without references inside them.
(test-exchange2 (DM-make DM-Nat1-class 32))
   #t
;;; send a repetition of characters
(test-exchange2 (DM-make DM-String-class 3 #\F #\o #\O))
   #t
;;; test object with a trivial reference
(test-exchange2 (DM-make DM-Reference-class DM-nil))
   #t
(test-exchange2 (DM-make DM-Reference-class DM-nat3-type))
   #t
;;; test repeated objects with trivial references
(test-exchange2 (DM-make DM-ImmutableVector-class
                        2 DM-nil DM-nat1-type ))
   #t
;;; test object with non trivial references
(test-exchange2 (DM-make DM-Reference-class 
                        (DM-make DM-Nat2-class 1234) ) )
   #t
(test-exchange2 (DM-make DM-Reference-class 
                        (DM-make DM-Reference-class
                                 (DM-make DM-Nat2-class 1234) ) ) )
   #t
;;; test repeated field with non trivial references.
(test-exchange2 (DM-make DM-ImmutableVector-class
                        4 (DM-make DM-Reference-class DM-nat3-type)
                          DM-nil
                          (DM-make DM-Nat2-class 1234)
                          (DM-make DM-Reference-class 
                                   (DM-make DM-Reference-class
                                            (DM-make DM-Nat2-class 1234) ) )))
   #t
;;; test object with non predefined class.
;;; Only an Exit is made on the receiver.
(define Pair-class (DM-create-class "Pair" DM-Object-class 2
                           "-field" "x" "-mutable"
                           "-field" "y" "-mutable" ))
   ---
(test-exchange2 (DM-make Pair-class
                        DM-nil DM-nat4-type ))
   #t
(test-exchange2 (DM-make Testf-class
                   11 
                   #x1234
                   #x12345678
                   #x12345678  ; nat4
                   11 
                   #x1234
                   #x12345678
                   #x12345678  ; int4
                   (- 11)
                   (- #x1234)
                   (- #x12345678)
                   (- #x12345678)  ; int4
                   #\Q
                   #x1234          ; netnat2
                   #x0abcde        ; netnat3
                   0.0          ; float1
                   0.0          ; float2
                   3.14         ; float1
                   2.78e+5      ; float2
                   3.14e-4
                   2.78e-7
                   -3.14e+4
                   -2.78e+7
                   -3.14e-4
                   -2.78e-7 ) )
   #t

;;; Send an object twice (received as an exit) and expects a decrement
;;; message the second time and a refcount of 2.
(let ()
  (define c 1)
  (define be 2)
  (define br 22)
  (define new 3)
  (define newnew 33)
  (define o (DM-make Pair-class DM-nil DM-nat4-type))
  (set! be (DM-allocate DM-Buffer-class 64))
  (set! br (DM-allocate DM-Buffer-class 64))
  (set! c (DM-make DM-Connection-class
                   0 5 0                ; 
                   0 0 0 0              ; sockaddr
                   0 0 br be 0 ))
  ;; for each of them, 
  (display `(sending ,o))(newline)
  (set-field-value! be 0 (->Field DM-Buffer-class "index"))
  (DM-send c o)
  (DM-send c o)
  ;;(show-bytes o)
  (set-field-value! be
                    (field-value be (->Field DM-Buffer-class "index"))
                    (->Field DM-Buffer-class "maximum") )
  (set-field-value! be 0 (->Field DM-Buffer-class "index"))
  ;;(show-bytes be)
  ;; take care of not disturbing the buffer we read if the receiver
  ;; wants to send back some commands to the emitter. Since the
  ;; field is immutable, bypass protection. We just exchange the
  ;; emission and reception buffers.
  (DM-local-write DM-reference-type
                  c
                  (DM-OffsetToTranslation
                   (DM-FieldToOffset
                    (->Field DM-Connection-class "emission") 0 ) ) 
                  br )
  (DM-local-write DM-reference-type
                  c
                  (DM-OffsetToTranslation
                   (DM-FieldToOffset
                    (->Field DM-Connection-class "reception") 0 ) ) 
                  be )
  (set-field-value! br 0 (->Field DM-Buffer-class "index"))
  (set! new (DM-receive c))
  (display `(received ,new))(newline)
  (show-bytes new)
  (set! newnew (DM-receive c))
  (display `(received ,newnew))(newline)
  (show-bytes newnew)
  ;; comparison
  (and (egal? o newnew)
       ;; must be the same exit:
       (eq? new newnew)
       ;; the refcount should be equal to 2
       (= 2 (field-value (Object.entry o) (->Field DM-Entry-class "counter")))
       ;; must contain a decrement message:
       (field-value br (->Field DM-Buffer-class "index")) ) )
   9 ; #xb0 followed by a nat4.

" *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
                      All migr.tst tests successfully run!
 *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***"
   ---

;;; end of migr.tst
