;;; $Id: newapi.tst,v 1.14 1997/08/17 19:30:50 queinnec Exp $
;;;///////////////////////////////////////////////////////////////////////
;;;                      //          //
;;;                     // DMeroon  //
;;;                    //          //
;;; A continuation-based causal distributed class-based object system.
;;; Christian.Queinnec@polytechnique.fr
;;;///////////////////////////////////////////////////////////////////////

;;; This file contains tests for the Scheme binding of DMeroon. It
;;; uses the new API for DMeroon.  This file was obtained from
;;; dmeroon.tst, this explains the auxiliary functions that appear
;;; here and there in this file.

(DM-initialize!)
   ---
;;; DM-site is predefined to hold the current site.
(eq? (DM-initialize!) DM-site)
   #t

;;; Test DMeroon membership
(DM-is-object? #t)
   #f
(DM-is-object? DM-Class-class)
   #t
(DM-is-object? DM-nat1-type)
   #t
(DM-is-object? car)
   #f
;;; no errors to be checked

;;; Check class membership
(DM-is-a? DM-Object-class DM-Class-class)
   #t
(DM-is-a? DM-Object-class DM-Object-class)
   #t
(DM-is-a? DM-Class-class DM-Object-class)
   #t
(DM-is-a? DM-Class-class DM-Class-class)
   #t
(DM-is-a? DM-Object-class DM-Field-class)
   #f
;;; check errors
(DM-is-a? #t DM-Class-class)
   *** ; not DMeroon instance
(DM-is-a? DM-Object-class DM-Class-name-field)
   *** ; not a Class

;;; check subclass relation
(DM-is-a-subclass? DM-Class-class DM-Class-class)
   #t
(DM-is-a-subclass? DM-Class-class DM-Object-class)
   #t
(DM-is-a-subclass? DM-Object-class DM-Class-class)
   #f
(DM-is-a-subclass? DM-Class-class DM-Field-class)
   #f
;;; check errors
(DM-is-a-subclass? DM-Class-class (list))
   *** ; not a DMeroon
(DM-is-a-subclass? DM-Class-class DM-site)
   *** ; not a Class
(DM-is-a-subclass? (list) DM-Class-class)
   *** ; not a DMeroon
(DM-is-a-subclass? DM-site DM-Class-class)
   *** ; not a Class
(DM-is-a-subclass? #f #t)
   *** ; not a DMeroon

;;; Check instantiation links
(eq? (DM-class-of DM-Object-class)
     (DM-class-of DM-Class-class) )
   #t
(eq? (DM-class-of DM-nat1-type)
     DM-DMeroonType-class )
   #t
;;; Check error
(DM-class-of #t)
   *** ; not DMeroon instance
(DM-class-of "foo")
   *** ; not DMeroon instance

;;; Try allocation.
;;; NOTE: DM-allocate is not a DMeroon API function but should be 
;;; offered in a Lisp binding.
DM-allocate
   ---

;;; allocation of a simple object (without indexed fields)
(DM-allocate DM-Object-class)
   ---
(DM-is-object? (DM-allocate DM-Object-class))
   #t
(DM-is-a? (DM-allocate DM-Object-class) DM-Object-class)
   #t
(DM-allocate DM-ImmutableLink-class)
   ---
(DM-is-a? (DM-allocate DM-ImmutableLink-class) DM-ImmutableLink-class)
   #t

;;; allocation of a non simple object
(DM-allocate DM-MutableVector-class 3)
   ---
(DM-is-a? (DM-allocate DM-MutableVector-class 3) DM-MutableVector-class)
   #t

;;; Exercize errors
(DM-allocate "foo" 3)
   *** ; not a DMeroon object
(DM-allocate DM-Class-name-field 3)
   *** ; not a class
(DM-allocate DM-ImmutableLink-class 3)
   *** ; too much sizes
(DM-allocate DM-MutableVector-class #t #f)
   *** ; not number
(DM-allocate DM-MutableVector-class 3 4)
   *** ; too much sizes
(DM-allocate DM-MutableVector-class)
   *** ; not enough sizes

;;; Check comparison on local objects only.
(DM-eq? DM-Object-class DM-Object-class)
   #t
;;; DM-eq? currently knows only how to compare eq? objects since
;;; not eq? may be DM-eq? if compared while migrating.
;(DM-eq? DM-Object-class DM-Class-class)
;   #f

;;; Check simple access
(DM-field-value DM-site DM-Site-support-field)
   ---
(DM-is-a? (DM-field-value DM-site DM-Site-support-field)
         DM-String-class )
   #t
;;; exercize errors
(DM-field-value #t DM-Site-support-field)
   *** ; not DMeroon instance
(DM-field-value DM-site "foo")
   *** ; not a DMeroon
(DM-field-value DM-site DM-Class-class)
   *** ; not a Field
(DM-field-value DM-site DM-Class-options-field)
   *** ; site is not a Class
(DM-field-value DM-Object-class DM-Class-super-field)
   *** ; not a regular Field

;;; check simple indexed access
(eq? (DM-indexed-field-value DM-Class-class DM-Class-super-field 0)
     DM-Object-class )
   #t
(eq? (DM-indexed-field-value DM-Class-class DM-Class-super-field 1)
     DM-Class-class )
   #t

;;; check non simple indexed access (fields are the second indexed
;;; field of a class).
(eq? (DM-indexed-field-value DM-Class-class DM-Class-field-field 3)
     DM-Class-name-field )
   #t
(eq? (DM-indexed-field-value DM-Class-class DM-Class-field-field 1)
     DM-Class-super-field )
   #t

;;; exercize errors
(DM-indexed-field-value #t DM-Class-field-field 2)
   *** ; not DMeroon instance
(DM-indexed-field-value DM-Class-class DM-Class-field-field #t)
   *** ; not an index
(DM-indexed-field-value DM-Class-class DM-Class-class 2)
   *** ; not a Field
(DM-indexed-field-value DM-Class-class "foo" 2)
   *** ; not a Field
(DM-indexed-field-value DM-Class-class DM-Class-options-field 2)
   *** ; not an indexed Field
(DM-indexed-field-value DM-Class-class DM-Dictionary-name-field 2)
   *** ; not a dictionary

;;; Try to read all kinds of DMeroon types.
;;; At allocation slots are zero-ed.
(let* ((class DM-nat1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0
(let* ((class DM-nat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0
(let* ((class DM-nat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0
(let* ((class DM-nat4-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0
(let* ((class DM-netnat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0
(let* ((class DM-netnat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0
(let* ((class DM-int1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0
(let* ((class DM-int2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0
(let* ((class DM-int3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0
(let* ((class DM-int4-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0
(let* ((class DM-char1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (char->integer (DM-field-value o f)) )
   0
(let* ((class DM-float1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0.0
(let* ((class DM-float2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-field-value o f) )
   0.0
;;; Pay attention to float3 which are not implemented everywhere.
(let* ((class DM-float3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (let ((size (DM-field-value DM-float3-type DM-Type-size-field)))
    (if (> size 0)
        ;; This type is implemented.
        (DM-field-value o f)
        0.0 ) ) )
   0.0

;;; Write now on all kind of DMeroon types.
(let* ((class DM-nat1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 11)
  (DM-field-value o f) )
   11
(let* ((class DM-nat1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 255)
  (DM-field-value o f) )
  255
(let* ((class DM-nat1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a number
(let* ((class DM-nat1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 256) )
  *** ; too big nat1
(let* ((class DM-nat1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -25)
  (DM-field-value o f) )
  *** ; not positive nat1
(let* ((class DM-nat1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -25) )
  *** ; not positive nat1

(let* ((class DM-nat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 65535)
  (DM-field-value o f) )
   65535
(let* ((class DM-nat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a number
(let* ((class DM-nat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 65536)
  (DM-field-value o f) )
  *** ; too big nat2
(let* ((class DM-nat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 65536) )
  *** ; too big nat2
(let* ((class DM-nat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -35)
  (DM-field-value o f) )
  *** ; not positive nat2
(let* ((class DM-nat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -35) )
  *** ; not positive nat2

(let* ((class DM-nat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 30541)
  (DM-field-value o f) )
   30541
;;; #x12345678 = 305419896
(let* ((class DM-nat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 305419896)
  (DM-field-value o f) )
   305419896
(let* ((class DM-nat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a number
(let* ((class DM-nat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -35)
  (DM-field-value o f) )
  ***  ; not positive nat3
(let* ((class DM-Nat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -35) )
  ***  ; not positive nat3

;;; No test on DM-nat4

(let* ((class DM-Reference-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f DM-site)
  (eq? DM-site (DM-field-value o f)) )
   #t
(let* ((class DM-reference-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a dmeroon 

(let* ((class DM-data-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f DM-site)
  (eq? DM-site (DM-field-value o f)) )
   #t
(let* ((class DM-data-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (cons #t #f))
  (DM-field-value o f) )
   (#t . #f)

(let* ((class DM-int1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 127)
  (DM-field-value o f) )
  127
(let* ((class DM-int1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a number
(let* ((class DM-int1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 128)
  (DM-field-value o f) )
  *** ; too big int1
(let* ((class DM-int1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 128) )
  *** ; too big int1
(let* ((class DM-int1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -127)
  (DM-field-value o f) )
  -127
(let* ((class DM-int1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -128)
  (DM-field-value o f) )
  *** ; too big int1
(let* ((class DM-int1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -128) )
  *** ; too big int1

;;; #x7fff = 32767
(let* ((class DM-int2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 32767)
  (DM-field-value o f) )
  32767
(let* ((class DM-int2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a number
(let* ((class DM-int2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 32768)
  (DM-field-value o f) )
  *** ; too big int2
(let* ((class DM-int2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 32768) )
  *** ; too big int2
(let* ((class DM-int2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -32767)
  (DM-field-value o f) )
  -32767
(let* ((class DM-int2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -32768)
  (DM-field-value o f) )
  *** ; too big int2
(let* ((class DM-int2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -32768) )
  *** ; too big int2

;;; #x7fffff = 8388607
(let* ((class DM-int3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 8388607)
  (DM-field-value o f) )
   8388607
(let* ((class DM-int3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -8388607)
  (DM-field-value o f) )
   -8388607
(let* ((class DM-int3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a number
(let* ((class DM-int3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
   *** ; not a number

(let* ((class DM-char1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f #\q)
  (DM-field-value o f) )
   #\q
(let* ((class DM-char1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f #t) )
   *** ; not a char1

(let* ((class DM-netnat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 65535)
  (DM-field-value o f) )
   65535
(let* ((class DM-netnat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 65536)
  (DM-field-value o f) )
  ***
(let* ((class DM-netnat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -35)
  (DM-field-value o f) )
  ***
(let* ((class DM-netnat2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a number

(let* ((class DM-netnat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 30541)
  (DM-field-value o f) )
   30541
;;; #x12345678 = 305419896
(let* ((class DM-netnat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 305419896)
  (DM-field-value o f) )
   305419896
(let* ((class DM-netnat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -35)
  (DM-field-value o f) )
  ***
(let* ((class DM-netnat3-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a number

(let* ((class DM-float1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 3.14e+2)
  (DM-field-value o f) )
  3.14e+2
(let* ((class DM-float1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -3.14e+2)
  (DM-field-value o f) )
  -3.14e+2
(let* ((class DM-float1-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a number
(let* ((class DM-float2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 3.14e+2)
  (DM-field-value o f) )
  3.14e+2
(let* ((class DM-float2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f -3.14e+2)
  (DM-field-value o f) )
  -3.14e+2
(let* ((class DM-float2-class)
       (o     (DM-allocate class))
       (f     (DM-indexed-field-value class DM-Class-field-field 0)) )
  (DM-set-field-value! o f (list)) )
  *** ; not a number

;;; Checking length of indexed fields.
(DM-indexed-field-length DM-Object-class DM-Class-super-field)
   1
(DM-indexed-field-length 
 (DM-allocate DM-Class-class 3 5 7) DM-Class-super-field )
   3
(DM-indexed-field-length 
 (DM-allocate DM-Class-class 3 5 7) DM-Class-field-field )
   5

;;; exercize errors
(DM-indexed-field-length "foo" DM-Class-super-field)
   *** ; not a DMeroon
(DM-indexed-field-length DM-Object-class "foo")
   *** ; not a DMeroon
(DM-indexed-field-length DM-Object-class DM-Class-class)
   *** ; not a Field
(DM-indexed-field-length DM-Object-class DM-Dictionary-name-field)
   *** ; wrong instance

;;; Simple modifications
(DM-allocate DM-MutableVector-class 4)
   ---
(let ((d (DM-allocate DM-Dictionary-class 2)))
  (DM-set-field-value! d DM-Dictionary-rest-field d)
  (eq? (DM-field-value d DM-Dictionary-rest-field) d) )
   #t
(let ((d (DM-allocate DM-Dictionary-class 2)))
  (DM-set-field-value! d DM-Dictionary-value-field d)
  (eq? (DM-field-value d DM-Dictionary-value-field) d) )
   #t
;;; check error
(let ((d (DM-allocate DM-Int1-class)))
  (DM-set-field-value! d DM-Int1-value-field 12)
  (DM-set-sharable! d)
  (DM-set-field-value! d DM-Int1-value-field 13) )
   *** ; immutable field

;;; modification on the first indexed field
(let ((mv    (DM-allocate DM-MutableVector-class 4))
      (value DM-Object-class) )
  (DM-set-indexed-field-value! mv DM-MutableVector-item-field 1 value)
  (eq? value (DM-indexed-field-value  mv DM-MutableVector-item-field 1)) )
   #t
(let ((mv    (DM-allocate DM-Class-class 4 2 1))
      (value DM-nat3-type) )
  (DM-set-indexed-field-value! mv DM-Class-super-field 1 value)
  (eq? value (DM-indexed-field-value mv DM-Class-super-field 1)) )
   *** ; value with bad class
(let ((mv    (DM-allocate DM-Class-class 4 2 1))
      (value DM-Object-class) )
  (DM-set-indexed-field-value! mv DM-Class-super-field 1 value)
  (eq? value (DM-indexed-field-value mv DM-Class-super-field 1)) )
   #t

;;; modifications in the second indexed field
(let ((mv    (DM-allocate DM-Class-class 4 2 1))
      (value DM-Object-class) )
  (DM-set-indexed-field-value! mv DM-Class-super-field 1 value)
  (eq? value (DM-indexed-field-value mv DM-Class-super-field 1)) )
   #t

;;; Checking object initialization
(DM-set-sharable! DM-site)
   ---
(let ((o (DM-allocate DM-Object-class)))
  (DM-set-sharable! o) )
   ---
(let ((o (DM-allocate DM-Object-class)))
  (DM-set-sharable! o)
  (DM-set-sharable! o) )
   ---
(let ((o (DM-allocate DM-Object-class)))
  (DM-set-copyable! o) )
   ---
(let ((o (DM-allocate DM-Object-class)))
  (DM-set-copyable! o)
  (DM-set-copyable! o) )
   ---
(let ((o (DM-allocate DM-Object-class)))
  (DM-set-sharable! o)
  (DM-set-copyable! o) )
   *** ; already sharable
(let ((o (DM-allocate DM-Object-class)))
  (DM-set-copyable! o)
  (DM-set-sharable! o) )
   *** ; already copyable

;;; Until an object is initialized, it may be mutated even if immutable.
(let ((v (DM-allocate DM-ImmutablePair-class)))
  (DM-set-field-value! v DM-ImmutablePair-value-field v)
  (eq? v (DM-field-value v DM-ImmutablePair-value-field)) )
   #t
(let ((v (DM-allocate DM-ImmutableVector-class 3)))
  (DM-set-indexed-field-value! v DM-ImmutableVector-item-field 1 v)
  (eq? v (DM-indexed-field-value v DM-ImmutableVector-item-field 1)) )
   #t
(let ((v (DM-allocate DM-ImmutablePair-class)))
  (DM-set-field-value! v DM-ImmutablePair-value-field v)
  (DM-set-sharable! v)
  (eq? v (DM-field-value v DM-ImmutablePair-value-field)) )
   #t
(let ((v (DM-allocate DM-ImmutableVector-class 3)))
  (DM-set-indexed-field-value! v DM-ImmutableVector-item-field 1 v)
  (DM-set-sharable! v)
  (eq? v (DM-indexed-field-value v DM-ImmutableVector-item-field 1)) )
   #t
;;; But this is no longer true after initialization
(let ((v (DM-allocate DM-ImmutablePair-class)))
  (DM-set-sharable! v)
  (DM-set-field-value! v DM-ImmutablePair-value-field v)
  (eq? v (DM-field-value v DM-ImmutablePair-value-field)) )
   *** ; immutable
(let ((v (DM-allocate DM-ImmutableVector-class 3)))
  (DM-set-sharable! v)
  (DM-set-indexed-field-value! v DM-ImmutableVector-item-field 1 v)
  (eq? v (DM-indexed-field-value v DM-ImmutableVector-item-field 1)) )
*** ; immutable

;;; exercize errors
(DM-set-sharable! "foo")
   *** ; not DMeroon

;;; Get site from local objects only
(eq? DM-site (DM-site-of DM-site))
   #t
(eq? DM-site (DM-site-of DM-Object-class))
   #t

;;; exercize errors
(DM-site-of "foo")
   *** ; not DMeroon

;;; setting site of local objects.   TO BE DONE
;(DM-set-site-of! (DM-allocate DM-ImmutablePair-class) DM-site)
;   ---

;;; Get clock of local objects
(DM-clock-of DM-Object-class)
   #f ; Classes are immutable.
(DM-clock-of (DM-field-value DM-site DM-Site-clock-field))
   #f ; clocks are immutable
(let ((d (DM-allocate DM-Dictionary-class 2)))
  (DM-set-sharable! d)
  (DM-clock-of d) )
   #f ; dictionary is mutable but has still no associated clock here.

;;; Setting clocks
(let ((d     (DM-allocate DM-Dictionary-class 2))
      (clock (DM-allocate DM-Clock-class)) )
  (DM-set-sharable! d)
  (DM-set-clock-of! d clock)
  (eq? clock (DM-clock-of d)) )
   #t
(DM-is-a? (DM-field-value DM-site DM-Site-clock-field) DM-Clock-class)
   #t

;;; exercize errors
(DM-set-clock-of! "foo" (DM-field-value DM-site DM-Site-clock-field))
   *** ; not a DMeroon
(let ((d     (DM-allocate DM-Dictionary-class 2))
      (clock (DM-allocate DM-Clock-class)) )
  (DM-set-sharable! d)
  (DM-set-clock-of! d "foo") )
  *** ; not a clock
(let ((d     (DM-allocate DM-ImmutableLink-class))
      (clock (DM-allocate DM-Clock-class)) )
  (DM-set-sharable! d)
  (DM-set-clock-of! d clock) )
  *** ; d is not mutable
; An uninitialized object may now be given a clock if mutable ???
;(let ((d     (DM-allocate DM-Dictionary-class 2))
;      (clock (DM-allocate DM-Clock-class)) )
;  (DM-set-clock-of! d clock) )
;  *** ; d not initialized

;;; Send an object to oneself.
(let ((o DM-Site-class))
  (DM-send o DM-site)
  (eq? o (DM-receive 0)) )
   #t
(let ((o DM-Site-class))
  (DM-send o DM-site)
  (DM-send o DM-site)
  (DM-send o DM-site)
  (DM-send o DM-site)
  (and (eq? o (DM-receive 0))
       (eq? o (DM-receive 1))
       (eq? o (DM-receive 0))
       (eq? o (DM-receive 1)) ) )
   #t

;;; Creating subclasses
(define Point1D-class
  (DM-create-subclass DM-Object-class
                      "Point1D"
                      0
                      "x"
                      0
                      DM-nought-type
                      DM-nat3-type
                      #f ) )
   ---
(define Point2D-class
  (DM-create-subclass Point1D-class
                      "Point2D"
                      0
                      "y"
                      0
                      DM-nought-type
                      DM-nat4-type
                      #f ) )
   ---
(define Point-class
  (DM-smash-class Point2D-class
                  "Point"
                  0
                  2 ) )
   ---
(DM-indexed-field-length Point-class DM-Class-super-field)
  2 ; Object and itself.
(DM-indexed-field-length Point-class DM-Class-field-field)
  2 ; x and y.
(DM-allocate Point-class)
   ---
(let ((pt (DM-allocate Point-class)))
  (DM-set-field-value! 
   pt (DM-indexed-field-value Point-class DM-Class-field-field 0) 11 )
  (DM-set-field-value! 
   pt (DM-indexed-field-value Point-class DM-Class-field-field 1) 22 )
  pt )
   ---

;;; Test a little inheritance
(define Point3D-class
  (DM-create-subclass Point-class
                      "Point3D"
                      0
                      "z"
                      0
                      DM-nought-type
                      DM-nat1-type
                      #f ) )
   ---
;;; Use the accessor of the superclass
(let* ((o (DM-allocate Point3D-class))
       (f (DM-indexed-field-value Point-class DM-Class-field-field 0)) )
  (DM-set-field-value! o f 11)
  (DM-field-value o f) )
   11
;;; Use the accessor of the superclass
(let* ((o (DM-allocate Point3D-class))
       (f (DM-indexed-field-value Point-class DM-Class-field-field 1)) )
  (DM-set-field-value! o f 121)
  (DM-field-value o f) )
   121
;;; Use the accessor of the class
(let* ((o (DM-allocate Point3D-class))
       (f (DM-indexed-field-value Point3D-class DM-Class-field-field 2)) )
  (DM-set-field-value! o f 121)
  (DM-field-value o f) )
   121

;;; Test new initialization primitive
(let ((o (DM-allocate Point3D-class)))
  (DM-is-initialized? o) )
   #f
(let ((o (DM-allocate Point3D-class)))
  (DM-set-initialized! o)
  (and (DM-is-initialized? o) #t) )
   #t
(let ((o (DM-allocate Point3D-class)))
  (DM-set-sharable! o)
  (DM-is-initialized? o) )
   1
(let ((o (DM-allocate Point3D-class)))
  (DM-set-copyable! o)
  (DM-is-initialized? o) )
   2

;;; Test references
(define PointWithRef-class
  (DM-create-subclass Point-class
                      "PointWithRef"
                      0
                      "next"
                      1
                      DM-nought-type
                      DM-reference-type
                      DM-Type-class ) )
   ---
(let* ((o (DM-allocate PointWithRef-class))
       (f (DM-indexed-field-value PointWithRef-class DM-Class-field-field 2)) )
  (DM-set-field-value! o f DM-nat1-type) )
   ---
(let* ((o (DM-allocate PointWithRef-class))
       (f (DM-indexed-field-value PointWithRef-class DM-Class-field-field 2)) )
  (DM-set-field-value! o f DM-Nat1-class) )
   ***
(let* ((o (DM-allocate PointWithRef-class))
       (f (DM-indexed-field-value PointWithRef-class DM-Class-field-field 2)) )
  (DM-set-initialized! o)
  (DM-set-field-value! o f DM-nat1-type) )
   ---
(let* ((o (DM-allocate PointWithRef-class))
       (f (DM-indexed-field-value PointWithRef-class DM-Class-field-field 2)) )
  (DM-set-initialized! o)
  (DM-set-field-value! o f DM-Nat1-class) )
   ***

;;; Test a little indexed fields
(define ColoredString-class
  (DM-create-subclass DM-String-class
                      "ColoredString"
                      0
                      "color"
                      0
                      DM-nat1-type
                      DM-char1-type
                      #f ) )
   ---
(define (test-indexed-colored-string n1 n2)
  (let* ((o (DM-allocate ColoredString-class n1 n2))
         (f (DM-indexed-field-value ColoredString-class 
                                    DM-Class-field-field 1 )) )
    (DM-set-indexed-field-value! o f (- n2 1) #\Q)
    (DM-indexed-field-value o f (- n2 1)) ) )
   ---
(test-indexed-colored-string 0 1)
   #\Q
(test-indexed-colored-string 0 2)
   #\Q
(test-indexed-colored-string 0 3)
   #\Q
(test-indexed-colored-string 0 4)
   #\Q
(test-indexed-colored-string 0 5)
   #\Q
(test-indexed-colored-string 1 1)
   #\Q
(test-indexed-colored-string 1 2)
   #\Q
(test-indexed-colored-string 1 3)
   #\Q
(test-indexed-colored-string 1 4)
   #\Q
(test-indexed-colored-string 1 4)
   #\Q
(test-indexed-colored-string 2 1)
   #\Q
(test-indexed-colored-string 2 2)
   #\Q
(test-indexed-colored-string 2 3)
   #\Q
(test-indexed-colored-string 2 4)
   #\Q
(test-indexed-colored-string 2 5)
   #\Q
(test-indexed-colored-string 3 1)
   #\Q
(test-indexed-colored-string 3 2)
   #\Q
(test-indexed-colored-string 3 3)
   #\Q
(test-indexed-colored-string 3 4)
   #\Q
(test-indexed-colored-string 3 5)
   #\Q
(test-indexed-colored-string 4 1)
   #\Q
(test-indexed-colored-string 4 2)
   #\Q
(test-indexed-colored-string 4 3)
   #\Q
(test-indexed-colored-string 4 4)
   #\Q
(test-indexed-colored-string 4 5)
   #\Q
(test-indexed-colored-string 5 1)
   #\Q
(test-indexed-colored-string 5 2)
   #\Q
(test-indexed-colored-string 5 3)
   #\Q
(test-indexed-colored-string 5 4)
   #\Q
(test-indexed-colored-string 5 5)
   #\Q
;;; Test again indexed fields
(define ColoredVector-class
  (DM-create-subclass DM-ImmutableVector-class
                      "ColoredVector"
                      0
                      "color"
                      0
                      DM-nat1-type
                      DM-char1-type
                      #f ) )
   ---
(define (test-indexed-colored-vector n1 n2)
  (let* ((o (DM-allocate ColoredVector-class n1 n2))
         (f (DM-indexed-field-value ColoredVector-class 
                                    DM-Class-field-field 1 )) )
    (DM-set-indexed-field-value! o f (- n2 1) #\Q)
    (DM-indexed-field-value o f (- n2 1)) ) )
   ---
(test-indexed-colored-vector 0 1)
   #\Q
(test-indexed-colored-vector 0 2)
   #\Q
(test-indexed-colored-vector 0 3)
   #\Q
(test-indexed-colored-vector 0 4)
   #\Q
(test-indexed-colored-vector 0 5)
   #\Q
(test-indexed-colored-vector 1 1)
   #\Q
(test-indexed-colored-vector 1 2)
   #\Q
(test-indexed-colored-vector 1 3)
   #\Q
(test-indexed-colored-vector 1 4)
   #\Q
(test-indexed-colored-vector 1 4)
   #\Q
(test-indexed-colored-vector 2 1)
   #\Q
(test-indexed-colored-vector 2 2)
   #\Q
(test-indexed-colored-vector 2 3)
   #\Q
(test-indexed-colored-vector 2 4)
   #\Q
(test-indexed-colored-vector 2 5)
   #\Q
(test-indexed-colored-vector 3 1)
   #\Q
(test-indexed-colored-vector 3 2)
   #\Q
(test-indexed-colored-vector 3 3)
   #\Q
(test-indexed-colored-vector 3 4)
   #\Q
(test-indexed-colored-vector 3 5)
   #\Q
(test-indexed-colored-vector 4 1)
   #\Q
(test-indexed-colored-vector 4 2)
   #\Q
(test-indexed-colored-vector 4 3)
   #\Q
(test-indexed-colored-vector 4 4)
   #\Q
(test-indexed-colored-vector 4 5)
   #\Q
(test-indexed-colored-vector 5 1)
   #\Q
(test-indexed-colored-vector 5 2)
   #\Q
(test-indexed-colored-vector 5 3)
   #\Q
(test-indexed-colored-vector 5 4)
   #\Q
(test-indexed-colored-vector 5 5)
   #\Q

" *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
                      All newapi.tst tests successfully run!
 *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***"
   ---

;;; end of dmeroon.tst
