;;; $Id: site.tst,v 1.9 1997/09/02 18:53:20 queinnec Exp $

;;; Check all the objects that are reachable through the site to be
;;; initialized. Ignore secret fields (this allows to skip Entries and
;;; Exits hashtables).

;;; Assume TOP/DMeroon/dmlib.scm file to be loaded.

(define (check-all o)
  (define all (list)) ; list of all checked objects
  (define (is-field-secret? field)
    (let ((options (DM-field-value field DM-Field-options-field)))
      (>= (modulo options 32) 16) ) )
  (call/cc
   (lambda (exit)
     (define (check o path)
       (if (and o (not (memq o all)))
           (let ((path (cons o path)))
             (define (check-item o field itemtyp)
               (if (and (eq? itemtyp DM-reference-type)
                        (not (is-field-secret? field)) )
                   (check (DM-field-value o field) path)
                   #f ) )
             (define (check-length o field reptyp itemtyp)
               #f )
             (define (check-indexed-item o field itemtyp j)
               (if (and (eq? itemtyp DM-reference-type)
                        (not (is-field-secret? field)) )
                   (check (DM-indexed-field-value o field j) path)
                   #f ) )
             (set! all (cons o all))
             (display `(checking ,path))(newline)
             ;; Check the class
             (check (DM-class-of o) path)
             ;; Check the object itself
             (if (DM-is-initialized? o)
                 (DM-iterate o check-item check-length check-indexed-item)
                 (exit `(--- not initialized: ,path)) ) ) ) )
     (check o (list))
     #f ) ) )
   ---

(and (check-all DM-site)(check-all DM-site))
   #f

;;; Testing Dictionaries. At the beginning the "user" dictionary is empty.

;;; Level 1
;;; When compiled, this function is no longer accessible, so I redefine it.
(define (DMinternal-mk-dict rest value name)
  (let* ((namelen (string-length name))
         (d (DM-allocate DM-Dictionary-class (+ 1 namelen))) )
    (DM-set-field-value! d DM-Dictionary-rest-field rest)
    (DM-set-field-value! d DM-Dictionary-value-field value)
    (do ((i 0 (+ i 1)))
        ((>= i namelen))
      (DM-set-indexed-field-value! 
       d DM-Dictionary-name-field i (string-ref name i) ) )
    (DM-set-initialized! d) ) )
   ---
(DM-publish! (DMinternal-mk-dict #f DM-site "site") "user")
   ---
(DM-is-a? (DM-retrieve "user") DM-Dictionary-class)
   #t
(eq? DM-site (DM-retrieve "user" "site"))
   #t
;;; test that we address the "user" dictionary be default.
(DM-is-a? (DM-retrieve) DM-Dictionary-class)
   #t
(eq? DM-site (DM-retrieve "site"))
   #t
;;; Level 2 (with creation of a subdictionary of "user")
(DM-publish! DM-true "user" "site")
   ---
(eq? DM-true (DM-retrieve "user" "site"))
   #t
(DM-publish! DM-false "site")
   ---
(eq? DM-false (DM-retrieve "site"))
   #t
;;; Level 3 (creation of a sibling subdictionary of "user")
(DM-publish! DM-nil "user" "foo")
   ---
(eq? DM-false (DM-retrieve "user" "site"))
   #t
(eq? DM-false (DM-retrieve "site"))
   #t
(eq? DM-nil (DM-retrieve "user" "foo"))
   #t
(eq? DM-nil (DM-retrieve "foo"))
   #t
;;; Level 4, create a dictionary at a deeper level
(DM-publish! DM-true "user" "a" "b" "c")
   ---
(eq? DM-true (DM-retrieve "user" "a" "b" "c"))
   #t


" *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
                      All site.tst tests successfully run!
 *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***"
   ---

;;; end of site.tst
