;;; $Id: vampyre.tst,v 1.10 1997/08/15 19:29:23 queinnec Exp $

;;; This file implements the vampyre behavior. A vampyre site is a
;;; site that, whenever it hears about another site, pulls everything
;;; it can out of that site and cache locally all its published
;;; objects. This is the contrary of the cicada behavior (so 'vampyre'
;;; may have been named 'ant' ?)  which tries asap to get rid of the
;;; objects it owns.

;;; Assume TOP/DMeroon/dmlib.scm file to be loaded.

;;; Check if bit at rank (counted from right end) is set or not.
;;; These are Bigloo functions.

(define (bit-option option rank)
  (let* ((m (bit-rsh option rank))
         (b (bit-and m 1)) )
    (= b 1) ) )
   ---
;;; Test: 12 is #b110 (Bigloo does not recognize this syntax).
;;;   (eq? #f (bit-option 6 0))
;;;   (eq? #t (bit-option 6 1))
;;;   (eq? #t (bit-option 6 2))

;;; NOTE for test only: do not recurse on local fields.
;;; Secrete fields are already ignored by DM-iterate.

(define (vamp o)
  (define all (list DM-site)) ; list of all pulled objects
  (define (is-field-local? field)
    (bit-option (DM-field-value field DM-Field-options-field) 2) )
  (define (check-item o field itemtyp)
    (if (and (eq? itemtyp DM-reference-type)
             (not (is-field-local? field)) )
        (fetch (DM-field-value o field)) ) )
  (define (check-length o field reptyp itemtyp) #f)
  (define (check-indexed-item o field itemtyp j)
    (if (and (eq? itemtyp DM-reference-type)
             (not (is-field-local? field)) )
        (fetch (DM-indexed-field-value o field j)) ) )
  (define (fetch o)
    (if (and o (not (memq o all)))
        (begin
          (display `(--------- vamp ,o))(newline) ; DEBUG
          (set! all (cons o all))
          (DM-iterate o check-item check-length check-indexed-item) ) ) )
  (fetch o)
  #t )
   ---
(vamp OTHER)
   #t
"Once again please!"
   ---
(vamp OTHER)
   #t

;;; Read OTHER's clock
(DM-describe (DM-field-value other DM-Site-clock-field))
   ---
(DM-describe (DM-field-value other DM-Site-clock-field))
   ---

;;; Read OTHER's options (this is the first volatile field of a site).
(DM-field-value other DM-Site-options-field)
   ---

;;; Read OTHER. This is different from (vamp OTHER) which walks only
;;; fields of type reference), DM-describe reads all fields (except
;;; secret fields of course).
(DM-describe OTHER)
   ---
;;; Describe (that is, re-read) the whole OTHER site.
(DM-describe-all OTHER)
   ---


" *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
                      All vampyre.tst tests successfully run!
 *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** ***"
   ---

;;; end of vampyre.tst
