/* $Id: dmcomm.h,v 1.55 1998/06/10 16:31:28 queinnec Exp $ 
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 */

#ifndef DM_COMM_H
#define DM_COMM_H

extern DM_Clock_wrapped DM_general_clock_object;

extern void
DM_increment_counter (DM_Context  *context,
                      DM_External *entry);

extern void
DM_decrement_counter (DM_Context *context,
                      DM_Entry   *entry);

/* This constant defines the size of the initial hashtables that are
 * allocated to hold Entry, Exit and other things.
 */

#define DM_HASHTABLE_INITIAL_SIZE 16

extern void
DM_propagate_information (DM_Context *context,
                          DM_OutBuffer *outbuffer);

/*
 * ENtry management
 */
extern DM_Entry*
DM_semi_externalize (DM_Context *context,
                     DM_Object  *o);

extern DM_Entry*
DM_externalize (DM_Context *context,
                DM_Object *o);

extern DM_Entry*
DM_extract_entry (DM_Context *context,
                  DM_key     number);

extern void
DM_remove_entry (DM_Context *context,
                 DM_External *external);

extern void
DM_register_entry (DM_Context *context,
                   DM_External *external);

/*
 * Exit management
 */

extern void
DM_remove_exit (DM_Context *context,
                DM_Exit *exit);

extern DM_Exit*
DM_extract_exit (DM_Context *context,
                 DM_key     number );

extern DM_Object*
DM_register_exit (DM_Context *context,
                  DM_Exit *exit);

/*
 * Site management
 */

extern void
DM_register_site (DM_Context *context,
                  DM_Site *site);

extern void
DM_remove_site (DM_Context *context,
                DM_Site *site);

extern DM_Site*
DM_extract_site (DM_Context *context,
                 DM_nat3 ip,
                 DM_nat2 port );

/*
 * Reifying values
 */

extern DM_Value*
DM_reify_value (DM_Context *context,
                DM_DMeroonType *type,
                void           *p );

extern DM_Value*
DM_reify_value_from_field (DM_Context *context,
                           DM_Object *o,
                           DM_Field  *field );

extern DM_Value*
DM_reify_value_from_indexed_field (DM_Context *context,
                                   DM_Object *o,
                                   DM_Field  *field,
                                   DM_nat3   index );

extern DM_Value*
DM_patch_from_reified_value (DM_Context *context,
                             DM_Object *o,
                             DM_Field  *field,
                             DM_Value  *value );

extern DM_Value*
DM_indexed_patch_from_reified_value (DM_Context *context,
                                     DM_Object *o,
                                     DM_Field  *field,
                                     DM_nat3   index,
                                     DM_Value  *value );

/*
 * Encoding/decoding 'data' field.
 */

extern void
DM_receive_raw_field (DM_Context *context,
                      DM_DMeroonType *type,
                      void           *p,
                      DM_InBuffer    *reception );

extern void
DM_send_raw_field (DM_Context *context,
                   DM_DMeroonType *type,
                   void           *p,
                   DM_OutBuffer   *outbuffer );

extern void DM_comm_initialize (DM_Context *context);

/* 
 * Two macros to encapsulate the use of Buffers.
 */

#if defined(DM_DEBUG) && (DM_DEBUG>9)

#define SendThing(context,emission, number_of_bytes, body) \
{ while ( emission->next ) { \
   emission = (DM_OutBuffer*)(emission->next); \
  } \
  if ( !(DM_CanSendBytes(context, emission, number_of_bytes)) ) { \
    emission = DM_ensure_large_enough_outbuffer( \
                                  context, \
                                  emission, \
                                  number_of_bytes ); \
  } \
  fprintf(DM_FindErrorStream(context), "{"); \
  body; \
  fprintf(DM_FindErrorStream(context), "}"); \
}

#define ReceiveThing(context,reception, number_of_bytes, body) \
  if ( DM_CanReceiveBytes(context,reception, number_of_bytes) ) { \
    fprintf(DM_FindErrorStream(context), "{"); \
    body; \
    fprintf(DM_FindErrorStream(context), "}"); \
  } else { \
    DM_Abort(context, DME_UNMARSHALING_ERROR); \
  }

#else

#define SendThing(context,emission,number_of_bytes,body) \
{ while ( emission->next ) { \
   emission = (DM_OutBuffer*)(emission->next); \
  } \
  if ( !(DM_CanSendBytes(context, emission, number_of_bytes)) ) { \
    emission = DM_ensure_large_enough_outbuffer( \
                                  context, \
                                  emission, \
                                  number_of_bytes ); \
  } \
  body; \
}

#define ReceiveThing(context,reception,number_of_bytes,body) \
  if ( DM_CanReceiveBytes(context, reception, number_of_bytes) ) { \
    body; \
  } else { \
    DM_Abort(context, DME_UNMARSHALING_ERROR); \
  }

#endif /* defined(DM_DEBUG) && (DM_DEBUG>9) */

/******************************************************************
 *                     Macros to read/write on tcp
 *
 * DM_CanSendBytes returns a C boolean telling whether one can add
 * bytes to a buffer without flushing it. This is a non-blocking test.
 *
 * DM_CanReceiveBytes tells if some bytes can be read from a buffer
 * without filling it. This is a non-blocking test.
 *
 * Once a buffer is known to have a correct size, it can be read/written
 * safely without rechecking its size.
 *
 * CAUTION: Pay attention to the arguments of these macros which are
 *          multiply evaluated.
 */

#define DM_CanSendBytes(context,buffer,bytes) \
  ( (((buffer)->index)+(bytes)) <= ((buffer)->byte_length) )

#define DM_CanReceiveBytes(context,buffer,bytes) \
  ( (((buffer)->index)+(bytes)) <= ((buffer)->maximum) )

#if defined(DM_DEBUG) && (DM_DEBUG>9)

#define DM_SendOneByte(context,buffer,nat1) \
  ( (buffer)->byte[(buffer)->index] = (nat1), \
    fprintf(DM_FindErrorStream(context), "[%#hx]", \
            buffer->byte[(buffer)->index++]) )

#define DM_ReceiveOneByte(context,buffer) \
  ( fprintf(DM_FindErrorStream(context), "[%#hx]", \
            buffer->byte[(buffer)->index]), \
    (buffer)->byte[((buffer)->index)++] )

#else

#define DM_SendOneByte(context,buffer,nat1) \
  ( (buffer)->byte[((buffer)->index)++] = (nat1) )

#define DM_ReceiveOneByte(context,buffer) \
  ( (buffer)->byte[((buffer)->index)++] )

#endif /* defined(DM_DEBUG) && (DM_DEBUG>9) */

/******************************************************************
 *               Special exportations
 *
 * They are there since now DM_Connection is known.
 * They should be really in dmcomm.h instead.
 */

#define DM_ExternalizeMethods(name) \
  extern void \
    DM_##name##_print (DM_Context *context, void *p, void *output_port); \
  extern void \
    DM_send_##name (DM_Context *context, DM_OutBuffer *emission, void *p); \
  extern void \
    DM_receive_##name (DM_Context *context, DM_InBuffer *reception, void *p)

DM_ExternalizeMethods(nought);
DM_ExternalizeMethods(nat1);
DM_ExternalizeMethods(nat2);
DM_ExternalizeMethods(nat3);
DM_ExternalizeMethods(nat4);
DM_ExternalizeMethods(reference);
DM_ExternalizeMethods(data);
DM_ExternalizeMethods(code);
DM_ExternalizeMethods(int1);
DM_ExternalizeMethods(int2);
DM_ExternalizeMethods(int3);
DM_ExternalizeMethods(int4);
DM_ExternalizeMethods(char1);
DM_ExternalizeMethods(netnat2);
DM_ExternalizeMethods(netnat3);
DM_ExternalizeMethods(unicode);
DM_ExternalizeMethods(float1);
DM_ExternalizeMethods(float2);
DM_ExternalizeMethods(float3);
DM_ExternalizeMethods(offset);
DM_ExternalizeMethods(date);
DM_ExternalizeMethods(key);
DM_ExternalizeMethods(class_options);
DM_ExternalizeMethods(field_options);
DM_ExternalizeMethods(port_options);
DM_ExternalizeMethods(site_options);
DM_ExternalizeMethods(external_options);
DM_ExternalizeMethods(context_options);
DM_ExternalizeMethods(file_options);
DM_ExternalizeMethods(request_options);
DM_ExternalizeMethods(handler_code);

#define DM_ExternalizeMethodsForC(name) \
  extern void \
    DM_##name##_print (DM_Context *context, void *p, void *output_port)

DM_ExternalizeMethodsForC(C_char);
DM_ExternalizeMethodsForC(C_signed_char);
DM_ExternalizeMethodsForC(C_short_int);
DM_ExternalizeMethodsForC(C_int);
DM_ExternalizeMethodsForC(C_long_int);
DM_ExternalizeMethodsForC(C_unsigned_char);
DM_ExternalizeMethodsForC(C_unsigned_short_int);
DM_ExternalizeMethodsForC(C_unsigned_int);
DM_ExternalizeMethodsForC(C_unsigned_long_int);
DM_ExternalizeMethodsForC(C_float);
DM_ExternalizeMethodsForC(C_double);
DM_ExternalizeMethodsForC(C_long_double);
DM_ExternalizeMethodsForC(C_void_star);
DM_ExternalizeMethodsForC(C_function_star);
#ifdef DM_HAS_LONG_LONG_ENTITIES
DM_ExternalizeMethodsForC(C_unsigned_long_long_int);
DM_ExternalizeMethodsForC(C_long_long_int);
#endif

#define DM_send_reference 	DM_send_nought
#define DM_receive_reference 	DM_receive_nought
#define DM_send_data 		DM_send_nought
#define DM_receive_data 	DM_receive_nought
#define DM_send_code 		DM_send_nought
#define DM_receive_code 	DM_receive_nought

#define DM_send_char1 		DM_send_nat1
#define DM_receive_char1 	DM_receive_nat1

#define DM_send_unicode 	DM_send_nought
#define DM_receive_unicode 	DM_receive_nought

#define DM_send_date	 	DM_send_nat3
#define DM_receive_date 	DM_receive_nat3

#define DM_send_class_options 	DM_send_nat3
#define DM_receive_class_options DM_receive_nat3

#define DM_send_field_options 	DM_send_nat3
#define DM_receive_field_options DM_receive_nat3

#define DM_send_port_options 	DM_send_nat3
#define DM_receive_port_options DM_receive_nat3

#define DM_send_site_options 	DM_send_nat3
#define DM_receive_site_options DM_receive_nat3

#define DM_send_external_options    DM_send_nat3
#define DM_receive_external_options DM_receive_nat3

#define DM_send_context_options    DM_send_nat3
#define DM_receive_context_options DM_receive_nat3

#define DM_send_file_options    DM_send_nat3
#define DM_receive_file_options DM_receive_nat3

#define DM_send_request_options    DM_send_nat3
#define DM_receive_request_options DM_receive_nat3

#define DM_send_handler_code 		DM_send_handler_code
#define DM_receive_handler_code 	DM_receive_handler_code

typedef struct DM_handler_code_descriptor {
  DM_nat1         code;
  DM_handler_code handler;
  char*           name;
} DM_handler_code_descriptor;

extern DM_handler_code_descriptor DM_handler_code_table[20];

#endif /*  DM_COMM_H */

/* end of dmcomm.h */
