/* $Id: dmeroon.h,v 1.34 1998/05/06 20:03:11 queinnec Exp $
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 */

#ifndef DM_DMEROON_H
#define DM_DMEROON_H

/* This file concentrates all the partial .h files into a single
 * multipurpose .h file that specifies the correct order of
 * inclusions.

 * NOTES:
 * (A) Every DMeroon specific thing is normally prefixed with DM_.

 * (B) Functions are always written in lower cases with _ between words:
 *     DM_index_to_type, DM_remove_entry.

 * (C) Macros are spelled with mixed case without spaces:
 *     DM_IsPortPeerless,  DM_TypeToIndex, DM_SetIndexOfType.

 * (D) Classes are capitalized as well as the associated C structure
 *     without intervening spaces:
 *     Type, DMeroonType,            DM_Type, DM_DMeroonType, 
 *     DM_Type_class_object  DM_Type_name_field_object
 
 * (E) Statically allocated DMeroon objects have a C name ending with
 *     _object. The penultimate word is often the type.

 */

/******************************************************************
 * This is the general type of a DMeroon object without its header.
 * It does not contain anything. There exist some macros to convert a
 * DMeroon_object* into a DM_Object* forth and back: these are
 * DM_FromSupport and DM_ToSupport. There are also macros DM_FromC
 * and DM_ToC to convert a structure allocated in C into a DMeroon
 * reference.
 *
 * NOTE1: This type is defined here so it can be used by DMeroon_SUPPORT.
 *
 * NOTE2: the _void field is only there since it seems necessary for C
 *        to have at least one field in a struct.
 */

typedef struct DM_Object {
  char _void;
} DM_Object;

/*
 * Import what depends on the operating system. This file tests the 
 * environment of the C compiler to deduce the type of the OS. 
 */

#include "dmos.h"

/*
 * Import the dependences relative to the C compiler ie the machine on
 * which DMeroon runs (this states what are the types used to implement
 * DMeroon types. Part of this mapping is dynamic and is used to confirm
 * if the static choices (needed to compile the C code) were correct.
 */

#ifndef DMeroon_SIZES
#error "Problem: No supporting machine for DMeroon."
#else 
#include DMeroon_SIZES
#endif /* DMeroon_SIZES */

/*
 * The DMeroon_SUPPORT macro specifies the support language.
 * Currently available: dmbgl.h for Bigloo, dmxla.h for Icslas, 
 * dmc.h for C, dmoscheme.h for OScheme ... Others to come!
 */

#ifndef DMeroon_SUPPORT
#error "Problem: No supporting language for DMeroon."
#else
#include DMeroon_SUPPORT
#endif /* DMeroon_SUPPORT */

/*
 * Include now all of dm*.h files (in the right order). 
 * None of these files must include dmeroon.h to void cycles.
 *
 * dm.h is where are gathered all internal macros used in the others.
 */

#include "dm.h"

#include "dmid.h"
#include "dmerr.h"
#include "dmstruct.h"
#include "dmtypes.h"
#include "dmdbg.h"
#include "dmkern.h"
#include "dmhash.h"
#include "dmprint.h"
#include "dmnet.h"
#include "dmzlib.h"
#include "dmcomm.h"
#include "dmextr.h"
#include "dmlib.h"
#include "dmprtcl.h"
#include "dmsite.h"
#include "dmrdobj.h"
#include "dmwrobj.h"
#include "dmcache.h"
#include "dmreq.h"
#include "dmsched.h"
#include "dmreqst.h"
#include "dmhtml.h"
#include "dmact.h"
#include "dmnewapi.h"
#include "dmgif.h"

/******************************************************************
 * In the dmxla.h header for example, some macros require the previous
 * files to be present. As a hack, we reinclude it with a macro to
 * conditionalize it. So dmxla.h is divided into two parts: -- one
 * which parameterizes the other dm*.h files, -- the other which
 * depends on these dm*.h files. This is also the case of the dmc.h
 * file.
 */

#define DMeroon_SUPPORT_INCLUDED_AGAIN 1

#include DMeroon_SUPPORT

#endif /* DM_DMEROON_H */

/* end of dmeroon.h */
