/* $Id: dmerr.h,v 1.57 1998/06/10 16:31:28 queinnec Exp $ 
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 */

/* DOCUMENTATION dmerr.h
 * This file defines all the error codes of DMeroon. Codes are gathered
 * as series.
 *     ??  kernel or interface errors,
 *    1??  
 *    200  timeout,
 *    3??  
 *    4??  
 *    5??  system call errors,
 *   15??  support language generic problems,
 *  13???  password problems,
 *  16???  DMeroon internal problems,
 *  169??  zlib problems,
 *  17???  URLscript problems,
 *  18???  Bigloo problems,
 *  20???  Icslas problems,
 *  32???  Facts.

 * Error codes are defined as nat3 numbers.
 */

#ifndef DM_DMERR_H
#define DM_DMERR_H

typedef DM_nat3 DM_ErrorCode;

/* General error codes among which the no-error code. */

#define DME_SUCCESS			0	/*
 Means that no error occurred. It should be seen often.

 Category: general

 */
#define DME_NOT_YET_IMPLEMENTED		2	/*
 Means that you called some unimplemented part of DMeroon.

 Category: general

 Solution: If you really need this code, ask me! If you badly need it,
 write it (and send it to me)!
 */
#define DME_INTERNAL_ERROR		3	/*
 Means that some internal inconsistency was remarked.

 Category: general
 */
#define DME_PERIOD_IS_ZERO		4	/*
 The period of the server is zero so it cannot serve anything useful.
 Check the value of the DMEROON_SERVING_PORT environment variable or
 the DM_DMEROON_FAVORITE_PERIOD cpp variable.
 
 Category: general
 */
#define DME_MEMORY_EXHAUSTED		5	/*
 Memory is exhausted, last allocation failed. Use a real GC or lessen
 your memory consumption.

 Category: general
 */

/* Codes returned when a value is given whose type is not the expected
 * one. The two first ones are generic and are returned when the
 * expected type cannot be statically known. */

#define DME_NOT_DMEROON_INSTANCE	9	/*
 Appears when calling an API function with a value not being a DMeroon
 object at all. 

 Category: api

 Solution: Fix your code. The file and line number should tell you which
 api function was invoked.
 */
#define DME_WRONG_INSTANCE		10	/*
 Appears when calling an API function with a DMeroon object not of the
 right class.

 category: api

 Solution: Fix your code. The file and line number should tell you which
 api function was invoked.
 */
#define DME_NOT_TYPE			11	/*
 Appears when calling an API function with a DMeroon object different from 
 a Type.

 category: api

 Solution: Fix your code. The file and line number should tell you which
 api function was invoked.
 */
#define DME_NOT_CONTEXT         	12	/*
 Appears when calling an API function with a DMeroon object different from 
 a Context.

 category: api

 Solution: Fix your code. The file and line number should tell you which
 api function was invoked.
 */
#define DME_NOT_CLASS           	13	/*
 Appears when calling an API function with a DMeroon object different from 
 a Class.

 category: api

 Solution: Fix your code. The file and line number should tell you which
 api function was invoked.
 */
#define DME_NOT_FIELD           	15	/*
 Appears when calling an API function with a DMeroon object different from 
 a Field.

 category: api

 Solution: Fix your code. The file and line number should tell you which
 api function was invoked.
 */
#define DME_NOT_SITE            	16	/*
 Appears when calling an API function with a DMeroon object different from 
 a Site.

 category: api

 Solution: Fix your code. The file and line number should tell you which
 api function was invoked.
 */
#define DME_NOT_CLOCK           	17	/*
 Appears when calling an API function with a DMeroon object different from 
 a Clock.

 category: api

 Solution: Fix your code. The file and line number should tell you which
 api function was invoked.
 */
#define DME_NOT_SYMBOL			18	/*
 The Symbol unmarshaling command was followed by a DMeroon object whose 
 class is not a Symbol. 

 category: marshaling internal

 Solution: Fix your code if you have your own marshaling routines. 
 */
#define DME_NOT_CONNECTION		19	/*
 A Port calls an operation while not being a Connection (a subclass of Port).

 category: internal 

 Solution: ask me.
 */
#define DME_NOT_QUEUELINK		21	/*
 I got an alien value where I expected a QueueLink.

 category: internal

 Solution: ask me.
 */
#define DME_BAD_CLASS			22	/*
 You try to store a value of a bad class in a constrained reference field.

 category: api set-field-value

 Solution: check your code.
 */

/* Codes returned when a system call was wrong. */

#define DME_FAILED_GETHOSTNAME		500	/*
 A call to gethostname failed.

 category: os internal

 Solution: look at your man page.
 */
#define DME_FAILED_GETHOSTBYNAME	501	/*
 A call to gethostbyname failed.

 category: os internal

 Solution: look at your man page.
 */
#define DME_HOST_NAME_CHOICE		502
#define DME_UNKNOWN_AF_TYPE		503
#define DME_FAILED_SOCKET		504
#define DME_FAILED_CONNECT		505
#define DME_FAILED_SETSOCKOPT		506
#define DME_FAILED_BIND			507
#define DME_FAILED_LISTEN		508
#define DME_FAILED_ACCEPT		509
#define DME_FAILED_CLOSE		510
#define DME_FAILED_READ			511
/*                                   no 512 since it is a multiple of 256. */
#define DME_FAILED_OPEN			513
#define DME_FAILED_SETJMP		514
#define DME_FAILED_SELECT		515
#define DME_FAILED_SIGPIPE_HANDLER	516
#define DME_FAILED_WRITE		517
#define DME_FAILED_LONGJMP		518

#define DME_TOO_MUCH_SIZES		50
#define DME_TOO_LESS_SIZES		51
#define DME_INCORRECT_NUMBER_OF_SIZES	52
#define DME_TOO_BIG_NAT1_SIZE		53
#define DME_TOO_BIG_NAT2_SIZE		54
#define DME_NOT_UNINSTANTIABLE		55
#define DME_NEGATIVE_SIZE		56
#define DM_SUPPORT_FIXNUM_MAXIMUM_PROBLEM	60
#define DME_BAD_INITIALIZATION_STATE	61
#define DME_PROBLEM_WITH_PROTECT	62

#define DME_WRONG_COMMAND		90
#define DME_MUTABLE_CLASS_WITHOUT_CLOCK	91

#define DME_BAD_PORT            	100
#define DME_SERVER_KO           	101
/* No IP number can be found for that site. */
#define DME_UNKNOWN_SITE		120
/* Cannot find a Connection towards this site (internal error). */
#define DME_UNREACHABLE_SITE		121
/* Cannot open a Connection towards this site. Probably there is no
 * DMeroon server on that remote machine. */
#define DME_UNCONNECTABLE_SITE		122
/* Something went wrong when establishing the DMeroon protocol
 * connection with a machine. Maybe a timeout due to the remote-ness
 * of the machine or a version incompatibility ? */
#define DME_UNHANDSHAKABLE_SITE		123

#define DME_CONNECTION_ALREADY_BOOTED	124
#define DME_BAD_SITE			128
#define DME_INCOMPATIBLE_SITENAME_LENGTH 129
#define DME_INCOMPATIBLE_SITE_DATES	130
#define DME_INCOMPATIBLE_KEY		131
#define DME_NOT_A_SITE			133
#define DME_INCOMPATIBLE_SITE		134
#define DME_CANNOT_FILL			135
#define DME_NOT_AN_EXIT			136
#define DME_UNFINISHED_MARK		137
#define DME_NOT_SUBCLASSABLE		138
#define DME_NOT_A_CLASS			139
#define DME_SITE_WITHOUT_ROUTE		140
#define DME_NOT_REGULAR_OBJECT		141	/*
 The object is an Entry or an Exit item whereas it was expected to be a 
 regular object.

 category: marshal

 Solutions: ask me.
 */

/* A request got no answer for the duration specified in the Context. */
#define DME_TIMEOUT			200

#define DME_NO_SUCH_SITE	       	308
#define DME_NO_SUCH_TYPE	       	309
#define DME_NO_SUCH_CLASS       	310
#define DME_SECRET_FIELD		311
#define DME_REMOTE_OBJECT		312
#define DME_NO_SUCH_FIELD       	313
#define DME_NOT_REGULAR_FIELD   	314
#define DME_BAD_INDEX           	315
#define DME_NOT_INDEXED_FIELD   	316
#define DME_BAD_REFERENCE       	317
#define DME_ALREADY_EXTERNALIZED	318
#define DME_ALREADY_COPYABLE		319
#define DME_ALREADY_SHARABLE		320
#define DME_NOT_MOTILE          	321
#define DME_NOT_MUTABLE         	322
#define DME_BAD_PROXY			323
#define DME_NOT_INITIALIZED		324
#define DME_NOT_NEIGHBOR_CLOCK		325
#define DME_BAD_CLASS_OPTIONS		328
#define DME_BAD_FIELD_OPTIONS		329
/* Repetition types may only be nought, nat1-4. */
#define DME_NOT_REPETITION_TYPE		330
#define DME_BAD_LEVEL			331
/* The field to mutate is not mutable. */
#define DME_NOT_MUTABLE_FIELD		334
/* Type and field are incompatible. */
#define DME_TYPE_MISMATCH		335

#define DME_NO_FILE_WAITER		336
#define DME_NO_TIME_WAITER		337
#define DME_NO_ANSWER_WAITER		338
#define DME_NO_DURATION_WAITER		339
#define DME_MISSING_WAITER		340
#define DME_BASIC_LOOP_RESTARTED	341

#define DME_THROW_OBJECT		401

#define DME_FLOAT3_NOT_SUPPORTED	402

#define DME_UNKNOWN_HANDLER_CODE	403

/* 
 * Errors coming from the Support language.
 */

#define DME_BAD_SUPPORT_REPRESENTATION	1500
#define DME_NOT_SUPPORT_STRING		1501
#define DME_NOT_SUPPORT_FIXNUM		1502
#define DME_NOT_NAT1_FIXNUM		1503
#define DME_NOT_NAT2_FIXNUM		1504
#define DME_NOT_NAT3_FIXNUM		1505
#define DME_NOT_NAT4_FIXNUM		1506
#define DME_NOT_INT1_FIXNUM		1507
#define DME_NOT_INT2_FIXNUM		1508
#define DME_NOT_INT3_FIXNUM		1509
#define DME_NOT_INT4_FIXNUM		1510
#define DME_UNKNOWN_TYPE		1511
#define DME_NOT_NETNAT2_FIXNUM		1512
#define DME_NOT_NETNAT3_FIXNUM		1513
#define DME_NOT_SUPPORT_CHARACTER	1514
#define DME_NOT_SUPPORT_FLOAT1		1515
#define DME_NOT_SUPPORT_FLOAT2		1516
#define DME_NOT_SUPPORT_FLOAT3		1517
#define DME_NOT_HANDLABLE_NAT4_FIXNUM	1520
#define DME_NOT_NUMBER			1521

/* 
 * Internal errors that is error codes that are not directly related
 * to the use of the API (though you may see them).
 */

#define DME_ADDITIONAL_PASSWORD_TOO_BIG 13001
#define DME_DMEROON_PASSWORD_TOO_BIG    13002
#define DME_PROTOCOL_NAME_TOO_BIG       13003

#define DME_DEAD_CODE_REACHED		16498
#define DME_NON_DEBUG_MODE		16499
/* An hashtable is full. */
#define DME_HASHTABLE_OVERFLOW		16500
/* Try to remove an absent object from a Hashtable. */
#define DME_NOT_IN_HASHTABLE		16501
/* Too large index for cache. */
#define DME_OUT_OF_CACHE		16550
#define DME_STACK_UNDERFLOW		16551
/* Cannot start to encode a message, the previous one is not finished. */
#define DME_MESSAGE_UNFINISHED		16600
/* Cannot finish a message, there is no current one being processed. */
#define DME_MESSAGE_UNBEGUN		16601
/* Unknown class of request. */
#define DME_UNKNOWN_REQUEST		16602
#define DME_UNKNOWN_ANSWER		16603
#define DME_UNEXPECTED_ANSWER		16604
#define DME_NO_ASSOCIATED_OUTBUFFER	16605
#define DME_INCORRECT_MESSAGE_SIZE	16606
#define DME_UNUSED_PROTOCOL_SUBCODE	16607
#define DME_NOT_AN_LF			16608
#define DME_OBSOLETE_SITE		16610
#define DME_MULTIPLE_WAIT		16613
#define DME_NOT_REQUEST_NOR_ANSWER	16620

#define DME_DISSIMILAR_CLASSES		16649
#define DME_DISSIMILAR_SKELETONS	16650
#define DME_SHOULD_BE_REMOTE		16651
#define DME_WEIRD_SITUATION		16652
#define DME_CANNOT_SETENV		16653
#define DME_CANNOT_CREATE_SERVER	16654
#define DME_INCORRECT_INDEX_TYPE	16656
#define DME_TOO_MUCH_INDEXES		16658
#define DME_WEIRD_DATE_FOR_SITE		16659
#define DME_MISSING_BYTES		16660
#define DME_LOST_ENTRY			16661
#define DME_UNUSABLE_CLASS		16662
#define DME_REMOTE_CLASS		16663
#define DME_TOO_LARGE_INDEX		16664
#define DME_ALREADY_SENT_DECREMENT	16665

/* INADDR_NONE is network-dependent. */
#define DME_INADDR_NONE_DEPENDENCY	16680
/* INADDR_ANY is network-dependent. */
#define DME_INADDR_ANY_DEPENDENCY	16681
/* FD_SETSIZE less than 3! */
#define DME_FD_SETSIZE_DEPENDENCY	16682

#define DME_NOT_MOST_STRINGENT_TYPE	16683	/*
 Means that the definition of the DM_MOST_STRINGENT_TYPE is incorrect.

 Category: check

 Solutions: Print the definitions of all types (with  DM_print_types(0,0)),
 select the type with the highest alignment constraint, define appropriately 
 the DM_MOST_STRINGENT_TYPE and DM_MOST_STRINGENT_TYPE_OBJECT cpp variables, 
 recompile everything.
 */

#define DME_INCORRECT_ALIGNMENT_LOG	16684	/*
 Means that the definition of the DM_MOST_STRINGENT_ALIGNMENT_LOG is incorrect.

 Category: check

 Solutions: Print the definitions of all types (with DM_print_types(0,0)),
 select the type with highest alignment constraint, define appropriately the
 DM_MOST_STRINGENT_ALIGNMENT_LOG to be the logarithm (in base 2) of that
 number and recompile everything. For a machine aligning float2 on 8 bytes
 boundaries, the log should be 3.
 */

#define DME_UNMARSHALING_ERROR		16686
#define DME_ALIGNMENT_ERROR		16687
#define DME_INCORRECT_CACHE_ARGUMENTS	16688	/*
 Error using the CACHE marshaling command. 

 Category: marshal

 Solutions: See code and don't violate the assertions of CACHE.
 */
#define DME_INCORRECT_CLASSES		16689	/*
 Error when associating an Exit item and an object with non-matching classes.

 Category: marshal

 Solutions: See code and don't violate the assertions of CACHE.
 */

/* Problem with allocation. */
#define DME_STATIC_ALLOCATION_PROBLEM	16799	/*
 Error when statically allocating an object containing an indexed field.

 Category: internal macro

 Solutions: Use a dynamic allocation, see DM_Allocate* macros.
 */
#define DME_ALLOCATION_PROBLEM		16800
/* Problem with implementation. */
#define DME_PRINT_CHAR_STAR_PROBLEM	16801

#define DME_USERNAME_PROBLEM		16802
#define DME_TOO_LONG_USERNAME		16803
#define DME_BASIC_CLASS_MUTABLE		16804
#define DME_NOT_DIRECT_INSTANCE		16805
#define DME_INCORRECT_0X		16806
#define DME_UNPRINTABLE_TYPE		16807
/* DM_SupportAddress cannot hold a DM_reference. */
#define DME_INCORRECT_SUPPORTADDRESS	16808
/* Prefix alignment problem. */
#define DME_PREFIX_ALIGNMENT_PROBLEM	16809
#define DME_TOC_FROMC_PROBLEM		16810
/* (To/From)Support macros conflict */
#define DME_TO_FROM_PROBLEM		16811
/* Incorrect DM_IsObject macro" */
#define DME_IS_OBJECT_PROBLEM		16812
/* DM_allocate returns a non-DMeroon value */
#define DME_ALLOCATE_PROBLEM		16813
#define DME_CANNOT_PRINT_NOUGHT		16814

#define DME_BAD_ACCESS			16816
#define DME_BAD_PATCH			16817
#define DME_UNSENDABLE_TYPE		16818
#define DME_UNRECEIVABLE_TYPE		16819
#define DME_FLOAT_DOMAIN_ERROR		16820
#define DME_REMOTE_CLOCK		16821
#define DME_BAD_REPETITION_TYPE		16822
#define DME_TOO_BIG_INDEX		16823
#define DME_NOT_ENOUGH_PREDEFINED_CLASSES	16824
#define DME_STATIC_SIZE_MISFIT		16825
/* The sockaddr_in must be perfectly mimicked by DMeroon. */
#define DME_INCORRECT_SOCKADDRIN_SIZE	16827
#define DME_INCORRECT_SOCKADDRIN_STRUCT	16828
#define DME_CANNOT_RECEIVE		16829	/*
 Means that you call DMeroon_receive while you have an active handler to
 Handle all incoming objects. 

 Category: DMeroon api

 Solution: Change your algorithm, you cannot use DMeroon_receive and
 DMeroon_install_receiver at the same time.
 */

/* These are zlib compression/decompression errors/return values. 
 * Implemented by Guillaume Rams, see dmzlib.c file.
 */

#define DME_Z_OK                        16901
#define DME_Z_STREAM_END                16902
#define DME_Z_NEED_DICT                 16903
#define DME_Z_ERRNO                     16904
#define DME_Z_STREAM_ERROR              16905
#define DME_Z_DATA_ERROR                16906
#define DME_Z_MEM_ERROR                 16907
#define DME_Z_BUF_ERROR                 16908
#define DME_Z_VERSION_ERROR             16909

/* These are URLscript errors. */
#define DME_URLSCRIPT_UNKNOWN_COMMAND		17000
#define DME_URLSCRIPT_BAD_CHARACTER_SYNTAX	17001
#define DME_URLSCRIPT_BAD_STRING_SYNTAX		17002
#define DME_URLSCRIPT_BAD_NUMBER_SYNTAX		17003
#define DME_URLSCRIPT_UNKNOWN_FIELD		17004
#define DME_URLSCRIPT_STACK_UNDERFLOW		17005
#define DME_URLSCRIPT_BAD_ARITY			17010
#define DME_URLSCRIPT_TOO_MUCH_ARGUMENTS	17011
#define DME_URLSCRIPT_WRONG_ARGUMENT		17012
#define DME_URLSCRIPT_UNCONSUMED_STACK		17013
#define DME_URLSCRIPT_NOT_AN_OBJECT		17014
#define DME_URLSCRIPT_INCOMPATIBLE_ARGUMENT	17015
#define DME_URLSCRIPT_UNHANDLED_ARGUMENT	17016
#define DME_URLSCRIPT_BAD_MODIFY_STRING		17017
#define DME_URLSCRIPT_BAD_MODIFY_VALUE		17018
#define DME_URLSCRIPT_EMPTY			17019
#define DME_URLSCRIPT_URLENV_UNDERFLOW		17020
#define DME_URLSCRIPT_COMMAND_REDEFINITION	17021

/* These are errors when binding with Bigloo. */
#define DME_NOT_BIGLOO_STRING			18001
#define DME_WRONG_CONTINUATION			18002
#define DME_BASIC_SERVER_LOOP_UNINITIALIZED	18003
#define DME_BASIC_SERVER_LOOP_REINITIALIZED	18004

/* Error codes 20xyz are reserved for Icslas. */
#define DME_ICSLAS_INTERNAL_PROBLEM		20000

/* These numbers characterize Facts */
#define DME_LOST_SITE				32001
#define DME_GOT_SITE				32002

/* 
 * When DMeroon raises an internal error, its associated number follows some
 * rules:
 *         1...  initialize errors.
 *
 */

#endif /* DM_DMERR_H */

/* end of dmerr.h */
