/* $Id: dmextr.h,v 1.44 1998/05/06 20:03:12 queinnec Exp $ 
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 */

#ifndef DM_EXTR_H
#define DM_EXTR_H

/*
 * This file contains the exportations of the dmextr.h file.
 * It defines Scheme-like classes and a bunch of related functions.
 * These functions are mainly used by the http server.
 */

#ifdef DM_HAS_LONG_LONG_ENTITIES
#define DM_SetNat4ToZero(nat4) \
  nat4 = 0
#else
#define DM_SetNat4ToZero(nat4) \
  nat4._high = nat4._low = 0
#endif

extern DM_nat4
DM_nat4_add (DM_nat4 a, DM_nat3 b);

extern DM_nat4
DM_nat4_subtract (DM_nat4 a, DM_nat3 b);

extern DM_nat4
DM_nat4_multiply (DM_nat4 a, DM_nat3 b);

extern DM_nat1
DM_nat4_last_nibble (DM_nat4 *a);

/******************************************************************
 *		Additional exportations.
 *
 * Two objects to mean True or False.
 */

extern DM_String*
DM_cstring_to_string (DM_Context *context,
                      char       *s,
                      DM_nat3    length );

extern DM_DMeroonType*
DM_cstring_to_type (DM_Context *context,
                    char *s);

extern DM_Class*
DM_cstring_to_predefined_class (DM_Context *context,
                                char *s);

extern DM_Field*
DM_cstring_to_field (DM_Context *context,
                     DM_Class* class,
                     char      *s );

extern DM_nat3
DM_string_to_nat3 (DM_Context *context,
                   char *p, char **q);

extern void
DM_nat4_to_string (DM_Context *context,
                   DM_nat4 number,
                   void    *output_port );

extern DM_nat3 
DM_hashpjw (DM_nat3 length, char *string);

/******************************************************************
 *  Symbol management routines.
 */

extern DM_nat3
DM_compute_hash_symbol (DM_Context *context,
                        DM_Symbol  *symbol);

extern DM_Symbol*
DM_intern_symbol (DM_Context *context,
                  DM_Symbol  *symbol);

/******************************************************************
 * conversion routines.
 */

extern DM_Field*
DM_number_to_field (DM_Context *context,
                    DM_Class *class,
                    DM_nat3  number );

extern DM_Field*
DM_cstring_to_field (DM_Context *context,
                     DM_Class *class,
                     char     *string );

extern DM_Field*
DM_string_to_field (DM_Context *context,
                    DM_Class  *class,
                    DM_String *string );

extern DM_Class*                     /* May return DM_NULL */
DM_index_to_predefined_class (DM_Context *context,
                              DM_nat1 index);

/******************************************************************
 * Various.
 */

extern DM_Object*
DM_remove_last_queuelink (DM_Context   *context,
                          DM_QueueLink **location );

extern void
DM_enqueue_received_object (DM_Context *context,
                            DM_Object  *o );

extern void
DM_enqueue_received_request_or_answer (DM_Context *context,
                                       DM_Object  *o );


DM_File*
DM_load_file (DM_Context *context,
              char       *filepath );

DM_c_boolean
DM_store_file (DM_Context *context,
               DM_File    *file,
               char       *dirpath );

/******************************************************************
 * initialization
 */

void DM_extr_initialize (DM_Context *context);

#endif /* DM_EXTR_H */

/* end of dmextr.h */
