/* $Id: dmhash.h,v 1.18 1998/05/25 19:01:26 queinnec Exp $ 
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 
 * These are the resources exported from the dmhash.c file.
 */

#ifndef DM_HASH_H
#define DM_HASH_H

/* 
 * Above this level, DM_DEBUG reports on hashtable usage. 
 */

#define DM_DEBUG_HASHTABLE 28

extern void
DM_hashtable_insert (DM_Context    *context,
                     DM_Hashtable* *hashtable_address,
                     DM_Object     *object );

extern void
DM_hashtable_double (DM_Context    *context,
                     DM_Hashtable* *hashtable_address);

extern void
DM_hashtable_remove (DM_Context    *context,
                     DM_Hashtable* *hashtable_address,
                     DM_Object     *object );

extern void 
DM_hash_initialize (DM_Context *context);

/* 
 * Macros to recover the hasher/extractor functions from a hashtable.
 */

#define DM_DeclareHasherFunction(name, value) \
  DM_nat3 (*name)(DM_Context*,DM_Object*) = \
    ((DM_nat3 (*)(DM_Context*,DM_Object*))(value))

#define DM_DeclareExtractorFunction(name, value) \
  void* (*name)(DM_Context*, DM_Object**) = \
      ((void* (*)(DM_Context*, DM_Object**))(value))

/* RESOURCE DM_HashtableExtract
 * Extracts an object from a Hashtable, returns DM_NULL if not found.

 * Since to extract an element depends on a comparison which is not easy
 * to abstract in a C function, extraction is provided with a macro with
 * three parameters:
 *     hashtable is the hashtable where to look,
 *     key is the hash of the looked for element,
 *     extractor allows to perform some treatment on the pointer (useful for
 *       Exit items that should not be referenced by the Exit table, see
 *       dmcomm.c file), (also useful in dmact.c to ensure that UrlActions
 *       are present in UrlEnv).
 *     comparison is the code that performs the comparison.
 * These two code fragments may use the visible local variables ie:
 *     hsize, object...
 */

#define DM_HashtableExtract(context, ht, type, key, comparison) \
  DM_Hashtable *hashtable  = (ht); \
  type (*extractor)(DM_Context*, type*) = \
      ((type (*)(DM_Context*, type*)) (hashtable->extractor)); \
  type         object; \
  DM_nat3      hsize      = hashtable->item_length; \
  DM_nat3      h          = hsize; \
  DM_nat3      hk         = (key) % hsize; \
  \
  while ( h-- > 0 ) { \
    if ( extractor ) { \
      object = (*extractor)(context, (type*)(&(hashtable->item[hk]))); \
    } else { \
      object = (type)(hashtable->item[hk]); \
    } \
 \
  DM_DebugAtLevel(DM_DEBUG_HASHTABLE, \
    fprintf(stderr, "Comparing with Object " DM_0X "%p from " \
                    DM_0X "%p[%lu]\n", \
          object, hashtable, (unsigned long)hk ); \
  ); \
 \
    if ( object ) { \
      if ( (comparison) ) { \
 \
  DM_DebugAtLevel(DM_DEBUG_HASHTABLE, \
    fprintf(stderr, "Found Object " DM_0X "%p in " DM_0X "%p[%lu]\n", \
          object, hashtable, (unsigned long)hk); \
 ); \
 \
        return object; \
      } else {     \
        hk = ( hk > 0 ) ? hk-1 : hsize-1; \
      } \
    } else { \
      /* No object in table. */ \
      return DM_NULL; \
    } \
  } \
  /* The hashtable is full and the looked for object is not there. */ \
  return DM_NULL

#endif /* DM_HASH_H */

/* end of dmhash.h */
