/* $Id: dmkern.h,v 1.56 1998/05/25 19:01:27 queinnec Exp $ 
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 */

#ifndef DM_KERNEL_H
#define DM_KERNEL_H

/*
 * These are the exportations of dmkern.c ie the fundamental DMeroon
 * kernel library.
 */

/*
 * Some primitives may limit the number of indexed fields that can be
 * handled (for instance, DM_allocate in dmbgl.c). This is the limit.
 */

#define DM_MAX_INDEXED_FIELDS 32

/******************************************************************
 *                 Other exportations.
 */

DM_DefineVerySpecializedClass(1,0,7);
extern DM_Class_1_0_7_wrapped DM_Object_class_object;

DM_ExportConstantObject(nil);
DM_ExportConstantObject(uninitialized);
DM_ExportConstantObject(missing);
DM_ExportConstantObject(true);
DM_ExportConstantObject(false);

typedef DM_GenerateImmutableVectorStruct(256) DM_ImmutableVector_256;
DM_DefineWrappedType(DM_ImmutableVector_256);

/******************************************************************
 *                      FUNCTIONS
 */

extern DM_c_boolean
DM_is_a (DM_Context *context,
         DM_Object *o, 
         DM_Class  *class);

extern DM_c_boolean
DM_is_subclass (DM_Context *context,
                DM_Class *class1, 
                DM_Class *class2);

extern DM_c_boolean
DM_is_up_to_date (DM_Context *context,
                  DM_Object *o);

extern void
DM_increment_clock_of (DM_Context *context,
                       DM_Object *o);

extern void
DM_increment_clock (DM_Context *context,
                    DM_Clock *clock);

extern DM_nat3
DM_read_repetition_factor (DM_Context *context,
                           void *p, 
                           DM_nat1 reptypindex);

extern DM_displacement 
DM_compute_offset (DM_Context *context,
                   DM_Object *o,
                   DM_Field  *field );

extern DM_displacement
DM_indexed_compute_offset (DM_Context *context,
                           DM_Object *o,
                           DM_Field  *field,
                           DM_nat3   index );

extern DM_nat3
DM_class_indexed_field_length (DM_Context *context,
                               DM_Class* class);

extern DM_Field**
DM_class_field_address (DM_Context *context,
                        DM_Class *class, 
                        DM_nat3  i );
extern void*
DM_fake_pointer (DM_Context*, void **p);

extern char*
DM_predefined_name (DM_Context *context,
                    DM_Object  *o );

extern DM_Field*
DM_class_field (DM_Context *context,
                DM_Class* class, 
                DM_nat3   i);

extern void
DM_set_class_field (DM_Context *context,
                    DM_Class *class , 
                    DM_nat3  i, 
                    DM_Field *field);

extern DM_nat3
DM_size (DM_Context *context,
         DM_Object *o);

extern DM_Object*
DM_clone (DM_Context *context,
          DM_Object *o);

extern DM_Object*
DM_zero (DM_Context *context,
         DM_Object  *o);

extern void
DM_compute_layout (DM_Context *context,
                   DM_Class *class);

extern void
DM_compute_characteristics (DM_Context *context,
                            DM_Class *class);

extern void
DM_structure_inheritance (DM_Context *context,
                          DM_Class *class,
                          DM_Class *superclass );

extern void
DM_update_object (DM_Context *context,
                  DM_Object *target, 
                  DM_Object *source );

extern DM_Object*
DM_allocate_instance (DM_Context *context,
                      DM_Class *class,
                      DM_nat3  number_of_sizes,
                      DM_nat3  sizes[] );

extern DM_Object*
DM_allocate0 (DM_Context *context,
              DM_Class *class);

extern DM_Object*
DM_allocate1 (DM_Context *context,
              DM_Class *class,
              DM_nat3  i0 );

extern DM_Object*
DM_allocate_direct1 (DM_Context *context,
                     DM_Class *class,
                     DM_nat3  i0 );

extern DM_Object*
DM_allocate2 (DM_Context *context,
              DM_Class *class,
              DM_nat3  i0,
              DM_nat3  i1 );

extern DM_Object*
DM_allocate3 (DM_Context *context,
              DM_Class *class,
              DM_nat3  i0,
              DM_nat3  i1,
              DM_nat3  i2 );

extern void DM_kernel_initialize(DM_Context *context);

#endif /* DM_KERNEL_H */

/* end of dmkern.h */
