/* $Id: dmlib.c,v 1.24 1998/04/09 14:35:23 queinnec Exp $
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////

 * This file contains functions not in all Un*xes or functions that are
 * quite dependent on the Un*x flavor.
 */

#include "dmeroon.h"

DM_DefineVersionString(dmlib,
  "$Id: dmlib.c,v 1.24 1998/04/09 14:35:23 queinnec Exp $");

/* RESOURCE DM_get_date
 * This function returns the time since 00:00:00 GMT, Jan. 1, 1970,
 * measured in seconds. It returns a DM_date (which is currently an 
 * alias for a DM_nat3).

 * NOTE: although time() returns a long, it really is an unsigned long
 * except in case of error (ie -1). 
 * NOTE: on alpha, time_t = int.
 */

DM_date
DM_get_date (DM_Context *context) {
  DM_TIME_T t;

  if ( time(&t) == -1 ) {
    return 0;
  } else {
    return (DM_date)t;
  }
}

/* RESOURCE DM_setenv
 * Set a variable in the system environment. Returns 0 if successful.
 * setenv exists on Sony, putenv on sun4_bsd ...
 *
 * Caution: Pay attention to the fact that name and value must not be
 * reclaimed so they are stored in this static list. Normally, I only
 * use once this function when recording the DMEROON variable in the 
 * environment for forked processes.
 */

#if defined(DM_HAS_PUTENV)
static DM_Object* strings = (DM_Object*)(DM_TheConstant(nil));
#endif

int
DM_setenv (DM_Context *context,
           char       *name, 
           char       *value ) {

#if defined(DM_HAS_SETENV)
  return setenv(name, value, 1);
#endif

#if defined(DM_HAS_PUTENV)
  { unsigned long    len1  = strlen(name);
    unsigned long    len2  = strlen(value);
    DM_String        *str  = DM_Allocate1(context,String, len1+1+len2+1);
    DM_ImmutablePair *pair = DM_Allocate0(context,ImmutablePair);

    strcpy(str->character, name);
    strcat(str->character, "=");        /* use strcpy(..[len1]..) */
    strcat(str->character, value);
    pair->value = (DM_Object*)str;
    pair->rest  = (DM_Object*)strings;
    strings = (DM_Object*)pair;
    return putenv(str->character);
  }
#endif
}

/* {{{ Missing library files
 */

#if defined(DM_MISSING_STRNCASECMP)

int
strncasecmp (const char *s1, const char *s2, int n)
{
  return 0; /* FUTURE */
}

#endif /* !defined(DM_MISSING_STRNCASECMP) */

/* }}}
 * Initialize this module.
 */

void
DM_lib_initialize (DM_Context *context) 
{
  DM_LinkVersionStringIntoSite(dmlib);
}

/* end of dmlib.c */
