/* $Id: dmnewapi.h,v 1.26 1998/06/01 19:30:44 queinnec Exp $
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////

 * These are the exportations from the DMeroon API.
 * Arguments immediately prefixed with a star correspond to returned 
 * values, they generally have a name prefixed with returned_. 
 */

extern DM_ErrorCode
DMeroon_initialize (DM_Context* context,
                    DM_Site*    *returned_site );

extern DM_ErrorCode
DMeroon_is_a (DM_Context*  context,
              DM_c_boolean *returned_boolean,
              DM_Object*   object,
              DM_Class*    class );

extern DM_ErrorCode
DMeroon_is_a_subclass (DM_Context*  context,
                       DM_c_boolean *returned_boolean,
                       DM_Class*    subclass,
                       DM_Class*    class );

extern DM_ErrorCode
DMeroon_class_of (DM_Context* context,
                  DM_Class*   *returned_class,
                  DM_Object*  object );

extern DM_ErrorCode
DMeroon_site_of (DM_Context* context,
                 DM_Site*    *returned_site,
                 DM_Object*  object );

extern DM_ErrorCode
DMeroon_clock_of (DM_Context* context,
                  DM_Clock*   *returned_clock,
                  DM_Object*  object );

extern DM_ErrorCode
DMeroon_receive (DM_Context* context,
                 DM_Object*  *returned_object,
                 DM_nat3     duration );

extern DM_ErrorCode
DMeroon_install_receiver (DM_Context* context,
                          DM_code     function );

extern DM_ErrorCode
DMeroon_serve (DM_Context*  context,
               DM_nat3      duration );

extern DM_ErrorCode
DMeroon_send (DM_Context* context,
              DM_Object*  o,
              DM_Site*    site );

extern DM_ErrorCode
DMeroon_connect (DM_Context* context,
                 DM_Site*    *returned_site,
                 char*       hostname,
                 DM_nat2     port_number,
                 char*       password );

extern DM_ErrorCode
DMeroon_create_subclass (DM_Context*      context,
                         DM_Class*        *returned_class,
                         DM_Class*        superclass,
                         char*            class_name,
                         DM_class_options class_options,
                         char*            field_name,
                         DM_field_options field_options,
                         DM_DMeroonType*  reptyp,
                         DM_DMeroonType*  itemtyp,
                         DM_Class*        referred_class );

extern DM_ErrorCode
DMeroon_smash_class (DM_Context*      context,
                     DM_Class*        *returned_class,
                     DM_Class*        class,
                     char*            class_name,
                     DM_class_options class_options,
                     DM_nat3          level );

extern DM_ErrorCode
DMeroon_set_site_of (DM_Context* context,
                     DM_Site*    *returned_previous_site,
                     DM_Object*  o,
                     DM_Site*    site );

extern DM_ErrorCode
DMeroon_set_clock_of (DM_Context* context,
                      DM_Clock*   *returned_previous_clock,
                      DM_Object*  object,
                      DM_Clock*   clock );

/* By default, the access is on a regular non indexed, read-only field. */

#define DM_DEFAULT_ACCESS	0
#define DM_INDEXED_ACCESS	(1<<1)
#define DM_LENGTH_ACCESS	(1<<2)
#define DM_WRITE_ACCESS		(1<<3)

extern DM_ErrorCode
DM_check_access (DM_Context *context,
                 DM_Object* o,
                 DM_Field*  field,
                 DM_nat3    access_status );

extern DM_ErrorCode
DMeroon_field_value (DM_Context*     context,
                     void            *returned_value,
                     DM_DMeroonType* *returned_type,
                     DM_Object*      object,
                     DM_Field*       field );

extern DM_ErrorCode
DMeroon_indexed_field_value (DM_Context*     context,
                             void            *returned_value,
                             DM_DMeroonType* *returned_type,
                             DM_Object*      object,
                             DM_Field*       field,
                             DM_nat3         index );

extern DM_ErrorCode
DMeroon_field_types (DM_Context*     context,
                     DM_DMeroonType* *returned_item_type,
                     DM_DMeroonType* *returned_repetition_type,
                     DM_Field*       field );

extern DM_ErrorCode
DMeroon_set_field_value (DM_Context*     context,
                         void            *returned_value,
                         DM_Object*      object,
                         DM_Field*       field,
                         void*           value );

extern DM_ErrorCode
DMeroon_set_indexed_field_value (DM_Context*     context,
                                 void            *returned_value,
                                 DM_Object*      object,
                                 DM_Field*       field,
                                 DM_nat3         index,
                                 void*           value );

extern DM_ErrorCode
DMeroon_indexed_field_length (DM_Context*     context,
                              void            *returned_length,
                              DM_DMeroonType* *returned_type,
                              DM_Object*      object,
                              DM_Field*       field );

extern DM_ErrorCode
DMeroon_do_allocate (DM_Context* context,
                     DM_Object*  *returned_object,
                     DM_Class*   class,
                     DM_nat3     number_of_sizes,
                     DM_nat3     sizes[] );

extern DM_ErrorCode
DMeroon_allocate (DM_Context* context,
                  DM_Object*  *returned_object,
                  DM_Class*   class,
                  DM_nat3     number_of_sizes,
                  DM_nat3     sizes[]  );

extern DM_ErrorCode
DMeroon_set_sharable (DM_Context* context,
                      DM_Object*  object );

extern DM_ErrorCode
DMeroon_set_copyable (DM_Context* context,
                      DM_Object*  object );

extern DM_ErrorCode
DMeroon_set_initialized (DM_Context* context,
                         DM_Object*  object );

#define DM_OBJECT_IS_SHARABLE	1
#define DM_OBJECT_IS_COPYABLE	2

extern DM_ErrorCode
DM_do_is_initialized (DM_Context*  context,
                      DM_nat3      *returned_boolean,
                      DM_Object*   object );

extern DM_ErrorCode
DMeroon_is_initialized (DM_Context*  context,
                        DM_nat3      *returned_boolean,
                        DM_Object*   object );

extern DM_ErrorCode
DMeroon_eq (DM_Context*  context,
            DM_c_boolean *returned_boolean,
            DM_Object*   object1,
            DM_Object*   object2 );

extern void 
DM_abort(DM_Context    *context,
         DM_ErrorCode  code,
         char*         file,
         unsigned long line ) __attribute__((noreturn));

extern void
DM_barf_if_reached (DM_Context* context);

extern DM_ErrorCode
DMeroon_interpret (DM_Context*  context,
                   DM_UrlState* *urlstate,
                   char*        urlprogram,
                   DM_nat3      nobjects,
                   DM_Object*   object, ... );

/******************************************************************
 *                    CONTEXT
 *
 * Options for contexts.
 * FUTURE: some options are there for debug.
 *
 * DM_CONTEXT_CHECK_DMEROON means that values that should be DMeroon
 * objects are checked. DM_CONTEXT_CHECK_CLASS checks that DMeroon
 * objects have the correct class.
 */

#define DM_CONTEXT_CHECK_DMEROON		(1<<1)
#define DM_CONTEXT_CHECK_CLASS			(1<<2)
#define DM_CONTEXT_ASYNCHRONOUS			(1<<3)
#define DM_CONTEXT_UNFINISHED			(1<<4)
#define DM_CONTEXT_ABORT_IMMEDIATELY		(1<<8)
#define DM_CONTEXT_VERBOSE			(1<<9)
#define DM_CONTEXT_DONT_INITIALIZE		(1<<16)
#define DM_CONTEXT_DONT_MARSHAL_DATA_CODE_FIELD	(1<<17)

/* This is the default options. The default options for contexts allocated
 * in dmscm.c are slightly different (see DM_PrepareLocalContext macro).
 * NOTE: If you set the DM_CONTEXT_DONT_MARSHAL_DATA_CODE_FIELD option on, 
 * then this is incompatible of some tests (particularly the tests that 
 * check how Bigloo values are transmitted).
 */

#define DM_CONTEXT_DEFAULT_OPTIONS	\
   (  DM_CONTEXT_CHECK_DMEROON		\
    | DM_CONTEXT_CHECK_CLASS		\
    | DM_CONTEXT_ABORT_IMMEDIATELY	\
    | DM_CONTEXT_VERBOSE 		\
    )

extern void 
DM_newapi_initialize (DM_Context *context);

/* end of dmnewapi.h */
