/* $Id: dmos.h,v 1.56 1998/06/05 18:38:26 queinnec Exp $
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 */

#ifndef DMeroon_OS_H
#define DMeroon_OS_H

/* DOCUMENTATION dmos.h
 * This header file tries to reconcile the different Operating Systems
 * header files as well as some general C compiler behavior. It is
 * highly conditionalized.
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>

#if defined(WIN32) 
#  include <winsock2.h>
#else /* Unix stuff here: */
#  include <unistd.h>
#  include <sys/time.h>
#  include <netdb.h>
#  include <sys/socket.h>
#  include <netinet/in.h>
#  include <arpa/inet.h>
#endif

#include <time.h>
#include <signal.h>
#include <stdarg.h>
#include <float.h>
#include <math.h>
#include <setjmp.h>
#include <sys/stat.h>

#define DM_TIME_T time_t

/* 
 * Does the compiler support the 'errno' identifier. See dm.h for
 * DM_ResetErrno(). 
 */

#include <errno.h>
#define DM_USE_OLD_ERRNO 1

/* 
 * DMeroon code uses sometimes gcc additional declarations. Just
 * redefine them as nothing if not using gcc.
 */

#ifndef __GNUC__
#define __attribute__(a) /* This characterizes a function */
#define __inline__       /* This indicates that the function may be inlined */
#endif

/* DOCUMENTATION
 * SUN Solaris 5.3 is recognized.

 * Example: youpou.ibp.fr
 */

#if (defined( SPARC ) || defined( sparc )) && defined( __svr4__ )

#define DM_SYSTEM "sun4_solaris"
#define DM_HAS_PUTENV 1
#define DM_HAS_FLOAT3 1

#define BADSIG      (void (*)(int))-1
#define INADDR_NONE ((long)-1)
extern int gethostname (char* name, int namelen);
/* To obtain memset that is used in FD_* macros */
#include <arpa/inet.h>
#include <memory.h>
/* random and initstate */
extern long random();
extern void srandom(unsigned int seed);
/* To obtain open  */
#include <fcntl.h>
/* To obtain O_RDONLY and other options for open */
#include <sys/fcntl.h>
/* To obtain getpwuid */
#include <pwd.h>
/* To include strncmp
 * index does not exist, and is replaced by strchr.  */
#include <string.h>
#define DM_strchr strchr
/* getdtablesize() is obsolete, should use getrlimit for POSIX. */
#include <limits.h>
#define getdtablesize() OPEN_MAX
/* To include isalnum() */
#include <ctype.h>
#endif

/* DOCUMENTATION
 * Sun BSD 4.1.2.1 is recognised.

 * Example: cornas.inria.fr
 */

#if (defined( SPARC ) || defined( sparc ))  && !(defined( __svr4__ ))

#define DM_SYSTEM "sun4_bsd"
#define DM_HAS_PUTENV 1

/* for MAXHOSTNAMELEN */
#include <limits.h>
#include <sys/param.h>
extern void perror (char* prefix);
#define INADDR_NONE ((long)-1)
extern int fputs (char *s, FILE *stream);
extern int gethostname (char* name, int namelen);
extern int connect (int s, struct sockaddr *name, int namelen);
extern int socket (int domain, int type, int protocol);
extern int bind (int sockfd, struct sockaddr* name, int namelen);
extern int listen (int sockfd, int backlog);
extern int accept (int sockfd, struct sockaddr* name, int* namelen);
extern void fprintf ();
extern int setsockopt (int sockfd, int level, int optname, 
                       void* optval, int optlen );
extern int select (int maxfdpl, fd_set *read, fd_set *write, fd_set *except,
                   struct timeval *timeout);
/* Used in FD_* macros */
extern bzero(char* b, int length);
/* random and initstate */
extern long random();
extern void srandom(int seed);
/* for time() */
extern time_t time(time_t *tloc);
extern fflush(FILE *stream);
/* To obtain getpwuid */
#include <pwd.h>
/* To obtain O_RDONLY .. */
#include <sys/fcntlcom.h>
/* getdtablesize is now only a system call */
extern int getdtablesize();
/* for strncmp and index */
#include <string.h>
#define DM_strchr strchr
#include <ctype.h>
extern int putenv(char *string);
#define DM_MISSING_STRNCASECMP 1 /* strncasecmp is missing */
#endif

/* DOCUMENTATION
 * Sony NewsOS 3200 is recognized.

 * Example: blaye.inria.fr
 */

#if defined( sony_news ) || defined( r3000 )

#define DM_SYSTEM "news_mips"
#define DM_HAS_SETENV 1

/* for MAXHOSTNAMELEN */
#include <sys/param.h>
extern char *getlogin();
extern uid_t getuid();
/* for getpwuid */
#include <pwd.h>
extern double frexp (double, int*);
extern double ldexp (double, int);
extern int getpid (void);
extern int close(int d);
extern int sleep(unsigned duration);
extern int open(char *path, int flags, int mode);
/* for O_RDONLY etc. */
#include <fcntl.h>
extern int read(int d, char *buf, int nbytes);
extern int write(int d, char *buf, int nbytes);
extern int sprintf(char *s, char *format, ...);
extern int fputs(char *s, FILE *stream);
extern void perror(char* prefix);
extern int gethostname (char* name, int namelen);
/* herror is not everywhere */
#define DM_HAS_HERROR 1
extern int herror(char *string);
extern int connect (int s, struct sockaddr* name, int namelen);
extern int socket (int domain, int type, int protocol);
extern int bind (int sockfd, struct sockaddr* name, int namelen);
extern int listen (int sockfd, int backlog);
extern int accept (int sockfd, struct sockaddr* name, int* namelen);
extern int setsockopt (int sockfd, int level, int optname, 
                       void* optval, int optlen );
extern int select (int maxfdpl, fd_set *read, fd_set *write, fd_set *except,
                   struct timeval *timeout);
/* Used in FD_* macros */
extern bzero(char* b, int length);
/* ntohs() and others are not in netinet/in.h on Sony 
   BIG_ENDIAN is redefined in bigloo.h */
#undef BIG_ENDIAN
#include <machine/endian.h>
/* random and initstate */
extern long random();
extern void srandom(int seed);
extern time_t time(time_t*);
/* These are generated by Bigloo */
/* #include <setjmp.h>
   int _setjmp(jmp_buf env);
   void _longjmp(jmp_buf env, int val);         */
extern int setjmp(jmp_buf env);
extern void longjmp(jmp_buf env, int val);
extern char *getenv(char *name);
extern int setenv(char *name, char *value, int overwrite);
extern int pipe(int fildes[2]);
extern int fork();
extern int dup2(int oldd, int newd);
extern int execv(char *name, char *argv[]);
extern int getdtablesize();
/* for strcmp, strncmp, ... */
#include <strings.h>
#define DM_strchr index
/* for isalnum ... */
#define toupper _toupper
#define tolower _tolower
#include <ctype.h>
#endif

/* DOCUMENTATION
 * DEC Alpha is recognized.

 * Examples: latour.inria.fr, hercule.polytechnique.fr
 */

#if defined(__alpha)

#define DM_SYSTEM "alpha"

#define DM_HAS_PUTENV 1

/* for MAXHOSTNAMELEN */
#include <sys/param.h>
/* for strlen and strcpy  (still a problem on the ffs function ???) */
#include <string.h>
#define DM_strchr strchr
extern int gethostname (char* name, int namelen);
extern int select (int maxfdpl, fd_set *read, fd_set *write, fd_set *except,
                   struct timeval *timeout);
/* for getpwuid */
#include <pwd.h>
/* for O_RDONLY etc. */
#include <fcntl.h>
/* To include isalnum */
#include <ctype.h>
#endif

/* DOCUMENTATION
 * DEC Ultrix is recognized.

 * Examples: lix.polytechnique.fr
 */

#if defined(ultrix) && defined(mips) && defined(host_mips)

#define DM_SYSTEM "ultrix"

/* for MAXHOSTNAMELEN */
#include <sys/param.h>
/* for strlen and strcpy  (still a problem on the ffs function ???) */
#include <string.h>
extern int gethostname (char* name, int namelen);
extern int select (int maxfdpl, fd_set *read, fd_set *write, fd_set *except,
                   struct timeval *timeout);
#endif

/* DOCUMENTATION
 * SGI is recognized.
 */

#if defined(sgi)

#define DM_SYSTEM "sgi"
#define DM_HAS_PUTENV 1

#include <stdio.h>
#include <bstring.h>
#define BADSIG SIG_ERR
#include <sys/param.h>
#include <limits.h>
/* To include isalnum() */
#include <ctype.h>
#include <string.h>
#define DM_strchr strchr

#endif

/* DOCUMENTATION
 * Linux is recognized.

 Example: borba.inria.fr
 */

#if defined(linux) 

#define DM_SYSTEM "Linux"
#define DM_HAS_SETENV 1 /* Linux also has putenv! */
/* for strlen and strcpy  (still a problem on the ffs function ???) */
#include <string.h>
#define DM_strchr strchr
/* To include isalnum */
#include <ctype.h>
/* for getpwuid */
#include <pwd.h>
/* for O_RDONLY etc. */
#include <fcntl.h>

#define DM_0X ""

/* DOCUMENTATION
 * redhat5 is recognized (see config/all.mkf) by testing the existence of
 * the /etc/redhat-release file and its content. Thanks to 
 * <Frederic.Raimbault@univ-ubs.fr> for this information.

 Example: pc-valoria-7.univ-ubs.fr
 Courtesy of Yves Maho <maheo@univ-ubs.fr>.
 */

#if defined(redhat5) 
#define BADSIG (void (*)(int))-1 
/* for MAXHOSTNAMELEN */
#include <asm/param.h>
#endif

#endif

/* DOCUMENTATION
 * FreeBSD is recognized.

 Example: absinthe.polytechnique.fr 
 */

#if defined(__FreeBSD__) 

#define DM_SYSTEM "FreeBSD"
#define DM_HAS_SETENV 1 /* FreeBSD also has putenv! */
#include <string.h>
#define DM_strchr strchr
/* for isalnum */
#include <ctype.h>
/* for MAXHOSTNAMELEN */
#include <sys/param.h>
/* For getpwuid */
#include <pwd.h>
/* for O_RDONLY ... */
#include <fcntl.h>

/* Special conditionalization: see DM_finish_port in dmnet.c */
#define DM_CLOSE_MAY_RETURN_EAGAIN 1

/* The %p directive prefixes addresses with 0x so define the following
 * prefix to nothing. */
#define DM_0X 

#endif

/* DOCUMENTATION
 * Windows is recognized.
 */

#if defined(WIN32) 

#define DM_SYSTEM "Windows/95/NT"
#define DM_HAS_SETENV 1 

#include <winsock2.h>
#define DM_ECONNRESET WSAECONNRESET
#define DM_EINTR      WSAEINTR
#define DM_EAGAIN     WSAEAGAIN
#define DM_EPIPE      WSAEPIPE
#define DM_EBADF      WSAEBADF

#endif

/*  Check or set default macros.
 *
 * DM_OX corresponds to the "0x" prefix that DMeroon prints before addresses.
 * 
 */

#ifndef DM_0X
#define DM_0X "0x"
#endif

#ifndef DM_SYSTEM
#error "Unknown system"
#endif

#ifndef DM_HAS_FLOAT3
#define DM_HAS_FLOAT3 0
#endif

#if !defined(WIN32)              /* for Unix: */
#define DM_ECONNRESET ECONNRESET
#define DM_EINTR      EINTR
#define DM_EAGAIN     EAGAIN
#define DM_EPIPE      EPIPE
#define DM_EBADF      EBADF
#endif

#endif /* DMeroon_OS_H */

/* end of dmos.h */
