/* $Id: dmprint.c,v 1.69 1998/06/10 16:31:29 queinnec Exp $
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 */

/* 
 * This file defines how are printed all DMeroon types.
 */

#include "dmeroon.h"

DM_DefineVersionString(dmprint,
  "$Id: dmprint.c,v 1.69 1998/06/10 16:31:29 queinnec Exp $");

void
DM_print_char (DM_Context *context, 
               char       character, 
               void       *output_port ) {
  char string[2];
  
  string[0] = character;
  /* This is useful if wrapped_port is a C stream for instance. */
  string[1] = 0;
  DM_print_char_star(context, string, 1, output_port);
}

/* FUTURE: fix the problem of the size of String!  The allocation size
 * includes the final null but we don't want to print that null
 * neither we can exclude non-null terminated strings.  Hence this
 * hack! Also check that all statically allocated Strings have their
 * final null character.
 */

void
DM_print_string (DM_Context *context, 
                 DM_String  *string,
                 void       *output_port ) {
  DM_nat3 size = string->character_length;

  if ( string->character[size-1] != '\0' ) {
    DM_print_char_star(context, string->character, size, output_port);
  } else {
    DM_print_char_star(context, string->character, size-1, output_port);
  }
}

/* The generic entry point. */

void
DM_local_print (DM_Context *context, 
                DM_DMeroonType  *type, 
                DM_Object       *o,
                DM_displacement t, 
                void            *output_port ) { 
  if ( output_port == NULL ) {
    return; 
  } else {
    void *p = DM_Displace(o, t);
    (type->printer)(context, p, output_port);
  }
}

/******************************************************************
 * Here  starts the long list of methods to print types.
 */

void 
DM_no_print (DM_Context *context, 
             void       *p, 
             void       *output_port) {
  DM_Abort(context, DME_UNPRINTABLE_TYPE);
}

void 
DM_C_unsigned_char_print (DM_Context *context, 
                          void *p, 
                          void *output_port) {
  char line[80];

  sprintf(line, "%hu", DM_Content(unsigned char,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_unsigned_short_int_print (DM_Context *context, 
                               void *p, 
                               void *output_port) {
  char line[80];

  sprintf(line, "%hu", DM_Content(unsigned short int,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_unsigned_int_print (DM_Context *context, 
                         void *p, 
                         void *output_port) {
  char line[80];

  sprintf(line, "%u", DM_Content(unsigned int,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_unsigned_long_int_print (DM_Context *context, 
                              void *p, 
                              void *output_port) {
  char line[80];

  sprintf(line, "%lu", DM_Content(unsigned long int,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_unsigned_long_long_int_print (DM_Context *context, 
                                   void *p, 
                                   void *output_port) {
  char line[80];
  char *q        = ((char*)(&line[79]));
  char *line_end = q;

#ifdef DM_HAS_LONG_LONG_ENTITIES
  unsigned long long int v = DM_Content(unsigned long long int,p);
  int n;

  /* The LL format is not provided by the OS printf utility.
   * sprintf(line, "%LLu", DM_Content(unsigned long long int,p)); */
  
  *q-- = '\0';
  do {
    n = v%((unsigned long long int) 10);
    v = v/((unsigned long long int) 10);
    *q-- = '0' + n;
    } while ( v > 0);
  DM_print_char_star(context, q+1, (line_end-q-1), output_port);
#endif

}

/*
 * To print the address of a C function, use the %p format and hope something
 * got printed!!!
 */

void 
DM_C_function_star_print (DM_Context *context, 
                          void *p, 
                          void *output_port) {
  char line[80];

  sprintf(line, DM_0X "%p", DM_Content(DM_code,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_void_star_print (DM_Context *context, 
                      void *p,
                      void *output_port) {
  char line[80];

  sprintf(line, DM_0X "%p", DM_Content(DM_data,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_signed_char_print (DM_Context *context, 
                        void *p,
                        void *output_port) {
  char line[80];

  sprintf(line, "%hd", DM_Content(signed char,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_short_int_print (DM_Context *context, 
                      void *p,
                      void *output_port) {
  char line[80];

  sprintf(line, "%hd", DM_Content(short int,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_int_print (DM_Context *context, 
                void *p,
                void *output_port) {
  char line[80];

  sprintf(line, "%d", DM_Content(int,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_long_int_print (DM_Context *context, 
                     void *p,
                     void *output_port) {
  char line[80];

  sprintf(line, "%ld", DM_Content(long int,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_long_long_int_print (DM_Context *context, 
                          void *p,
                          void *output_port) {
  char line[80];
  char *q        = ((char*)(&line[79]));
  char *line_end = q;

#ifdef DM_HAS_LONG_LONG_ENTITIES
  long long int v = DM_Content(long long int,p);
  int n;
  int negative = 0;

  /* The LL format  does not exist.
   * sprintf(line,"%LLd", DM_Content(long long int,p)); */
  
  *q-- = '\0';
  if ( v < 0 ) {
    v        = -v;
    negative = 1;
  }
  do {
    n = v%((long long int) 10);
    v = v/((long long int) 10);
    *q-- = '0' + n;
    } while ( v > 0);
  *q-- = negative ? '-' : '+';
  DM_print_char_star(context, q+1, (line_end-q-1), output_port);
#endif

}

void 
DM_C_char_print (DM_Context *context, 
                 void *p,
                 void *output_port) {
  char line[80];

  sprintf(line, "%c", DM_Content(char,p));
  DM_print_char_star(context, line, 1, output_port);
}

void 
DM_netnat2_print (DM_Context *context, 
                  void *p,
                  void *output_port) {
  char line[80];
  DM_nat2 port = ntohs(DM_Content(DM_netnat2,p));

  sprintf(line, "%u(%hu.%hu)", port, port>>8, port&0xff);
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_netnat3_print (DM_Context *context, 
                  void *p,
                  void *output_port) {
  char line[80];
  DM_netnat3 ip = DM_Content(DM_netnat3,p);
  struct in_addr* address = (struct in_addr*)&ip;

  sprintf(line, "%s", inet_ntoa(*address));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_unicode_print (DM_Context *context, 
                  void *p,
                  void *output_port) {
  DM_Abort(context, DME_NOT_YET_IMPLEMENTED);
}

void 
DM_C_float_print (DM_Context *context, 
                  void *p,
                  void *output_port) {
  char line[80];

  sprintf(line, "%g", ((double)DM_Content(float,p)));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_double_print (DM_Context *context, 
                   void *p,
                   void *output_port) {
  char line[80];

  sprintf(line, "%g", DM_Content(double,p));
  DM_print_char_star(context, line, 0, output_port);
}

void 
DM_C_long_double_print (DM_Context *context, 
                        void *p,
                        void *output_port) {
  char line[80];

  sprintf(line, "%Lg", DM_Content(long double,p));
  DM_print_char_star(context, line, 0, output_port);
}

/* 
 * There is a particular method to (pretty-)print offsets.
 */

void 
DM_offset_print (DM_Context *context, 
                 void *p, 
                 void *output_port) {
  char line[80];
  DM_ImplementedTypesVector *implemented_types = DM_TheImplementedTypes();
  DM_offset offset = DM_Content(DM_offset,p);
  unsigned reptypindex = DM_OffsetToRepetitionTypeIndex(offset);
  DM_DMeroonType* reptype = implemented_types->item[reptypindex];
  unsigned itemtypindex = DM_OffsetToItemTypeIndex(offset);
  DM_DMeroonType* itemtype = implemented_types->item[itemtypindex];

  if ( reptypindex != DM_nought_index ) {
    sprintf(line, "(%s*)%s@%u",
            DM_predefined_name(context, (DM_Object*)reptype),
            itemtype->name,
            DM_OffsetToTranslation(offset) );
  } else {
    sprintf(line, "%s@%u",
            DM_predefined_name(context, (DM_Object*)itemtype),
            DM_OffsetToTranslation(offset) );
  }
  DM_print_char_star(context, line, 0, output_port);
}

void
DM_date_print (DM_Context *context, 
               void *p,
               void *output_port) {
  DM_TIME_T date  = (DM_TIME_T)(DM_Content(DM_date, p));
  char      *line = ctime(&date);
  char      *q;

  /* Remove the newline. */
  q = DM_strchr(line, '\n');
  if ( q ) 
    *q = '\0';
  DM_print_char_star(context, line, 0, output_port);
}

static void
DM_nat2_hex_print (DM_Context *context, 
                   void       *p, 
                   void       *output_port) {
  char line[80];
  DM_nat2 n = DM_Content(DM_nat2, p);
  
  sprintf(line, "%04lx", (unsigned long)n);
  DM_print_char_star(context, line, 4, output_port);
}

static void
DM_nat3_hex_print (DM_Context *context, 
                   void       *p, 
                   void       *output_port) {
  char line[80];
  DM_nat3 n = DM_Content(DM_nat3, p);
  
  sprintf(line, "%08lx", (unsigned long)n);
  DM_print_char_star(context, line, 8, output_port);
}

void
DM_key_print (DM_Context *context, 
              void       *p, 
              void       *output_port) {
  DM_key key = DM_Content(DM_key, p);

  DM_print_char_star(context, "0x", 2, output_port);
  DM_nat2_hex_print(context, &(key._idx), output_port);
  DM_nat2_hex_print(context, &(key._port), output_port);
  DM_nat3_hex_print(context, &(key._ip), output_port);
  DM_nat3_hex_print(context, &(key._date), output_port);
  DM_nat3_hex_print(context, &(key._rand), output_port);
}

void
DM_class_options_print (DM_Context *context, 
                        void *p, 
                        void *output_port) {
  DM_class_options options = DM_Content(DM_class_options, p);
  DM_nat1 cn;

  if ( options & DM_CLASS_IS_MUTABLE ) {
    DM_print_char_star(context, " MUTABLE", 0, output_port);
  }
  if ( options & DM_CLASS_IS_IMMOTILE ) {
    DM_print_char_star(context, " IMMOTILE", 0, output_port);
  }
  if ( options & DM_CLASS_IS_STRINGENT ) {
    DM_print_char_star(context, " STRINGENT", 0, output_port);
  }
  if ( options & DM_CLASS_IS_VOLATILE ) {
    DM_print_char_star(context, " VOLATILE", 0, output_port);
  }
  if ( options & DM_CLASS_HAS_NO_REFERENCE ) {
    DM_print_char_star(context, " WITHOUT_REFERENCE", 0, output_port);
  }
  if ( options & DM_CLASS_IS_USABLE ) {
    DM_print_char_star(context, " USABLE", 0, output_port);
  }
  if ( options & DM_CLASS_IS_NOT_SUBCLASSABLE ) {
    DM_print_char_star(context, " NOT SUBCLASSABLE", 0, output_port);
  }
  if ( options & DM_CLASS_IS_NOT_INSTANTIABLE ) {
    DM_print_char_star(context, " NOT INSTANTIABLE", 0, output_port);
  }
  cn = options & DM_CLASS_PREDEFINED;
  if ( cn ) {
    DM_print_char_star(context, " PREDEFINED=", 0, output_port);
    DM_nat1_print(context, &cn, output_port);
  }
}

void
DM_field_options_print (DM_Context *context, 
                        void *p, 
                        void *output_port) {
  DM_field_options options = DM_Content(DM_field_options, p);

  if ( options & DM_FIELD_IS_MUTABLE ) {
    DM_print_char_star(context, " MUTABLE", 0, output_port);
  }
  if ( options & DM_FIELD_IS_VOLATILE ) {
    DM_print_char_star(context, " VOLATILE", 0, output_port);
  }
  if ( options & DM_FIELD_IS_LOCAL ) {
    DM_print_char_star(context, " LOCAL", 0, output_port);
  }
  if ( options & DM_FIELD_IS_PRE_SENDABLE ) {
    DM_print_char_star(context, " PRESENDABLE", 0, output_port);
  }
  if ( options & DM_FIELD_IS_SECRET ) {
    DM_print_char_star(context, " SECRET", 0, output_port);
  }
}

void
DM_port_options_print (DM_Context *context, 
                       void *p,
                       void *output_port) {
  DM_port_options options = DM_Content(DM_port_options, p);

  if ( options & DM_PORT_TO_READ ) {
    DM_print_char_star(context, " INPUT", 0, output_port);
  }
  if ( options & DM_PORT_TO_WRITE ) {
    DM_print_char_star(context, " OUTPUT", 0, output_port);
  }
  if ( options & DM_PORT_IS_WRITABLE ) {
    DM_print_char_star(context, " WRITABLE", 0, output_port);
  }
  if ( options & DM_PORT_IS_READABLE ) {
    DM_print_char_star(context, " READABLE", 0, output_port);
  }
  if ( options & DM_PORT_IS_DRIED ) {
    DM_print_char_star(context, " DRIED", 0, output_port);
  }
  if ( options & DM_PORT_IS_PEERLESS ) {
    DM_print_char_star(context, " PERRLESS", 0, output_port);
  }
  if ( options & DM_PORT_IS_CLOSED ) {
    DM_print_char_star(context, " CLOSED", 0, output_port);
  }
  if ( options & DM_PORT_RUNS_DMEROON ) {
    DM_print_char_star(context, " runs_DMEROON", 0, output_port);
  }
  if ( options & DM_PORT_FAILED_SETSOCKOPT ) {
    DM_print_char_star(context, " failed_SETSOCKOPT", 0, output_port);
  }
  if ( options & DM_PORT_FAILED_RETRIED_WRITE ) {
    DM_print_char_star(context, " failed_RETRIED_WRITE", 0, output_port);
  }
  if ( options & DM_PORT_FAILED_WRITE ) {
    DM_print_char_star(context, " failed_WRITE", 0, output_port);
  }
  if ( options & DM_PORT_FAILED_READ ) {
    DM_print_char_star(context, " failed_READ", 0, output_port);
  }
  if ( options & DM_PORT_FAILED_BIND ) {
    DM_print_char_star(context, " failed_BIND", 0, output_port);
  }
  if ( options & DM_PORT_FAILED_LISTEN ) {
    DM_print_char_star(context, " failed_LISTEN", 0, output_port);
  }
  if ( options & DM_PORT_FAILED_ACCEPT ) {
    DM_print_char_star(context, " failed_ACCEPT", 0, output_port);
  }
  if ( options & DM_PORT_FAILED_CLOSE ) {
    DM_print_char_star(context, " failed_CLOSE", 0, output_port);
  }
  if ( options & DM_PORT_FAILED_CONNECT ) {
    DM_print_char_star(context, " failed_CONNECT", 0, output_port);
  }
  if ( options & DM_PORT_FAILED_SOCKET ) {
    DM_print_char_star(context, " failed_SOCKET", 0, output_port);
  }
  if ( options & DM_PORT_PARTIALLY_DRAINED ) {
    DM_print_char_star(context, " partially_DRAINED", 0, output_port);
  }
  if ( options & DM_IP_FAILED_GETHOSTBYNAME ) {
    DM_print_char_star(context, " failed_gethostbyname", 0, output_port);
  }
  if ( options & DM_PORT_WITH_RECURSIVE_ENCODING ) {
    DM_print_char_star(context, " recENCODING", 0, output_port);
  }
  if ( options & DM_PORT_WITHOUT_PREFETCHING ) {
    DM_print_char_star(context, " withoutPREFETCH", 0, output_port);
  }
  if ( options & DM_PORT_WITHOUT_CACHING ) {
    DM_print_char_star(context, " withoutCACHING", 0, output_port);
  }
}

void
DM_site_options_print (DM_Context *context, 
                       void *p,
                       void *output_port) {
  DM_site_options options = DM_Content(DM_site_options, p);

  if ( options & DM_SITE_IS_USABLE ) {
    DM_print_char_star(context, " USABLE", 0, output_port);
  }
  if ( options & DM_SITE_WANTS_FACTS ) {
    DM_print_char_star(context, " WANTS-FACTS", 0, output_port);
  }
}

void
DM_external_options_print (DM_Context *context, 
                           void *p,
                           void *output_port) {
  DM_external_options options = DM_Content(DM_external_options, p);

  if ( options & DM_ENTRY_IS_COPYABLE ) {
    DM_print_char_star(context, " COPYABLE", 0, output_port);
  }
  if ( options & DM_EXTERNAL_IS_EXTERNALIZED ) {
    DM_print_char_star(context, " EXTERNALIZED", 0, output_port);
  }
}

void
DM_context_options_print (DM_Context *context, 
                          void       *p, 
                          void       *output_port) {
  DM_context_options options = DM_Content(DM_context_options, p);

  if ( options & DM_CONTEXT_CHECK_DMEROON ) {
    DM_print_char_star(context, " CHECK_DMEROON", 0, output_port);
  }
  if ( options & DM_CONTEXT_CHECK_CLASS ) {
    DM_print_char_star(context, " CHECK_CLASS", 0, output_port);
  }
  if ( options & DM_CONTEXT_ABORT_IMMEDIATELY ) {
    DM_print_char_star(context, " ABORT_IMMEDIATELY", 0, output_port);
  }
  if ( options & DM_CONTEXT_VERBOSE ) {
    DM_print_char_star(context, " VERBOSE", 0, output_port);
  }
}

void
DM_file_options_print (DM_Context *context,
                       void       *p,
                       void       *output_port )
{
  DM_file_options options = DM_Content(DM_file_options, p);
  options = options;
}

void
DM_request_options_print (DM_Context *context,
                          void       *p,
                          void       *output_port )
{
  DM_request_options options = DM_Content(DM_request_options, p);

  if ( options & DM_REQUEST_WANTS_ANSWER ) {
    DM_print_char_star(context, " WantsAnswer", 0, output_port);
  }
}

void
DM_handler_code_print (DM_Context *context,
                       void       *p,
                       void       *output_port )
{
  DM_handler_code fn = DM_Content(DM_handler_code, p);
  DM_nat1 i = 0;

  for ( i=0 ; DM_handler_code_table[i].code != 0 ; i++ ) {
    if ( fn == DM_handler_code_table[i].handler ) {
      DM_print_char_star(context, DM_handler_code_table[i].name, 
                         0, output_port);
        return;
    } else {
      continue;
    }
  }
  DM_Abort(context, DME_UNKNOWN_HANDLER_CODE);
}

/* This procedure is used for tests from the DM_prtcl_initialize
 * procedure to check the DM_ProtectWithFreshContext macro that poses
 * problems with sparc machines wrt automatic variables. It is defined
 * there to be not inlined by too clever compilers. */

void
DM_prtcl_test (DM_Context *context,
               DM_Object  *o1,
               DM_Object  *o2 ) 
{
  if ( o1 == DM_TheConstant(true) ) {
    if ( o2 == DM_TheConstant(false) ) {
      return;	/* OK */
    }
  }
  DM_Abort(context, DME_PROBLEM_WITH_PROTECT);
}

/*
 * Initialization
 */

void
DM_print_initialize (DM_Context *context)
{
  DM_LinkVersionStringIntoSite(dmprint);

  /* Test whether the %p printf directive is conformant to the DM_0X
   * setting. On most machine, %p does not prefix the address with "0x".
   */
  { char line [80];

    sprintf(line, "%p", line);
    if (   (line[0] == '0') 
        && (   (line[1] == 'x')
            || (line[1] == 'X') ) ) {
      /* %p prefixes with 0x so DM_0X should be empty. */
      if ( strlen("" DM_0X) != 0 ) {
        DM_Abort(context, DME_INCORRECT_0X);
      } 
    } else {
      /* %p does not prefix the address with 0x so DM_0X should define it. */
      if ( strlen("" DM_0X) == 0 ) {
        DM_Abort(context, DME_INCORRECT_0X);
      }
    }
  }

}

/* end of dmprint.c */
