/* $Id: dmrdobj.h,v 1.36 1998/05/25 19:01:29 queinnec Exp $
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 
 * These are the constants used to encode/decode DMeroon objects.
 * The encoding has been choosen to be humanly writable!
 */

#define DM_NIL_COMMAND			65	/* A 	0x41 */
#define DM_TRUE_COMMAND			66	/* B 	0x42 */
#define DM_FALSE_COMMAND		67	/* C 	0x43 */
#define DM_UNINITIALIZED_COMMAND	68	/* D 	0x44 */
#define DM_NULL_COMMAND			69	/* E 	0x45 */
#define DM_EMITTING_SITE_COMMAND	70	/* F	0x46 */
#define DM_RECEIVING_SITE_COMMAND	71	/* G	0x47 */
#define DM_SITE_COMMAND			72	/* H	0x48 */

#define DM_BIND_COMMAND			77	/* M	0x4d */
#define DM_LIMIT_COMMAND		78	/* N	0x4e */
#define DM_PREDEFINED_FIELD_COMMAND	79	/* O 	0x4f */
#define DM_PREDEFINED_CLASS_COMMAND	80	/* P 	0x50 */
#define DM_PREDEFINED_TYPE_COMMAND	81	/* Q 	0x51 */
#define DM_ALLOCATE_COMMAND		82	/* R 	0x52 */
#define DM_CLASS_COMMAND		83	/* S 	0x53 */
#define DM_SYMBOL_COMMAND		84	/* T 	0x54 */
#define DM_EMITTER_REFERENCE_COMMAND	85	/* U 	0x55 */
#define DM_REMOTE_REFERENCE_COMMAND	86	/* V 	0x56 */
#define DM_SET_COPYABLE_COMMAND		87	/* W 	0x57 */
#define DM_SET_SHARABLE_COMMAND		88	/* X 	0x58 */
#define DM_DEFINE_EMITTER_COMMAND	89	/* Y 	0x59 */
#define DM_PROG1_COMMAND		90	/* Z	0x5a */
#define DM_PROG2_COMMAND		91	/* [	0x5b */
#define DM_RECEIVER_REFERENCE_COMMAND	92	/* \	0x5c */
#define DM_DEFINE_REMOTE_SITE_COMMAND	93	/* ] 	0x5d */
#define DM_FILL_COMMAND			94	/* ^	0x5e */
#define DM_FIELD_COMMAND		95	/* _ 	0x5f */

#define DM_RESET_STACK_COMMAND		96	/* `	0x60 */
#define DM_REFER_WITH_NAT1_COMMAND	97	/* a 	0x61 */
#define DM_REFER_WITH_NAT2_COMMAND	98	/* b 	0x62 */
#define DM_REFER_WITH_NAT3_COMMAND	99	/* c 	0x63 */
#define DM_INSERT_WITH_NAT1_COMMAND	100	/* d	0x64 */
#define DM_INSERT_WITH_NAT2_COMMAND	101	/* e	0x65 */
#define DM_INSERT_WITH_NAT3_COMMAND	102	/* f	0x66 */
#define DM_PUSH_STACK_COMMAND		103	/* g 	0x67 */
#define DM_POP_STACK_COMMAND		104	/* h	0x68 */
#define DM_SWAP_STACK_COMMAND		105	/* i	0x69 */
#define DM_DUP_STACK_COMMAND		106	/* j	0x6a */
#define DM_RESET_CACHE_COMMAND		107	/* k	0x6b */

#define DM_DECREMENT_COMMAND		112	/* p 	0x70 */
#define DM_RECEIVE_REQUEST_COMMAND	114	/* r	0x72 */
#define DM_RECEIVE_ANSWER_COMMAND	115	/* s	0x73 */
#define DM_UPDATE_CLOCK_COMMAND		116	/* t	0x74 */

/* Additional commands that help writing self-executable modules
 * ie modules using the #! convention.
 * CR-LF-whitespace are ignored if in command position
 * # ignores the rest of the line until the next CR or LF.
 */

#define DM_CR_COMMAND			'\r'	/*	0x0d */
#define DM_LF_COMMAND			'\n'	/*	0x0a */
#define DM_BLANK_COMMAND		' '	/*	0x20 */
#define DM_SHARP_COMMAND		'#'	/*	0x23 */

/*
 * Function entry point.
 */

extern DM_Object*
DM_read_object (DM_Context  *context,
                DM_InBuffer *inbuffer);

extern void 
DM_rdobj_initialize (DM_Context *context);

/* end of dmrdobj.h */
