/* $Id: dmscm.h,v 1.13 1998/03/19 19:03:53 queinnec Exp $ 
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////

 * This file defines the generic Scheme interface. Here follow the
 * signatures of the Scheme functions; they are processed to give the
 * additional glue code that may be needed to interface DMeroon. Names
 * of functions with side-effects and names of predicates are changed
 * as specified in Bigloo/sig2bgl.scm for Bigloo.

 * ATTENTION: despite the Scheme appearance, these are the names of the
 * C functions or variables to externalize from this file. The precise
 * equivalent Scheme name may easily be deduced as in sig2bgl.scm. 

	SIGNATURES:
 (boolean  (DMscm_is_object anything))
 (Site     (DMscm_initialize))
 (boolean  (DMscm_is_a Object Class))
 (boolean  (DMscm_is_a_subclass Class Class))
 (Class    (DMscm_class_of Object))
 (Site     (DMscm_site_of Object))
 (Clock    (DMscm_clock_of Object))
 (Object   (DMscm_receive nat3))
 (void     (DMscm_send Object Site))
 (Site     (DMscm_connect string fixnum string))
 (Class    (DMscm_create_subclass Class string options 
                                  string options Type Type Class ))
 (Class    (DMscm_smash_class Class string options index))
 (Site     (DMscm_set_site_of Object Site))
 (Clock    (DMscm_set_clock_of Object Clock))
 (boolean  (DMscm_set_sharable Object))
 (boolean  (DMscm_set_copyable Object))
 (boolean  (DMscm_set_initialized Object))
 (anything (DMscm_is_initialized Object))
 (boolean  (DMscm_eq Object Object))
 (anything (DMscm_field_value Object Field))
 (anything (DMscm_indexed_field_value Object Field index))
 (anything (DMscm_set_field_value Object Field anything))
 (anything (DMscm_set_indexed_field_value Object Field index anything))
 (index    (DMscm_indexed_field_length Object Field))
 (Type     (DMscm_field_item_type Field))
 (Type     (DMscm_field_repetition_type Field))
 (boolean  (DMscm_serve anything))
 (fixnum   (DMscm_set_site_debug_level nat3))

 * Many additional variables are useful for the interface but, currently,
 * classes, fields are automatically extracted from the dmstruct.c file.
 * We just give a selection of Types here.

	SIGNATURES:
 (Type DM_nought_type)
 (Type DM_nat1_type)
 (Type DM_nat2_type)
 (Type DM_nat3_type)
 (Type DM_nat4_type)
 (Type DM_reference_type)
 (Type DM_data_type)
 (Type DM_code_type)
 (Type DM_int1_type)
 (Type DM_int2_type)
 (Type DM_int3_type)
 (Type DM_int4_type)
 (Type DM_char1_type)
 (Type DM_netnat2_type)
 (Type DM_netnat3_type)
 (Type DM_float1_type)
 (Type DM_float2_type)
 (Type DM_float3_type)
 (Type DM_date_type)
 (Type DM_key_type)

 * Since the Object class is done by hand, its signature does not appear
 * in dmstruct.c. Add also some other variables worthwhile for Scheme

 SIGNATURES:
 (Class DM_Object_class)
 (Object DM_true)
 (Object DM_false)
 (Object DM_nil)

 */

extern DM_SupportPointer
DMscm_is_object (DM_SupportPointer wrapped_p);

extern DM_SupportPointer
DMscm_initialize (void);

extern DM_SupportPointer
DMscm_is_a (DM_SupportPointer wrapped_object,
            DM_SupportPointer wrapped_class );

extern DM_SupportPointer
DMscm_is_a_subclass (DM_SupportPointer wrapped_subclass,
                     DM_SupportPointer wrapped_class );

extern DM_SupportPointer
DMscm_class_of (DM_SupportPointer wrapped_object);

extern DM_SupportPointer
DMscm_site_of (DM_SupportPointer wrapped_object);

extern DM_SupportPointer
DMscm_clock_of (DM_SupportPointer wrapped_object);

extern DM_SupportPointer
DMscm_receive (DM_SupportPointer wrapped_duration);

extern DM_SupportPointer
DMscm_send (DM_SupportPointer wrapped_object,
            DM_SupportPointer wrapped_site );

extern DM_SupportPointer
DMscm_connect (DM_SupportPointer wrapped_hostname,
               DM_SupportPointer wrapped_portnumber,
               DM_SupportPointer wrapped_password );

extern DM_SupportPointer
DMscm_create_subclass (DM_SupportPointer wrapped_superclass,
                       DM_SupportPointer wrapped_classname,
                       DM_SupportPointer wrapped_classoptions,
                       DM_SupportPointer wrapped_fieldname,
                       DM_SupportPointer wrapped_fieldoptions,
                       DM_SupportPointer wrapped_reptyp,
                       DM_SupportPointer wrapped_itemtyp,
                       DM_SupportPointer wrapped_referred_class );

extern DM_SupportPointer
DMscm_smash_class (DM_SupportPointer wrapped_class,
                   DM_SupportPointer wrapped_classname,
                   DM_SupportPointer wrapped_classoptions,
                   DM_SupportPointer wrapped_level );

extern DM_SupportPointer
DMscm_set_site_of (DM_SupportPointer wrapped_object,
                   DM_SupportPointer wrapped_site );

extern DM_SupportPointer
DMscm_set_clock_of (DM_SupportPointer wrapped_object,
                    DM_SupportPointer wrapped_clock );

extern DM_SupportPointer
DMscm_set_sharable (DM_SupportPointer wrapped_object);

extern DM_SupportPointer
DMscm_set_copyable (DM_SupportPointer wrapped_object);

extern DM_SupportPointer
DMscm_is_initialized (DM_SupportPointer wrapped_object);

extern DM_SupportPointer
DMscm_eq (DM_SupportPointer wrapped_object1,
          DM_SupportPointer wrapped_object2 );

extern DM_SupportPointer
DMscm_field_value (DM_SupportPointer wrapped_object,
                   DM_SupportPointer wrapped_field );

extern DM_SupportPointer
DMscm_indexed_field_value (DM_SupportPointer wrapped_object,
                           DM_SupportPointer wrapped_field,
                           DM_SupportPointer wrapped_index );

extern DM_SupportPointer
DMscm_set_field_value (DM_SupportPointer wrapped_object,
                       DM_SupportPointer wrapped_field,
                       DM_SupportPointer wrapped_value );

extern DM_SupportPointer
DMscm_set_indexed_field_value (DM_SupportPointer wrapped_object,
                               DM_SupportPointer wrapped_field,
                               DM_SupportPointer wrapped_index,
                               DM_SupportPointer wrapped_value );

extern DM_SupportPointer
DMscm_indexed_field_length (DM_SupportPointer wrapped_object,
                            DM_SupportPointer wrapped_field );

extern DM_SupportPointer
DMscm_serve (DM_SupportPointer wrapped_duration);

extern DM_SupportPointer
DMscm_set_site_debug_level (DM_SupportPointer wrapped_natural);

extern void
DM_scm_initialize (DM_Context* context);

/* end of dmscm.h */
