/* $Id: dmsite.c,v 1.75 1998/05/31 14:51:28 queinnec Exp $ 
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 *
 * This file defines the site object which is the prominent intercessor
 * with DMeroon. The site allows to inquire DMeroon, to publish information.
 * Some predefined information is exported as defined in this file.
 * As usual there is a DM_site_initialize procedure.
 */

#include "dmeroon.h"

DM_DefineVersionString(dmsite,
  "$Id: dmsite.c,v 1.75 1998/05/31 14:51:28 queinnec Exp $");

/* {{{ Static objects
 *     {{{ Name of the support language

 * Record the name of the support language.
 * It is first turned into a DMeroon string that will be referenced 
 * by the site object.
 */

DM_DefineStringWithCstring(support_language, DM_SUPPORT_LANGUAGE);

/* }}} 
 *     {{{ Published information
 * Publish statically some information.
 * The structure of the initial dictionary is:
 *  DMeroon                       [ reserved for DMeroon ]
 *    site
 *    classes				040
 *      Object_class			100
 *      Class_class			101
 *      ...
 *    constants				030
 *      nil 				031
 *    sources				020
 *      rt				021
 *      ...
 *    vector of DMeroon types		010
 *    DMeroon original repository	
 *  Icslas                         [ reserved for Icslas ]
 *  user                           [ free for users ]
 *
 * NOTE: add a comment field to explain the published value ???
 * NOTE: define immutable variants of dictionaries ???
 */

DM_Publish(010,"vector of DMeroon types",
        DM_ReferenceTo(DM_implemented_types),
        DM_NULL
        );
/* Create a node to be filled later on. */
DM_Publish(020,"sources",
        DM_NULL,
        DM_ReferenceTo(DM_010_dictionary)
        );
/* Start a subdictionary for constants. */
DM_Publish(031,"nil",
        DM_NULL,
        DM_TheConstant(nil)
        );
DM_Publish(032,"true",
        DM_ReferenceTo(DM_031_dictionary),
        DM_TheConstant(true)
        );
DM_Publish(033,"false",
        DM_ReferenceTo(DM_032_dictionary),
        DM_TheConstant(false)
        );
DM_Publish(034,"uninitialized",
        DM_ReferenceTo(DM_033_dictionary),
        DM_TheConstant(uninitialized)
        );
DM_Publish(030,"constants",
        DM_ReferenceTo(DM_034_dictionary),
        DM_ReferenceTo(DM_020_dictionary)
        );
/* Start a subdictionary for classes. */
DM_Publish(040,"classes",
        DM_ReferenceTo(DM_predefined_classes),
        DM_ReferenceTo(DM_030_dictionary)
        );
/* to be continued ... */

/* The site object itself.
 */
DM_Publish(060,"site",
        DM_ReferenceTo(DM_site),
        DM_ReferenceTo(DM_040_dictionary)
        );

/* RESOURCE DM_user_dictionary
 * This is the user dictionary reserved for users. It is initially empty. 
 */

DM_Publish(200,"user",
        DM_NULL,
        DM_NULL                 /* To be filled below. */
        );

/* The final node, leave the DMeroon dictionary as first entry. */
DM_Publish(DMeroon,"DMeroon",
        DM_ReferenceTo(DM_060_dictionary),
        DM_ReferenceTo(DM_200_dictionary)
        );

/* }}} 
 *     {{{ The Site instance
 * This object describes the current site where is run DMeroon. It is
 * initialized in the dmnet.c file. 

 * NOTE: the 'version' field is, so far, the only float used in DMeroon.
 * Is it really useful for float-less languages ???         FUTURE

 * RESOURCE DM_clonable_context_object
 * This is an instance of Context that will be cloned whenever DMeroon
 * needs a Context and none is provided. This Context is uninitialized
 * and is reachable via the current site, it may therefore be mutated.
 */

DM_Context_wrapped DM_clonable_context_object = {
  DM_SupportPrefixInitialization(DM_Context_wrapped, 
                                 DM_clonable_context_object)
  DM_ReferenceToClass(Context),
  DM_ProxyInitialization(),
  { { { 
    DM_CodeOf(DM_abort),                /* handler */
    DM_CONTEXT_DEFAULT_OPTIONS,         /* options */
    0,					/* errcode */
    NULL,				/* file */
    0,					/* line */
    0,					/* last errno */
    DM_NULL,				/* previous Context */
    NULL,				/* continuation */
    (void*)stderr			/* error stream */
  } } }
};

/* RESOURCE DM_site_object
 * This is the current site object.
 */

DM_Site_255_wrapped DM_site_object = {
  DM_SupportPrefixInitialization(DM_Site_255_wrapped,
                                 DM_site_object)
  DM_ReferenceToClass(Site),
  DM_ProxyInitialization(),
  { { {
    0L,					/* IP address (to be filled) */
    56423,				/* port (to be patched) */
    0,					/* date (to be filled) */
    DM_SITE_DEFAULT_OPTIONS|DM_SITE_IS_USABLE|DM_SITE_WANTS_FACTS, /* options */
    0,					/* processid (to be filled) */
    DM_RELEASE_NUMBER,			/* version */
    DM_NULL,				/* versions */
    (DM_Clock*)DM_ReferenceTo(DM_general_clock),  /* clock (see dmcomm.c) */
    0,				/* clocker */
    0,				/* ticker */
    DM_CYCLIC_DURATION,		/* cyclic_duration */
    DM_NORMAL_DURATION,		/* normal_duration */
    DM_ABNORMAL_DURATION,	/* abnormal_duration */
    DM_DMEROON_FAVORITE_PERIOD, /* period (initialized in dmsched.c) */
    0,                          /* debug level (initialized below) */
    0,                          /* compress threshold (see dmzlib) */
    (DM_Context*)DM_ReferenceTo(DM_clonable_context),
                                /* context (initialized by support language) */
    DM_ReferenceToString(support_language),  /* support */
    (DM_Dictionary*)DM_ReferenceTo(DM_DMeroon_dictionary),  /* information */
    DM_NULL,    		/* connection */
    DM_NULL,			/* route */
    0,				/* amount */
    DM_NULL,    		/* exits (to be filled) */
    DM_NULL,    		/* entries (to be filled) */
    DM_NULL,    		/* ports (filled in dmnet.c) */
    DM_NULL,    		/* objects */
    DM_CodeOf(DM_accumulate_object), 	/* handler */
    DM_NULL,    		/* sites (to be filled) */
    DM_NULL,    		/* oblist (to be filled) */
    DM_NULL,    		/* protocol (filled in dmnet.c) */
    DM_NULL,                    /* urlenv (filled in dmact.c) */
    DM_NULL,			/* arrived */
    NULL,  	  		/* threads */
    DM_NULL,			/* waiter */
    DM_NULL,			/* freed */
    DM_PrefixSize(),		/* prefix size */
    (DM_DMeroonType*)DM_FromC(&DM_MOST_STRINGENT_TYPE_OBJECT),	/* type */
    DM_NULL,                    /* password */
    255,			/* name length */
    { "1234567890" "1234567890" "1234567890" "1234567890" "1234567890"
      "1234567890" "1234567890" "1234567890" "1234567890" "1234567890"
      "1234567890" "1234567890" "1234567890" "1234567890" "1234567890"
      "1234567890" "1234567890" "1234567890" "1234567890" "1234567890"
      "1234567890" "1234567890" "1234567890" "1234567890" "1234567890"
      "12345"
    }
  } } }
};

/* }}}
 * }}}
 * {{{ Initialization
 *
 * Get the name of the current host and store it in the DMeroon string
 * instance named DM_site_name_string. Fetch and store the IP number
 * as well.
 */

void
DM_site_initialize (DM_Context *context) {
  DM_Site        *site      = DM_TheCurrentSite();
  char* const    hostname  = &(site->name[0]);
  struct hostent *host;
  struct in_addr *ip;
  
  DM_LinkVersionStringIntoSite(dmsite);

#if defined(DM_DEBUG)
  DM_TheCurrentSite()->debug_level = DM_DEBUG;
  DM_DebugAtLevel(18,
    fprintf(DM_FindErrorStream(context), "Debug_level=%d.\n", 
            DM_TheCurrentSite()->debug_level );
);
#endif /* DM_DEBUG */

  /* Record starting time. */
  site->date = DM_get_date(context);

  /* Get the name of the current host. */
  if ( gethostname(hostname, MAXHOSTNAMELEN) != 0 ) {
    DM_Abort(context, DME_FAILED_GETHOSTNAME);
  };

  DM_DebugAtLevel(50,
    fprintf(DM_FindErrorStream(context), "Host name=\"%s\".\n", hostname);
);

  /* Get the official name of the current host. If found it will
     replace the previously stored one. */
  if ( (host = gethostbyname(hostname)) == ((struct hostent*)0) ) {
    DM_Abort(context, DME_FAILED_GETHOSTBYNAME);
  };

  DM_DebugAtLevel(5,
    fprintf(DM_FindErrorStream(context), 
            "Official host name=\"%s\".\n", host->h_name);
);

  /* Record this full name in the site DMeroon instance. */
  strcpy(hostname, host->h_name);

  /* Record the IP address in the site DMeroon instance */
  ip = ((struct in_addr*) host->h_addr_list[0]);
  if ( AF_INET != host->h_addrtype ) 
    DM_Abort(context, DME_UNKNOWN_AF_TYPE);
  site->address = (DM_nat3)ip->s_addr;

  DM_DebugAtLevel(0,
    fprintf(DM_FindErrorStream(context), 
            "Internet address=\"%s\".\n", inet_ntoa(*ip));
);

  /* Unfortunately some machines do not answer their real official
     host name (fqdn for fully qualified domain name). If there is no
     dot in the official name then we use the IP dotted name
     instead. This is sufficient for the generation of correct html
     links. But we may also ask the DNS for the real name. */
  if ( strchr(hostname, '.') == NULL ) {
    char *ipname = inet_ntoa(*ip);

    /* No dot means incorrect cannonical host name: ask the DNS. */
    if (    (host = gethostbyaddr(ipname, strlen(ipname), AF_INET))
         == ((struct hostent*)0) ) {
      /* Backup: take the IP name as fqdn. 
       * BTW regenerate the ipname since gethostbyaddr modifies it! */
      ipname = inet_ntoa(*ip);
      strcpy(hostname, ipname);
    } else {
      /* Record this fqdn in the site DMeroon instance. */
      strcpy(hostname, host->h_name);
    }

    DM_DebugAtLevel(5,
      fprintf(DM_FindErrorStream(context),
              "Fixed final host name=\"%s\".\n", hostname);
);

  }
  /* Record the length of the host name. */
  site->name_length = (DM_nat3) (strlen(hostname)+1);

  /* record the processid in the site instance. 
   * Record the name (email) of the USER ???                  FUTURE */
  site->processid = getpid();

  /* The site->sites hashtable will be initialized in dmcomm. */

#if defined(DM_MONOTHREAD_CONTEXT)
  { DM_BoundedStack *stack = DM_Allocate1(context, BoundedStack, 2);

    /* initialize the stack of pending requests. */
    stack->index  = 0;
    site->threads = stack;
    if ( context && DM_HasNotOption(context, DM_CONTEXT_DONT_INITIALIZE) ) {
      DM_SetSharable(context, stack);
    }
  }
#endif

  DM_DebugAtLevel(50,
    fprintf(DM_FindErrorStream(context), 
            "Site is " DM_0X "%p\n", site);
);

}

/* }}}
 * end of dmsite.c */
