/* $Id: dmsite.h,v 1.31 1998/05/25 19:01:30 queinnec Exp $ 
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 */

#ifndef DM_SITE_H
#define DM_SITE_H

/*
 * This file defines the exportations of the dmsite.c file.

 * An unknown site is a Site instance which is not completely filled.
 * It is created when receiving a remote reference from a never seen
 * Site. See the dmrdobj.c file.
 */

#define DM_SITE_DEFAULT_OPTIONS 0

#define DM_SITE_IS_USABLE	(1<<0)
#define DM_SITE_WANTS_FACTS	(1<<1)
#define DM_SITE_SUPPORTS_ZLIB	(1<<2)

/******************************************************************
 *                  Additional configuration parameters.
 *
 * DM_SAFETY should be set to 1 if some safety is wanted. A zero value
 * means that (1) some internal values are published in the site object to
 * ease debug (2) URLscripts are allowed.
 */

#define DM_SAFETY 0

/******************************************************************
 *                        Data structures
 *
 * This is the type for preallocated site that will be initialized
 * at initialization time. Generally the preallocated string will be
 * chopped to the correct length.
 */

typedef DM_GenerateSiteStruct(255) DM_Site_255;

DM_DefineWrappedType(DM_Site_255);

extern DM_Site_255_wrapped DM_site_object;

extern void DM_site_initialize (DM_Context *context);

/*******************************************************************
 * Macro to publish informations.
 */

#define DM_GenerateSpecialDictionaryStruct(_0,code) \
struct DM_Dictionary_for_##code { \
  DM_Dictionary* rest; \
  DM_Object* value; \
  DM_nat1 name_length; \
  DM_char1 name[_0]; \
}

#define DM_Publish(code,string,val,next) \
typedef DM_GenerateSpecialDictionaryStruct(sizeof(string),code) \
        DM_Dictionary_for_##code; \
DM_DefineWrappedType(DM_Dictionary_for_##code); \
static DM_Dictionary_for_##code##_wrapped DM_##code##_dictionary_object = { \
  DM_SupportPrefixInitialization(DM_Dictionary_for_##code##_wrapped, \
                                 DM_##code##_dictionary_object) \
  DM_ReferenceToClass(Dictionary), \
  DM_ProxyInitialization(), \
  { { { \
    (DM_Dictionary*)(next), \
    (DM_Object*)(val), \
    sizeof(string), \
    string, \
  } } } \
}

#endif /* DM_SITE_H */

/* end of dmsite.h */
