/* $Id: dmtypes.c,v 1.32 1998/06/10 16:31:31 queinnec Exp $ 
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////

 * DOCUMENTATION dmtypes.c
 * This file defines and initializes the DMeroon (and C) Types. 
 */

#include "dmeroon.h"

DM_DefineVersionString(dmtypes,
  "$Id: dmtypes.c,v 1.32 1998/06/10 16:31:31 queinnec Exp $");

/* {{{ Type objects
 *    {{{ NOUGHT type
 *
 * It is defined by hand since it is very particular. Its size is zero,
 * its index is zero.
 */

DM_DMeroonType_nought_struct_wrapped DM_nought_type_object = { 
  DM_SupportPrefixInitialization(DM_DMeroonType_wrapped,
                                 DM_nought_type_object)
  DM_ReferenceToClass(DMeroonType),
  DM_ProxyInitialization(),
  { { {
    DM_nought_index,
    1,				/* no alignment restriction */
    0,				/* size */
    0,				/* repeated size */
    DM_CodeOf(DM_no_print),
    DM_CodeOf(DM_send_nought),
    DM_CodeOf(DM_receive_nought),
    DM_NULL,			/* No associated class */
    DM_NULL,			/* No associated Ctype */
    sizeof("nought"),
    "nought"
  } } }
};

/*    }}}
 *    {{{ other regular DMeroon Types

 * NOTE: some of these types may be invalidated by DM_types_initialize if
 * there is no convenient C type.
 */

DM_DefineTypeForDMeroon(nat1,Nat1);
DM_DefineTypeForDMeroon(nat2,Nat2);
DM_DefineTypeForDMeroon(nat3,Nat3);
DM_DefineTypeForDMeroon(nat4,Nat4);
DM_DefineTypeForDMeroon(reference,Reference);
DM_DefineTypeForDMeroon(data,Data);
DM_DefineTypeForDMeroon(code,Code);
DM_DefineTypeForDMeroon(int1,Int1);
DM_DefineTypeForDMeroon(int2,Int2);
DM_DefineTypeForDMeroon(int3,Int3);
DM_DefineTypeForDMeroon(int4,Int4);
DM_DefineTypeForDMeroon(char1,Char1);
DM_DefineTypeForDMeroon(netnat2,NetNat2);
DM_DefineTypeForDMeroon(netnat3,NetNat3);
#ifdef DM_HAS_UNICODE_ENTITIES
DM_DefineTypeForDMeroon(unicode,Unicode);
#else
DM_DefineAbsentTypeForDMeroon(unicode,Unicode);
#endif
DM_DefineTypeForDMeroon(float1,Float1);
DM_DefineTypeForDMeroon(float2,Float2);
/* float3 will be made absent if incorrect (see dm_kernel_initialize) */
DM_DefineTypeForDMeroon(float3,Float3);
DM_DefineSpecialTypeForDMeroon(offset);
DM_DefineSpecialTypeForDMeroon(date);
DM_DefineSpecialTypeForDMeroon(key);
DM_DefineSpecialTypeForDMeroon(class_options);
DM_DefineSpecialTypeForDMeroon(field_options);
DM_DefineSpecialTypeForDMeroon(port_options);
DM_DefineSpecialTypeForDMeroon(site_options);
DM_DefineSpecialTypeForDMeroon(external_options);
DM_DefineSpecialTypeForDMeroon(context_options);
DM_DefineSpecialTypeForDMeroon(file_options);
DM_DefineSpecialTypeForDMeroon(request_options);
DM_DefineSpecialTypeForDMeroon(handler_code);

/*    }}}
 *    {{{ C Types
 * They are needed by users of DMeroon that want to access raw C structures.

 * C types also have an index (that may change in some future that's
 * why they are not exported in dmtypes.h).  These C types must be
 * inserted in the implemented_types vector at this position.
 */

#define DM_C_char_index				240
#define DM_C_signed_char_index			241
#define DM_C_short_int_index			242
#define DM_C_int_index				243
#define DM_C_long_int_index			244
#define DM_C_unsigned_char_index		245
#define DM_C_unsigned_short_int_index		246
#define DM_C_unsigned_int_index			247
#define DM_C_unsigned_long_int_index		248
#define DM_C_float_index			249
#define DM_C_double_index			250
#define DM_C_long_double_index			251
#define DM_C_void_star_index			252
#define DM_C_function_star_index		253
#define DM_C_unsigned_long_long_int_index	254
#define DM_C_long_long_int_index		255  /* -1 for a int1 */

DM_DefineTypeForC(C_char);
DM_DefineTypeForC(C_signed_char);
DM_DefineTypeForC(C_short_int);
DM_DefineTypeForC(C_int);
DM_DefineTypeForC(C_long_int);
DM_DefineTypeForC(C_unsigned_char);
DM_DefineTypeForC(C_unsigned_short_int);
DM_DefineTypeForC(C_unsigned_int);
DM_DefineTypeForC(C_unsigned_long_int);
DM_DefineTypeForC(C_float);
DM_DefineTypeForC(C_double);
DM_DefineTypeForC(C_long_double);
DM_DefineTypeForC(C_void_star);
DM_DefineTypeForC(C_function_star);
#ifdef DM_HAS_LONG_LONG_ENTITIES
DM_DefineTypeForC(C_unsigned_long_long_int);
DM_DefineTypeForC(C_long_long_int);
#else
DM_DefineAbsentTypeForC(C_unsigned_long_long_int);
DM_DefineAbsentTypeForC(C_long_long_int);
#endif

/*    }}}
 * }}}
 * {{{ Vector of types
 *
 * RESOURCE DM_implemented_types_object
 * The ImmutableVector of all implemented types. If at a given index
 * there is no type then it contains DM_NULL.  The type of this vector
 * allows for 256 different types.

 * There's a hack here that skeletizes the vector to only contain the
 * DMeroon types.  This leaves room for additional types but to add
 * them on a site will not make them migratable since the other sites
 * will ignore these types.  The C types are near the end of the
 * vector but no definite index is associated to them.

 * Another hack is that the C type is DM_ImplementedTypesVector_256
 * while the DMeroon class is ImmutableVector. This is to ease
 * compilation checks since its type tells that it contains DMeroon
 * types only.
 */

DM_ImplementedTypesVector_256_wrapped DM_implemented_types_object = {
  DM_SupportPrefixInitialization(DM_ImplementedTypesVector_256_wrapped,
                                 DM_implemented_types_object)
  DM_ReferenceToClass(ImmutableVector),
  DM_ProxyInitialization(),
  { { {
    DM_MAXIMUM_TYPES_NUMBER,
    {
      DM_ReferenceToDMeroonType(nought),
      DM_ReferenceToDMeroonType(nat1),
      DM_ReferenceToDMeroonType(nat2),
      DM_ReferenceToDMeroonType(nat3),
      DM_ReferenceToDMeroonType(nat4),
      DM_ReferenceToDMeroonType(reference),
      DM_ReferenceToDMeroonType(data),
      DM_ReferenceToDMeroonType(code),
      DM_ReferenceToDMeroonType(int1),
      DM_ReferenceToDMeroonType(int2),
      DM_ReferenceToDMeroonType(int3),
      DM_ReferenceToDMeroonType(int4),
      DM_ReferenceToDMeroonType(char1),
      DM_ReferenceToDMeroonType(netnat2),
      DM_ReferenceToDMeroonType(netnat3),
      DM_ReferenceToDMeroonType(unicode),
      DM_ReferenceToDMeroonType(float1),
      DM_ReferenceToDMeroonType(float2),
      DM_ReferenceToDMeroonType(float3),
      DM_ReferenceToDMeroonType(offset),
      DM_ReferenceToDMeroonType(date),
      DM_ReferenceToDMeroonType(key),
      DM_ReferenceToDMeroonType(class_options),
      DM_ReferenceToDMeroonType(field_options),
      DM_ReferenceToDMeroonType(port_options),
      DM_ReferenceToDMeroonType(site_options),
      DM_ReferenceToDMeroonType(external_options),
      DM_ReferenceToDMeroonType(context_options),
      DM_ReferenceToDMeroonType(file_options),
      DM_ReferenceToDMeroonType(request_options),
      DM_ReferenceToDMeroonType(handler_code),
      
#define FillWith1Null() DM_NULL
#define FillWith2Null() FillWith1Null(),FillWith1Null()
#define FillWith3Null() FillWith2Null(),FillWith1Null()
#define FillWith4Null() FillWith2Null(),FillWith2Null()
#define FillWith5Null() FillWith4Null(),FillWith1Null()
#define FillWith9Null() FillWith5Null(),FillWith4Null()
#define FillWith10Null() FillWith5Null(),FillWith5Null()
#define FillWith23Null() FillWith10Null(),FillWith10Null(),FillWith3Null()
#define FillWith25Null() FillWith10Null(),FillWith10Null(),FillWith5Null()
#define FillWith48Null() FillWith25Null(),FillWith23Null()
#define FillWith50Null() FillWith25Null(),FillWith25Null()
#define FillWith98Null() FillWith50Null(),FillWith48Null()
#define FillWith100Null() FillWith50Null(),FillWith50Null()

      FillWith100Null(),
      FillWith98Null(),
      FillWith2Null(), FillWith9Null(),

      /* 16 additional C raw types.
       * They must be placed at their right place. They are referenced
       * with DM_ReferenceToDMeroonType even if they are CType instead.
       */

      DM_ReferenceToDMeroonType(C_char),
      DM_ReferenceToDMeroonType(C_signed_char),
      DM_ReferenceToDMeroonType(C_short_int),
      DM_ReferenceToDMeroonType(C_int),
      DM_ReferenceToDMeroonType(C_long_int),
      DM_ReferenceToDMeroonType(C_unsigned_char),
      DM_ReferenceToDMeroonType(C_unsigned_short_int),
      DM_ReferenceToDMeroonType(C_unsigned_int),
      DM_ReferenceToDMeroonType(C_unsigned_long_int),
      DM_ReferenceToDMeroonType(C_float),
      DM_ReferenceToDMeroonType(C_double),
      DM_ReferenceToDMeroonType(C_long_double),
      DM_ReferenceToDMeroonType(C_void_star),
      DM_ReferenceToDMeroonType(C_function_star),
      DM_ReferenceToDMeroonType(C_unsigned_long_long_int),
      DM_ReferenceToDMeroonType(C_long_long_int),           /* index=255 */
    }
  } } }
};

/* }}}
 * {{{ Initialization
 * RESOURCE DM_types_initialize
 * This function dynamically initializes DMeroon types from their C 
 * counterparts.
 */

void
DM_types_initialize (DM_Context *context) 
{
  DM_ImplementedTypesVector *implemented_types = DM_TheImplementedTypes();
  unsigned int i;

  DM_LinkVersionStringIntoSite(dmtypes);

  /* Check indexes of types.
   * Any type must have an index corresponding to the position where it
   * is stored in the DM_implemented_types vector. These indexes are
   * recorded in the dmtypes.h file and must be checked. */

  for ( i=0 ; i<=255 ; i++ ) {
    DM_DMeroonType *type = implemented_types->item[i];

    if ( ( type ) && ( type->index != i ) )
      DM_Abort(context, DME_INCORRECT_INDEX_TYPE);
  }

  /* For all DMeroon types, associate (if possible) a C type that 
   * represents it. Check sizes, initialize fields etc. 
   */

#define AssociateCTypeToDMeroonType(dmname,sz,body) \
  { DM_DMeroonType *dmtype = DM_ReferenceToDMeroonType(dmname); \
    DM_nat1 wanted_size = (sz); \
    \
    do { /* escape from body with break. */ \
      body; \
    } while ( 0 ); \
    if ( dmtype->c_type ) { \
      /* copy DMeroon characteristics from the found CType */ \
      DM_CType *ctype = dmtype->c_type; \
      dmtype->size      = ctype->size; \
      dmtype->alignment = ctype->alignment; \
      dmtype->printer   = ctype->printer; \
    } else { \
      /* make this DMeroon type absent */ \
      dmtype->alignment = 0; \
      dmtype->size      = 0; \
    } \
  }

/* NOTE: One may take a larger C type to implement a DMeroon type.
 * Pay attention to choose the smallest C type that represents
 * correctly a DMeroon type ie test them first.
 */

#define CheckIfCTypeFitsDMeroonType(cname) \
  { DM_CType *ctype = DM_ReferenceToCType(cname); \
    if ( ctype->size >= wanted_size ) { \
      dmtype->c_type      = ctype; \
      ctype->dmeroon_type = dmtype ;\
      break; \
    } \
  }
  
  /* Some unlikely associations are commented to save time and room.
   */
  AssociateCTypeToDMeroonType(nat1,1,
   CheckIfCTypeFitsDMeroonType(unsigned_char)
   CheckIfCTypeFitsDMeroonType(unsigned_short_int)
/* CheckIfCTypeFitsDMeroonType(unsigned_int)		*/
/* CheckIfCTypeFitsDMeroonType(unsigned_long_int)	*/
/* CheckIfCTypeFitsDMeroonType(unsigned_long_long_int)	*/
                              )
  AssociateCTypeToDMeroonType(nat2,2,
/* CheckIfCTypeFitsDMeroonType(unsigned_char)		*/
   CheckIfCTypeFitsDMeroonType(unsigned_short_int)
   CheckIfCTypeFitsDMeroonType(unsigned_int)
/* CheckIfCTypeFitsDMeroonType(unsigned_long_int)	*/
/* CheckIfCTypeFitsDMeroonType(unsigned_long_long_int)	*/
                              )
  AssociateCTypeToDMeroonType(nat3,4,
/* CheckIfCTypeFitsDMeroonType(unsigned_char)		*/
/* CheckIfCTypeFitsDMeroonType(unsigned_short_int)	*/
   CheckIfCTypeFitsDMeroonType(unsigned_int)
   CheckIfCTypeFitsDMeroonType(unsigned_long_int)
/* CheckIfCTypeFitsDMeroonType(unsigned_long_long_int)	*/
                              )
  AssociateCTypeToDMeroonType(nat4,8,
/* CheckIfCTypeFitsDMeroonType(unsigned_char)		*/
/* CheckIfCTypeFitsDMeroonType(unsigned_short_int)	*/
/* CheckIfCTypeFitsDMeroonType(unsigned_int)		*/
   CheckIfCTypeFitsDMeroonType(unsigned_long_int)
   CheckIfCTypeFitsDMeroonType(unsigned_long_long_int)
                              )

    /* char1 is just C char. */
  AssociateCTypeToDMeroonType(char1,1,
   CheckIfCTypeFitsDMeroonType(char)
                              )

    /* int* are signed types. */
  AssociateCTypeToDMeroonType(int1,1,
   CheckIfCTypeFitsDMeroonType(signed_char)
   CheckIfCTypeFitsDMeroonType(short_int)
/* CheckIfCTypeFitsDMeroonType(int)			*/
/* CheckIfCTypeFitsDMeroonType(long_int)		*/
/* CheckIfCTypeFitsDMeroonType(long_long_int)		*/
                              )
  AssociateCTypeToDMeroonType(int2,2,
/* CheckIfCTypeFitsDMeroonType(signed_char)		*/
   CheckIfCTypeFitsDMeroonType(short_int)
   CheckIfCTypeFitsDMeroonType(int)
/* CheckIfCTypeFitsDMeroonType(long_int)		*/
/* CheckIfCTypeFitsDMeroonType(long_long_int)		*/
                              )
  AssociateCTypeToDMeroonType(int3,4,
/* CheckIfCTypeFitsDMeroonType(signed_char)		*/
/* CheckIfCTypeFitsDMeroonType(short_int)		*/
   CheckIfCTypeFitsDMeroonType(int)
   CheckIfCTypeFitsDMeroonType(long_int)
/* CheckIfCTypeFitsDMeroonType(long_long_int)		*/
                              )
  AssociateCTypeToDMeroonType(int4,8,
/* CheckIfCTypeFitsDMeroonType(signed_char)		*/
/* CheckIfCTypeFitsDMeroonType(short_int)		*/
/* CheckIfCTypeFitsDMeroonType(int)			*/
   CheckIfCTypeFitsDMeroonType(long_int)
   CheckIfCTypeFitsDMeroonType(long_long_int)
                              )

    /* float* types are among C floating point types. */
  AssociateCTypeToDMeroonType(float1,4,
   CheckIfCTypeFitsDMeroonType(float)
/* CheckIfCTypeFitsDMeroonType(double)			*/
/* CheckIfCTypeFitsDMeroonType(long_double)		*/
                              )
  AssociateCTypeToDMeroonType(float2,8,
/* CheckIfCTypeFitsDMeroonType(float)			*/
   CheckIfCTypeFitsDMeroonType(double)
/* CheckIfCTypeFitsDMeroonType(long_double)		*/
                              )
  AssociateCTypeToDMeroonType(float3,16,
/* CheckIfCTypeFitsDMeroonType(float)			*/
/* CheckIfCTypeFitsDMeroonType(double)			*/
   CheckIfCTypeFitsDMeroonType(long_double)
                              )

    /* Pointer types data and code. */
  AssociateCTypeToDMeroonType(reference,sizeof(DM_C_void_star),
   CheckIfCTypeFitsDMeroonType(void_star)
                              )
  AssociateCTypeToDMeroonType(data,sizeof(DM_C_void_star),
   CheckIfCTypeFitsDMeroonType(void_star)
                              )
  AssociateCTypeToDMeroonType(code,sizeof(DM_C_function_star),
   CheckIfCTypeFitsDMeroonType(function_star)
                              )
  AssociateCTypeToDMeroonType(handler_code,sizeof(DM_C_function_star),
   CheckIfCTypeFitsDMeroonType(function_star)
                              )

  /* Alias a DMeroon type with another DMeroon type. For instance, 
   * netnat2 is copied from nat2 while date is copied from nat3...
   * They generally differ by their associated printer.
   */

#define DM_CopyDMeroonType(org,dest) \
 { DM_DMeroonType *dmtype = DM_ReferenceToDMeroonType(dest); \
   DM_DMeroonType *otype  = DM_ReferenceToDMeroonType(org);   \
 \
    dmtype->size      = otype->size; \
    dmtype->alignment = otype->alignment; \
    dmtype->c_type    = otype->c_type; \
    /* Don't copy the printer which is usually different. */ \
  }

DM_CopyDMeroonType(nat2,netnat2)
DM_CopyDMeroonType(nat3,netnat3)
DM_CopyDMeroonType(nat3,offset)
DM_CopyDMeroonType(nat3,date)
DM_CopyDMeroonType(nat3,class_options)
DM_CopyDMeroonType(nat3,field_options)
DM_CopyDMeroonType(nat3,port_options)
DM_CopyDMeroonType(nat3,site_options)
DM_CopyDMeroonType(nat3,external_options)
DM_CopyDMeroonType(nat3,context_options)
DM_CopyDMeroonType(nat3,file_options)
DM_CopyDMeroonType(nat3,request_options)

  /* Check if the most stringent type was well choosen ie check that
   * there is no DMeroonType with strictly more stringent alignment
   * requirements. */
  { 
    DM_DMeroonType *mst = 
      (DM_DMeroonType*)(DM_FromC(&DM_MOST_STRINGENT_TYPE_OBJECT));

    for (i=0 ; i<=255 ; i++ ) {
      DM_DMeroonType *type = implemented_types->item[i];

      if ( ( type ) && DM_IsA(context, type, DMeroonType) ) {
        if ( mst->alignment < type->alignment ) {
          DM_Abort(context, DME_NOT_MOST_STRINGENT_TYPE);
        }
      }
    }

    /* Check now if the alignment logarithm is well choosen. */
    if ( mst->alignment != (1<<DM_MOST_STRINGENT_ALIGNMENT_LOG) ) {
      DM_Abort(context, DME_INCORRECT_ALIGNMENT_LOG);
    }
  }
}

/* }}}
 * end of dmtypes.c */
