/* $Id: dmtypes.h,v 1.21 1998/06/10 16:31:31 queinnec Exp $ 
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 */

#ifndef DM_TYPES_H
#define DM_TYPES_H

/*  
 * This file exports all the Type instances describing DMeroon (and the
 * C compiler that compiled it.
 */

extern void DM_types_initialize(DM_Context *context);

/******************************************************************
 *                      INDEXES FOR TYPES
 *
 * All basic types have a given small number to identify them.
 * This encoding must not be changed or DMeroon will become very ill.
 * Moreover, nought should be 0 and unsigned integers used to represent
 * lengths of indexed fields should be intances of nat1, nat2, nat3 with
 * these indexes.
 */

#define DM_nought_index			0
#define DM_nat1_index			1
#define DM_nat2_index			2
#define DM_nat3_index			3
#define DM_nat4_index			4
#define DM_reference_index		5
#define DM_data_index			6
#define DM_code_index			7
#define DM_int1_index			8
#define DM_int2_index			9
#define DM_int3_index			10
#define DM_int4_index			11
#define DM_char1_index			12
#define DM_netnat2_index		13
#define DM_netnat3_index		14
#define DM_unicode_index		15
#define DM_float1_index			16
#define DM_float2_index			17
#define DM_float3_index			18
#define DM_offset_index			19
#define DM_date_index			20
#define DM_key_index			21
#define DM_class_options_index		22
#define DM_field_options_index		23
#define DM_port_options_index		24
#define DM_site_options_index		25
#define DM_external_options_index	26
#define DM_context_options_index	27
#define DM_file_options_index           28
#define DM_request_options_index        29
#define DM_handler_code_index		30

/* 
 * This is the number of above types (just 1 + the highest number above).
 * It is used to dimension the vector of types.
 */

#define DM_MAXIMUM_TYPES_NUMBER           31

/*
 * This is the class for the vector of implemented types
 */

#define DM_GenerateImplementedTypesVectorStruct(_0) \
struct DM_ImplementedTypesVector_##_0 { \
  DM_nat3               item_length; \
  struct DM_DMeroonType *item[_0]; \
}

typedef DM_GenerateImplementedTypesVectorStruct(256) 
     DM_ImplementedTypesVector_256;
DM_DefineWrappedType(DM_ImplementedTypesVector_256);

typedef DM_ImplementedTypesVector_256 DM_ImplementedTypesVector;

extern DM_ImplementedTypesVector_256_wrapped DM_implemented_types_object;

/* 
 * And the vector of predefined classes.
 */

#define DM_GeneratePredefinedClassesVectorStruct(_0) \
struct DM_PredefinedClassesVector_##_0 { \
  DM_nat3         item_length; \
  struct DM_Class *item[_0]; \
}

typedef DM_GeneratePredefinedClassesVectorStruct(256) 
     DM_PredefinedClassesVector_256;
DM_DefineWrappedType(DM_PredefinedClassesVector_256);

typedef DM_PredefinedClassesVector_256 DM_PredefinedClassesVector;

extern DM_PredefinedClassesVector_256_wrapped DM_predefined_classes_object;

/******************************************************************
 *      Exporting types, associates classes and fields.
 */

#define DM_DefineDMeroonTypeStruct(nm) \
typedef struct DM_DMeroonType_##nm##_struct { \
  DM_nat1  index; \
  DM_nat1  alignment; \
  DM_nat1  size; \
  DM_nat1  repsize; \
  DM_code  printer; \
  DM_code  encoder; \
  DM_code  decoder; \
  struct DM_Class* dmeroon_class; \
  struct DM_CType* c_type; \
  DM_nat1  name_length; \
  DM_char1 name[sizeof(#nm)+1]; \
} DM_DMeroonType_##nm##_struct; \
DM_DefineWrappedType(DM_DMeroonType_##nm##_struct)

#define DM_DefineDMeroonTypeExterns(name) \
  DM_DefineDMeroonTypeStruct(name); \
  extern DM_DMeroonType_##name##_struct_wrapped DM_##name##_type_object

DM_DefineDMeroonTypeExterns(nought);
DM_DefineDMeroonTypeExterns(nat1);
DM_DefineDMeroonTypeExterns(nat2);
DM_DefineDMeroonTypeExterns(nat3);
DM_DefineDMeroonTypeExterns(nat4);
DM_DefineDMeroonTypeExterns(reference);
DM_DefineDMeroonTypeExterns(data);
DM_DefineDMeroonTypeExterns(code);
DM_DefineDMeroonTypeExterns(int1);
DM_DefineDMeroonTypeExterns(int2);
DM_DefineDMeroonTypeExterns(int3);
DM_DefineDMeroonTypeExterns(int4);
DM_DefineDMeroonTypeExterns(char1);
DM_DefineDMeroonTypeExterns(netnat2);
DM_DefineDMeroonTypeExterns(netnat3);
DM_DefineDMeroonTypeExterns(unicode);
DM_DefineDMeroonTypeExterns(offset);
DM_DefineDMeroonTypeExterns(float1);
DM_DefineDMeroonTypeExterns(float2);
DM_DefineDMeroonTypeExterns(float3);
DM_DefineDMeroonTypeExterns(date);
DM_DefineDMeroonTypeExterns(key);
DM_DefineDMeroonTypeExterns(class_options);
DM_DefineDMeroonTypeExterns(field_options);
DM_DefineDMeroonTypeExterns(port_options);
DM_DefineDMeroonTypeExterns(site_options);
DM_DefineDMeroonTypeExterns(external_options);
DM_DefineDMeroonTypeExterns(context_options);
DM_DefineDMeroonTypeExterns(file_options);
DM_DefineDMeroonTypeExterns(request_options);
DM_DefineDMeroonTypeExterns(handler_code);

/******************************************************************
 *    Exporting C types.
 */

#define DM_DefineCTypeStruct(nm) \
typedef struct DM_CType_##nm##_struct { \
  DM_nat1  index; \
  DM_nat1  alignment; \
  DM_nat1  size; \
  DM_nat1  repsize; \
  DM_code  printer; \
  struct DM_DMeroonType* dmeroon_type; \
  DM_nat1  name_length; \
  DM_char1 name[sizeof(#nm)+1]; \
} DM_CType_##nm##_struct; \
DM_DefineWrappedType(DM_CType_##nm##_struct)

#define DM_DefineCTypeExterns(name) \
  DM_DefineCTypeStruct(name); \
  extern DM_CType_##name##_struct_wrapped DM_##name##_type_object

DM_DefineCTypeExterns(C_char);
DM_DefineCTypeExterns(C_signed_char);
DM_DefineCTypeExterns(C_short_int);
DM_DefineCTypeExterns(C_int);
DM_DefineCTypeExterns(C_long_int);
DM_DefineCTypeExterns(C_unsigned_char);
DM_DefineCTypeExterns(C_unsigned_short_int);
DM_DefineCTypeExterns(C_unsigned_int);
DM_DefineCTypeExterns(C_unsigned_long_int);
DM_DefineCTypeExterns(C_float);
DM_DefineCTypeExterns(C_double);
DM_DefineCTypeExterns(C_long_double);
DM_DefineCTypeExterns(C_void_star);
DM_DefineCTypeExterns(C_function_star);
DM_DefineCTypeExterns(C_unsigned_long_long_int);
DM_DefineCTypeExterns(C_long_long_int);
DM_DefineCTypeExterns(C_long_long_double);

/* The type used to represent C booleans. */

typedef int DM_c_boolean;

/* DOCUMENTATION DM_NULL 
 * Null DMeroon references are allowed. 
 * I try to differentiate the use of NUL (the '\0' character), 
 * NULL a null C pointer and DM_NULL a null reference to a 
 * DMeroon object. */

#define DM_NULL NULL

/******************************************************************
 *   Additional internal macros.
 *
 * The DM_DefineTypeObject macro allocates a DMeroon object, instance of
 * the Type class. This object contains everything that is needed to 
 * manage this type. All primary C types have a corresponding DMeroon type.
 */

/* nought type cannot be printed, sent nor received. 
 * FUTURE should change the order of names in *_print names      ???
 */
#define DM_nought_print   DM_no_print
#define DM_send_nought    DM_send_nought
#define DM_receive_nought DM_receive_nought

#define DM_DefineTypeObjectForC(name,index,ref) \
DM_CType_##name##_struct_wrapped DM_##name##_type_object = { \
  DM_SupportPrefixInitialization(DM_CType_wrapped,\
                                 DM_##name##_type_object) \
  DM_ReferenceToClass(CType), \
  DM_ProxyInitialization(), \
  { { { \
    index, \
    DM_AlignConstraintFor(name), \
    sizeof(DM_##name), \
    DM_RepeatedSizeFor(name), \
    DM_CodeOf(DM_##name##_print), \
    (ref), \
    sizeof(#name), \
    #name \
  } } } \
}

#define DM_DefineTypeForC(name) \
  DM_DefineConstrainedStructure(name,DM_##name); \
  DM_DefineTypeObjectForC(name,DM_##name##_index,DM_NULL)

/* 
 * Definition of Type instances (but the nought type which is
 * hand-made) is totally repetitive so a macro automatizes it all. 
 * Any type has an associated class with a single field of that type.
 * This class has name DM_<typename>_class.
 */

#define DM_DefineTypeObjectForDMeroon(name,index,ref) \
DM_DMeroonType_##name##_struct_wrapped DM_##name##_type_object = { \
  DM_SupportPrefixInitialization(DM_DMeroonType_wrapped,\
                                 DM_##name##_type_object) \
  DM_ReferenceToClass(DMeroonType), \
  DM_ProxyInitialization(), \
  { { { \
    index, \
    DM_AlignConstraintFor(name), \
    sizeof(DM_##name), \
    DM_RepeatedSizeFor(name), \
    DM_CodeOf(DM_##name##_print), \
    DM_CodeOf(DM_send_##name), \
    DM_CodeOf(DM_receive_##name), \
    ref, \
    DM_NULL, \
    sizeof(#name), \
    #name \
  } } } \
}

/* These definitions are necessary for the compilation of the previous
 * DM_DefineTypeObjectForDMeroon macro. The value is unimportant since
 * printers are dynamically associated in DM_types_initialize(). 
 */

#define DM_char1_print          DM_C_char_print
#define DM_int1_print           DM_C_signed_char_print
#define DM_int2_print           DM_C_short_int_print
#define DM_int3_print           DM_C_int_print
#define DM_int4_print           DM_C_long_long_int_print
#define DM_nat1_print           DM_C_unsigned_char_print
#define DM_nat2_print           DM_C_unsigned_short_int_print
#define DM_nat3_print           DM_C_unsigned_int_print
#define DM_nat4_print           DM_C_unsigned_long_long_int_print
#define DM_float1_print         DM_C_float_print
#define DM_float2_print         DM_C_double_print
#define DM_float3_print         DM_C_long_double_print
#define DM_reference_print      DM_C_void_star_print
#define DM_data_print           DM_C_void_star_print
#define DM_code_print           DM_C_function_star_print

#define DM_DefineTypeForDMeroon(name,classname) \
  DM_DefineConstrainedStructure(name,DM_##name); \
  DM_DefineTypeObjectForDMeroon(name,DM_##name##_index,\
                                DM_ReferenceToClass(classname))

#define DM_DefineSpecialTypeForDMeroon(name) \
  DM_DefineConstrainedStructure(name,DM_##name); \
  DM_DefineTypeObjectForDMeroon(name,DM_##name##_index, DM_NULL)

/*
 * Definition of a Type instance for which no representation exists.
 * This is encoded with a null alignment constraint (to let
 * DM_nought_type be acceptable).
 */

#define DM_DefineAbsentTypeForC(name) \
DM_CType_##name##_struct_wrapped DM_##name##_type_object = { \
  DM_SupportPrefixInitialization(DM_CType_wrapped,\
                                 DM_##name##_type_object) \
  DM_ReferenceToClass(CType), \
  DM_ProxyInitialization(), \
  { { { \
    DM_##name##_index, \
    0,    /* Zero means absent */ \
    0,    /* Zero means absent */ \
    0,    /* Zero means absent */ \
    DM_NULL, \
    sizeof(#name), \
    #name \
  } } } \
}

#define DM_DefineAbsentTypeForDMeroon(name,classname) \
DM_DMeroonType_##name##_struct_wrapped DM_##name##_type_object = { \
  DM_SupportPrefixInitialization(DM_DMeroonType_wrapped,\
                                 DM_##name##_type_object) \
  DM_ReferenceToClass(DMeroonType), \
  DM_ProxyInitialization(), \
  { { { \
    DM_##name##_index, \
    0,    /* Zero means absent */ \
    0,    /* Zero means absent */ \
    0,    /* Zero means absent */ \
    NULL, NULL, NULL, \
    DM_NULL, DM_NULL, \
    sizeof(#name), \
    #name \
  } } } \
}

#endif /* DM_TYPES_H */

/* end of dmtypes.h */
