#! /bin/sh
# $Id: demo2.sh,v 1.10 1996/07/18 13:56:10 queinnec Exp $
# This is the second DMeroon demo with two interacting servers.
# This script must be run from the TARGET/DMeroon/demos directory.

# Load configuration parameters if not yet done (in demo1.sh).
# The first script sets up the main parameters describing how DMeroon was
# regenerated, the second one sets up the parameters of the demos.

if [ -z "${DMEROON_SERVER1}" ]
then
     if [ -f ../../config.sh ]
     then . ../../config.sh
     else 
           echo "You are probably not in the TARGET/DMeroon/demos directory."
           exit 1
     fi
     . paths.sh
fi

next=2

# Prepare screen for text

clear

cat <<EOF
                        Welcome to DMeroon

This demo will start two DMeroon servers and an associated HTTP client
to inspect the objects exported by these DMeroon servers. The exact
executables that are used for that demo are defined in the
TARGET/demos/paths.sh file.

EOF

# Start two DMeroon servers and one HTTP client.

LAUNCH_ARGS=2
. launch.sh

cat <<EOF
Wait a little for all these three windows then type <return> when they 
are present.
EOF

eval ${CONTINUE}

cat <<EOF 
You should now have three windows. Two xterm windows displaying a
trace of the activity of a DMeroon server, and one HTTP client
displaying the initial page of one of this DMeroon server.  You may
also take time to rearrange or resize these windows.  You may also use
the following urls if the HTTP clients do not display the right pages.

        $url1
 or     $url2

Your screen may look like:
              
site A :   +--------+
           |  +-------+
           |  |       |
           +--|       |
              |       |
site B :   +--|       |
           |  |       |
           |  +-------+
           +--------+                         Type <return> when ready.
EOF

eval ${CONTINUE}

cat <<EOF
Each DMeroon server currently ignores the other one. Let ask site A to
connect to the site B. Observe the furious exchange of messages between them.
Type <return> to send an URLscript to connect them.
EOF

eval ${CONTINUE}

${TOP}/DMeroon/Commands/urlscript ${DMEROON_HOST1}:${DMEROON_PORT1} <<EOF
        "${DMEROON_HOST2}" ${DMEROON_PORT2} connect
        "connected." print
EOF

cat <<EOF

Now the two DMeroon servers are connected and know each other. You may
verify that with your HTTP clients. For instance, display the site
instance of site A with the A browser and click in its 'sites'
field. You will notice two entries in the displayed Hashtable, one
corresponds to site A (the current site), the other corresponds to
site B: click on it to see this object.

Another way is to use the A client with the following url to reach
directly and display that object:

        http://${DMEROON_HOST1}:${DMEROON_PORT1}/"${DMEROON_HOST2}"/${DMEROON_PORT2}/site/print

More explanation with <return>
EOF

eval ${CONTINUE}

cat <<EOF 
The displayed object is the replica of a remote object (owned by B)
representing the site B locally cached on site A. More picturally:

On Site A                        On Site B

local cached object
            +--------+    +------> +------+ entry item
            | site B |    |         \    /
            +--------+    |          \  /
                ^         |           \/
    exit item   V         |            |
            +-------+     |            V  original Site B object
            \      /      |         +--------+
             \  ----------+         | site B |
              \  /                  +--------+
               \/

To know more, type <return>.
EOF

eval ${CONTINUE}

cat <<EOF 
Observe that some fields (address, version ...) of site B are already
known on site A so you see it directly. This is not the case of other
references such as the 'support' field.

Click on the (owner) link to follow it, you will automatically switch
your A browser to B, requesting B to display this local String (the
'support' field holds the name of the binding language). If you go
backward (returning on site A), click on the (Exit item) hyperlink,
you will see the Exit item object on A (an object internal to DMeroon
that materializes a remote pointer) and you may click on the "Fetch"
hyperlink. This will fetch the remote String of B and cache it locally
on A. You may notice that site B is qualified as "remote" by the A
browser while they are local if seen on B.

This is rather confusing, isn't it ? Type <return> to be more confused.
EOF

eval ${CONTINUE}

cat <<EOF
Now we allocate a MutableVector of length 3 on site A and we publish
it ie we insert it in the 'value' field of the 'user' sub-dictionary
of the 'information' field of site A.

Type <return> to let me perform this for you.
EOF

eval ${CONTINUE}

${TOP}/DMeroon/Commands/urlscript ${DMEROON_HOST1}:${DMEROON_PORT1} <<EOF
        "MutableVector" Class 3 allocate "mv" define
        mv setsharable
        thesite .information .rest "user-dictionary" define
        user-dictionary dup "value" Field mv set
EOF

cat <<EOF
You may now browse site A to verify the presence of this MutableVector.
This MutableVector is owned by A since it has been created on A.
You may alternatively jump directly on it with the following URL:

  http://${DMEROON_HOST1}:${DMEROON_PORT1}/thesite.information.rest.value/print

Since a MutableVector is mutable, it is associated to a Clock. By default,
this is the general clock of the site. You may see it with the following
URL:

  http://${DMEROON_HOST1}:${DMEROON_PORT1}/thesite.information.rest.value/clockof/print

We now ask site B to locally cache this vector and to mutate it so its
first slot now contains the DMeroon "true" constant.  Before typing
<return>, position the second browser on that mutable vector or if you
are not sure how to do it, just send the following URL to the second
browser:

  http://${DMEROON_HOST2}:${DMEROON_PORT2}/"${DMEROON_HOST1}"/${DMEROON_PORT1}/site/.information.rest.value/print
EOF

eval ${CONTINUE}

${TOP}/DMeroon/Commands/urlscript ${DMEROON_HOST2}:${DMEROON_PORT2} <<EOF
        "${DMEROON_HOST1}" ${DMEROON_PORT1} site "othersite" define
        othersite .clock pop # Externalize the clock associated to mv.
        othersite .information .rest .value "mv" define
        mv dup "item" Field 0 true xset
EOF

cat <<EOF
Reload now this object in the A browser: it is now mutated since the
owner always have an accurate vision of all the objects it owns. All
exported mutable objects are monitored by a clock which, by default,
is the general clock of the site (the one held in the 'clock' field of
the site object). You may verify that this clock has been incremented
by this mutation. This clock may be seen with the following URL:

  http://${DMEROON_HOST1}:${DMEROON_PORT1}/thesite.information.rest.value/clockof/print

Reload now the mutable vector object with the B browser: it is not
changed but it is marked as "obsolete". This is so because the second
site knows that the monitoring clock of this cached object was
incremented. You may see that (remote) clock (owned by site A)
associated to the mutable vector with the following URL:

  http://${DMEROON_HOST2}:${DMEROON_PORT2}/"${DMEROON_HOST1}"/${DMEROON_PORT1}/site/.information.rest.value/clockof/print

If you want to have an uptodate cache of this mutable vector on B,
then use the "fetch" hyperlink via the (exit item) hyperlink or
alternatively with the following URL:

  http://${DMEROON_HOST2}:${DMEROON_PORT2}/"${DMEROON_HOST1}"/${DMEROON_PORT1}/site/.information.rest.value/fetch/print

I'll do it in a moment.
EOF

eval ${CONTINUE}

${TOP}/DMeroon/Commands/urlscript ${DMEROON_HOST2}:${DMEROON_PORT2} <<EOF
        "${DMEROON_HOST1}" ${DMEROON_PORT1} site "othersite" define
        othersite .information .rest .value fetch
EOF

cat <<EOF
This is the end of the second demonstration!
EOF

eval ${NEXT_DEMO}

# end of demo2.sh
