#! /bin/sh
# $Id: demo3.sh,v 1.4 1996/07/18 13:56:12 queinnec Exp $
# This is the third DMeroon demo with three interacting servers.
# This script must be run from the TARGET/DMeroon/demos directory.

# Load configuration parameters if not yet done (in demo2.sh).
# The first script sets up the main parameters describing how DMeroon was
# regenerated, the second one sets up the parameters of the demos.

if [ -z "${DMEROON_SERVER1}" ]
then
     if [ -f ../../config.sh ]
     then . ../../config.sh
     else 
           echo "You are probably not in the TARGET/DMeroon/demos directory."
           exit 1
     fi
     . paths.sh
fi

next=3

# Prepare screen for text

clear

cat <<EOF
                        Welcome to DMeroon

This demo will start three DMeroon servers and one associated HTTP
client to inspect the objects exported by these DMeroon servers. The
exact executables that are used for that demo are defined in the
TARGET/demos/paths.sh file.

EOF

# Start three DMeroon servers. 

LAUNCH_ARGS=3
. launch.sh

cat <<EOF
Arrange your windows so that your screen looks like:
              
site A :   +--------+        +--------+  : site C
           |        |        |        |
           |  +-------+      |        |
           +--|       |      +--------+
              |       |
site B :   +--|       |
           |  |       |
           |  +-------+
           +--------+                         Type <return> when ready.
EOF

eval ${CONTINUE}

cat <<EOF
Type <return> to connect A to B and A to C.
EOF

eval ${CONTINUE}

${TOP}/DMeroon/Commands/urlscript ${DMEROON_HOST1}:${DMEROON_PORT1} <<EOF
        "${DMEROON_HOST2}" ${DMEROON_PORT2} connect
        "${DMEROON_HOST3}" ${DMEROON_PORT3} connect
EOF

cat <<EOF
The three sites are connected altogether. You may inspect the 'sites' 
Hashtable of site A to discover the two other connected sites. Sites
are propagated lazily, that's why site B does not know of C nor C of B. 
A simple message from A to B will let B learn about C (while C is still not
aware of B).

Type <return> to ask A to send something to B and thus inform B of the
existence of C.
EOF

eval ${CONTINUE}

${TOP}/DMeroon/Commands/urlscript ${DMEROON_HOST1}:${DMEROON_PORT1} <<EOF
        nil "${DMEROON_HOST2}" ${DMEROON_PORT2} site send
EOF

cat <<EOF
After the previous message from A to B, B now knows of C and may access it.
You may verify this fact with the following URL (to be sent to the B
HTTP client):

  http://${DMEROON_HOST2}:${DMEROON_PORT2}/"${DMEROON_HOST3}"/${DMEROON_PORT3}/site/print

However C is not yet aware of B (unless your B browser asks C for some
data). Type <return> when convinced.  
EOF

eval ${CONTINUE}

cat <<EOF
Type <return> to let A and B both publish a (different) MutableVector
and ask B to locally cache the MutableVector from A.
EOF

eval ${CONTINUE}

${TOP}/DMeroon/Commands/urlscript ${DMEROON_HOST1}:${DMEROON_PORT1} <<EOF
        "MutableVector" Class 5 allocate "mv" define
        mv setsharable
        "A" String "s" define 
        s setsharable
        mv dup "item" Field 0 s xset pop
        thesite .information .rest "user-dictionary" define
        user-dictionary dup "value" Field mv set
EOF

${TOP}/DMeroon/Commands/urlscript ${DMEROON_HOST2}:${DMEROON_PORT2} <<EOF
        "MutableVector" Class 2 allocate "mv" define
        mv setsharable
        "B" String "s" define
        s setsharable
        mv dup "item" Field 0 s xset pop
        thesite .information .rest "user-dictionary" define
        user-dictionary dup "value" Field mv set
        # Ask B to cache the MutableVector published by A
        "${DMEROON_HOST1}" ${DMEROON_PORT1} site "A_site" define
        A_site .information .rest .value 
EOF

cat <<EOF
These MutableVectors are installed in their respective 'user'
dictionary. Their associated clock are the general Clocks of their
respective sites. I let you use your browser to see these
MutableVectors with the following URLs:

 http://${DMEROON_HOST1}:${DMEROON_PORT1}/thesite.information.rest.value/print
 http://${DMEROON_HOST2}:${DMEROON_PORT2}/thesite.information.rest.value/print

You may also position your B browser on the cached replica of the
MutableVector exported by A. Use the following URL for that:

  http://${DMEROON_HOST2}:${DMEROON_PORT2}/"${DMEROON_HOST1}"/${DMEROON_PORT1}/site.information.rest.value/print

Type <return> to see causality at work...
EOF

eval ${CONTINUE}

cat <<EOF 
Type <return> to ask C to mutate the MutableVector published
by A then to mutate the MutableVector published by B (when A will
acknowledge the mutation, it will inform C of the existence of B). The
tenet of causal coherence is that B cannot see the modification made
on its own published MutableVector and still consider the cached copy of
the MutableVector published by A to be up to date since, viewed from
C, A was mutated before B. 
EOF

eval ${CONTINUE}

${TOP}/DMeroon/Commands/urlscript ${DMEROON_HOST3}:${DMEROON_PORT3} <<EOF
        "C1" String "s" define
        s setsharable
        "${DMEROON_HOST1}" ${DMEROON_PORT1} site "A_site" define
        A_site .information .rest .value "A_mv" define
        A_mv dup "item" Field 0 s xset
EOF

cat <<EOF
C had just modified the mutable vector exported by A and heard of B.
EOF

eval ${CONTINUE}

${TOP}/DMeroon/Commands/urlscript ${DMEROON_HOST3}:${DMEROON_PORT3} <<EOF
        "C2" String "s" define
        s setsharable
        "${DMEROON_HOST2}" ${DMEROON_PORT2} site "B_site" define
        B_site .information .rest .value "B_mv" define
        B_mv dup "item" Field 0 s xset
EOF

cat <<EOF
Use now your browser to inspect the results of this mutation. You may
first check it on B and see that the cached replica of the MutableVector
published by A is obsolete with the following URL:

  http://${DMEROON_HOST2}:${DMEROON_PORT2}/"${DMEROON_HOST1}"/${DMEROON_PORT1}/site.information.rest.value/print

You may also inspect A or C and type <return> when fed up!
EOF

eval ${CONTINUE}

cat <<EOF
This is the end of the third demonstration!
EOF

eval ${NEXT_DEMO}

# end of demo3.sh
