#! /bin/sh
# This file sets various parameters to be used by the various demos.
# You may of course change them.

# This is the absolute path naming a DMeroon executable. Their name is
# generally made of the name of the binding language followed by
# +DMeroon. 

eval "DMEROON_SERVER1=`ls -t ${TARGET}/*+DMeroon | head -1`"

# By default, DMeroon servers use the well-known port 56423. But if
# you run multiple DMeroon servers on the same machine, they cannot
# simultaneously serve the same port number so they use alternate
# ones. 

DMEROON_PORT1=56423

# This is the machine on which to run the DMeroon server.

eval "DMEROON_HOST1=`hostname`"

# Now define two other DMeroon servers. They may reside on the same
# machine or on others (they may then use the same well-known port
# number). They may use the same binding language or others. By
# default, the demos will be on the current machine with the same
# binding language. To use different machines, change the following
# 'true' to 'false' and set accordingly the DMEROON_*[23] variables.
# To use the same machine and different binding languages, just change
# the DMEROON_SERVER[23] variables in the then part of the following
# alternative.

if      true
then 
        # This is the setting for a unique machine.
        DMEROON_SERVER2=${DMEROON_SERVER1}
        DMEROON_PORT2=56222
        DMEROON_HOST2=${DMEROON_HOST1}

        DMEROON_SERVER3=${DMEROON_SERVER1}
        DMEROON_PORT3=56333
        DMEROON_HOST3=${DMEROON_HOST1}
else
        # This is the setting for different machines using the same
        # port number. 
        DMEROON_SERVER2=${DMEROON_SERVER1}
        DMEROON_PORT2=${DMEROON_PORT1}
        DMEROON_HOST2=otherhost

        DMEROON_SERVER3=${DMEROON_SERVER1}
        DMEROON_PORT3=${DMEROON_PORT1}
        DMEROON_HOST3=thirdhost
fi

# Patch now these definitions for the machines I use for demos. 

case "X${SITE}" in
  Xyoupou)
        DMEROON_SERVER2=${HOME}/icslas/o/icslas.sun4/Icslas+DMeroon
        DMEROON_PORT2=${DMEROON_PORT1}
        DMEROON_HOST2=iza.ibp.fr

        DMEROON_SERVER3=${HOME}/icslas/o/icslas.intel386/Icslas+DMeroon
        DMEROON_PORT3=${DMEROON_PORT1}
        DMEROON_HOST3=peche.ibp.fr
        ;;
esac

# Define now where are the html pages describing the sources. These
# pages are automatically extracted from DMeroon sources.

DMEROON_SOURCES_HTML_DOCUMENTATION=${TARGET}/WWW

# Define what is your favorite http client (netscape, mosaic ...)
# If netscape or xmosaic is in your PATH, then the HTTP_CLIENT variable 
# will be automatically set up.

case X`which netscape` in
  X/*)
        HTTP_CLIENT=netscape ;;
  *)
        case X`which xmosaic` in
          X/*)
                HTTP_CLIENT=xmosaic ;;
          *)
            case X`which redbaron` in
              X/*)
                    HTTP_CLIENT=redbaron ;;
              *)
                    echo Cannot find an http client for the HTTP_CLIENT variable
                    echo Change its definition in TOP/DMeroon/demos/paths.sh
                    exit 1 
                    ;;
                ;;
        esac ;;
esac

################################# End of configurable section
# Check now the given paths.

if [ -f ${DMEROON_SERVER1} ]
then :
else    echo DMEROON_SERVER1 does not exist or is not executable.
        echo Currently DMEROON_SERVER1="\"${DMEROON_SERVER1}\"."
        echo Change its definition in TOP/DMeroon/demos/paths.sh
        exit 1
fi

################################# Predefined paramters
# Common definitions to demos.

# Define the trap numbers to catch while running demos.

TRAP_NUMBERS="0 1 2 3 4 5 6 7 8 10 12 13 14 15"

# Set xterm default parameters

XTERM_FLAGS="+ls +l -si -bg pink"

# Define geometry for the sites.
XTERM_FLAGS1="${XTERM_FLAGS} -geometry 80x20+0+30"
XTERM_FLAGS2="${XTERM_FLAGS} -geometry 80x20+0-100"
XTERM_FLAGS3="${XTERM_FLAGS} -geometry 80x20+600+30"

# Define geometry for the HTTP client.
url1="http://${DMEROON_HOST1}:${DMEROON_PORT1}/help"
HTTP_FLAGS="-geometry 500x350+100+0 ${url1}"

# Command for the next step of the demo.  Initialize it unless
# defined. It is defined for test.

if [ -z "$CONTINUE" ]
then 
     CONTINUE='echo --------------------------- ; read cr ; case X$cr in X[qQ]*) exit 0 ;; *) : ;; esac'
fi

# Command to chain on the next demo. Initialize it unless defined.
# It is defined for test.

if [ -z "$NEXT_DEMO" ]
then 
     NEXT_DEMO='echo "Do you want to see the next demo (yes,no)[y] ? " ; read answer ; case X$answer in X[nNqQ]*) exit 0 ;; *) next=`expr ${next} + 1` ; eval ${EXPORTS} ; exec /bin/sh demo${next}.sh ;; esac'
fi

# Time for DMeroon to linger. The LINGERTIME variable may stay undefined.

# The number of the next demo to run

next=1

# Export all these variables

EXPORTS='export DMEROON_SERVER1 DMEROON_SERVER2 DMEROON_SERVER3 DMEROON_PORT1 DMEROON_PORT2 DMEROON_PORT3 DMEROON_HOST1 DMEROON_HOST2 DMEROON_HOST3 XTERM_FLAGS1 XTERM_FLAGS2 XTERM_FLAGS3 HTTP_FLAGS url1 url2 url3 HTTP_CLIENT CONTINUE NEXT_DEMO next httpclientpid xtermpid1 xtermpid2 xtermpid3 '

eval ${EXPORTS}

# end of paths.sh
