;;; $Id: rtgam.scm,v 1.3 1997/12/09 13:13:36 queinnec Exp $

;;; This is the adaptation to Gambit of the language of Icslas.

(define stdout ##stdout)
(define stderr ##stderr)

;;; How to flush a port.

(define (flush-output-port out)
  (flush-output out) )

;;; Icslas use the 'monitor' special form that is implemented as a macro
;;; in Scheme. The implementation should define two helper functions:
;;;   a protected-eval function
;;; 	(protected-eval thunk) computes the thunk and returns a vector 
;;;     of its value. If an error occurs, it returns a list describing
;;;     the error.
;;;   a protected-error function that signals an error to the underlying 
;;;     Scheme implementation.

(define (protected-eval thunk)
  (call/cc
   (lambda (exit)
     (##catch-all (lambda (s args)
                    ;; args contain cyclic values: don't try to print them!
                    (define (simplify args)
                      (map ##decomp args) )
                    (exit (list s (simplify args))) )
                  (lambda () (vector (thunk))) ) ) ) )

(define (protected-error . hints)
  (##runtime-error "Uncaught exception" 'diagnose hints) )

;;; Others utilities

(define call/cc call-with-current-continuation)

;;; The system() function is Un*x-related, you will have to run it
;;; by hand.

(define (system string)
  (format stderr
          "~%Can you run the following shell command:~%   ~S~%~%"
          string )
  (protected-error "No system function here" 'system '()) )

;;; end of rtgam.scm
