;;; $Id: javaenv.scm,v 1.4 1997/12/09 19:47:20 queinnec Exp $
;;; By Christian.Queinnec@inria.fr (http://pauillac.inria.fr/~queinnec/)
;;; This file is part of a Scheme->Java compiler derived from the Scheme->C
;;; compiler described in the LiSP book. 

;;; Define the predefined global environment for Java compilation.

;;; Exportations:
;;;                   (predefined-environment) : Global-Environment
;;; end of exportations.

;;; {{{ Classes

;;; This function creates a new global environment. Its predefined
;;; part is static and shared (but it is fortunately immutable).

(define (predefined-global-env)
  (make-Global-Environment predefined-env '()) )

;;; The predefined environment is just a list of bindings. 

(define predefined-env '())

;;; Descriptions of functions for Java are richer, they know how to
;;; generate a direct call to the Java function.

(define-class Functional-Description-for-Java Functional-Description
  ( generator
    ) )

;;; }}}
;;; {{{ Definers

;;; This macro enriches the predefined environment
;;; with variables, 
;;;     with fixed arity functions,
;;;         or with variable arity functions.

(define-icslas-macro (definitial name value)
  `(let ((v (make-Predefined-Variable 
             ',name (make-Constant-Description ,value) )))
     (set! predefined-env (cons v predefined-env))
     ',name ) )

(define-icslas-macro (defprimitive name Javaname arity)
  `(let ((v (make-Predefined-Variable 
             ',name (make-Functional-Description-for-Java
                    = ,arity 
                    (make-predefined-application-generator ',Javaname) ) )))
     (set! predefined-env (cons v predefined-env))
     ',name ) )

(define-icslas-macro (defnaryprimitive name Javaname arity)
  `(let ((v (make-Predefined-Variable 
             ',name (make-Functional-Description-for-Java
                    >= ,arity 
                    (make-predefined-nary-application-generator
                     ',Javaname ) ) )))
     (set! predefined-env (cons v predefined-env))
     ',name ) )

;;; }}}
;;; {{{ Variables

(defprimitive cons "cons" 2)
(defprimitive car "car" 1)
(defprimitive cdr "cdr" 1)
(defprimitive set-car! "set_car" 2)
(defprimitive set-cdr! "set_cdr" 2)
(defprimitive pair? "pairp" 1)
(defprimitive null? "nullp" 1)
(defprimitive symbol? "symbolp" 1)
(defprimitive eq? "eqp" 2)
(defprimitive procedure? "procedurep" 1)
(defprimitive boolean? "booleanp" 1)
(defprimitive string? "stringp" 1)

(defprimitive char? "charp" 1)
(defprimitive integer->char "integer_to_char" 1)
(defprimitive char->integer "char_to_integer" 1)

(defprimitive symbol->string "symbol_to_string" 1)
(defprimitive string->symbol "string_to_symbol" 1)

(defprimitive number? "numberp" 1)
(defprimitive integer? "fixnump" 1)
(defprimitive real?    "floatnump" 1)
(defnaryprimitive + "plus" 0)
(defprimitive - "minus" 2)
(defnaryprimitive * "times" 0)
(defprimitive / "quotient" 2)
(defprimitive quotient "quotient" 2)
(defprimitive divide "divide" 2)
(defprimitive remainder "remainder" 2)
(defprimitive modulo "modulo" 2)
(defprimitive <= "lep" 2)
(defprimitive >= "gep" 2)
(defprimitive = "eqnp" 2)
(defprimitive < "ltp" 2)
(defprimitive > "gtp" 2)

(defprimitive vector?         "vectorp" 1)
(defnaryprimitive make-vector "make_vector" 1)
(defprimitive vector-ref      "vector_ref"  2)
(defprimitive vector-set!     "vector_set"  3)
(defprimitive vector-length   "vector_length" 1)
(defnaryprimitive vector "vector" 0)

(defnaryprimitive string "string" 0)
(defnaryprimitive make-string "make_string" 1)
(defprimitive string-length "string_length" 1)
(defprimitive string-ref "string_ref" 2)
(defprimitive string-set! "string_set" 3)

(defprimitive call/ep "callep" 1)
(defnaryprimitive apply "apply" 2)
(defnaryprimitive list "list" 0)

(defprimitive eof-object? "eofp" 1)
(defnaryprimitive newline "newline" 0)  ; should be 1+1       FUTURE
(defnaryprimitive display "display" 1)  ; should be 1+1       FUTURE
(defnaryprimitive write   "write" 1)    ; should be 1+1	      FUTURE
(defnaryprimitive read "read" 0)        ; should be 1+1	      FUTURE
(defprimitive current-input-port "current_input_port" 0)
(defprimitive current-output-port "current_output_port" 0)
(defprimitive open-input-file "open_input_file" 1)
(defprimitive open-output-file "open_output_file" 1)
(defprimitive close-input-file "close_input_file" 1)
(defprimitive close-output-file "close_ouput_file" 1)

;;; One is needed for objectify: the `unspecified' variable which holds
;;; a regular value with no specific properties. 

(definitial unspecified "UNSPECIFIED")

;;; Not really useful for r4rs.

(definitial t   "TRUE")
(definitial f   "FALSE")
(definitial nil "NIL")

;;; }}}
;;; end of javaenv.scm
