;;; $Id: javatest.scm,v 1.4 1996/09/04 07:22:35 queinnec Exp $
;;; By Christian.Queinnec@inria.fr (http://pauillac.inria.fr/~queinnec/)
;;; This file is part of a Scheme->Java compiler derived from the Scheme->C
;;; compiler described in the LiSP book. 

;;; Quick and Dirty tests of the Java code generation.

;;; The generic function to handle exceptions while testing.

(define-generic (test-Java-compiler-handler (exc) c?))

(define-method (test-Java-compiler-handler (exc Compilation-Warning) c?)
  (flush-output-port stdout)
  (display (Compilation-Warning-phase exc) stderr)
  (display " Warning: " stderr)
  (display (Compilation-Warning-message exc) stderr)
  (newline stderr)
  (for-each (lambda (hint) 
              (show hint stderr)
              (newline stderr) )
            (Compilation-Warning-culprits exc) ) )

(define *tests-to-skip*
  '( (SET! XYZZY 3)
     ) )

;;; (compile->Java e (current-output-port))

(define (test-Java-compiler file)
  (suite-test
   file
   "Scheme? "
   "Scheme= "
   #t
   (lambda (read check err)
     (define-method (test-Java-compiler-handler (exc Exception) c?)
       (err exc) )
     (lambda ()
       (monitor test-Java-compiler-handler
         (let ((e (read)))
           (cond ((member e *tests-to-skip*)
                  (compiler-main-error "Test to skip") )
                 ((appear? 'call/cc e)
                  (check (test-expression 
                          `((lambda (call/cc) ,e) call/ep) )) )
                 (else (check (test-expression e))) ) ) ) ) )
   naive-match ) )
;;; Test: (test-Java-compiler "../DEA/scheme.tst")

;;; Compile a single Sexpression E. 
;;; Write it in a file, compile then run this program, reads its output 
;;; (normally a single Sexpression) and return it.

(define (test-expression e)
  (define tmpfile "/tmp/Main.scm")
  (call-with-output-file tmpfile
    (lambda (out) (write e out)) )
  ;; This is the body of (compiler-main (list tmpfile))
  (let ((cmd (make-Java-Compilation #f #f #f #f)))
    (scan-options! cmd (list tmpfile))
    (when (Compilation-verbose? cmd)
      (display "Generate Java code...")(newline) )
    (let ((e `(begin . ,(file->list (Compilation-source cmd)))))
      (call-with-output-file (Java-Compilation-javafile cmd)
        (lambda (out) (compile->Java e (predefined-global-env) out)) ) )
    (when (Compilation-verbose? cmd)
      (display "Calling Java compiler...")(newline) )
    (call-external-compiler cmd) )
  (run-compiled-program "/tmp/Main") )

;;; Things are complex since the Java compiler is not on blaye but on
;;; youpou but I don't have bigloo on youpou. So I generate Java on
;;; blaye:/tmp/Main.java, transport it into youpou:/tmp/Main.java,
;;; compile and execute there. Care must be taken to the classpath
;;; which should be expanded correctly.

(set! Java-compiler 
      (string-append "rcp /tmp/Main.java youpou:/tmp ; "
                     "rsh youpou \"rm /tmp/Main.class >& /dev/null \"; "
                     "rsh youpou javac " ) )

(set! Java-flags 
  (string-append " -g -verbose -classpath "
                 "/usr/local/java/lib/classes.zip:"
                 "/home/queinnec/icslas/o/Java.sun4_solaris:. " ) )

(define-method (call-external-compiler (cmd Java-Compilation))
  (let* ((status 0)
         (java-cmd (string-append Java-compiler
                                  Java-flags
                                  (Java-Compilation-javafile cmd) ))
         (complete-shell-command java-cmd) )
    (set! status (system complete-shell-command))
    (unless (= (modulo status 256) 0)
      (compiler-main-error "Java compilation problem" status) )
    cmd ) )

(define (run-compiled-program aout)
  (let ((status 0)
        (run-cmd (string-append
                  "rsh youpou \" cd /tmp ; java -verbose -classpath "
                  "/usr/local/java/lib/classes.zip:"
                  "/home/queinnec/icslas/o/Java.sun4_solaris:. "
                  " Main \" | tee > /tmp/tmplog ; "
                  "sed -e 's:#<:\<:g' -e 's:@::g' < /tmp/tmplog > /tmp/log" )) )
    (set! status (system run-cmd))
    (unless (= status 0)
      (compiler-main-error "Java execution problem" status) )
    (let ((e (call-with-input-file "/tmp/log" read)))
      ;; The file may contain "Uncaught Exception: ..."
      (if (or (eof-object? e) 
              (eq? e 'Uncaught) )
          (compiler-main-error "No result" e)
          e ) ) ) )

;;; end of javatest.scm
