;;; $Id: mkboot.scm,v 1.9 1998/11/24 15:47:58 queinnec Exp $

;;;              Expand boot files.
;;; This file is made of parts snarfed from ~/icslas/Bootstrap/*.scm
;;; This file is supposed to run with Bigloo. It expands some Scheme
;;; files containing the unessential library procedures (those that may
;;; be defined in pure Scheme) and a macroexpander able to remove all
;;; derived syntaxes of Scheme. In particular, this macroexpander must
;;; be expanded itself. Two files are produced: boot1.scm and bootm1.scm.

;;; Special for Bigloo
;(define stderr (current-error-port))
;(define *current-scheme* '(Bigloo))

;;; Special for Elk
(define stderr (current-output-port))
(set! print-length -1) ; -1 means infinite
(set! print-depth  -1) ; -1 means infinite
 (define gensym
   (let ((counter 99))
     (lambda args
       (set! counter (+ counter 1))
       (symbol-concatenate (if (pair? args) (car args) 'G)
                           counter ) ) ) )
(define *current-scheme* '(Elk))

;;; Special for Gambit
;(define stderr ##stderr)
;(define reverse! reverse)
;(define *current-scheme* '(Gambit))

(define (symbol-concatenate . names)
  (string->symbol
   (apply string-append
          (map (lambda (thing)
                 (cond ((symbol? thing) 
                        (symbol->string thing) )
                       ((string? thing) thing)
                       ((number? thing)
                        (number->string thing) )
                       (else (error 
                              "non coercible to String"
                              thing )) ) )
               names ) ) ) )

;;; (1) Load the neutral macro-expander (it does not define any macros
;;; (nor uses macros for its own definition) but uses non-essential
;;; syntax such as (and), (let) ...). 

(load "../../Macros/clexpand.scm")

;;; (2) Define define-icslas-macro as a macro for bigloo.

(define-macro (define-icslas-macro call . body)
  `(define-macro ,call . ,body) )

;;; (3) and load the macro-expander that customizes the neutral
;;; macro-expander so it can remove non-essential syntaxes such as let
;;; forms, internal define forms, etc.

(load "../../Macros/clmacros.scm")

;;; Read a whole file into a list of expressions.

(define (file->list filename)
  (call-with-input-file filename
    (lambda (in)
      (let loop ((e (read in)))
        (if (eof-object? e)
            '()
            (cons e (loop (read in))) ) ) ) ) )

;;; Print a series of expressions into a file

(define (list->file expressions filename)
  (call-with-output-file filename
    (lambda (out)
      (for-each (lambda (e)
                  (write e out)
                  (newline out) )
                expressions )
      (newline out)
      filename ) ) )

;;; (4) Expand the basic library (that contains all the predefined but
;;; non primitive Scheme functions (append, map, etc.)) The difficulty
;;; is that we want to expand for a dialect that offers the
;;; uninitialized-let special form.

(set! enriched-icslas-expansion-environment
      (extend-with-one-shot-macro
       enriched-icslas-expansion-environment
       'uninitialized-let
       (lambda (macroexpand variables . body)
         `(uninitialized-let ,variables 
                 . ,(walk* body macroexpand) ) ) ) )

(let* ((e* (file->list "../../Library/libscm1.scm"))
       (ne* (map (lambda (e)
                   (walk e enriched-icslas-expansion-environment) )
                 e* )) )
  (list->file ne* "boot1.escm") )

;;; (4bis) En passant, expand a naive Scheme interpreter.
;;; Attention natScheme.scm uses some macros defined by define-macro.

(define expansion-environment-for-natScheme
  (extend-with-one-shot-macro 
   enriched-icslas-expansion-environment
   'define-macro
   (lambda (macroexpand call . body)
     (set! expansion-environment-for-natScheme
           (extend-with-repetitive-macro 
            expansion-environment-for-natScheme
            (car call)
            (eval `(lambda (macroexpand . ,(cdr call))
                     ,(walk (analyze-body body)
                            (extend-with-variables macroexpand 
                                                   (cdr call) ) ) ) ) ) )
     `',(car call) ) ) )

(let* ((e* (file->list "../../../DEA/natScheme.scm"))
       (ne* (map (lambda (e)
                   (walk e expansion-environment-for-natScheme) )
                 e* )) )
  (list->file ne* "natScheme.escm") )

;;; (5) Expand the expander itself (and keep it).

(define bootm1
  (let* ((e* (file->list "../../Macros/clexpand.scm"))
         (ne* (map (lambda (e)
                     (walk e enriched-icslas-expansion-environment) )
                   e* )) )
    ;; (list->file ne* "bootm1.escm")
    ne* ) )

;;; (6) Expand the expanders of the non essential syntaxes. Skip the
;;; needless expander for uninitialized-let since the target language
;;; offers it.

(set! enriched-icslas-expansion-environment
      (extend-with-one-shot-macro 
       enriched-icslas-expansion-environment
       'define-icslas-expander
       (lambda (macroexpand call . body)
         `(set! enriched-icslas-expansion-environment
                (extend-with-repetitive-macro
                 enriched-icslas-expansion-environment
                 ',(car call)
                 (lambda (macroexpand . ,(cdr call))
                   ,(walk (analyze-body body)
                          (extend-with-variables macroexpand 
                                                 (cdr call) ) ) ) ) ) ) ) )

(define bootm2
  (let* ((e* (cdr (member "Expanders section after uninitialized-let"
                          (file->list "../../Macros/clmacros.scm") )))
         (ne* (map (lambda (e)
                     (walk e enriched-icslas-expansion-environment) )
                   e* )) )
    ;; (list->file ne* "bootm2.escm")
    ne* ) )

;;; Produce the complete bootm1.escm file.
;;; Before introduce if-icslas-feature to process Bind/utils.scm

(set! enriched-icslas-expansion-environment
      (extend-with-repetitive-macro 
       enriched-icslas-expansion-environment
       'if-icslas-feature
       (lambda (macroexpand condition th-en . el-se)
         (if (feature-present? condition) 
             th-en
             (if (pair? el-se) (car el-se) '#f) ) ) ) )

(define *icslas-features* 
  (append *current-scheme*
          '(direct-mode 
            Jaja Icslas
            (not AddLibrary) 
            Utilities FrameWork Meroonet FrontEnd 
            (not QDeval) (not TreeCode)
            (not ByteCode) Java-generation ) ) )

;;; Unfortunately, there are many macros in these files. The
;;; if-icsla-feature macro must be expanded immediately to get rid of
;;; the needless functions. Unfortunately, the definition of
;;; if-icsla-feature use (as it appears in Bind/macros.scm uses an
;;; internal define so if it is expanded towards Icslas, an
;;; uninitialized-let will appear that Bigloo cannot interpret). We
;;; split expansion into two phases).

(define (feature-present? condition)
  (if (pair? condition) 
      (case (car condition)
        ((and) 
         (if (pair? (cdr condition))
             (if (pair? (cddr condition))
                 (and (feature-present? (cadr condition))
                      (feature-present? (cons 'and (cddr condition))) )
                 (feature-present? (cadr condition)) )
             #f ) )
        ((or) 
         (if (pair? (cdr condition))
             (if (pair? (cddr condition))
                 (or (feature-present? (cadr condition))
                     (feature-present? (cons 'or (cddr condition))) )
                 (feature-present? (cadr condition)) )
             #t ) )
        ((not) (not (feature-present? (cadr condition))))
        (else (member condition *icslas-features*)) )
      (member condition *icslas-features*) ) )

(list->file
 `(,@ bootm1  ; the neutral macro-expander
   (set! enriched-icslas-expansion-environment
         neutral-walking-env )
   ,@ bootm2  ; the derived syntaxes
   ,@(map (lambda (e)
            (walk e enriched-icslas-expansion-environment) )
          `(
            ;; Need reverse! and other helper functions
            ,@(file->list "../../Bind/utils.scm")
            (define (expand-error . messages)
              (diagnose messages #f) )
            ;; add eval-in-expansion-world as a macro so it may be used
            ;; not only at toplevel but deeper within expressions.
            (set! enriched-icslas-expansion-environment
                  (extend-with-one-shot-macro
                   enriched-icslas-expansion-environment
                   'eval-in-expansion-world
                   (lambda (macroexpand . forms)
                     (eval (cons 'begin forms)) ) ) )
            ;; Update expand to hold this new macro-expander.
            (set! expand 
                  (lambda (e)
                    (walk e enriched-icslas-expansion-environment) ) ) )
          )
   )
 "bootm1.escm" )

;;; The compiler is not a strict part of the bootstrap. With the
;;; previous files, we have a Jaja interpreter which is nearly a
;;; complete Scheme implementation (but for full-fledged
;;; continuations). It is therefore possible to load it as it is
;;; (probably prefixed with the definition of define-macro).  The
;;; problem is that the compiler can only compile a single file so we
;;; need to gather all the files of the compiler into a single file.

;;; First gather the names of all needed files so, load the generic
;;; galley file with suitable parameters to obtain the list of all
;;; files to load. Use these features to parameterize the Icslas
;;; source files.

(set! *icslas-features* 
  (append *current-scheme*
          '(direct-mode
            Jaja Icslas
            (not AddLibrary) 
            Utilities FrameWork Meroonet FrontEnd 
            (not QDeval) (not TreeCode)
            (not ByteCode) Java-generation ) ) )

(load "../../icslas.scm")

;;; Collect the names of the files. (THESE ARE THE FILES FOR THE
;;; COMPILER: Bind/macros Bind/utils Bootstrap/filutl Bind/meroonet
;;; Bind/prepare Bind/except Bind/glue Ast/progr Ast/predenv Ast/check
;;; Ast/objify Analyzes/walk Analyzes/alphat Analyzes/anorm
;;; Analyzes/box Analyzes/inian Analyzes/colltemp Analyzes/lift
;;; Analyzes/extract Analyzes/quotes Analyzes/rmseq
;;; Java/Javagen/javagen Java/Javagen/javaenv Java/Javagen/javacomp)

(define compiler-files
  (let ((files '()))
    (process-icslas-sources 
     *icslas-features*
     (lambda (filename)
       (set! files (append files (list filename))) ) )
     (newline)
     (display `(These are the files for the compiler: . ,files))
     (newline)
     files ) )

;;; Gather and expand all these files. This is mainly for debug.

(define compiler*
  (map (lambda (filename)
         `(begin . ,(file->list 
                     (string-append "../../" filename ".scm") )) )
       compiler-files ) )

;(list->file compiler* "compiler.gscm")    ; for DEBUG

;;; This new expansion environment contains if-icslas-feature. Since
;;; the definition of if-icslas-feature also appears in
;;; Bind/macros.scm, we redefine define-icslas-macro to skip it.

(define pre-compiler-expansion-environment
  (extend-with-repetitive-macro
   neutral-walking-env
   'if-icslas-feature
   (lambda (macroexpand condition th-en . el-se)
     (if (feature-present? condition) 
         th-en 
         (if (pair? el-se) (car el-se) '#f) ) ) ) )

(set! pre-compiler-expansion-environment
  (extend-with-one-shot-macro
   pre-compiler-expansion-environment
   'define-icslas-macro
   (lambda (macroexpand call . body)
     (if (eq? (car call) 'if-icslas-feature)
         `#f
         `(define-icslas-macro ,call . ,body) ) ) ) )

;;; Remove if-icslas-feature

(begin
  (set! compiler*
        (walk compiler* pre-compiler-expansion-environment) )
  ;; (list->file compiler* "compiler.ggscm")   ;; for DEBUG
  'compiler* )

;;; Define a new environment to remove all other macros. They should
;;; generate Icslas code. Macro-defining macros that are still present
;;; are define-icslas-macro (for when, unless, define-class,
;;; define-generic, define-method, define-special-form,
;;; between-parentheses, within-block, with-possible-return,
;;; definitial, defprimitive, defnaryprimitive), define-macro (for
;;; case-cs). Unfortunately, to expand Meroonet, Meroonet must be
;;; present so we load it for Bigloo and meroonet needs
;;; symbol-concatenate which is conditionally defined in
;;; Bind/utils.scm (so we need to load Bind/macros.scm to have
;;; if-icslas-feature available).

(load "../../Bind/macros.scm")
(load "../../Bind/utils.scm")
(load "../../Bind/meroonet.scm")

(define enhanced-icslas-expansion-environment
  (extend-with-one-shot-macro 
   enriched-icslas-expansion-environment
   'define-icslas-macro
   (lambda (macroexpand call . body)
     (set! enhanced-icslas-expansion-environment
           (extend-with-repetitive-macro 
            enhanced-icslas-expansion-environment
            (car call)
            (eval `(lambda (macroexpand . ,(cdr call))
                     ,(walk (analyze-body body)
                            (extend-with-variables macroexpand 
                                                   (cdr call) ) ) ) ) ) )
     `',(car call) ) ) )

(set! enhanced-icslas-expansion-environment
  (extend-with-one-shot-macro 
   enhanced-icslas-expansion-environment
   'define-macro
   (lambda (macroexpand call . body)
     (set! enhanced-icslas-expansion-environment
           (extend-with-repetitive-macro 
            enhanced-icslas-expansion-environment
            (car call)
            (eval `(lambda (macroexpand . ,(cdr call))
                     ,(walk (analyze-body body)
                            (extend-with-variables macroexpand 
                                                   (cdr call) ) ) ) ) ) )
     `',(car call) ) ) )

;;; Must use this relay so new macros that are added to
;;; enhanced-icslas-expansion-environment are immediately available to
;;; other expressions (in a begin form for instance).

(define compiler-expansion-environment
  (lambda (e)
    (enhanced-icslas-expansion-environment e) ) )

;;; The compiler also needs to expand the expressions it has to
;;; compile, so incorporate also a macroexpander into it. Use the
;;; already produced macroexpander.

(list->file 
 (map (lambda (e) (walk e compiler-expansion-environment))
      `(
        ,@ bootm1                           ; the neutral macro-expander
        (set! enriched-icslas-expansion-environment
              neutral-walking-env )
        ,@ bootm2 ; the derived syntaxes
        ,@compiler*        
        ) )
 "compiler.escm" )

;;; end of mkboot.scm
