;;; $Id: libscm1.scm,v 1.9 1998/11/24 14:17:44 queinnec Exp $

;;; This file contains the functions of R4RS that are not primitive.

(define (not o) (eq? o #f))

(define (eqv? x y)
  (or (eq? x y)
      (and (number? x) 
           (number? y)
           (= x y) ) ) )

(define (equal? x y)
  (or (eq? x y)
      (cond 
       ((number? x) (and (number? y) (= x y)))
       ;; Already handled:
       ;;((symbol? x) (and (symbol? y) (eq? x y)))
       ;; Same for input-port, output-port, boolean and procedure.
       ((pair? x) (and (pair? y)
                       (equal? (car x) (car y))
                       (equal? (cdr x) (cdr y)) ))
       ((vector? x)
        (and (vector? y)
             (let ((xn (vector-length x))
                   (yn (vector-length y)) )
               (and (= xn yn)
                    (let compare ((i 0))
                      (or (>= i xn)
                          (and (equal? (vector-ref x i) (vector-ref y i))
                               (compare (+ i 1)) ) ) ) ) ) ) )
       ((string? x) (and (string? y) (string=? x y)))
       ((char? x) (and (char? y) (char=? x y)))
       (else #f) ) ) )

(define (caar o) (car (car o)))
(define (cadr o) (car (cdr o)))
(define (cdar o) (cdr (car o)))
(define (cddr o) (cdr (cdr o)))

(define (caaar o) (car (car (car o))))
(define (caadr o) (car (car (cdr o))))
(define (cadar o) (car (cdr (car o))))
(define (caddr o) (car (cdr (cdr o))))
(define (cdaar o) (cdr (car (car o))))
(define (cdadr o) (cdr (car (cdr o))))
(define (cddar o) (cdr (cdr (car o))))
(define (cdddr o) (cdr (cdr (cdr o))))

(define (caaaar o) (car (car (car (car o)))))
(define (caaadr o) (car (car (car (cdr o)))))
(define (caadar o) (car (car (cdr (car o)))))
(define (caaddr o) (car (car (cdr (cdr o)))))
(define (cadaar o) (car (cdr (car (car o)))))
(define (cadadr o) (car (cdr (car (cdr o)))))
(define (caddar o) (car (cdr (cdr (car o)))))
(define (cadddr o) (car (cdr (cdr (cdr o)))))
(define (cdaaar o) (cdr (car (car (car o)))))
(define (cdaadr o) (cdr (car (car (cdr o)))))
(define (cdadar o) (cdr (car (cdr (car o)))))
(define (cdaddr o) (cdr (car (cdr (cdr o)))))
(define (cddaar o) (cdr (cdr (car (car o)))))
(define (cddadr o) (cdr (cdr (car (cdr o)))))
(define (cdddar o) (cdr (cdr (cdr (car o)))))
(define (cddddr o) (cdr (cdr (cdr (cdr o)))))

(define (null? o) (eq? o '()))

(define (list? o) (or (pair? o) (null? o)))

(define (length l)
  (define (iterate l n)
    (if (pair? l)
        (iterate (cdr l) (+ n 1))
        n ) )
  (iterate l 0) )

(define (reverse l)
  (define (rev l r)
    (if (pair? l)
        (rev (cdr l) (cons (car l) r))
        r ) )
  (rev l '()) )

;;; Handle errors in list-{tail,ref} ???

(define (list-tail l n)
  (if (> n 0)
      (list-tail (cdr l) (- n 1))
      l ) )

(define (list-ref l n)
  (if (> n 0)
      (list-ref (cdr l) (- n 1))
      (car l) ) )

(define (memq o l)
  (if (pair? l)
      (if (eq? o (car l))
          l
          (memq o (cdr l)) )
      #f ) )

(define (memv o l)
  (if (pair? l)
      (if (eqv? o (car l))
          l
          (memv o (cdr l)) )
      #f ) )

(define (member o l)
  (if (pair? l)
      (if (equal? o (car l))
          l
          (member o (cdr l)) )
      #f ) )

(define (assq o l)
  (if (pair? l)
      (if (pair? (car l))
          (if (eq? o (caar l))
              (car l)
              (assq o (cdr l)) )
          ;; ignore atomic things
          (assq o (cdr l)) )
      #f ) )

(define (assv o l)
  (if (pair? l)
      (if (pair? (car l))
          (if (eqv? o (caar l))
              (car l)
              (assv o (cdr l)) )
          ;; ignore atomic things
          (assv o (cdr l)) )
      #f ) )

(define (assoc o l)
  (if (pair? l)
      (if (pair? (car l))
          (if (equal? o (caar l))
              (car l)
              (assoc o (cdr l)) )
          ;; ignore atomic things
          (assoc o (cdr l)) )
      #f ) )

(define (append . ll)
  (define (append2 l1 l2)
    (if (pair? l1)
        (cons (car l1) (append2 (cdr l1) l2))
        l2 ) )
  (define (appendn l ll)
    (if (pair? ll)
        (let ((l2 (car ll))
              (ll (cdr ll)) )
          (if (pair? ll)
              (append2 l (appendn l2 ll))
              (append2 l l2) ) )
        l ) )
  (if (pair? ll)
      (appendn (car ll) (cdr ll))
      '() ) )

;;; Functions for numbers.

(define exact? integer?)

(define inexact? real?)

(define (zero? o)
  (= o 0) )

(define (positive? o)
  (>= o 0) )

(define (negative? o)
  (<= o 0) )

(define (odd? o)
  (= 1 (modulo o 2)) )

(define (even? o)
  (= 0 (modulo o 2)) )

(define (max num . numbers)
  (define (find-max num numbers)
    (if (pair? numbers)
        (find-max (if (>= num (car numbers))
                      num (car numbers) )
                  (cdr numbers) )
        num ) )
  (find-max num numbers) )

(define (min num . numbers)
  (define (find-min num numbers)
    (if (pair? numbers)
        (find-min (if (<= num (car numbers))
                      num (car numbers) )
                  (cdr numbers) )
        num ) )
  (find-min num numbers) )

(define (abs o)
  (if (>= o 0) o (- o)) )

;;; adapted from Bigloo.

(define (gcd . x)
  (define (gcd2 m n)
    (if (zero? n)
        m
        (let ((r (remainder m n)))
          (if (= r 0)
              n
              (gcd2 n r)))))
  (case (length x)
    ((0) 0)
    ((1) (abs (car x)))
    (else (let loop ((result (gcd2 (abs (car x)) (abs (cadr x))))
                     (left (cddr x)))
            (if (pair? left)
                (loop (gcd2 result (abs (car left))) (cdr left))
                result)))))

(define (lcm . x)
  (define (lcm2 m n)
    (let ((m (abs m)) (n (abs n)))
      (cond ((= m n) m)
            ((= (remainder m n) 0) m)
            ((= (remainder n m) 0) n)
            (else (* (quotient m (gcd m n)) n)))))
  (case (length x)
    ((0) 1)
    ((1) (abs (car x)))
    (else (let loop ((result (lcm2 (car x) (cadr x))) (left (cddr x)))
            (if (pair? left)
                (loop (lcm2 result (car left)) (cdr left))
                result)))))

;;; Functions for characters.

(define char-whitespace? 
  (let ((whitespace-characters 
         (string #\space
                 (integer->char 9);;  #\tab
                 (integer->char 13);; #\cr
                 (integer->char 12);; #\ff
                 (integer->char 10);; #\lf
                 ) ))
    (lambda (c)
      (char-member? c whitespace-characters) ) ) )

(define char-alphabetic? 
  (let ((alphabetic-characters 
         "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ" ))
    (lambda (c)
      (char-member? c alphabetic-characters) ) ) )

(define char-numeric?
  (let ((numeric-characters "0123456789"))
    (lambda (c)
      (char-member? c numeric-characters) ) ) )

;;; These functions rely on char-member? which is defined as:

(define (char-member? char string)
  (let ((len (string-length string)))
    (let search ((i 0))
      (if (>= i len)
          #f
          (if (char=? char (string-ref string i))
              i
              (search (+ i 1)) ) ) ) ) )

(define (char-upper-case? c)
  (char=? c (char-upcase c)) )

(define (char-lower-case? c)
  (char=? c (char-downcase c)) )

;;; Case change.

(define (char-upcase c)
  (string-ref to-upper-case-characters (char->integer c)) )

(define (char-downcase c)
  (string-ref to-lower-case-characters (char->integer c)) )

;;; Comparison predicates.

(define (char=? c1 c2)
  (= (char->integer c1) (char->integer c2)) )

(define (char<=? c1 c2)
  (<= (char->integer c1) (char->integer c2)) )

(define (char>=? c1 c2)
  (>= (char->integer c1) (char->integer c2)) )

(define (char<? c1 c2)
  (< (char->integer c1) (char->integer c2)) )

(define (char>? c1 c2)
  (> (char->integer c1) (char->integer c2)) )

(define (char-ci=? c1 c2)
  (char=? (char-downcase c1) (char-downcase c2)) )

(define (char-ci>=? c1 c2)
  (char>=? (char-downcase c1) (char-downcase c2)) )

(define (char-ci<=? c1 c2)
  (char<=? (char-downcase c1) (char-downcase c2)) )

(define (char-ci>? c1 c2)
  (char>? (char-downcase c1) (char-downcase c2)) )

(define (char-ci<? c1 c2)
  (char<? (char-downcase c1) (char-downcase c2)) )

;;; NOTE: For the two following functions, radix should be less than
;;; or equal to 16.
;;;                    Share the figures string.          FUTURE

(define (number->string n . radix)
  (define figures "0123456789ABCDEF")
  (let ((radix     (if (pair? radix) (car radix) 10))
        ;; next position to write.
        (index     0)
        ;; 65 is sufficient to hold -/+ 2^64 (a int4 or nat4) in base 2.
        (result    (make-string 65)) )
    (define (stringify! n)
      (cond ((< n radix)
             (string-set! result index (string-ref figures n))
             (set! index (+ index 1)) )
            (else
             (stringify! (quotient n radix))
             (stringify! (modulo n radix)) ) ) )
    (cond ((< n 0)
           (string-set! result 0 #\-)
           (set! index (+ index 1))
           (stringify! (- n)) )
          (else 
           ;;(string-set! result 0 #\+)
           ;;(set! index (+ index 1))
           (stringify! n) ) )
    (substring result 0 index) ) )

(define (string->number s . radix)
  (define figures "0123456789ABCDEF")
  (let ((radix (if (pair? radix) (car radix) 10))
        (len   (string-length s)) )
    (define (convert-figure char result)
      (if (< result radix)
          (if (char-ci=? char (string-ref figures result))
              result
              (convert-figure char (+ 1 result)) )
          #f ) )
    (define (read-number index result)
      (if (< index len)
          (let* ((char (string-ref s index))
                 (figure (convert-figure char 0)) )
            (if figure
                (if result
                    (read-number (+ index 1) (+ figure (* radix result)))
                    (read-number (+ index 1) figure) )
                result ) )
          result ) )
    (let ((char (string-ref s 0)))
      (case char
        ((#\+) (read-number 1 #f))
        ((#\-) (let ((r (read-number 1 #f)))
                 (if r (- r) #f) ))
        (else (read-number 0 #f)) ) ) ) )
;;; Tests: (= 421 (string->number "421"))
;;;        (= 421 (string->number "+421"))
;;;        (= -421 (string->number "-421"))
;;;        (= #x421 (string->number "421" 16))
;;;        (= #o421 (string->number "421" 8))
;;;        (= 10 (string->number "a" 16))
;;;        (eq? #f (string->number "z"))
;;;        (eq? #f (string->number "a"))
;;;        (eq? #f (string->number "-infinity"))
    
(define (string=? str1 str2)
  (let ((len1 (string-length str1))
        (len2 (string-length str2)) )
    (and (= len1 len2)
         (let compare ((i 0))
           (if (< i len1)
               (and (char=? (string-ref str1 i) (string-ref str2 i))
                    (compare (+ i 1)) )
               #t ) ) ) ) )

(define (string=-ci? str1 str2)
  (let ((len1 (string-length str1))
        (len2 (string-length str2)) )
    (and (= len1 len2)
         (let compare ((i 0))
           (if (< i len1)
               (and (char=-ci? (string-ref str1 i) (string-ref str2 i))
                    (compare (+ i 1)) )
               #t ) ) ) ) )

(define (compare-strings str1 str2)
  (let ((len1 (string-length str1))
        (len2 (string-length str2)) )
    (let compare ((i 0))
      (if (< i len2)
          (if (< i len1)
              (if (char<? (string-ref str1 i) (string-ref str2 i))
                  -1                    ; str1... < str2...
                  (if (char=? (string-ref str1 i) (string-ref str2 i))
                      (compare (+ i 1))
                      +1 ) )            ; str1... > str2...
              -1 )                      ; str1 < str2...
          (if (< i len1)
              +1                        ; str1... > str2
              0 ) ) ) ) )               ; str1 = str2

(define (string<? str1 str2)
  (< (compare-strings str1 str2) 0) )

(define (string<=? str1 str2)
  (<= (compare-strings str1 str2) 0) )

(define (string>? str1 str2)
  (> (compare-strings str1 str2) 0) )

(define (string>=? str1 str2)
  (>= (compare-strings str1 str2) 0) )

(define (compare-strings-ci str1 str2)
  (let ((len1 (string-length str1))
        (len2 (string-length str2)) )
    (let compare ((i 0))
      (if (< i len2)
          (if (< i len1)
              (if (char<-ci? (string-ref str1 i) (string-ref str2 i))
                  -1                    ; str1... < str2...
                  (if (char=-ci? (string-ref str1 i) (string-ref str2 i))
                      (compare (+ i 1))
                      +1 ) )            ; str1... > str2...
              -1 )                      ; str1 < str2...
          (if (< i len1)
              +1                        ; str1... > str2
              0 ) ) ) ) )               ; str1 = str2

(define (string<-ci? str1 str2)
  (< (compare-strings str1 str2) 0) )

(define (string<=-ci? str1 str2)
  (<= (compare-strings str1 str2) 0) )

(define (string>-ci? str1 str2)
  (> (compare-strings str1 str2) 0) )

(define (string>=-ci? str1 str2)
  (>= (compare-strings str1 str2) 0) )

(define (substring string start end)
  (let* ((size   (- end start))
         (result (make-string size)) )
    (do ((i 0 (+ i 1)))
        ((>= i size))
      (string-set! result i (string-ref string (+ start i))) )
    result ) )

(define (string-append . strings)
  (let* ((sizes (let count ((strings strings))
                  (if (pair? strings)
                      (+ (string-length (car strings))
                         (count (cdr strings)) )
                      0 ) ))
         (result (make-string sizes))
         (index 0) )
    (let fill ((strings strings))
      (if (pair? strings)
          (let* ((str (car strings))
                 (n (string-length str)) )
            (do ((i 0 (+ i 1)))
                ((>= i n))
              (string-set! result index (string-ref str i))
              (set! index (+ index 1)) )
            (fill (cdr strings)) )
          result ) ) ) )

(define (string->list str)
  (let ((n (string-length str)))
    (let loop ((i 0))
      (if (< i n)
          (cons (string-ref str i) (loop (+ i 1)))
          '() ) ) ) )

(define (list->string chars)
  (apply string chars) )

(define (string-fill! str ch)
  (let ((n (string-length str)))
    (do ((i 0 (+ i 1)))
        ((>= i n))
      (string-set! str i ch) ) ) )

(define (vector->list vec)
  (let ((n (vector-length vec)))
    (let loop ((i 0))
      (if (< i n)
          (cons (vector-ref vec i) (loop (+ i 1)))
          '() ) ) ) )

(define (list->vector items)
  (apply vector items) )

(define (vector-fill! str item)
  (let ((n (vector-length str)))
    (do ((i 0 (+ i 1)))
        ((>= i n))
      (vector-set! str i item) ) ) )

(define (every? f l)
  (if (pair? l)
      (and (f (car l))
           (every? f (cdr l)) )
      #t ) )

(define (any? f l)
  (if (pair? l)
      (or (f (car l))
          (any? f (cdr l)) )
      #f ) )

(define (map f l . ll)
  (define (map2 f l)
    (define (mp l)
      (if (pair? l)
          (let ((r (f (car l))))
            (cons r (mp (cdr l))) )
          '() ) )
    (mp l) )
  (define (mapn ll)
    (if (every? pair? ll)
        (let ((r (apply f (map2 car ll))))
          (cons r (mapn (map2 cdr ll))) )
        '() ) )
  (if (pair? ll)
      (mapn (cons l ll))
      (map2 f l) ) )

;;; map is already defined to be from left to right.

(define for-each map)

(define (force promise) (promise))

(define (call-with-input-file filename proc)
  (let ((in (open-input-file filename)))
    (let ((result (proc in)))
      (close-input-port in)
      result ) ) )

(define (call-with-output-file filename proc)
  (let ((in (open-output-file filename)))
    (let ((result (proc in)))
      (close-output-port in)
      result ) ) )

;;; Missing functions are those that need dynamic binding (for ports).

;;; end of lib.scm
