;;; $Id: print.scm,v 1.2 1995/07/06 18:25:05 queinnec Exp $

;;;                   A simple printer.  This file is based on the
;;; printer of LiSP2TeX: LiSP2TeX/emit.scm The only primitives that
;;; are used are display-char, display-string and display-newline.

;;; FUTURE: This file will disappear since MeroonV3 will bring its own
;;; printer. The generic function show will have to be extended to
;;; print numbers (with a given output base) and quasiquotation.

;;;==================================================================
;;; Needs: display-char, display-string and display-newline.
;;;        Meroon for show, object?

;;; Exports variables *output-base*
;;;                   *display-for-read*
;;;                   display, write, write-char, newline.

;;;==================================================================
;;; Some global settings.

;;; By default, the output base for numbers is ten.

(define *output-base* 10)

;;; Control how strings are displayed (with or without double quotes).
;;; This variable should be replaced by a dynamic binding to avoid
;;; shared mutation in concurrent computations.                     FUTURE

(define *display-for-read* #f)

;;;==================================================================
;;; Interface functions.

(define (display o . stream)
  (set! *display-for-read* #f)
  (display-value o (if (pair? stream) (car stream) (current-output-port))) )

(define (write o . stream)
  (set! *display-for-read* #t)
  (display-value o (if (pair? stream) (car stream) (current-output-port))) )

(define (write-char c . stream)
  (display-char c (if (pair? stream) (car stream) (current-output-port))) )

(define (newline . stream)
  (display-newline (if (pair? stream) (car stream) (current-output-port))) )

;;;==================================================================
;;; The real printing engine.

(define (display-value o stream)
  (cond 
   ((integer? o)     (display-integer o stream))
   ((input-port? o)  (display-string "#<an input Port>" stream))
   ((output-port? o) (display-string "#<an output Port>" stream))
   ((char? o)        (display-char o stream))
   ((symbol? o)      (display-string (symbol->string o) stream))
   ((null? o)        (display-string "()" stream))
   ((pair? o)        (display-form o stream))
   ((string? o)      (if *display-for-read*
                         (begin (display-char #\" stream)
                                (display-string o stream)
                                (display-char #\" stream) )
                         (display-string o stream) ) )
   ((boolean? o)     (display-string (if o "#T" "#F") stream))
   ((procedure? o)   (display-string "#<a Function>" stream))
   ((vector? o)      (display-string "#(" stream)
                     (display-vector-content o stream)
                     (display-string ")" stream) )
   ;;((Object? o)      (show o stream))                       ;; FUTURE
   (else             (display-string "#<UFO>" stream)) ) )

(define (display-form o stream)
  ;; (assume (pair? o))
  (case (car o)
    ((quote) 
     (cond ((and (pair? (cdr o)) (null? (cddr o)))
            (display-char #\' stream)
            (display-value (cadr o) stream) )
           (else (display-normal-list o stream)) ) )
    ((quasiquote) 
     (cond ((and (pair? (cdr o)) (null? (cddr o)))
            (display-char #\` stream)
            (display-value (cadr o) stream) )
           (else (display-normal-list o stream)) ) )
    ((unquote)  
     (cond ((and (pair? (cdr o)) (null? (cddr o)))
            (display-char #\, stream)
            (display-value (cadr o) stream) )
           (else (display-normal-list o stream)) ) )
    ((unquote-splicing)  
     (cond ((and (pair? (cdr o)) (null? (cddr o)))
            (display-string ",@" stream)
            (display-value (cadr o) stream) )
           (else (display-normal-list o stream)) ) )
    (else (display-normal-list o stream)) ) )

(define (display-normal-list o stream)
  (display-char #\( stream)
  (display-value (car o) stream)
  (display-list-content (cdr o) stream)
  (display-char #\) stream) )

(define (display-list-content o stream)
  (define (display-dotted-list o)
    (display-string " . " stream)
    (display-value o stream) )
  (cond ((null? o) o)
        ((pair? o) 
         (if (memq (car o) '(quote quasiquote unquote unquote-splicing))
             (display-dotted-list o)
             (begin (display-char #\space stream)
                    (display-value (car o) stream)
                    (display-list-content (cdr o) stream) ) ) )
        (else (display-dotted-list o)) ) )

(define (display-vector-content o stream)
  (let ((leng (vector-length o)))
    (do ((i 0 (+ i 1))
         (j 1 (+ j 1)) )
        ((= i leng) o)
      (display-value (vector-ref o i) stream)
      (if (not (= j leng)) (display-char #\space stream)) ) ) )

(define (display-integer n stream)
  ;; These are the characters used to display numbers.
  (define *number-images* "0123456789ABCDEF")
  (if (< n 0) (begin (display-char #\- stream)
                     (display-integer (- n) stream) ))
  (let ((p (quotient n *output-base*)))
    (if (> p 0) (display-integer p stream))
    (display-char (string-ref *number-images* (remainder n *output-base*))
                  stream ) ) )

;;; end of print.scm
