;;; $Id: derstand.scm,v 1.2 1995/06/21 19:33:16 queinnec Exp $

;;; This file defines an utility program that allows to analyse a
;;; whole file, check for some errors, extract global variables etc.
;;; It should help to produce module directives to declare what to
;;; import or export. 

;;; This tool is quite intricated with the rest of Icslas.

(define (ramses)
  (display "Filename to analyse? ")
  (let ((filename (read)))
    (cond ((string? filename)
           (ramses-analyze filename '(expand) (current-output-port))
           "Done" )
          (else "Not a string!") ) ) )

;;; This is the main function: it takes the name of a file, analyze
;;; it, then produces some information.

(define (ramses-analyze filename options out)
  (call/cc
   (lambda (exit)
     (monitor ramses-handler
       (define-method (ramses-handler (exc Exception) c?)
         (exit "Failed!") )
       (display "Reading..." out)
       (let* ((e*       (file->list filename))
              (g        (make-Global-Environment g.r4rs '()))
              (expanded #f)
              (o        #f) )
         (newline out)
         (display "Expanding..." out)
         (set! expanded 
               (walk `(begin . ,e*)
                     enriched-icslas-expansion-environment ) )
         (newline out)
         ;; Show expanded code ?
         (if (memq 'expand options) (pp expanded out))
         ;; Parse and analyze the program.
         (set! o (objectify expanded (make-Local-Environment '()) g))
         ;; Report the global variables.
         (display "Global variables:" out)(newline out)
         (display-global-variables (Global-environment-global g) out)
         o ) ) ) ) )

;;; Report global variables.

(define (display-global-variables g out)
  (for-each (lambda (gv)
              (display (Variable-name gv) out)
              (display " " out) )
            g )
  (newline out) )

;;;==================================================================
;;; The exception handler.

(define-generic (ramses-handler (exc) c?)
  (basic-handler exc c?) )

(define-method (ramses-handler (exc Unknown-Variable) c?)
  (let* ((g   (Unknown-Variable-g exc))
         (gvs (Global-Environment-global g))
         (v   (make-Global-Variable (Unknown-Variable-name exc) 
                                    #f )) )
    (set-Global-Environment-global! g (cons v gvs))
    (make-Global-Reference v) ) )

;;; end of derstand.scm
