;;; $Id: genenv.scm,v 1.6 1994/10/15 14:51:14 queinnec Exp $

;;; This file defines the predefined environment for Tree-Code compilation.

;;; This is the initial global compile-time environment.

(define tcg.current
  (make-Global-Environment '() '()) )

;;;==================================================================
;;; New classes.

(define-class TC-Global-Variable Global-Variable 
  ( box      ; an immutable box containing the value 
    ) )

(define-class TC-Predefined-Variable Predefined-Variable
  ( box      ; an immutable box containing the value 
    offset   ; an index that encodes the Predefined-Variable
    ) )

;;;==================================================================
;;; Macros to define the initial compile-time (and run-time) environments.
;;; They enrich the predefined part of tcg.current. For constants they also
;;; define global variables RT-{t,f,nil} holding icslas values.

(define-icslas-macro (definitial name)
  `(define ,(symbol-concatenate 'RT- name)
     (let* ((box (make-RT-Predefined-Box 
                  (make-RT-Constant ',name)
                  ',name ))
            (v (make-TC-Predefined-Variable
                ',name
                #f       ; no description
                box
                (length (Global-Environment-predefined tcg.current)) )) )
       (set-Global-Environment-predefined! 
        tcg.current (cons v (Global-Environment-predefined tcg.current)) )
       (RT-Box-value box) ) ) )

(define-icslas-macro (defprimitive name value desc)
  (let ((comparator (if (memq desc '(0 1 2 3)) '= '>=))
        (arity (case desc ((0 1 2 3) desc) ((>=0) 0) ((>=2) 2)
                 (else (Tree-Code-error 'incorrect-arity arity)) ))
        (coded-arity (case desc ((0 1 2 3) desc) ((>=0) -1) ((>=2) -3))) )
    `(let* ((box (make-RT-Predefined-Box
                  (,(symbol-concatenate 'make-RT-Primitive coded-arity)
                   ,value ',name )
                  ',name ))
            (v (make-TC-Predefined-Variable
                ',name
                (make-Functional-Description ,comparator ,arity)
                box
                (length (Global-Environment-predefined tcg.current)) )) )
       (set-Global-Environment-predefined!
        tcg.current (cons v (Global-Environment-predefined tcg.current)) )
       ',name ) ) )

(define-icslas-macro (defpredicate name value desc)
  (let ((comparator (if (memq desc '(0 1 2 3)) '= '>=))
        (arity (case desc ((0 1 2 3) desc) ((>=0) 0) ((>=2) 2)
                 (else (Tree-Code-error 'incorrect-arity arity)) ))
        (coded-arity (case desc ((0 1 2 3) desc) ((>=0) -1) ((>=2) -3))) )
    `(let* ((box (make-RT-Predefined-Box 
                  (,(symbol-concatenate 'make-RT-Primitive coded-arity)
                   ,(case coded-arity
                      ((0) `(lambda () (if (,value) RT-t RT-f)))
                      ((1) `(lambda (a) (if (,value a) RT-t RT-f)))
                      ((2) `(lambda (a b) (if (,value a b) RT-t RT-f)))
                      ((3) `(lambda (a b c) (if (,value a b c) RT-t RT-f)))
                      ((-1 -3) `(lambda a (if (apply ,value a) RT-t RT-f))) )
                   ,name )
                  ',name ))
            (v (make-TC-Predefined-Variable
                ',name
                (make-Functional-Description ,comparator ,arity)
                box
                (length (Global-Environment-predefined tcg.current)) )) )
       (set-Global-Environment-predefined!
        tcg.current (cons v (Global-Environment-predefined tcg.current)) )
       ',name ) ) )

(definitial t)
(definitial f)
(definitial nil)

;;; This is a constant but it must not be the value of a user-visible
;;; variable. It is used to test if variables are initialized or not.

(define RT-uninitialized (make-RT-Constant 'uninitialized))

;;; This variable is needed for (begin) and (if #f p)
(definitial unspecified)

(defprimitive cons cons 2)
(defprimitive car car 1)
(defprimitive cdr cdr 1)
(defpredicate pair? pair? 1)
(defpredicate symbol? symbol? 1)
(defpredicate eq? eq? 2)
(defprimitive set-car! set-car! 2)
(defprimitive set-cdr! set-cdr! 2)
(defprimitive + + 2)
(defprimitive - - 2)
(defpredicate = = 2)
(defpredicate < < 2)
(defpredicate > > 2)
(defprimitive * * 2)
(defpredicate <= <= 2)
(defpredicate >= >= 2)
(defprimitive remainder remainder 2)
(defprimitive display display 1)
(defprimitive list icslas-list >=0)

;;; These are special primitives.

(defprimitive apply icslas-apply >=2)

(defprimitive call/cc icslas-call/cc 1)

;;; end of genenv.scm
