;;; $Id: rtobj.scm,v 1.8 1994/10/24 13:26:24 queinnec Exp $

;;; This file defines the classes of the entities needed for the
;;; runtime of the Tree-Code interpreter. They are all prefixed with
;;; RT-.

(define-class RT-Entity Object () :virtual)

;;; This class is used for constants such as T, F, nil. The
;;; `uninitialized' internal constant is also a RT-Constant.

(define-class RT-Constant RT-Entity 
  ( name      ; (a string) the name of that constant
    ) )

;;;==================================================================
;;; The class of activation-frames. The environment is a linked chain
;;; of RT-Environments possibly finishing with a flat-closure. They
;;; are chained through the `next' field.

;;; FUTURE: Meroon requires the :maybe-uninitialized keywords but this
;;; is unefficient since it forces accessors to check whether there is
;;; a value or not. In principle, references to possibly uninitialized
;;; values know that they have to check this fact and it is their duty
;;; to do it, not to RT-Environment to check that.

(define-class RT-Environment RT-Entity 
  ( (= next)
    (* value)
    ) :maybe-uninitialized )

;;;==================================================================
;;; Closures.

(define-class RT-Invokable RT-Entity
  ( coded-arity        ; an integer
    body               ; a tree-code
    ) :virtual )

(define-class RT-Closure RT-Invokable
  ( closed             ; an environment
    ) :virtual )

(define-class RT-Fix-Closure RT-Closure ())
(define-class RT-Nary-Closure RT-CLosure ())

(define-class RT-Fix-Combinator RT-Invokable ())
(define-class RT-Nary-Combinator RT-Invokable ())

;;; Flat closures do not capture an environment but copies the needed
;;; variables in itself. They are probably far better wrt migration
;;; since they lessen the number of values to migrate.

;;; FUTURE: as for RT-Environment, Meroon requires the
;;; :maybe-uninitialized keyword which is a waste of time since all
;;; these values will be written during the initialization of the
;;; flat-closure.

(define-class RT-Flat-Invokable RT-Invokable
  ( (* value :maybe-uninitialized)
    ) )

(define-class RT-Fix-Flat-Closure RT-Flat-Invokable ())
(define-class RT-Nary-Flat-Closure RT-Flat-Invokable ())

;;;==================================================================
;;; The class of predefined invokable entities.
;;; This is very dependent of the underlying implementation and this is
;;; different in deval.scm or keval.scm

(define-class RT-Primitive RT-Entity
  ( address      ; an address
    name         ; (a string) that names the primitive.
    ) :virtual )

(define-class RT-Primitive0 RT-Primitive ())
(define-class RT-Primitive1 RT-Primitive ())
(define-class RT-Primitive2 RT-Primitive ())
(define-class RT-Primitive3 RT-Primitive ())
(define-class RT-Primitive-1 RT-Primitive ())
(define-class RT-Primitive-3 RT-Primitive ())

(define-class RT-Continuation RT-Entity
  ( stack     ; a continuation
    ) )

;;;==================================================================
;;; The class of mutable data structures.

(define-class RT-Box RT-Entity
  ( value
    ) :virtual )

;;; Mutable boxes are used by mutable local variables.

(define-class RT-Mutable-Box RT-Box ())

;;; Global boxes are used for global variables. They are subdivided into
;;; immutable predefined boxes and general boxes which hold their
;;; mutability in an additional field.

(define-class RT-Global-Box RT-Box
  ( name     ; (a string)  the name of the associated global variable
    ) :virtual )

(define-class RT-Predefined-Box RT-Global-Box ())

(define-class RT-General-Box RT-Global-Box 
  ( mutable?       ; a Boolean telling if the box is mutable or not.
    ) )

;;; end of rtobj.scm
