/* $Id: dmxxx.c,v 1.10 1998/04/13 20:31:58 queinnec Exp $
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////

 * This file is a template for dmxxx.c files. It must provide some functions
 * described below.
 */

#include "dmeroon.h"

DM_DefineStringWithCstring(XXX_version,
  "$Id: dmxxx.c,v 1.10 1998/04/13 20:31:58 queinnec Exp $");

/* RESOURCE DM_allocate_bare_instance
 * This function allocates an instance of a given class with a given size.
 * This size is expressed in bytes and only corresponds to the size of the
 * content of the DMeroon object. To this size is added the size of 
 * the DMeroon header (two void* fields) and the size required by the 
 * support language (held in the DM_PrefixSize() macro).

 SIGNATURE:
 (Object (DM_allocate_bare_instance Class nat3 Object))

 */

DM_SupportPointer 
DM_allocate_bare_instance (DM_SupportPointer wrapped_class, 
                           DM_nat3           size,
                           DM_SupportPointer wrapped_reference ) {
  DM_Class *class   = DM_FromSupport(wrapped_class);
  DM_nat3  sz       = size;
  void     *address;
  
  /* This code uses the XXX_allocator of the XXX support language.
   * It might be malloc or Boehm's GC_malloc or whatever.
   */

#ifdef _GNUC__
  if ( DM_HasOption(class, DM_CLASS_IS_STRINGENT) ) {
    sz += DM_ROOM_FOR_REALIGNMENT;
    address = (void*) XXX_allocate (DM_PrefixSize() + sz);
    if ( address == NULL )
      return DM_Abort(context, DME_ALLOCATION_PROBLEM);
    address = (void*) DM_OffsetRealign((DM_SupportAddress)address);
  } else 
#endif /* __GNUC__ */

    {
      address = (void*) XXX_allocate ( DM_PrefixSize() + sz );
      if ( address == NULL )
        return DM_Abort(context, DME_ALLOCATION_PROBLEM);
    }

  /* Structure the object for XXX.
   * Insert its header and any other missing fields.
   */
                     ???

  /* Initialize DMeroon header, its class and proxy, etc. 
   * NOTE: If you want a greater speed, inline the next call. */
  return DM_structure_bare_instance(wrapped_o, 
                                    wrapped_class, 
                                    wrapped_reference );
}


/* RESOURCE DM_print_char_star
 * This procedure allows DMeroon to use the printing machinery of the 
 * support language. string is the address of a C string, len is the number 
 * characters to be printed (if zero the whole string (until the next null
 * character) is printed), wrapped_port is an XXX value telling where to 
 * print (for debug, this may be stdout or stderr). The function returns
 * true if it succeeds printing.

 SIGNATURE:
 (boolean (DM_print_char_star string nat3 anything))

 */

DM_SupportBoolean
DM_print_char_star (char              *string, 
                    DM_nat3           len,
                    DM_SupportPointer wrapped_port) {
  int n = len ? len : strlen(string);

  if ( wrapped_port == NULL ) return;

  DM_DebugAtLevel(0,
    /* Check if wrapped_port is not a standard printing C port. If this
     * is the case, output the characters of the string with fprintf and
     * don't forget to immediately flush the output to allow the user to
     * notice it quickly.
     */
    if (   (wrapped_port == (DM_SupportPointer)stdout)
        || (wrapped_port == (DM_SupportPointer)stderr) ) {
      FILE *stream = (FILE*) wrapped_port;

      fprintf(stream, "%*s", n, string);
      fflush(stream);
      return DM_TRUE;
    }
);

  /* If wrapped_port is an XXX port, then write on it if possible.
   */
  ???

  /* To allow for remote debugging, if the port is a socket, use
   * DMeroon itself to print through this socket.
   */
  { DM_OutBuffer *outbuffer = DM_FromSupport(wrapped_port);
    if ( DM_IsA(outbuffer, OutBuffer) ) {
      DM_print_char_star_in_outbuffer(string, len, wrapped_port);
      return DM_TRUE;
    }
  }

  /* Unknown way of printing on wrapped_port. */
  return DM_Abort(context, DME_UNPRINTABLE_OBJECT);
}

/* RESOURCE DM_raise_exception
 * The error function. It takes an error code, a message identifying the 
 * problem as well as the filename and the linenumber indicating where
 * the error was signalled. This function may be very crude and may exit
 * the current process (after reporting the error).

 SIGNATURE:
 (anything (DM_raise_exception nat3 string string nat3))

 */

DM_SupportPointer
DM_raise_exception (DM_nat3 code, 
                    char    *message, 
                    char    *file, 
                    DM_nat3 line) {
  DM_DebugAtLevel(10,
    fflush(stdout);
    fprintf(stderr,"Error %lu in %s(%lu)\n%s\n",code,file,line,message);
);
  exit(1);
}

/* RESOURCE DM_initialize
 * This is the single function that initializes entirely DMeroon from the
 * support language. DM_initialize initializes individually the various
 * parts of DMeroon. It returns the Site instance describing reflectively
 * the DMeroon site. It also checks some assumptions to ease porting, you
 * may remove these tests when successful.

 SIGNATURE:
 (Site (DM_initialize))

 */

DM_SupportPointer
DM_initialize () {
  
  /* Only initialize once! */
  static DM_nat1 already_done = 0;
  if ( already_done ) return DM_ToSupport(DM_ReferenceTo(DM_site));
  already_done = 1;

  /* XXX-dependent initializations. */
  ???

  /* Initialize now the various packages (in order).  */
  DM_types_initialize();
  DM_kernel_initialize();
  DM_hash_initialize();
  DM_extr_initialize();
  DM_site_initialize();
  DM_net_initialize();
  DM_comm_initialize();
  DM_rdobj_initialize();
  DM_wrobj_initialize();
  DM_lib_initialize();
  DM_debug_initialize();
  DM_api_initialize();

  /* Return the site object. */
  return DM_ToSupport(DM_ReferenceTo(DM_site));
}

/* end of dmxxx.c */
