/* $Id: dmxxx.h,v 1.1 1995/12/01 18:55:35 queinnec Exp $
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////

 * This file is a template for a dmXXX.h file. Substitute the XXX and
 * ??? occurrences to what they stand for. The main difficulty of the
 * binding is to clearly understand the different existing viewpoints.
 * A DMeroon object generally appears within some XXX objects so there
 * is at least (1) a DMeroon-reference that is the address of the
 * first byte of the content of the DMeroon instance, (2) an
 * XXX-reference that, quite often, is the address of the XXX object
 * encapsulating the DMeroon instance. In this binding must be
 * specified how to convert between (1) and (2) back and forth (these
 * are the DM_FromSupport and DM_ToSupport macros).
 
 * See additional explanations in the DMeroon/Doc/BIND.txt file.
 */

#ifndef DMeroon_XXX_H
#define DMeroon_XXX_H

/* DOCUMENTATION
 * This is the XXX header file for DMeroon.
 * It defines how DMeroon objects are represented, accessed, managed, etc.
 * It only contains macros that will be used to customize DMeroon files.
 */

/* 
 * Define the name of this binding.
 * It will appear in the site.support field.
 */

#define DM_SUPPORT_LANGUAGE "XXX"

/* 
 * Include the files that are required to compile C code using resources
 * of the support language. Also adjoin here missing prototypes if any.
 */

#include "XXX.h"

/*
 * DMeroon objects are represented within an XXX object as follows:
 *      ???
 */

/*
 * This is the header used to recognize DMeroon instances among XXX values.
 */

#define DM_HEADER (???)

/* 
 * This is the C macro that recognizes an XXX object as a DMeroon instance.
 * Here wrapped_o is a XXX-reference towards a DMeroon object. The result
 * must be a C boolean.
 */ 

#define DM_IsDMeroonInstance(wrapped_o) \
  ( ??? )

/* 
 * Here are the types of the various kinds of values handled by XXX.
 
 * DM_SupportPointer is the type used by XXX to refer to a Scheme value
 * whatever it is: a char, a fixnum or a heap-allocated structure.

 * DM_SupportPointer is the C type that XXX uses for fixnums.

 * DM_SupportFloatnum is the C type that XXX uses for floatnums.

 * DM_SupportBoolean means that the results are DM_TRUE or DM_FALSE
 * DM_SupportVoid means that the result is DM_DONE. These two types are
 * for documentation purpose only and must be similar to DM_SupportPointer.
 */

#define DM_SupportPointer  ???
#define DM_SupportFixnum   ???
#define DM_SupportFloatnum ???
#define DM_SupportBoolean  DM_SupportPointer
#define DM_SupportVoid     DM_SupportPointer

/*
 * This is the C type big enough to contain an address of the current 
 * machine. long is generally the appropriate one; if this is not the
 * case, you will have compilation errors and moreover this is checked
 * when DMeroon comes up running.
 */

#define DM_SupportAddress  long

/* 
 * This is how XXX values are recognized according to their types.
 * These macros return a C boolean.
 */

#define DM_IsSupportFixnum(wrapped_o) \
    ???
#define DM_IsSupportCharacter(wrapped_o) \
    ???
#define DM_IsSupportFloat(wrapped_o) \
    ???
#define DM_IsSupportDouble(wrapped_o) \
    ???
#define DM_IsSupportLongDouble(wrapped_o) \
    ???

/* 
 * This is how XXX values are built.
 * They take a C value and converts it into a legal XXX value.
 */

#define DM_ToSupport(dmeroon) \
    ???
#define DM_ToSupportFixnum(int) \
    ???
#define DM_ToSupportCharacter(char) \
    ???
#define DM_ToSupportFloat(float) \
    ???
#define DM_ToSupportDouble(double) \
    ???
#define DM_ToSupportLongDouble(longdouble) \
  

/*
 * This is how XXX values are decoded to recover the underlying 
 * corresponding C value.
 */

#define DM_FromSupport(wrapped_o) \
  ???
#define DM_FromSupportFixnum(wrapped_int) \
  ???
#define DM_FromSupportCharacter(wrapped_char) \
  ???
#define DM_FromSupportFloat(wrapped_float) \
  ???
#define DM_FromSupportDouble(wrapped_double) \
  ???
#define DM_FromSupportLongDouble(wrapped_longdouble) \
  ???

/*
 * DM_FALSE and DM_TRUE designates the XXX representations of #f and #t.
 */

#define DM_FALSE ???
#define DM_TRUE  ???

/*
 * When a C procedure has nothing better to return, it returns DM_DONE.
 * Generally such a procedure has type DM_SupportVoid corresponding to
 * C void. The C void type should be avoided since it makes invocation
 * of procedures harder: it is better to always return a meaningless value.
 */

#define DM_DONE DM_TRUE

/*
 * This macro specifies what is the prefix DMeroon objects should have to 
 * be embedded inside XXX values. This macro may define more than one field
 * but must suffix them with a colon for DM_SupportPrefix() and with a comma
 * for DM_SupportPrefixInitialization(t,v).
 */

#define DM_SupportPrefix() \
   ???

#define DM_SupportPrefixInitialization(type, varname) \
   ???

/*
 * Limits set by XXX.
 * DM_SUPPORT_FIXNUM_MAXIMUM is the most positive small integer XXX 
 * can represent. DM_SUPPORT_FIXNUM_MINIMUM is the least negative one
 * and is defined as its opposite.

 * On a 32bits cpu with a 2bits tag, this yields:
 *   DM_SUPPORT_FIXNUM_MAXIMUM = 536870911
 * Observe that we are insensitive to integer representation such as
 * complement to 1 or 2 (but we loose one integer!).
 * On a 64bits cpu with a 3bits tag, this yields:
 *   DM_SUPPORT_FIXNUM_MAXIMUM = 1152921504606846976

 * The following definition is correct but cannot be used in #if statement 
 * since it contains casts and sizeof operators. 
 *#define DM_SUPPORT_FIXNUM_MAXIMUM \
 * ((DM_SupportFixnum) \
 * ((((DM_SupportFixnum)1)<<( (CHAR_BIT*sizeof(DM_SupportFixnum)) \
 *                           - TAG_SHIFT -1 ))-1))

 * If all C DM_SupportFixnum are legal XXX values then undefine these
 * two limits.

 * NOTE: on alpha (a 64bit/word cpu) long takes 8 bytes so is equivalent
 * to long long on a 32bit/word cpu. Therefore this definition holds:
 */

#define DM_SUPPORT_FIXNUM_MAXIMUM \
   ???

#ifdef  DM_SUPPORT_FIXNUM_MAXIMUM
#define DM_SUPPORT_FIXNUM_MINIMUM (-DM_SUPPORT_FIXNUM_MAXIMUM)
#endif  /* DM_SUPPORT_FIXNUM_MAXIMUM */

/*
 * Unfortunately long long are not handled by cpp except on machines with
 * 64bit/word, even Gnu's cpp does not appreciate the official definition
 * of long_long_max entities with their LL suffix. If you redefine them
 * without the LL suffix then, on 32bit/word cpus, these definitions lead
 * to too big unsigned numbers that's why I remove them.
 */

#ifdef __GNUC__
#undef LONG_LONG_MAX
#undef ULONG_LONG_MAX
/*	#define LONG_LONG_MAX  9223372036854775807	*/
/*	#define ULONG_LONG_MAX 18446744073709551615	*/
#endif /* __GNUCC__ */

/*
 * These are the signature of the functions that must appear in the dmxxx.c
 * file.
 *
 * DM_initialize should be called before you use DMeroon (somewhere in
 *   your main). It returns the current site object.
 *
 * DM_allocate_bare_instance allocates an instance of <class> with a given
 *   size (expressed in bytes).
 *
 */

extern DM_SupportPointer 
DM_allocate_bare_instance (DM_SupportPointer wrapped_class, 
                           DM_nat3           size);

extern void
DM_finalize_exit (void *wrapped_exit, void* client_data);

extern DM_SupportVoid
DM_set_exit_finalization (DM_SupportPointer wrapped_exit);

extern DM_SupportPointer
DM_raise_exception (DM_nat3 code, 
                    char    *message, 
                    char    *file, 
                    DM_nat3 line);

extern DM_SupportBoolean
DM_print_char_star (char              *string, 
                    DM_nat3           len,
                    DM_SupportPointer wrapped_port);

extern DM_SupportPointer
DM_initialize (void);

#endif /* DMeroon_XXX_H */

/* end of dmXXX.h */
