;;; $Id: runxla.scm,v 1.4 1996/08/29 15:33:51 queinnec Exp $

;;; This file is evaluated when Icslas is started (but after the
;;; icslas.scm file). It analyzes the command line to determine which
;;; subsystem is to be run. It also receives the name of the currently
;;; running Scheme system (see the Commands/cxla command).

(define (runxla interpreter options)
  (runxla-banner interpreter)
  (set! *icslas-features* (list interpreter))
  (runxla-process-options options) )

;;; {{{ Analyzing options

;;; This variable will hold the desired features.

(define *icslas-features* '())

;;; The default extension. All common Icslas files have this extension.

(define *icslas-extension* ".scm")

;;; This variable tells *icslas-processor* to display the name of the
;;; files that are loaded.

(define *icslas-see-load* #f)

;;; The basic processor of Icslas sources is to load them.

(define *icslas-processor*
  (lambda (filename)
    (if *icslas-see-load*
        (begin (display ";;; Loading ")
               (display (string-append filename *icslas-extension*))
               (display " ...")
               (newline) ) )
    (load (string-append filename *icslas-extension*))
    (if *icslas-see-load*
        (begin (display ";;; Loaded ")
               (display (string-append filename *icslas-extension*))
               (newline) ) )
    'done ) )

;;; The list of features will specify what to run. Processing the
;;; options will augment it. Features are exploited in the icslas.scm
;;; file.

(define (runxla-process-options options)
  (let ((automatic? #t))
    (let scan ((options options))
      (if (pair? options)
          (cond 

           ((equal? (car options) "-not-auto")
            (set! automatic? #f)
            (scan (cdr options)) )

           ((equal? (car options) "-debug")
            (set! *icslas-features* (cons 'debug *icslas-features*))
            (scan (cdr options)) )

           ;; This option allows to know the files that are loaded.
           ;; This is useful when porting Icslas to another Scheme
           ;; implementation.
           ((equal? (car options) "-see-load")
            (set! *icslas-see-load* #t)
            (scan (cdr options)) )

           ((equal? (car options) "-C")
            (display ";;; Compiler from Scheme to C")(newline)
            (set! *icslas-features* (cons 'compiler-to-c *icslas-features*))
            (process-icslas-sources 
             *icslas-features*
             *icslas-processor* )
            (if automatic?
                (compiler-main (cdr options)) ) )
       
           ((equal? (car options) "-Java")
            (display ";;; Compiler from Scheme to Java")(newline)
            (set! *icslas-features* (cons 'compiler-to-java *icslas-features*))
            (process-icslas-sources 
             *icslas-features*
             *icslas-processor* )
            (if automatic?
                (compiler-main (cdr options)) ) )
       
           ;; Unrecognized option.
           (else 
            (runxla-error "Unrecognized option" (car options)) ) ) ) ) ) )

;;; }}}
;;; {{{ Miscellaneous

;;; Report a problem. Force a portable error to stop the evaluation.
;;; This error must be subtle enough or some compilers will complain
;;; at compile-time. This one supposes error messages to be short.

(define (runxla-error message . hints)
  (newline)
  (display ";;;;; ERROR in Icslas: ")
  (display message)
  (display " ")
  (if (pair? hints) (write hints))
  (newline)
  ;; Force an error!
  (display ";;; Please, ignore next error!")
  (newline)
  (string-ref message 987654) )

;;; Display banner.

(define (runxla-banner interpreter)
  (display ";;; Icslas multi-system $Revision: 1.4 $ [")
  (display interpreter)
  (display "]")
  (newline)
  (display ";;; (c) C. Queinnec, 1994, 1995, 1996.")
  (newline) )

;;; }}}
;;; end of runxla.scm
